@@ -1,6 +1,6 @@
 /* FSNBrowser.m
  *  
- * Copyright (C) 2004 Free Software Foundation, Inc.
+ * Copyright (C) 2004-2010 Free Software Foundation, Inc.
  *
  * Author: Enrico Sersale <enrico@imago.ro>
  * Date: July 2004
@@ -19,19 +19,23 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
  */
 
-#include <Foundation/Foundation.h>
-#include <AppKit/AppKit.h>
+
 #include <math.h>
-#include "FSNBrowser.h"
-#include "FSNBrowserColumn.h"
-#include "FSNBrowserMatrix.h"
-#include "FSNBrowserCell.h"
-#include "FSNIcon.h"
-#include "FSNFunctions.h"
-#include "GNUstep.h"
+#include <unistd.h>
+
+#import <Foundation/Foundation.h>
+#import <AppKit/AppKit.h>
+#import <GNUstepBase/GNUstep.h>
+#import "FSNBrowser.h"
+#import "FSNBrowserColumn.h"
+#import "FSNBrowserMatrix.h"
+#import "FSNBrowserCell.h"
+#import "FSNIcon.h"
+#import "FSNFunctions.h"
+
 
 #define DEFAULT_ISIZE 24
 
@@ -45,12 +49,12 @@
 - (void)dealloc
 {
   RELEASE (baseNode);
-  TEST_RELEASE (extInfoType);  
-  TEST_RELEASE (lastSelection);
+  RELEASE (extInfoType);  
+  RELEASE (lastSelection);
   RELEASE (columns);
-  TEST_RELEASE (nameEditor);
+  RELEASE (nameEditor);
   RELEASE (cellPrototype);
-  TEST_RELEASE (charBuffer);
+  RELEASE (charBuffer);
   RELEASE (backColor);
 
   [super dealloc];
@@ -71,7 +75,9 @@
     NSString *selName = [defaults stringForKey: @"DesktopApplicationSelName"];
     id defentry;
 		int i;
-
+    
+    fsnodeRep = [FSNodeRep sharedInstance];
+    
     if (appName && selName) {
 		  Class desktopAppClass = [[NSBundle mainBundle] classNamed: appName];
       SEL sel = NSSelectorFromString(selName);
@@ -88,7 +94,7 @@
       defentry = [defaults objectForKey: @"extended_info_type"];
 
       if (defentry) {
-        NSArray *availableTypes = [[FSNodeRep sharedInstance] availableExtendedInfoNames];
+        NSArray *availableTypes = [fsnodeRep availableExtendedInfoNames];
       
         if ([availableTypes containsObject: defentry]) {
           ASSIGN (extInfoType, defentry);
@@ -161,7 +167,7 @@
   ASSIGN (baseNode, [FSNode nodeWithPath: [node path]]);
   [self readNodeInfo];
   [self loadColumnZero];
-  [self notifySelectionChange: [NSArray arrayWithObject: [node path]]];
+  [self notifySelectionChange: [NSArray arrayWithObject: node]];
 }
 
 - (void)setUsesCellsIcons:(BOOL)cicns
@@ -294,12 +300,32 @@
 - (void)showSelection:(NSArray *)selection
 {
   if (selection && [selection count]) {
+    FSNode *node = [selection objectAtIndex: 0];
     FSNBrowserColumn *bc;
-    NSArray *selPaths;
+    NSArray *selNodes;
     
     updateViewsLock++;
-
-    [self showSubnode: [selection objectAtIndex: 0]];
+    
+    if ([selection count] > 1) {
+      BOOL alldirs = YES;
+      int i;
+      
+      for (i = 0; i < [selection count]; i++) {
+        FSNode *nd = [selection objectAtIndex: i];  
+        
+        if ([nd isDirectory] == NO) {
+          node = nd;
+          alldirs = NO;
+          break;
+        }
+      }
+    
+      if (alldirs) {
+        node = [FSNode nodeWithPath: [node parentPath]];
+      }
+    }
+    
+    [self showSubnode: node];
 
     bc = [self lastLoadedColumn];
     [bc selectCellsOfNodes: selection sendAction: NO];
@@ -320,11 +346,11 @@
     updateViewsLock--;
     [self tile];
 
-    selPaths = [bc selectedPaths];
-    if (selPaths == nil) {
-      selPaths = [NSArray arrayWithObject: [[bc shownNode] path]];
+    selNodes = [bc selectedNodes];
+    if (selNodes == nil) {
+      selNodes = [NSArray arrayWithObject: [bc shownNode]];
     }
-    [self notifySelectionChange: selPaths];
+    [self notifySelectionChange: selNodes];
   }
 }
 
@@ -333,10 +359,29 @@
   if (selpaths && [selpaths count]) {
     FSNode *node = [FSNode nodeWithPath: [selpaths objectAtIndex: 0]];      
     FSNBrowserColumn *bc;
-    NSArray *selPaths;
+    NSArray *selNodes;
 
     updateViewsLock++;
 
+    if ([selpaths count] > 1) {
+      BOOL alldirs = YES;
+      int i;
+      
+      for (i = 0; i < [selpaths count]; i++) {
+        FSNode *nd = [FSNode nodeWithPath: [selpaths objectAtIndex: i]];
+        
+        if ([nd isDirectory] == NO) {
+          node = nd;
+          alldirs = NO;
+          break;
+        }
+      }
+    
+      if (alldirs) {
+        node = [FSNode nodeWithPath: [node parentPath]];
+      }
+    }
+
     [self showSubnode: node];
 
     bc = [self lastLoadedColumn];
@@ -356,11 +401,11 @@
     updateViewsLock--;
     [self tile];
 
-    selPaths = [bc selectedPaths];
-    if (selPaths == nil) {
-      selPaths = [NSArray arrayWithObject: [[bc shownNode] path]];
+    selNodes = [bc selectedNodes];
+    if (selNodes == nil) {
+      selNodes = [NSArray arrayWithObject: [bc shownNode]];
     }
-    [self notifySelectionChange: selPaths];
+    [self notifySelectionChange: selNodes];
   }
 }
 
@@ -506,6 +551,7 @@
 
 - (void)reloadFromColumn:(FSNBrowserColumn *)col
 {
+  CREATE_AUTORELEASE_POOL(arp);
 	int index = [col index];
   int i = 0;
 
@@ -532,7 +578,7 @@
       int last = (i > 0) ? i - 1 : 0;
       int shift = 0;
       int leftscr = 0;
-
+    
       if (last >= visibleColumns) {
         if (last < firstVisibleColumn) {
           shift = visibleColumns - 1;
@@ -555,14 +601,11 @@
       break;
     }
 	}
-  
-  updateViewsLock--;
-  [self tile];
-  
+
   col = [self lastLoadedColumn];
 
   if (col) {
-    NSArray *selection = [col selectedPaths];
+    NSArray *selection = [col selectedNodes];
     int index = [col index];
 
     if (index < firstVisibleColumn) {
@@ -572,13 +615,30 @@
     [[self window] makeFirstResponder: [col cmatrix]];
 
     if (selection) {
+      if (selColumn && (index == lastColumnLoaded)) {
+        if ([selection count] == 1) {
+          FSNode *node = [selection objectAtIndex: 0];
+        
+          if (([node isDirectory] == NO) || [node isPackage]) {
+            [self addFillingColumn];
+          }
+        
+        } else {
+          [self addFillingColumn];
+        }
+      }
+    
       [self notifySelectionChange: selection];	
-
+    
     } else {
       FSNode *node = [col shownNode];
-      [self notifySelectionChange: [NSArray arrayWithObject: [node path]]];
+      [self notifySelectionChange: [NSArray arrayWithObject: node]];
     }
   }
+
+  updateViewsLock--;
+  [self tile];
+  RELEASE (arp);
 }
 
 - (void)reloadFromColumnWithNode:(FSNode *)anode
@@ -608,9 +668,11 @@
 
 - (void)tile
 {
+  updateViewsLock = (updateViewsLock < 0) ? 0 : updateViewsLock;
+
   if (updateViewsLock == 0) {
     NSWindow *window = [self window];
-    NSRect r = [self frame];
+    NSRect r = [self bounds];
     float frameWidth = r.size.width - visibleColumns;
     int count = [columns count];
     NSRect colrect;
@@ -659,6 +721,7 @@
 - (void)scrollViaScroller:(NSScroller *)sender
 {
   NSScrollerPart hit = [sender hitPart];
+  BOOL needsDisplay = NO;
   
   updateViewsLock++;
   skipUpdateScroller = YES;
@@ -678,6 +741,7 @@
 		case NSScrollerIncrementLine:
 		case NSScrollerIncrementPage:
       [self scrollColumnsRightBy: 1];
+      needsDisplay = YES;
 			break;
       
 		// The knob or knob slot
@@ -693,6 +757,8 @@
           [self setLastColumn: (lastColumnLoaded - currentshift)];			
 		      currentshift = 0;
         }
+        
+        needsDisplay = YES;
 			}
       break;
       
@@ -704,6 +770,7 @@
   
   updateViewsLock--;
   [self tile];
+  [self setNeedsDisplay: needsDisplay];
 } 
 
 - (void)updateScroller
@@ -825,7 +892,7 @@
 	} else {
     NSMatrix *matrix = [selCol cmatrix];
     
-    if (matrix) {
+    if ([[matrix cells] count]) {
       int index = [selCol index];
       
       [matrix sendAction];
@@ -833,7 +900,8 @@
       if (index < ([columns count] - 1)) {
         selCol = [columns objectAtIndex: index + 1];
         matrix = [selCol cmatrix];
-        if (matrix) {
+        
+        if ([[matrix cells] count]) {
           if ([selCol selectFirstCell]) {
             [matrix sendAction];  
             [[self window] makeFirstResponder: matrix];
@@ -874,7 +942,12 @@
 - (NSString *)pathToLastColumn
 {
   FSNode *node = [self nodeOfLastColumn];
-  return node ? [node path] : nil;
+
+  if (node) {
+    return [node path];
+  }
+  
+  return nil;
 }
 
 - (NSArray *)selectionInColumnBeforeColumn:(FSNBrowserColumn *)col
@@ -1044,6 +1117,17 @@
   return nil;
 }
 
+- (void)clickInColumn:(FSNBrowserColumn *)col
+{
+  if (viewer) {
+    NSArray *selection = [col selectedNodes];
+
+    if (selection && [selection count]) {
+      [viewer multipleNodeViewDidSelectSubNode: [col shownNode]];
+    }
+  }
+}
+
 - (void)clickInMatrixOfColumn:(FSNBrowserColumn *)col
 {
   int index = [col index];
@@ -1053,7 +1137,7 @@
   NSArray *selection = [col selectedNodes];
   
   if ((selection == nil) || ([selection count] == 0)) {
-    [self notifySelectionChange: [NSArray arrayWithObject: [[col shownNode] path]]];
+    [self notifySelectionChange: [NSArray arrayWithObject: [col shownNode]]];
     return;
   }
 
@@ -1109,7 +1193,7 @@
   updateViewsLock--;
   [self tile];
   
-  [self notifySelectionChange: [col selectedPaths]];		  
+  [self notifySelectionChange: [col selectedNodes]];		  
 }
 
 - (void)doubleClickInMatrixOfColumn:(FSNBrowserColumn *)col
@@ -1222,10 +1306,15 @@
 	    } else {
 	      if (([theEvent timestamp] - lastKeyPressed < 500.0)
 		  											      && (alphaNumericalLastColumn == index)) {
-		  		ASSIGN (charBuffer, ([charBuffer stringByAppendingString:
-				    																[characters substringToIndex: 1]]));
+          NSString *transition = [charBuffer stringByAppendingString:
+				                                      [characters substringToIndex: 1]];
+		      RELEASE (charBuffer);
+		      charBuffer = transition;
+		      RETAIN (charBuffer);
 				} else {
-		  		ASSIGN (charBuffer, ([characters substringToIndex: 1]));
+		      RELEASE (charBuffer);
+		      charBuffer = [characters substringToIndex: 1];
+		      RETAIN (charBuffer);
 				}														
 			}
 			
@@ -1262,7 +1351,7 @@
     matrix = [selCol cmatrix];
 	}
 	
-  if (matrix) {
+  if ([[matrix cells] count]) {
     [[self window] makeFirstResponder: matrix];
 	}
 	
@@ -1326,7 +1415,8 @@
   NSString *prefsname = [NSString stringWithFormat: @"viewer_at_%@", [baseNode path]];
   NSDictionary *nodeDict = nil;
 
-  if ([baseNode isWritable]) {
+  if ([baseNode isWritable]
+        && ([[fsnodeRep volumes] containsObject: [baseNode path]] == NO)) {
     NSString *infoPath = [[baseNode path] stringByAppendingPathComponent: @".gwdir"];
   
     if ([[NSFileManager defaultManager] fileExistsAtPath: infoPath]) {
@@ -1355,7 +1445,7 @@
       entry = [nodeDict objectForKey: @"ext_info_type"];
 
       if (entry) {
-        NSArray *availableTypes = [[FSNodeRep sharedInstance] availableExtendedInfoNames];
+        NSArray *availableTypes = [fsnodeRep availableExtendedInfoNames];
 
         if ([availableTypes containsObject: entry]) {
           ASSIGN (extInfoType, entry);
@@ -1371,15 +1461,18 @@
   return nodeDict;
 }
 
-- (void)updateNodeInfo
+- (NSMutableDictionary *)updateNodeInfo:(BOOL)ondisk
 {
+  CREATE_AUTORELEASE_POOL(arp);
+  NSMutableDictionary *updatedInfo = nil;
+
   if ([baseNode isValid]) {
     NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];	      
     NSString *prefsname = [NSString stringWithFormat: @"viewer_at_%@", [baseNode path]];
     NSString *infoPath = [[baseNode path] stringByAppendingPathComponent: @".gwdir"];
-    NSMutableDictionary *updatedInfo = nil;
-
-    if ([baseNode isWritable]) {
+    BOOL writable = ([baseNode isWritable] && ([[fsnodeRep volumes] containsObject: [baseNode path]] == NO));
+    
+    if (writable) {
       if ([[NSFileManager defaultManager] fileExistsAtPath: infoPath]) {
         NSDictionary *dict = [NSDictionary dictionaryWithContentsOfFile: infoPath];
 
@@ -1406,15 +1499,19 @@
     if (infoType == FSNInfoExtendedType) {
       [updatedInfo setObject: extInfoType forKey: @"ext_info_type"];
     }
-    
-    if ([baseNode isWritable]) {
-      [updatedInfo writeToFile: infoPath atomically: YES];
-    } else {
-      [defaults setObject: updatedInfo forKey: prefsname];
-    }
 
-    RELEASE (updatedInfo);
+    if (ondisk) {    
+      if (writable) {
+        [updatedInfo writeToFile: infoPath atomically: YES];
+      } else {
+        [defaults setObject: updatedInfo forKey: prefsname];
+      }
+    }
   }
+      
+  RELEASE (arp);
+  
+  return (AUTORELEASE (updatedInfo));
 }
 
 - (void)reloadContents
@@ -1498,6 +1595,7 @@
         || [operation isEqual: @"NSWorkspaceDuplicateOperation"]
         || [operation isEqual: @"GWorkspaceCreateDirOperation"]
         || [operation isEqual: @"GWorkspaceCreateFileOperation"]
+        || [operation isEqual: @"NSWorkspaceRecycleOperation"]
         || [operation isEqual: @"GWorkspaceRenameOperation"]
 			  || [operation isEqual: @"GWorkspaceRecycleOutOperation"]) { 
     FSNBrowserColumn *bc = [self columnWithPath: destination];        
@@ -1516,8 +1614,6 @@
           NSString *newname = [files objectAtIndex: 0];
           NSString *newpath = [destination stringByAppendingPathComponent: newname];
           
-          NSLog(@"newpath %@", newpath);
-          
           selectCell = ([bc cellWithPath: newpath] != nil);
         }
         
@@ -1546,16 +1642,32 @@
   NSString *event = [info objectForKey: @"event"];
   NSString *path = [info objectForKey: @"path"];
 
-  if ([event isEqual: @"GWWatchedDirectoryDeleted"]) {
+  if ([event isEqual: @"GWWatchedPathDeleted"]) {
     NSString *s = [path stringByDeletingLastPathComponent];
 
     if ([self isShowingPath: s]) {
       [self reloadFromColumnWithPath: s]; 
     }
     
-  } else if ([event isEqual: @"GWFileDeletedInWatchedDirectory"]) {
-    [self removeCellsWithNames: [info objectForKey: @"files"]
-              inColumnWithPath: path];
+  } else if ([event isEqual: @"GWFileDeletedInWatchedDirectory"]) {    
+    if ([self isShowingPath: path]) {
+      FSNBrowserColumn *col;
+    
+      [self reloadFromColumnWithPath: path]; 
+       
+      col = [self lastLoadedColumn];
+
+      if (col) {
+        NSArray *selection = [col selectedNodes];
+        
+        if (selection == nil) {
+          selection = [NSArray arrayWithObject: [col shownNode]];
+        }
+        
+        [viewer selectionChanged: selection];
+        [self synchronizeViewer];
+      }  
+    }      
                  
   } else if ([event isEqual: @"GWFileCreatedInWatchedDirectory"]) {   
     [self addCellsWithNames: [info objectForKey: @"files"]
@@ -1733,7 +1845,7 @@
   
   if (bc) {
     [[bc cmatrix] deselectAllCells];
-    [self notifySelectionChange: [NSArray arrayWithObject: [[bc shownNode] path]]];
+    [self notifySelectionChange: [NSArray arrayWithObject: [bc shownNode]]];
   }
 }
 
@@ -1898,7 +2010,7 @@
 - (void)restoreLastSelection
 {
   if (lastSelection) {
-    [self selectRepsOfPaths: lastSelection];
+    [self selectRepsOfSubnodes: lastSelection];
   }
 }
 
@@ -1914,12 +2026,12 @@
     if (prev) {
       if ([prev selectCellOfNode: anode sendAction: YES] == nil) {
         [self setLastColumn: [prev index]];
-        [self notifySelectionChange: [NSArray arrayWithObject: [[prev shownNode] path]]];
+        [self notifySelectionChange: [NSArray arrayWithObject: [prev shownNode]]];
       }
     } else {
       [self setLastColumn: 0];
       [bc unselectAllCells];
-      [self notifySelectionChange: [NSArray arrayWithObject: [baseNode path]]];
+      [self notifySelectionChange: [NSArray arrayWithObject: baseNode]];
     }
     
     updateViewsLock--;
@@ -2023,7 +2135,7 @@
     if (canedit) {   
       NSMatrix *matrix = [col cmatrix];
       NSFont *edfont = [nameEditor font];
-      float fnheight = [edfont defaultLineHeightForFont];
+      float fnheight = [fsnodeRep heighOfFont: edfont];
       NSRect r = [cell labelRect];
       
       r = [matrix convertRect: r toView: self];
@@ -2053,7 +2165,6 @@
     [nameEditor setNode: nil stringValue: @"" index: -1];
     [nameEditor removeFromSuperview];
     [self setNeedsDisplayInRect: [nameEditor frame]];
-    [[NSCursor arrowCursor] set];
   }
 }
 
@@ -2068,90 +2179,104 @@
 
 - (void)controlTextDidEndEditing:(NSNotification *)aNotification
 {
-  NSFileManager *fm = [NSFileManager defaultManager];
   FSNode *ednode = [nameEditor node];
+  BOOL writable = [ednode isWritable];
 
 #define CLEAREDITING \
   [self stopCellEditing]; \
   return 
     
-  if ([ednode isWritable] == NO) {
-    NSRunAlertPanel(NSLocalizedString(@"Error", @""), 
-          [NSString stringWithFormat: @"%@\"%@\"!\n", 
-              NSLocalizedString(@"You have not write permission for ", @""), 
-                    [ednode name]], NSLocalizedString(@"Continue", @""), nil, nil);   
-    CLEAREDITING;
+  if (writable == NO) {
+    /* check for broken symlink */     
+    if ([ednode isLink] && ([ednode hasValidPath] == NO)) { 
+      BOOL iamRoot;
+      
+      #ifdef __WIN32__
+		    iamRoot = YES;
+	    #else
+		    iamRoot = (geteuid() == 0);
+	    #endif
+      
+      writable = (iamRoot || [[ednode owner] isEqual: NSUserName()]);          
+    }
     
-  } else if ([fm isWritableFileAtPath: [ednode parentPath]] == NO) {
-    NSRunAlertPanel(NSLocalizedString(@"Error", @""), 
-          [NSString stringWithFormat: @"%@\"%@\"!\n", 
-              NSLocalizedString(@"You have not write permission for ", @""), 
-                  [ednode parentName]], NSLocalizedString(@"Continue", @""), nil, nil);   
-    CLEAREDITING;
-    
-  } else {
-    NSString *newname = [nameEditor stringValue];
-    NSString *newpath = [[ednode parentPath] stringByAppendingPathComponent: newname];
-    NSCharacterSet *notAllowSet = [NSCharacterSet characterSetWithCharactersInString: @"/\\*:?"];
-    NSRange range = [newname rangeOfCharacterFromSet: notAllowSet];
-    NSFileManager *fm = [NSFileManager defaultManager];    
-    NSArray *dirContents = [fm directoryContentsAtPath: [ednode parentPath]];
-    NSMutableDictionary *userInfo = [NSMutableDictionary dictionary];
-    
-    if (range.length > 0) {
+    if (writable == NO) {
       NSRunAlertPanel(NSLocalizedString(@"Error", @""), 
-                NSLocalizedString(@"Invalid char in name", @""), 
-                          NSLocalizedString(@"Continue", @""), nil, nil);   
+            [NSString stringWithFormat: @"%@\"%@\"!\n", 
+                NSLocalizedString(@"You do not have write permission for ", @""), 
+                      [ednode name]], NSLocalizedString(@"Continue", @""), nil, nil);   
+      CLEAREDITING;
+    }
+  } 
+  
+  if (writable) {  
+    if ([ednode isParentWritable] == NO) {
+      NSRunAlertPanel(NSLocalizedString(@"Error", @""), 
+            [NSString stringWithFormat: @"%@\"%@\"!\n", 
+                NSLocalizedString(@"You do not have write permission for ", @""), 
+                    [ednode parentName]], NSLocalizedString(@"Continue", @""), nil, nil);   
       CLEAREDITING;
-    }	
 
-    if ([dirContents containsObject: newname]) {
-      if ([newname isEqual: [ednode name]]) {
-        CLEAREDITING;
-      } else {
+    } else if ([ednode isSubnodeOfPath: [desktopApp trashPath]]) {
+      NSRunAlertPanel(NSLocalizedString(@"Error", @""), 
+              NSLocalizedString(@"You can't rename an object that is in the Recycler", @""), 
+              NSLocalizedString(@"Continue", @""), nil, nil);   
+      CLEAREDITING;
+
+    } else {
+      NSString *newname = [nameEditor stringValue];
+      NSString *newpath = [[ednode parentPath] stringByAppendingPathComponent: newname];
+      NSString *extension = [newpath pathExtension];
+      NSCharacterSet *notAllowSet = [NSCharacterSet characterSetWithCharactersInString: @"/\\*:?\33"];
+      NSRange range = [newname rangeOfCharacterFromSet: notAllowSet];
+      NSArray *dirContents = [ednode subNodeNamesOfParent];
+      NSMutableDictionary *opinfo = [NSMutableDictionary dictionary];
+
+      if (([newname length] == 0) || (range.length > 0)) {
         NSRunAlertPanel(NSLocalizedString(@"Error", @""), 
-          [NSString stringWithFormat: @"%@\"%@\" %@ ", 
-              NSLocalizedString(@"The name ", @""), 
-              newname, NSLocalizedString(@" is already in use!", @"")], 
+                  NSLocalizedString(@"Invalid name", @""), 
                             NSLocalizedString(@"Continue", @""), nil, nil);   
         CLEAREDITING;
+      }	
+
+      if (([extension length] 
+              && ([ednode isDirectory] && ([ednode isPackage] == NO)))) {
+        NSString *msg = NSLocalizedString(@"Are you sure you want to add the extension ", @"");
+
+        msg = [msg stringByAppendingFormat: @"\"%@\" ", extension];
+        msg = [msg stringByAppendingString: NSLocalizedString(@"to the end of the name?", @"")];
+        msg = [msg stringByAppendingString: NSLocalizedString(@"\nif you make this change, your folder may appear as a single file.", @"")];
+
+        if (NSRunAlertPanel(@"", msg, 
+                            NSLocalizedString(@"Cancel", @""), 
+				                    NSLocalizedString(@"OK", @""), 
+                            nil) == NSAlertDefaultReturn) {
+          CLEAREDITING;
+        }
       }
+
+      if ([dirContents containsObject: newname]) {
+        if ([newname isEqual: [ednode name]]) {
+          CLEAREDITING;
+        } else {
+          NSRunAlertPanel(NSLocalizedString(@"Error", @""), 
+            [NSString stringWithFormat: @"%@\"%@\" %@ ", 
+                NSLocalizedString(@"The name ", @""), 
+                newname, NSLocalizedString(@" is already in use!", @"")], 
+                              NSLocalizedString(@"Continue", @""), nil, nil);   
+          CLEAREDITING;
+        }
+      }
+
+	    [opinfo setObject: @"GWorkspaceRenameOperation" forKey: @"operation"];	
+      [opinfo setObject: [ednode path] forKey: @"source"];	
+      [opinfo setObject: newpath forKey: @"destination"];	
+      [opinfo setObject: [NSArray arrayWithObject: @""] forKey: @"files"];	
+
+      [self stopCellEditing];
+      [desktopApp performFileOperation: opinfo];
     }
-
-	  [userInfo setObject: @"GWorkspaceRenameOperation" forKey: @"operation"];	
-    [userInfo setObject: [ednode path] forKey: @"source"];	
-    [userInfo setObject: newpath forKey: @"destination"];	
-    [userInfo setObject: [NSArray arrayWithObject: @""] forKey: @"files"];	
-    
-//    [[NSDistributedNotificationCenter defaultCenter]
-// 				postNotificationName: @"GWFileSystemWillChangeNotification"
-//	 								    object: nil 
-//                    userInfo: userInfo];
-
-    [fm movePath: [ednode path] toPath: newpath handler: self];
-
-    [[NSDistributedNotificationCenter defaultCenter]
- 				postNotificationName: @"GWFileSystemDidChangeNotification"
-	 								    object: nil 
-                    userInfo: userInfo];
   }
 }
 
-- (BOOL)fileManager:(NSFileManager *)manager 
-              shouldProceedAfterError:(NSDictionary *)errorDict
-{
-	NSString *title = NSLocalizedString(@"Error", @"");
-	NSString *msg1 = NSLocalizedString(@"Cannot rename ", @"");
-  NSString *name = [[nameEditor node] name];
-	NSString *msg2 = NSLocalizedString(@"Continue", @"");
-
-  NSRunAlertPanel(title, [NSString stringWithFormat: @"%@'%@'!", msg1, name], msg2, nil, nil);   
-
-	return NO;
-}
-
-- (void)fileManager:(NSFileManager *)manager willProcessPath:(NSString *)path
-{
-}
-
 @end
