@@ -1,6 +1,6 @@
 /* FSNBrowserCell.m
  *  
- * Copyright (C) 2004 Free Software Foundation, Inc.
+ * Copyright (C) 2004-2010 Free Software Foundation, Inc.
  *
  * Author: Enrico Sersale <enrico@imago.ro>
  * Date: March 2004
@@ -19,14 +19,16 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
  */
 
-#include <Foundation/Foundation.h>
-#include <AppKit/AppKit.h>
 #include <math.h>
-#include "FSNBrowserCell.h"
-#include "FSNode.h"
+
+#import <Foundation/Foundation.h>
+#import <AppKit/AppKit.h>
+
+#import "FSNBrowserCell.h"
+#import "FSNode.h"
 
 #define DEFAULT_ISIZE (24)
 #define HLIGHT_H_FACT (0.8125)
@@ -40,18 +42,21 @@ static NSDictionary *fontAttr = nil;
 static NSFont *infoFont = nil;
 static int infoheight = 0;
 
+static SEL cutTitleSel = NULL;
+static cutIMP cutTitle = NULL;
+
 
 @implementation FSNBrowserCell
 
 - (void)dealloc
 {
-  TEST_RELEASE (selection);
-  TEST_RELEASE (selectionTitle);
-  TEST_RELEASE (uncuttedTitle);
-  TEST_RELEASE (extInfoType);
-  TEST_RELEASE (infoCell); 
-  TEST_RELEASE (icon); 
-  TEST_RELEASE (openicon); 
+  RELEASE (selection);
+  RELEASE (selectionTitle);
+  RELEASE (uncuttedTitle);
+  RELEASE (extInfoType);
+  RELEASE (infoCell); 
+  RELEASE (icon); 
+  RELEASE (selectedicon); 
   RELEASE (dots);
   
   [super dealloc];
@@ -59,26 +64,35 @@ static int infoheight = 0;
 
 + (void)initialize
 {
-  if (desktopApp == nil) {
-    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
-    NSString *appName = [defaults stringForKey: @"DesktopApplicationName"];
-    NSString *selName = [defaults stringForKey: @"DesktopApplicationSelName"];
+  static BOOL initialized = NO;
 
-    if (appName && selName) {
-		  Class desktopAppClass = [[NSBundle mainBundle] classNamed: appName];
-      SEL sel = NSSelectorFromString(selName);
-      desktopApp = [desktopAppClass performSelector: sel];
+  if (initialized == NO) {
+    cutTitleSel = @selector(cutTitle:toFitWidth:);
+    cutTitle = (cutIMP)[self instanceMethodForSelector: cutTitleSel];   
+  
+    if (desktopApp == nil) {
+      NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
+      NSString *appName = [defaults stringForKey: @"DesktopApplicationName"];
+      NSString *selName = [defaults stringForKey: @"DesktopApplicationSelName"];
+
+      if (appName && selName) {
+		    Class desktopAppClass = [[NSBundle mainBundle] classNamed: appName];
+        SEL sel = NSSelectorFromString(selName);
+        desktopApp = [desktopAppClass performSelector: sel];
+      }
+
+      fontAttr = [NSDictionary dictionaryWithObject: [NSFont systemFontOfSize: 12]
+                                             forKey: NSFontAttributeName];
+      RETAIN (fontAttr);
+      dtslenght = [dots sizeWithAttributes: fontAttr].width;     
+      infoFont = [NSFont systemFontOfSize: 10];
+      infoFont = [[NSFontManager sharedFontManager] convertFont: infoFont 
+                                                    toHaveTrait: NSItalicFontMask];
+      RETAIN (infoFont);
+      infoheight = floor([[FSNodeRep sharedInstance] heighOfFont: infoFont]);
     }
-    
-    fontAttr = [NSDictionary dictionaryWithObject: [NSFont systemFontOfSize: 12]
-                                           forKey: NSFontAttributeName];
-    RETAIN (fontAttr);
-    dtslenght = [dots sizeWithAttributes: fontAttr].width;     
-    infoFont = [NSFont systemFontOfSize: 10];
-    infoFont = [[NSFontManager sharedFontManager] convertFont: infoFont 
-                                                  toHaveTrait: NSItalicFontMask];
-    RETAIN (infoFont);
-    infoheight = floor([infoFont defaultLineHeightForFont]);
+   
+    initialized = YES;
   }
 }
 
@@ -86,17 +100,14 @@ static int infoheight = 0;
 {
   self = [super init];
   
-  if (self) {
-    cutTitleSel = @selector(cutTitle:toFitWidth:);
-    cutTitle = (cutIMP)[self methodForSelector: cutTitleSel]; 
-       
+  if (self) {       
     node = nil;
     selection = nil;
     selectionTitle = nil;
     showType = FSNInfoNameType;
     extInfoType = nil;
     icon = nil;
-    openicon = nil;
+    selectedicon = nil;
     icnsize = DEFAULT_ISIZE;
     
     isLocked = NO;
@@ -117,7 +128,7 @@ static int infoheight = 0;
   if (node) {
     ASSIGN (icon, [fsnodeRep iconOfSize: icnsize forNode: node]);
     icnh = [icon size].height;
-    DESTROY (openicon);
+    DESTROY (selectedicon);
   }
 }
 
@@ -135,12 +146,12 @@ static int infoheight = 0;
     return NO;
   }
   
-  if (openicon == nil) {
+  if (selectedicon == nil) {
     NSImage *opicn = [fsnodeRep openFolderIconOfSize: icnsize forNode: node];
 
     if (opicn) {
-      ASSIGN (openicon, opicn);
-      icnh = [openicon size].height;
+      ASSIGN (selectedicon, opicn);
+      icnh = [selectedicon size].height;
     }
   }
   
@@ -204,157 +215,159 @@ static int infoheight = 0;
 - (void)drawInteriorWithFrame:(NSRect)cellFrame 
 		                   inView:(NSView *)controlView
 {
-  NSColor *backcolor = [(NSMatrix *)controlView cellBackgroundColor];
-  float textlenght = cellFrame.size.width;
-  BOOL showsFirstResponder = [self showsFirstResponder];
-  NSString *cuttitle;  
-
 #define MARGIN (2.0)
 #define LEAF_MARGIN (5.0)
-  
-  titleRect = cellFrame;
-  
-  if (icon) {
-    textlenght -= ([icon size].width + (MARGIN * 2));
-  }
-  if ([self isLeaf]) {
-    textlenght -= LEAF_MARGIN; 
-  } else {
-    textlenght -= (LEAF_MARGIN * 3); 
-  }
-  
-  textlenght -= MARGIN;
-  ASSIGN (uncuttedTitle, [self stringValue]);
-  cuttitle = (*cutTitle)(self, cutTitleSel, uncuttedTitle, textlenght);
-  [self setStringValue: cuttitle];        
 
-  [self setShowsFirstResponder: NO];
+  NSWindow *cvwin = [controlView window];
 
-  if (icon == nil) {
-    if (nameEdited == NO) {
-      if (infoCell) {
-        if (([self isHighlighted] || [self state]) && (nameEdited == NO)) {
-	        [[self highlightColorInView: controlView] set];
-          NSRectFill(cellFrame);
+  if (cvwin) {
+    NSColor *backcolor = [cvwin backgroundColor];
+    float textlenght = cellFrame.size.width;
+    BOOL showsFirstResponder = [self showsFirstResponder];
+    NSString *cuttitle;  
+
+    titleRect = cellFrame;
+
+    if (icon) {
+      textlenght -= ([icon size].width + (MARGIN * 2));
+    }
+    if ([self isLeaf]) {
+      textlenght -= LEAF_MARGIN; 
+    } else {
+      textlenght -= (LEAF_MARGIN + 16); 
+    }
+
+    textlenght -= MARGIN;
+    ASSIGN (uncuttedTitle, [self stringValue]);
+    cuttitle = (*cutTitle)(self, cutTitleSel, uncuttedTitle, textlenght);
+    [self setStringValue: cuttitle];        
+
+    [self setShowsFirstResponder: NO];
+
+    if (icon == nil) {
+      if (nameEdited == NO) {
+        if (infoCell) {
+          if (([self isHighlighted] || [self state]) && (nameEdited == NO)) {
+	          [[self highlightColorInView: controlView] set];
+            NSRectFill(cellFrame);
+          }
+
+          titleRect.size.height -= infoheight;
+
+          if ([controlView isFlipped]) {
+            titleRect.origin.y += cellFrame.size.height;
+            titleRect.origin.y -= (titleRect.size.height + infoheight);
+          } else {
+            titleRect.origin.y += infoheight;
+          }
+
+          [super drawInteriorWithFrame: titleRect inView: controlView];
+
+        } else {
+          [super drawInteriorWithFrame: titleRect inView: controlView];
         }
-      
+
+      } else {
+        [backcolor set];
+        NSRectFill(cellFrame);
+      }
+
+      if (infoCell) {
+        infoRect = NSMakeRect(cellFrame.origin.x + 2, cellFrame.origin.y + 3,
+                                        cellFrame.size.width - 2, infoheight);
+
+        if ([controlView isFlipped]) {
+	        infoRect.origin.y += (cellFrame.size.height - infoRect.size.height);
+          infoRect.origin.y -= 6;
+        }
+
+        [infoCell drawInteriorWithFrame: infoRect inView: controlView];
+      } 
+
+    } else {
+      NSRect icon_rect;    
+
+      if (([self isHighlighted] || [self state]) && (nameEdited == NO)) {
+	      [[self highlightColorInView: controlView] set];
+        NSRectFill(cellFrame);
+      } 
+	    
+      if (infoCell) {
         titleRect.size.height -= infoheight;
-        
+
         if ([controlView isFlipped]) {
           titleRect.origin.y += cellFrame.size.height;
           titleRect.origin.y -= (titleRect.size.height + infoheight);
         } else {
           titleRect.origin.y += infoheight;
         }
-     
-        [super drawInteriorWithFrame: titleRect inView: controlView];
-
-      } else {
-        [super drawInteriorWithFrame: titleRect inView: controlView];
       }
-      
-    } else {
-      [backcolor set];
-      NSRectFill(cellFrame);
-    }
-    
-    if (infoCell) {
-      infoRect = NSMakeRect(cellFrame.origin.x + 2, cellFrame.origin.y + 3,
-                                      cellFrame.size.width - 2, infoheight);
+
+      icon_rect.origin = titleRect.origin;
+      icon_rect.size = NSMakeSize(icnsize, icnh);
+      icon_rect.origin.x += MARGIN;
+      icon_rect.origin.y += ((titleRect.size.height - icon_rect.size.height) / 2.0);
 
       if ([controlView isFlipped]) {
-	      infoRect.origin.y += (cellFrame.size.height - infoRect.size.height);
-        infoRect.origin.y -= 6;
+        if (infoCell) {
+          icon_rect.origin.y += cellFrame.size.height;
+          icon_rect.origin.y -= (titleRect.size.height + infoheight);
+        }
+
+	      icon_rect.origin.y += icon_rect.size.height;
       }
 
-      [infoCell drawInteriorWithFrame: infoRect inView: controlView];
-    } 
-    
-  } else {
-    NSRect icon_rect;    
+      titleRect.origin.x += (icon_rect.size.width + (MARGIN * 2));	
+      titleRect.size.width -= (icon_rect.size.width + (MARGIN * 2));	
 
-    if (([self isHighlighted] || [self state]) && (nameEdited == NO)) {
-	    [[self highlightColorInView: controlView] set];
-    } else {
-      [backcolor set];
-	  }
-	  NSRectFill(cellFrame);
-
-    if (infoCell) {
-      titleRect.size.height -= infoheight;
-
-      if ([controlView isFlipped]) {
-        titleRect.origin.y += cellFrame.size.height;
-        titleRect.origin.y -= (titleRect.size.height + infoheight);
-      } else {
-        titleRect.origin.y += infoheight;
+      if (nameEdited == NO) {        
+        [super drawInteriorWithFrame: titleRect inView: controlView];
       }
-    }
-  
-    icon_rect.origin = titleRect.origin;
-    icon_rect.size = NSMakeSize(icnsize, icnh);
-    icon_rect.origin.x += MARGIN;
-    icon_rect.origin.y += ((titleRect.size.height - icon_rect.size.height) / 2.0);
 
-    if ([controlView isFlipped]) {
       if (infoCell) {
-        icon_rect.origin.y += cellFrame.size.height;
-        icon_rect.origin.y -= (titleRect.size.height + infoheight);
+        infoRect = NSMakeRect(cellFrame.origin.x + 2, cellFrame.origin.y + 3,
+                                        cellFrame.size.width - 2, infoheight);
+
+        if ([controlView isFlipped]) {
+	        infoRect.origin.y += (cellFrame.size.height - infoRect.size.height);
+          infoRect.origin.y -= 6;
+        }
+
+        [infoCell drawInteriorWithFrame: infoRect inView: controlView];
       }
 
-	    icon_rect.origin.y += icon_rect.size.height;
-    }
-    
-    titleRect.origin.x += (icon_rect.size.width + (MARGIN * 2));	
-    titleRect.size.width -= (icon_rect.size.width + (MARGIN * 2));	
-    
-    if (nameEdited == NO) {        
-      [super drawInteriorWithFrame: titleRect inView: controlView];
-    }
+      [controlView lockFocus];
 
-    if (infoCell) {
-      infoRect = NSMakeRect(cellFrame.origin.x + 2, cellFrame.origin.y + 3,
-                                      cellFrame.size.width - 2, infoheight);
-
-      if ([controlView isFlipped]) {
-	      infoRect.origin.y += (cellFrame.size.height - infoRect.size.height);
-        infoRect.origin.y -= 6;
-      }
-
-      [infoCell drawInteriorWithFrame: infoRect inView: controlView];
-    }
-    
-    [controlView lockFocus];
-        
-    if ([self isEnabled]) {
-      if (iconSelected) {
-        if (isOpened == NO) {	
-          [openicon compositeToPoint: icon_rect.origin 
-	                         operation: NSCompositeSourceOver];
+      if ([self isEnabled]) {
+        if (iconSelected) {
+          if (isOpened == NO) {	
+            [selectedicon compositeToPoint: icon_rect.origin 
+	                           operation: NSCompositeSourceOver];
+          } else {
+            [selectedicon dissolveToPoint: icon_rect.origin fraction: 0.5];
+          }
         } else {
-          [openicon dissolveToPoint: icon_rect.origin fraction: 0.5];
+          if (isOpened == NO) {	
+            [icon compositeToPoint: icon_rect.origin 
+	                       operation: NSCompositeSourceOver];
+          } else {              
+            [icon dissolveToPoint: icon_rect.origin fraction: 0.5];
+          }
         }
       } else {
-        if (isOpened == NO) {	
-          [icon compositeToPoint: icon_rect.origin 
-	                     operation: NSCompositeSourceOver];
-        } else {              
-          [icon dissolveToPoint: icon_rect.origin fraction: 0.5];
-        }
+			  [icon dissolveToPoint: icon_rect.origin fraction: 0.3];
       }
-    } else {
-			[icon dissolveToPoint: icon_rect.origin fraction: 0.3];
+
+      [controlView unlockFocus];
     }
 
-    [controlView unlockFocus];
-  }
+    if (showsFirstResponder) {
+      [self setShowsFirstResponder: showsFirstResponder];
+      NSDottedFrameRect(cellFrame);
+    }
 
-  if (showsFirstResponder) {
-    [self setShowsFirstResponder: showsFirstResponder];
-    NSDottedFrameRect(cellFrame);
-  }
-
-  [self setStringValue: uncuttedTitle];          
+    [self setStringValue: uncuttedTitle]; 
+  }         
 }
 
 
@@ -592,16 +605,11 @@ static int infoheight = 0;
 
 - (void)setOpened:(BOOL)value
 {
-/*
-  NSLog(@"setOpened %i", value);
-
+  /* This was commented. (To know if something goes wrong) */
   if (isOpened == value) {
     return;
   }
   isOpened = value;
-  
-  NSLog(@"setOpened %@ %i", [node name], value);
-*/
 }
 
 - (BOOL)isOpened
@@ -637,37 +645,37 @@ static int infoheight = 0;
   return 0;
 }
 
-- (int)compareAccordingToName:(FSNBrowserCell *)aCell
+- (int)compareAccordingToName:(id)aCell
 {
   return [node compareAccordingToName: [aCell node]];
 }
 
-- (int)compareAccordingToKind:(FSNBrowserCell *)aCell
+- (int)compareAccordingToKind:(id)aCell
 {
   return [node compareAccordingToKind: [aCell node]];
 }
 
-- (int)compareAccordingToDate:(FSNBrowserCell *)aCell
+- (int)compareAccordingToDate:(id)aCell
 {
   return [node compareAccordingToDate: [aCell node]];
 }
 
-- (int)compareAccordingToSize:(FSNBrowserCell *)aCell
+- (int)compareAccordingToSize:(id)aCell
 {
   return [node compareAccordingToSize: [aCell node]];
 }
 
-- (int)compareAccordingToOwner:(FSNBrowserCell *)aCell
+- (int)compareAccordingToOwner:(id)aCell
 {
   return [node compareAccordingToOwner: [aCell node]];
 }
 
-- (int)compareAccordingToGroup:(FSNBrowserCell *)aCell
+- (int)compareAccordingToGroup:(id)aCell
 {
   return [node compareAccordingToGroup: [aCell node]];
 }
 
-- (int)compareAccordingToIndex:(FSNBrowserCell *)aCell
+- (int)compareAccordingToIndex:(id)aCell
 {
   return NSOrderedSame;
 }
@@ -679,7 +687,7 @@ static int infoheight = 0;
 
 - (void)dealloc
 {
-  TEST_RELEASE (node);
+  RELEASE (node);
   [super dealloc];
 }
 
@@ -705,7 +713,7 @@ static int infoheight = 0;
   return index;
 }
 
-- (void)mouseDown:(NSEvent*)theEvent
+- (void)mouseDown:(NSEvent *)theEvent
 {
   if ([self isEditable]) {
 	  [self setAlignment: NSLeftTextAlignment];
