@@ -1,6 +1,6 @@
 /* FSNBrowserColumn.m
  *  
- * Copyright (C) 2004 Free Software Foundation, Inc.
+ * Copyright (C) 2004-2010 Free Software Foundation, Inc.
  *
  * Author: Enrico Sersale <enrico@imago.ro>
  * Date: July 2004
@@ -19,17 +19,18 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
  */
 
-#include <AppKit/AppKit.h>
 #include <math.h>
-#include "FSNBrowserColumn.h"
-#include "FSNBrowserCell.h"
-#include "FSNBrowserMatrix.h"
-#include "FSNBrowserScroll.h"
-#include "FSNBrowser.h"
-#include "FSNFunctions.h"
+
+#import <AppKit/AppKit.h>
+#import "FSNBrowserColumn.h"
+#import "FSNBrowserCell.h"
+#import "FSNBrowserMatrix.h"
+#import "FSNBrowserScroll.h"
+#import "FSNBrowser.h"
+#import "FSNFunctions.h"
 
 #define ICON_CELL_HEIGHT 28
 
@@ -47,11 +48,10 @@ static id <DesktopApplication> desktopApp = nil;
 
 - (void)dealloc
 {
-  TEST_RELEASE (matrix);
   RELEASE (cellPrototype);
-  TEST_RELEASE (shownNode);
-  TEST_RELEASE (oldNode);
-  TEST_RELEASE (extInfoType);  
+  RELEASE (shownNode);
+  RELEASE (oldNode);
+  RELEASE (extInfoType);  
   RELEASE (backColor);
     
   [super dealloc];
@@ -59,16 +59,22 @@ static id <DesktopApplication> desktopApp = nil;
 
 + (void)initialize
 {
-  if (desktopApp == nil) {
-    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
-    NSString *appName = [defaults stringForKey: @"DesktopApplicationName"];
-    NSString *selName = [defaults stringForKey: @"DesktopApplicationSelName"];
+  static BOOL initialized = NO;
 
-    if (appName && selName) {
-		  Class desktopAppClass = [[NSBundle mainBundle] classNamed: appName];
-      SEL sel = NSSelectorFromString(selName);
-      desktopApp = [desktopAppClass performSelector: sel];
+  if (initialized == NO) {
+    if (desktopApp == nil) {
+      NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
+      NSString *appName = [defaults stringForKey: @"DesktopApplicationName"];
+      NSString *selName = [defaults stringForKey: @"DesktopApplicationSelName"];
+
+      if (appName && selName) {
+		    Class desktopAppClass = [[NSBundle mainBundle] classNamed: appName];
+        SEL sel = NSSelectorFromString(selName);
+        desktopApp = [desktopAppClass performSelector: sel];
+      }
     }
+  
+    initialized = YES;
   }
 }
 
@@ -82,48 +88,68 @@ static id <DesktopApplication> desktopApp = nil;
 {
   self = [super init];
   
-  if (self) {
-	  NSRect rect = NSMakeRect(0, 0, 150, 100);
-    int lineh = floor([[acell font] defaultLineHeightForFont]);
-                
-    browser = abrowser;
-    index = ind;
-    ASSIGN (cellPrototype, acell);
-    cellsIcon = cicon;
-    ASSIGN (backColor, acolor);
+  if (self)
+    {
+      NSRect rect = NSMakeRect(0, 0, 150, 100);
+      int lineh;
+               
+      browser = abrowser;
+      index = ind;
+      ASSIGN (cellPrototype, acell);
+      cellsIcon = cicon;
+      ASSIGN (backColor, acolor);
 
-    infoType = type;
-    extInfoType = nil;
-    if (exttype) {
-      ASSIGN (extInfoType, exttype);
-    }
+      infoType = type;
+      extInfoType = nil;
+      if (exttype)
+	ASSIGN (extInfoType, exttype);
 
-    shownNode = nil;
-    oldNode = nil;
-    matrix = nil;
-    isLoaded = NO;
+      shownNode = nil;
+      oldNode = nil;
+      scroll = nil;
+      matrix = nil;
+      isLoaded = NO;
     
-    [self setFrame: rect];
+      [self setFrame: rect];
     
-    fsnodeRep = [FSNodeRep sharedInstance];
+      fsnodeRep = [FSNodeRep sharedInstance];
+      lineh = floor([fsnodeRep heighOfFont: [acell font]]);
     
-    scroll = [[FSNBrowserScroll alloc] initWithFrame: rect 
-                                    inColumn: self acceptDnd: cellsIcon];
-    [self addSubview: scroll];
-    RELEASE (scroll);
+      scroll = [[FSNBrowserScroll alloc] initWithFrame: rect 
+					      inColumn: self acceptDnd: cellsIcon];
+      [self addSubview: scroll];
+      RELEASE (scroll);
     
-    if (cellsIcon) {
-      cellsHeight = ICON_CELL_HEIGHT;
-    } else {
-      cellsHeight = lineh;
+      if (cellsIcon)
+	cellsHeight = ICON_CELL_HEIGHT;
+      else
+	cellsHeight = lineh;
+    
+      if (infoType != FSNInfoNameType)
+	cellsHeight += (lineh +1);
+    
+      isDragTarget = NO;
+
+      matrix = [[FSNBrowserMatrix alloc] initInColumn: self 
+					    withFrame: [self bounds]
+						 mode: NSListModeMatrix 
+					    prototype: cellPrototype
+					 numberOfRows: 0 
+				      numberOfColumns: 0 
+					    acceptDnd: cellsIcon];
+
+      [matrix setIntercellSpacing: NSMakeSize(0, 0)];
+      [matrix setCellSize: NSMakeSize([scroll contentSize].width, cellsHeight)];  
+      [matrix setAutoscroll: YES];
+      [matrix setAllowsEmptySelection: YES];
+      [matrix setBackgroundColor: backColor];
+      [matrix setCellBackgroundColor: backColor];
+      [matrix setTarget: self];
+      [matrix setAction: @selector(doClick:)];
+      [matrix setDoubleAction: @selector(doDoubleClick:)];
+      [scroll setDocumentView: matrix];
+      RELEASE (matrix);
     }
-    
-    if (infoType != FSNInfoNameType) {
-      cellsHeight += (lineh +1);
-    }
-    
-    isDragTarget = NO;
-  }
   
   return self;
 }
@@ -131,7 +157,9 @@ static id <DesktopApplication> desktopApp = nil;
 - (void)setShowType:(FSNInfoType)type
 {
   if (infoType != type) {
-    int lineh = floor([[cellPrototype font] defaultLineHeightForFont]);
+    NSArray *cells = [matrix cells];
+    int lineh = floor([fsnodeRep heighOfFont: [cellPrototype font]]);
+    int i;
     
     infoType = type;
     DESTROY (extInfoType);
@@ -146,24 +174,21 @@ static id <DesktopApplication> desktopApp = nil;
       cellsHeight += (lineh +1);
     }
     
-    if (matrix) {
-      NSArray *cells = [matrix cells];
-      int i;
+    [self adjustMatrix];
 
-      [self adjustMatrix];
-
-	    for (i = 0; i < [cells count]; i++) {
-        [[cells objectAtIndex: i] setNodeInfoShowType: infoType];
-      }
-    }   
+	  for (i = 0; i < [cells count]; i++) {
+      [[cells objectAtIndex: i] setNodeInfoShowType: infoType];
+    }
   }
 }
 
 - (void)setExtendedShowType:(NSString *)type
 {
   if ((extInfoType == nil) || ([extInfoType isEqual: type] == NO)) {
-    int lineh = floor([[cellPrototype font] defaultLineHeightForFont]);  
-  
+    NSArray *cells = [matrix cells];    
+    int lineh = floor([fsnodeRep heighOfFont: [cellPrototype font]]);  
+    int i;
+    
     infoType = FSNInfoExtendedType;
     ASSIGN (extInfoType, type);
 
@@ -175,17 +200,12 @@ static id <DesktopApplication> desktopApp = nil;
 
     cellsHeight += (lineh +1);
 
-    if (matrix) {
-      NSArray *cells = [matrix cells];
-      int i;
-      
-      [self adjustMatrix];
-      
-	    for (i = 0; i < [cells count]; i++) {
-        FSNBrowserCell *cell = [cells objectAtIndex: i];
-        [cell setExtendedShowType: extInfoType];
-      }
-    }   
+    [self adjustMatrix];
+
+	  for (i = 0; i < [cells count]; i++) {
+      FSNBrowserCell *cell = [cells objectAtIndex: i];
+      [cell setExtendedShowType: extInfoType];
+    }       
   }
 }
 
@@ -197,30 +217,26 @@ static id <DesktopApplication> desktopApp = nil;
   int i = 0;
 
   if (oldNode && anode && [oldNode isEqualToNode: anode] && [anode isValid]) {
+    NSArray *vnodes = nil;  
+    
     savedSelection = [self selectedNodes];
     
     if (savedSelection) {
       RETAIN (savedSelection);
     }
     
-    if (matrix) {
-      NSArray *vnodes = nil;
+    [matrix visibleCellsNodes: &vnodes scrollTuneSpace: &scrollTune];
 
-      [matrix visibleCellsNodes: &vnodes scrollTuneSpace: &scrollTune];
-    
-      if (vnodes) {
-        visibleNodes = [NSMutableArray new];
-        [visibleNodes addObjectsFromArray: vnodes];
-      }
-    }
+    if (vnodes) {
+      visibleNodes = [NSMutableArray new];
+      [visibleNodes addObjectsFromArray: vnodes];
+    }    
   }
-    
-  if (matrix) {
-    [matrix removeFromSuperviewWithoutNeedingDisplay];  
-    [scroll setDocumentView: nil];	  
-    DESTROY (matrix);
+      
+  if ([matrix numberOfColumns] > 0) {
+    [matrix removeColumn: 0];
   }
-
+  
   DESTROY (shownNode); 
   DESTROY (oldNode);
   isLoaded = NO;
@@ -240,7 +256,7 @@ static id <DesktopApplication> desktopApp = nil;
 
     if (visibleNodes) {
       int count = [visibleNodes count];
-
+      
       for (i = 0; i < count; i++) {
         FSNode *node = [visibleNodes objectAtIndex: i];
 
@@ -254,14 +270,20 @@ static id <DesktopApplication> desktopApp = nil;
       if ([visibleNodes count]) {
         cell = [self cellOfNode: [visibleNodes objectAtIndex: 0]];
         [matrix scrollToFirstPositionCell: cell withScrollTune: scrollTune];
+      
+      } else if ([[matrix cells] count]) {
+        [matrix scrollCellToVisibleAtRow: 0 column: 0];
       }
+      
+    } else if ([[matrix cells] count]) {
+      [matrix scrollCellToVisibleAtRow: 0 column: 0];
     }
                
     isLoaded = YES;
   }
   
-  TEST_RELEASE (savedSelection);
-  TEST_RELEASE (visibleNodes);  
+  RELEASE (savedSelection);
+  RELEASE (visibleNodes);  
 }
 
 - (FSNode *)shownNode
@@ -277,60 +299,48 @@ static id <DesktopApplication> desktopApp = nil;
   SEL compSel = [fsnodeRep compareSelectorForDirectory: [shownNode path]];
   int i;
 
-	matrix = [[FSNBrowserMatrix alloc] initInColumn: self 
-                                        withFrame: [self frame]
-		                                         mode: NSListModeMatrix 
-                                        prototype: cellPrototype
-		      					                 numberOfRows: 0 
-                                  numberOfColumns: 0 
-                                        acceptDnd: cellsIcon];
+  if ([matrix numberOfColumns] > 0)
+    [matrix removeColumn: 0];
+
+
+  if (count == 0)
+    {
+      [matrix setNeedsDisplay: YES];
+      return;
+    }
   
-	[matrix setIntercellSpacing: NSMakeSize(0, 0)];
-  [matrix setCellSize: NSMakeSize([scroll contentSize].width, cellsHeight)];  
-	[matrix setAutoscroll: YES];
-	[matrix setAllowsEmptySelection: YES];
-  [matrix setBackgroundColor: backColor];
-  [matrix setCellBackgroundColor: backColor];
-	[matrix setTarget: self];
-	[matrix setAction: @selector(doClick:)];
-	[matrix setDoubleAction: @selector(doDoubleClick:)];
-	[scroll setDocumentView: matrix];
-
-  if (count == 0) {
-		return;
-	}
-
   [matrix addColumn]; 
 
-  for (i = 0; i < count; ++i) {
-    FSNode *subnode = [subNodes objectAtIndex: i];
-    id cell;
+  for (i = 0; i < count; ++i)
+    {
+      FSNode *subnode = [subNodes objectAtIndex: i];
+      id cell;
     
-    if (i != 0) {
-		  [matrix insertRow: i];
-    } 
+      if (i != 0)
+	[matrix insertRow: i];
 
-    cell = [matrix cellAtRow: i column: 0];   
-    [cell setLoaded: YES];
-		[cell setEnabled: YES]; 
-    [cell setNode: subnode nodeInfoType: infoType extendedType: extInfoType];
+      cell = [matrix cellAtRow: i column: 0];   
+      [cell setLoaded: YES];
+      [cell setEnabled: YES]; 
+      [cell setNode: subnode nodeInfoType: infoType extendedType: extInfoType];
    
-    if ([subnode isDirectory]) {
-      if ([subnode isPackage]) {
-        [cell setLeaf: YES]; 
-      } else {
-        [cell setLeaf: NO]; 
+      if ([subnode isDirectory])
+	{
+	  if ([subnode isPackage])
+	    {
+	      [cell setLeaf: YES]; 
+	    } else {
+	    [cell setLeaf: NO]; 
+	  }
+	} else {
+	[cell setLeaf: YES];
       }
-    } else {
-		  [cell setLeaf: YES];
-    }
     
-    if (cellsIcon) {
-      [cell setIcon];
-    }
+      if (cellsIcon)
+	[cell setIcon];
     
-    [cell checkLocked];	
-  }
+      [cell checkLocked];	
+    }
 
   [matrix sortUsingSelector: compSel];
   RELEASE (pool);
@@ -407,7 +417,7 @@ static id <DesktopApplication> desktopApp = nil;
   FSNBrowserColumn *col = nil;
   id cell = nil;
   float scrollTune = 0;
-  BOOL updatesel = NO;
+  BOOL updated = NO;
   int i;
   
   selcells = [matrix selectedCells];
@@ -430,26 +440,27 @@ static id <DesktopApplication> desktopApp = nil;
     if (cell) {
       FSNode *node = [cell node];
 			int row, col;
-
+      
 			if (visibleNodes && [visibleNodes containsObject: node]) {
 				[visibleNodes removeObject: node];
 			}
 			
 			if (selectedCells && [selectedCells containsObject: cell]) {
 				[selectedCells removeObject: cell];
-        updatesel = YES;
 			}
       
       [matrix getRow: &row column: &col ofCell: cell];  
-      [matrix removeRow: row];    			
+      [matrix removeRow: row];  
+      updated = YES;  			
     }
   }
 
   [matrix sizeToCells];
   [matrix setNeedsDisplay: YES];
   
-  if (updatesel) {
-	  if ([selectedCells count] > 0) {      
+  if (updated) {
+	  if ([selectedCells count] > 0) {    
+    
       [self selectCells: selectedCells sendAction: NO];    
       [matrix setNeedsDisplay: YES];
       
@@ -475,9 +486,9 @@ static id <DesktopApplication> desktopApp = nil;
     cell = [self cellOfNode: [visibleNodes objectAtIndex: 0]];
     [matrix scrollToFirstPositionCell: cell withScrollTune: scrollTune];
   }
-  
-  TEST_RELEASE (selectedCells); 
-  TEST_RELEASE (visibleNodes);
+
+  RELEASE (selectedCells); 
+  RELEASE (visibleNodes);
   RELEASE (arp);
 }
 
@@ -730,7 +741,7 @@ static id <DesktopApplication> desktopApp = nil;
 
 - (BOOL)selectFirstCell
 {
-  if (matrix && [[matrix cells] count]) {
+  if ([[matrix cells] count]) {
     [matrix selectCellAtRow: 0 column: 0];
     [matrix sendAction];
     return YES;
@@ -741,7 +752,7 @@ static id <DesktopApplication> desktopApp = nil;
 
 - (BOOL)selectCellWithPrefix:(NSString *)prefix
 {
-  if (matrix && [[matrix cells] count]) {
+  if ([[matrix cells] count]) {
     int n = [matrix numberOfRows];
     int s = [matrix selectedRow];
     NSString *cellstr = nil;
@@ -785,9 +796,66 @@ static id <DesktopApplication> desktopApp = nil;
 
 - (void)selectAll
 {
-  if (matrix && [[matrix cells] count]) {
-    [matrix selectAll: nil];
-	  [matrix sendAction];
+  if ([[matrix cells] count]) {
+    NSArray *cells = [matrix cells];
+    unsigned count = [cells count];
+    FSNBrowserCell *cell;
+    int selstart = 0;
+    int selend = 0;
+    int i;
+        
+    [matrix deselectAllCells];
+
+    while (selstart < count) {
+      cell = [cells objectAtIndex: selstart];
+      
+      if ([[cell node] isReserved] == NO) {
+        break;
+      } 
+      
+      selstart++;
+    } 
+    
+    i = selstart;
+
+    while (i < count) {
+      cell = [cells objectAtIndex: i];
+      
+      if ([[cell node] isReserved] == NO) {
+        selend = i;
+
+      } else {
+        [matrix setSelectionFrom: selstart 
+                              to: selend
+                          anchor: selstart 
+                       highlight: YES];
+        
+        selstart = i + 1;
+        
+        while (selstart < count) {        
+          cell = [cells objectAtIndex: selstart];
+
+          if ([[cell node] isReserved] == NO) {
+            break;
+          }
+           
+          selstart++;
+          i++;
+        }
+      }
+      
+      i++;
+    }
+    
+    if (selstart < count) {
+      [matrix setSelectionFrom: selstart 
+                            to: selend 
+                        anchor: selstart 
+                     highlight: YES];
+    }
+
+    [matrix sendAction];
+        
   } else {
     FSNBrowserColumn *col = [browser columnBeforeColumn: self];
   
@@ -799,9 +867,7 @@ static id <DesktopApplication> desktopApp = nil;
 
 - (void)unselectAllCells
 {
-  if (matrix) {
-    [matrix deselectAllCells];
-  }
+  [matrix deselectAllCells];
 }
 
 - (void)setEditorForCell:(FSNBrowserCell *)cell
@@ -930,38 +996,34 @@ static id <DesktopApplication> desktopApp = nil;
 
 - (void)lock
 {
-  if (matrix) {
-    NSArray *cells = [matrix cells];
-    int i;  
-  
-  	for (i = 0; i < [cells count]; i++) {
-			id cell = [cells objectAtIndex: i];
+  NSArray *cells = [matrix cells];
+  int i;  
 
-			if ([cell isEnabled]) {
-    		[cell setEnabled: NO];
-			}
-  	}
+  for (i = 0; i < [cells count]; i++) {
+		id cell = [cells objectAtIndex: i];
 
-  	[matrix setNeedsDisplay: YES];   
-	}
+		if ([cell isEnabled]) {
+    	[cell setEnabled: NO];
+		}
+  }
+
+  [matrix setNeedsDisplay: YES];   
 }
 
 - (void)unlock
 {
-  if (matrix) {
-    NSArray *cells = [matrix cells];
-    int i;  
-  
-  	for (i = 0; i < [cells count]; i++) {
-			id cell = [cells objectAtIndex: i];
+  NSArray *cells = [matrix cells];
+  int i;  
 
-			if ([cell isEnabled] == NO) {
-    		[cell setEnabled: YES];
-			}
-  	}
+  for (i = 0; i < [cells count]; i++) {
+		id cell = [cells objectAtIndex: i];
 
-  	[matrix setNeedsDisplay: YES];   
-	}
+		if ([cell isEnabled] == NO) {
+    	[cell setEnabled: YES];
+		}
+  }
+
+  [matrix setNeedsDisplay: YES];   
 }
 
 - (FSNBrowserCell *)cellOfNode:(FSNode *)node
@@ -1001,19 +1063,26 @@ static id <DesktopApplication> desktopApp = nil;
   NSArray *cells = [matrix cells];
   int i;
 
-	for (i = 0; i < [cells count]; i++) {
-	  FSNBrowserCell *cell = [cells objectAtIndex: i];
+  for (i = 0; i < [cells count]; i++)
+    {
+      FSNBrowserCell *cell = [cells objectAtIndex: i];
     
-    if ([[[cell node] name] isEqual: name]) {
-      return cell;
-    } 
-	}
+      if ([[[cell node] name] isEqual: name])
+	return cell;
+    
+    }
 
   return nil;
 }             
 
 - (void)adjustMatrix
 {
+  if (scroll == nil)
+    {
+      NSLog(@"FSNBrowserColumn adjustMatrix: scroll is nil");
+      return;
+    }
+  
   [matrix setCellSize: NSMakeSize([scroll contentSize].width, cellsHeight)];  
   [matrix sizeToCells];
 }
@@ -1055,9 +1124,19 @@ static id <DesktopApplication> desktopApp = nil;
 - (void)setBackgroundColor:(NSColor *)acolor
 {
   ASSIGN (backColor, acolor);
-  if (matrix) {
-    [matrix setBackgroundColor: backColor];
-    [matrix setCellBackgroundColor: backColor];
+  [matrix setBackgroundColor: backColor];
+  [matrix setCellBackgroundColor: backColor];
+}
+
+- (void)mouseUp:(NSEvent *)theEvent
+{
+  NSPoint p = [theEvent locationInWindow];
+  int row, col;
+
+  p = [matrix convertPoint: p fromView: nil];
+
+  if ([matrix getRow: &row column: &col forPoint: p] == NO) {
+    [browser clickInColumn: self];
   }
 }
 
@@ -1065,33 +1144,34 @@ static id <DesktopApplication> desktopApp = nil;
 {
   NSRect r = NSMakeRect(1, 0, frameRect.size.width -1, frameRect.size.height);
   
-  if (index == [browser firstVisibleColumn]) {
-    r.origin.x = 0;
-    r.size.width += 1;
-  }
+  if (index == [browser firstVisibleColumn])
+    {
+      r.origin.x = 0;
+      r.size.width += 1;
+    }
   
   CHECKRECT (frameRect);
   [super setFrame: frameRect]; 
    
   CHECKRECT (r);
-  [scroll setFrame: r];  
-  
-  if (matrix) {
-    [self adjustMatrix];
-  }
+
+  if (scroll != nil)
+    {
+      [scroll setFrame: r];  
+      [self adjustMatrix];
+    }
 }
 
 - (void)drawRect:(NSRect)rect
 {
   [super drawRect: rect];
-  [backColor set];
-  NSRectFill(rect);
   
-  if (index != [browser firstVisibleColumn]) {
-	  [[NSColor blackColor] set];  
-    [NSBezierPath strokeLineFromPoint: NSMakePoint(0, 0) 
-                              toPoint: NSMakePoint(0, rect.size.height)];
-  }
+  if (index != [browser firstVisibleColumn])
+    {
+      [[NSColor blackColor] set];  
+      [NSBezierPath strokeLineFromPoint: NSMakePoint(0, 0) 
+				toPoint: NSMakePoint(0, rect.size.height)];
+    }
 }
 
 @end
@@ -1099,21 +1179,27 @@ static id <DesktopApplication> desktopApp = nil;
 
 @implementation FSNBrowserColumn (DraggingDestination)
 
-- (unsigned int)draggingEntered:(id <NSDraggingInfo>)sender
+- (NSDragOperation)draggingEntered:(id <NSDraggingInfo>)sender
 {
-	NSPasteboard *pb;
+  NSPasteboard *pb;
   NSDragOperation sourceDragMask;
-	NSArray *sourcePaths;
+  NSArray *sourcePaths;
   NSString *basePath;
   NSString *nodePath;
   NSString *prePath;
-	int count;
+  int count;
   
-	isDragTarget = NO;	
+  isDragTarget = NO;	
   
   if ((shownNode == nil) || ([shownNode isValid] == NO)) {
     return NSDragOperationNone;
   }
+
+  if ([shownNode isDirectory]) {
+    if ([shownNode isSubnodeOfPath: [desktopApp trashPath]]) { 
+      return NSDragOperationNone;
+    }
+  }	
   
  	pb = [sender draggingPasteboard];
 
@@ -1168,6 +1254,27 @@ static id <DesktopApplication> desktopApp = nil;
     prePath = [prePath stringByDeletingLastPathComponent];
   }
 
+  if ([shownNode isDirectory] && [shownNode isParentOfPath: basePath]) {
+    NSArray *subNodes = [shownNode subNodes];
+    int i;
+    
+    for (i = 0; i < [subNodes count]; i++) {
+      FSNode *nd = [subNodes objectAtIndex: i];
+      
+      if ([nd isDirectory]) {
+        int j;
+        
+        for (j = 0; j < count; j++) {
+          NSString *fname = [[sourcePaths objectAtIndex: j] lastPathComponent];
+          
+          if ([[nd name] isEqual: fname]) {
+            return NSDragOperationNone;
+          }
+        }
+      }
+    }
+  }	
+
   isDragTarget = YES;	
   forceCopy = NO;
     
@@ -1190,7 +1297,7 @@ static id <DesktopApplication> desktopApp = nil;
   return NSDragOperationNone;
 }
 
-- (unsigned int)draggingUpdated:(id <NSDraggingInfo>)sender
+- (NSDragOperation)draggingUpdated:(id <NSDraggingInfo>)sender
 {
   NSDragOperation sourceDragMask = [sender draggingSourceOperationMask];
   
@@ -1295,8 +1402,8 @@ static id <DesktopApplication> desktopApp = nil;
   [desktopApp performFileOperation: opDict];
 }
 
-- (unsigned int)draggingEntered:(id <NSDraggingInfo>)sender
-                   inMatrixCell:(id)cell
+- (NSDragOperation)draggingEntered:(id <NSDraggingInfo>)sender
+                      inMatrixCell:(id)cell
 {
 	NSPasteboard *pb = [sender draggingPasteboard];
   NSDragOperation sourceDragMask = [sender draggingSourceOperationMask];
@@ -1305,29 +1412,42 @@ static id <DesktopApplication> desktopApp = nil;
 	NSArray *sourcePaths;
 	NSString *fromPath;
   NSString *prePath;
-	int count;
+	unsigned i, count;
 
   if (([cell isEnabled] == NO) || ([node isDirectory] == NO) 
-                    || [node isPackage] || ([node isWritable] == NO)) {
+            || ([node isPackage] && ([node isApplication] == NO))
+            || (([node isWritable] == NO) && ([node isApplication] == NO))) {
     return NSDragOperationNone;
   }
 
+  if ([node isDirectory]) {
+    if ([node isSubnodeOfPath: [desktopApp trashPath]]) { 
+      return NSDragOperationNone;
+    }
+  }	
+ 
+  sourcePaths = nil;
+
   if ([[pb types] containsObject: NSFilenamesPboardType]) {
     sourcePaths = [pb propertyListForType: NSFilenamesPboardType]; 
        
   } else if ([[pb types] containsObject: @"GWRemoteFilenamesPboardType"]) {
-    NSData *pbData = [pb dataForType: @"GWRemoteFilenamesPboardType"]; 
-    NSDictionary *pbDict = [NSUnarchiver unarchiveObjectWithData: pbData];
+    if ([node isApplication] == NO) {
+      NSData *pbData = [pb dataForType: @"GWRemoteFilenamesPboardType"]; 
+      NSDictionary *pbDict = [NSUnarchiver unarchiveObjectWithData: pbData];
     
-    sourcePaths = [pbDict objectForKey: @"paths"];
-
+      sourcePaths = [pbDict objectForKey: @"paths"];
+    }
   } else if ([[pb types] containsObject: @"GWLSFolderPboardType"]) {
-    NSData *pbData = [pb dataForType: @"GWLSFolderPboardType"]; 
-    NSDictionary *pbDict = [NSUnarchiver unarchiveObjectWithData: pbData];
+    if ([node isApplication] == NO) {
+      NSData *pbData = [pb dataForType: @"GWLSFolderPboardType"]; 
+      NSDictionary *pbDict = [NSUnarchiver unarchiveObjectWithData: pbData];
     
-    sourcePaths = [pbDict objectForKey: @"paths"];
+      sourcePaths = [pbDict objectForKey: @"paths"];
+    }
+  }
 
-  } else {
+  if (sourcePaths == nil) {
     return NSDragOperationNone;
   }
   
@@ -1349,21 +1469,59 @@ static id <DesktopApplication> desktopApp = nil;
   prePath = [NSString stringWithString: nodePath];
 
   while (1) {
+    CREATE_AUTORELEASE_POOL(arp);
+
     if ([sourcePaths containsObject: prePath]) {
+      RELEASE (arp);
       return NSDragOperationNone;
     }
     if ([prePath isEqual: path_separator()]) {
+      RELEASE (arp);
       break;
     }            
     prePath = [prePath stringByDeletingLastPathComponent];
   }
 
+  if ([node isApplication]) {
+    for (i = 0; i < count; i++) {
+      CREATE_AUTORELEASE_POOL(arp);
+      FSNode *nd = [FSNode nodeWithPath: [sourcePaths objectAtIndex: i]];
+      
+      if (([nd isPlain] == NO) && ([nd isPackage] == NO)) {
+        RELEASE (arp);
+        return NSDragOperationNone;
+      }
+      RELEASE (arp);
+    }
+  }
+
+  if ([node isDirectory] && [node isParentOfPath: fromPath]) {
+    NSArray *subNodes = [node subNodes];
+    
+    for (i = 0; i < [subNodes count]; i++) {
+      FSNode *nd = [subNodes objectAtIndex: i];
+      
+      if ([nd isDirectory]) {
+        int j;
+        
+        for (j = 0; j < count; j++) {
+          NSString *fname = [[sourcePaths objectAtIndex: j] lastPathComponent];
+          
+          if ([[nd name] isEqual: fname]) {
+            return NSDragOperationNone;
+          }
+        }
+      }
+    }
+  }	
+
 	if (sourceDragMask == NSDragOperationCopy) {
-		return NSDragOperationCopy;
+		return ([node isApplication] ? NSDragOperationMove : NSDragOperationCopy);
 	} else if (sourceDragMask == NSDragOperationLink) {
-		return NSDragOperationLink;
+		return ([node isApplication] ? NSDragOperationMove : NSDragOperationLink);
 	} else {
-    if ([[NSFileManager defaultManager] isWritableFileAtPath: fromPath]) {
+    if ([[NSFileManager defaultManager] isWritableFileAtPath: fromPath]
+                                                    || [node isApplication]) {
       return NSDragOperationAll;			
     } else {
 			return NSDragOperationCopy;			
@@ -1376,68 +1534,94 @@ static id <DesktopApplication> desktopApp = nil;
 - (void)concludeDragOperation:(id <NSDraggingInfo>)sender
                  inMatrixCell:(id)cell
 {
-	NSPasteboard *pb = [sender draggingPasteboard];
+  FSNode *node = [cell node];
+  NSPasteboard *pb = [sender draggingPasteboard];
   NSDragOperation sourceDragMask = [sender draggingSourceOperationMask];
-	NSArray *sourcePaths;
+  NSArray *sourcePaths;
   NSString *operation, *source;
   NSMutableArray *files;
-	NSMutableDictionary *opDict;
-	NSString *trashPath;
-  int i;
+  NSMutableDictionary *opDict;
+  NSString *trashPath;
+  unsigned i;
 
-  if (([cell isEnabled] == NO) || [cell isLeaf]) {
+  if (([cell isEnabled] == NO) 
+                  || ([cell isLeaf] && ([node isApplication] == NO))) {
     return;
   }
 
-  if ([[pb types] containsObject: @"GWRemoteFilenamesPboardType"]) {  
-    NSData *pbData = [pb dataForType: @"GWRemoteFilenamesPboardType"]; 
+  if ([node isApplication] == NO) {    
+    if ([[pb types] containsObject: @"GWRemoteFilenamesPboardType"]) {  
+      NSData *pbData = [pb dataForType: @"GWRemoteFilenamesPboardType"]; 
 
-    [desktopApp concludeRemoteFilesDragOperation: pbData
-                                     atLocalPath: [[cell node] path]];
-    return;
-    
-  } else if ([[pb types] containsObject: @"GWLSFolderPboardType"]) {  
-    NSData *pbData = [pb dataForType: @"GWLSFolderPboardType"]; 
+      [desktopApp concludeRemoteFilesDragOperation: pbData
+                                       atLocalPath: [[cell node] path]];
+      return;
 
-    [desktopApp lsfolderDragOperation: pbData
-                      concludedAtPath: [[cell node] path]];
-    return;
+    } else if ([[pb types] containsObject: @"GWLSFolderPboardType"]) {  
+      NSData *pbData = [pb dataForType: @"GWLSFolderPboardType"]; 
+
+      [desktopApp lsfolderDragOperation: pbData
+                        concludedAtPath: [[cell node] path]];
+      return;
+    }
   }
-
+  
   sourcePaths = [pb propertyListForType: NSFilenamesPboardType];
 
-  source = [[sourcePaths objectAtIndex: 0] stringByDeletingLastPathComponent];
-  
-  trashPath = [desktopApp trashPath];
+  if ([node isApplication] == NO) {  
+    source = [[sourcePaths objectAtIndex: 0] stringByDeletingLastPathComponent];
+    trashPath = [desktopApp trashPath];
 
-  if ([source isEqual: trashPath]) {
-  		operation = @"GWorkspaceRecycleOutOperation";
-	} else {	
-		if (sourceDragMask == NSDragOperationCopy) {
-			operation = NSWorkspaceCopyOperation;
-		} else if (sourceDragMask == NSDragOperationLink) {
-			operation = NSWorkspaceLinkOperation;
-		} else {
-      if ([[NSFileManager defaultManager] isWritableFileAtPath: source]) {
-			  operation = NSWorkspaceMoveOperation;
-      } else {
+    if ([source isEqual: trashPath]) {
+  		  operation = @"GWorkspaceRecycleOutOperation";
+	  } else {	
+		  if (sourceDragMask == NSDragOperationCopy) {
 			  operation = NSWorkspaceCopyOperation;
-      }
-		}
+		  } else if (sourceDragMask == NSDragOperationLink) {
+			  operation = NSWorkspaceLinkOperation;
+		  } else {
+        if ([[NSFileManager defaultManager] isWritableFileAtPath: source]) {
+			    operation = NSWorkspaceMoveOperation;
+        } else {
+			    operation = NSWorkspaceCopyOperation;
+        }
+		  }
+    }
+
+    files = [NSMutableArray arrayWithCapacity: 1];    
+    for(i = 0; i < [sourcePaths count]; i++) {    
+      [files addObject: [[sourcePaths objectAtIndex: i] lastPathComponent]];
+    }  
+
+	  opDict = [NSMutableDictionary dictionaryWithCapacity: 4];
+	  [opDict setObject: operation forKey: @"operation"];
+	  [opDict setObject: source forKey: @"source"];
+	  [opDict setObject: [[cell node] path] forKey: @"destination"];
+	  [opDict setObject: files forKey: @"files"];
+
+    [desktopApp performFileOperation: opDict];
+
+  } else {
+    for (i = 0; i < [sourcePaths count]; i++) {    
+      NSString *path = [sourcePaths objectAtIndex: i];
+    
+      NS_DURING
+        {
+      [[NSWorkspace sharedWorkspace] openFile: path 
+                              withApplication: [node name]];
+        }
+      NS_HANDLER
+        {
+      NSRunAlertPanel(NSLocalizedString(@"error", @""), 
+          [NSString stringWithFormat: @"%@ %@!", 
+                    NSLocalizedString(@"Can't open ", @""), [node name]],
+                                        NSLocalizedString(@"OK", @""), 
+                                        nil, 
+                                        nil);                                     
+        }
+      NS_ENDHANDLER  
+    }
   }
-  
-  files = [NSMutableArray arrayWithCapacity: 1];    
-  for(i = 0; i < [sourcePaths count]; i++) {    
-    [files addObject: [[sourcePaths objectAtIndex: i] lastPathComponent]];
-  }  
-
-	opDict = [NSMutableDictionary dictionaryWithCapacity: 4];
-	[opDict setObject: operation forKey: @"operation"];
-	[opDict setObject: source forKey: @"source"];
-	[opDict setObject: [[cell node] path] forKey: @"destination"];
-	[opDict setObject: files forKey: @"files"];
-
-  [desktopApp performFileOperation: opDict];
 }
 
 @end
