@@ -1,6 +1,6 @@
 /* FSNIcon.m
  *  
- * Copyright (C) 2004 Free Software Foundation, Inc.
+ * Copyright (C) 2004-2010 Free Software Foundation, Inc.
  *
  * Author: Enrico Sersale <enrico@imago.ro>
  * Date: March 2004
@@ -19,17 +19,19 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
  */
 
-#include <Foundation/Foundation.h>
-#include <AppKit/AppKit.h>
 #include <math.h>
-#include "FSNIcon.h"
-#include "FSNTextCell.h"
-#include "FSNode.h"
-#include "FSNFunctions.h"
-#include "GNUstep.h"
+
+#import <Foundation/Foundation.h>
+#import <AppKit/AppKit.h>
+#import <GNUstepBase/GNUstep.h>
+
+#import "FSNIcon.h"
+#import "FSNTextCell.h"
+#import "FSNode.h"
+#import "FSNFunctions.h"
 
 #define BRANCH_SIZE 7
 #define ARROW_ORIGIN_X (BRANCH_SIZE + 4)
@@ -45,13 +47,16 @@ static NSImage *branchImage;
 
 - (void)dealloc
 {
+  if (trectTag != -1) {
+    [self removeTrackingRect: trectTag];
+  }
   RELEASE (node);
-	TEST_RELEASE (hostname);
-  TEST_RELEASE (selection);
-  TEST_RELEASE (selectionTitle);
-  TEST_RELEASE (extInfoType);
+  RELEASE (hostname);
+  RELEASE (selection);
+  RELEASE (selectionTitle);
+  RELEASE (extInfoType);
   RELEASE (icon);
-  TEST_RELEASE (openicon);
+  RELEASE (selectedicon);
   RELEASE (highlightPath);
   RELEASE (label);
   RELEASE (infolabel);
@@ -60,22 +65,28 @@ static NSImage *branchImage;
 
 + (void)initialize
 {
-  NSBundle *bundle = [NSBundle bundleForClass: [FSNodeRep class]];
-  NSString *imagepath = [bundle pathForResource: @"ArrowRight" ofType: @"tiff"];
+  static BOOL initialized = NO;
 
-  if (desktopApp == nil) {
-    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
-    NSString *appName = [defaults stringForKey: @"DesktopApplicationName"];
-    NSString *selName = [defaults stringForKey: @"DesktopApplicationSelName"];
+  if (initialized == NO) {
+    NSBundle *bundle = [NSBundle bundleForClass: [FSNodeRep class]];
+    NSString *imagepath = [bundle pathForResource: @"ArrowRight" ofType: @"tiff"];
 
-    if (appName && selName) {
-		  Class desktopAppClass = [[NSBundle mainBundle] classNamed: appName];
-      SEL sel = NSSelectorFromString(selName);
-      desktopApp = [desktopAppClass performSelector: sel];
+    if (desktopApp == nil) {
+      NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
+      NSString *appName = [defaults stringForKey: @"DesktopApplicationName"];
+      NSString *selName = [defaults stringForKey: @"DesktopApplicationSelName"];
+
+      if (appName && selName) {
+		    Class desktopAppClass = [[NSBundle mainBundle] classNamed: appName];
+        SEL sel = NSSelectorFromString(selName);
+        desktopApp = [desktopAppClass performSelector: sel];
+      }
     }
+
+    branchImage = [[NSImage alloc] initWithContentsOfFile: imagepath];
+  
+    initialized = YES;
   }
-    
-  branchImage = [[NSImage alloc] initWithContentsOfFile: imagepath];
 }
 
 + (NSImage *)branchImage
@@ -115,7 +126,7 @@ static NSImage *branchImage;
     
     ASSIGN (icon, [fsnodeRep iconOfSize: iconSize forNode: node]);
     drawicon = icon;
-    openicon = nil;
+    selectedicon = nil;
     
     dndSource = dndsrc;
     acceptDnd = dndaccept;
@@ -166,7 +177,7 @@ static NSImage *branchImage;
     
     labelRect = NSZeroRect;
     labelRect.size.width = [label uncuttedTitleLenght] + [fsnodeRep labelMargin];
-    labelRect.size.height = [[label font] defaultLineHeightForFont];
+    labelRect.size.height = [fsnodeRep heighOfFont: [label font]];
     labelRect = NSIntegralRect(labelRect);
 
     infoRect = NSZeroRect;
@@ -175,7 +186,7 @@ static NSImage *branchImage;
     } else {
       infoRect.size.width = labelRect.size.width;
     }
-    infoRect.size.height = [[infolabel font] defaultLineHeightForFont];
+    infoRect.size.height = [fsnodeRep heighOfFont: [infolabel font]];
     infoRect = NSIntegralRect(infoRect);
 
     icnPosition = ipos;
@@ -219,7 +230,8 @@ static NSImage *branchImage;
     } else {
       r.size = icnBounds.size;
     }
-
+    
+    trectTag = -1;
     [self setFrame: NSIntegralRect(r)];
     
     if (acceptDnd) {
@@ -262,7 +274,7 @@ static NSImage *branchImage;
 
 - (void)tile
 {
-  NSRect frameRect = [self frame];
+  NSRect frameRect = [self bounds];
   NSSize sz = [icon size];
   int lblmargin = [fsnodeRep labelMargin];
   BOOL hasinfo = ([[infolabel stringValue] length] > 0);
@@ -448,7 +460,18 @@ static NSImage *branchImage;
   brImgBounds.origin.x = frameRect.size.width - ARROW_ORIGIN_X;
   brImgBounds.origin.y = myrintf(icnBounds.origin.y + (icnBounds.size.height / 2) - (BRANCH_SIZE / 2));
   brImgBounds = NSIntegralRect(brImgBounds);
+
+  if ([self window]) {
+    if (trectTag != -1) {
+      [self removeTrackingRect: trectTag];
+    }
   
+    trectTag = [self addTrackingRect: icnBounds 
+                               owner: self 
+                            userData: nil
+                        assumeInside: NO]; 
+  }
+    
   [self setNeedsDisplay: YES]; 
 }
 
@@ -501,17 +524,17 @@ static NSImage *branchImage;
 - (void)mouseDown:(NSEvent *)theEvent
 {
   NSPoint location = [theEvent locationInWindow];
+  NSPoint selfloc = [self convertPoint: location fromView: nil];
   BOOL onself = NO;
 	NSEvent *nextEvent = nil;
   BOOL startdnd = NO;
-
-  location = [self convertPoint: location fromView: nil];
+  NSSize offset;
 
   if (icnPosition == NSImageOnly) {
-    onself = [self mouse: location inRect: icnBounds];
+    onself = [self mouse: selfloc inRect: icnBounds];
   } else {
-    onself = ([self mouse: location inRect: icnBounds]
-                        || [self mouse: location inRect: labelRect]);
+    onself = ([self mouse: selfloc inRect: icnBounds]
+                        || [self mouse: selfloc inRect: labelRect]);
   }
 
   if (onself) {
@@ -577,22 +600,30 @@ static NSImage *branchImage;
             
             break;
 
-          } else if ([nextEvent type] == NSLeftMouseDragged) {
+          } else if (([nextEvent type] == NSLeftMouseDragged)
+                            && ([self mouse: selfloc inRect: icnBounds])) {
 	          if (dragdelay < 5) {
               dragdelay++;
-            } else {     
+            } else {    
+              NSPoint p = [nextEvent locationInWindow];
+              offset = NSMakeSize(p.x - location.x, p.y - location.y); 
               startdnd = YES;        
               break;
             }
-          }
+          } 
         }
-      }
+      } 
       
-      if (startdnd == YES) {  
+      if (startdnd) {  
         if ([container respondsToSelector: @selector(stopRepNameEditing)]) {
           [container stopRepNameEditing];
         }
-        [self startExternalDragOnEvent: nextEvent];    
+        
+        if ([container respondsToSelector: @selector(setFocusedRep:)]) {
+          [container setFocusedRep: nil];
+        }
+        
+        [self startExternalDragOnEvent: theEvent withMouseOffset: offset];
       }
       
       editstamp = [theEvent timestamp];       
@@ -603,6 +634,20 @@ static NSImage *branchImage;
   }
 }
 
+- (void)mouseEntered:(NSEvent *)theEvent
+{
+  if ([container respondsToSelector: @selector(setFocusedRep:)]) {
+    [container setFocusedRep: self];
+  }
+}
+
+- (void)mouseExited:(NSEvent *)theEvent
+{
+  if ([container respondsToSelector: @selector(setFocusedRep:)]) {
+    [container setFocusedRep: nil];
+  }
+}
+
 - (BOOL)acceptsFirstMouse:(NSEvent *)theEvent
 {
   return YES;
@@ -640,8 +685,6 @@ static NSImage *branchImage;
     if (icnPosition != NSImageOnly) {
       if (nameEdited == NO) {
         [[container backgroundColor] set];
-        NSFrameRect(labelRect);
-        NSRectFill(labelRect);
         [label drawWithFrame: labelRect inView: self];
       }
       
@@ -679,7 +722,7 @@ static NSImage *branchImage;
   ASSIGN (node, anode);
   ASSIGN (icon, [fsnodeRep iconOfSize: iconSize forNode: node]);
   drawicon = icon;
-  DESTROY (openicon);
+  DESTROY (selectedicon);
   
   if ([[node path] isEqual: path_separator()] && ([node isMountPoint] == NO)) {
     NSHost *host = [NSHost currentHost];
@@ -731,7 +774,7 @@ static NSImage *branchImage;
                   [selection count], NSLocalizedString(@"elements", @"")]));
   ASSIGN (icon, [fsnodeRep multipleSelectionIconOfSize: iconSize]);
   drawicon = icon;
-  DESTROY (openicon);
+  DESTROY (selectedicon);
   
   [label setStringValue: selectionTitle];
   [infolabel setStringValue: @""];
@@ -789,7 +832,7 @@ static NSImage *branchImage;
   [infolabel setFont: infoFont];
 
   labelRect.size.width = myrintf([label uncuttedTitleLenght] + lblmargin);
-  labelRect.size.height = myrintf([[label font] defaultLineHeightForFont]);
+  labelRect.size.height = myrintf([fsnodeRep heighOfFont: [label font]]);
   labelRect = NSIntegralRect(labelRect);
 
   infoRect = NSZeroRect;
@@ -798,7 +841,7 @@ static NSImage *branchImage;
   } else {
     infoRect.size.width = labelRect.size.width;
   }
-  infoRect.size.height = [infoFont defaultLineHeightForFont];
+  infoRect.size.height = [fsnodeRep heighOfFont: infoFont];
   infoRect = NSIntegralRect(infoRect);
 
   [self tile];
@@ -830,7 +873,7 @@ static NSImage *branchImage;
     ASSIGN (icon, [fsnodeRep multipleSelectionIconOfSize: iconSize]);
   }
   drawicon = icon;
-  DESTROY (openicon);
+  DESTROY (selectedicon);
   hlightRect.size.width = myrintf(iconSize / 3 * 4);
   hlightRect.size.height = myrintf(hlightRect.size.width * [fsnodeRep highlightHeightFactor]);
   if ((hlightRect.size.height - iconSize) < 4) {
@@ -841,7 +884,7 @@ static NSImage *branchImage;
   ASSIGN (highlightPath, [fsnodeRep highlightPathOfSize: hlightRect.size]); 
 
   labelRect.size.width = [label uncuttedTitleLenght] + [fsnodeRep labelMargin];
-  labelRect.size.height = [[label font] defaultLineHeightForFont];
+  labelRect.size.height = [fsnodeRep heighOfFont: [label font]];
 
   [self tile];
 }
@@ -1054,37 +1097,37 @@ static NSImage *branchImage;
   return gridIndex;
 }
 
-- (int)compareAccordingToName:(FSNIcon *)aIcon
+- (int)compareAccordingToName:(id)aIcon
 {
   return [node compareAccordingToName: [aIcon node]];
 }
 
-- (int)compareAccordingToKind:(FSNIcon *)aIcon
+- (int)compareAccordingToKind:(id)aIcon
 {
   return [node compareAccordingToKind: [aIcon node]];
 }
 
-- (int)compareAccordingToDate:(FSNIcon *)aIcon
+- (int)compareAccordingToDate:(id)aIcon
 {
   return [node compareAccordingToDate: [aIcon node]];
 }
 
-- (int)compareAccordingToSize:(FSNIcon *)aIcon
+- (int)compareAccordingToSize:(id)aIcon
 {
   return [node compareAccordingToSize: [aIcon node]];
 }
 
-- (int)compareAccordingToOwner:(FSNIcon *)aIcon
+- (int)compareAccordingToOwner:(id)aIcon
 {
   return [node compareAccordingToOwner: [aIcon node]];
 }
 
-- (int)compareAccordingToGroup:(FSNIcon *)aIcon
+- (int)compareAccordingToGroup:(id)aIcon
 {
   return [node compareAccordingToGroup: [aIcon node]];
 }
 
-- (int)compareAccordingToIndex:(FSNIcon *)aIcon
+- (int)compareAccordingToIndex:(id)aIcon
 {
   return (gridIndex <= [aIcon gridIndex]) ? NSOrderedAscending : NSOrderedDescending;
 }
@@ -1095,6 +1138,7 @@ static NSImage *branchImage;
 @implementation FSNIcon (DraggingSource)
 
 - (void)startExternalDragOnEvent:(NSEvent *)event
+                 withMouseOffset:(NSSize)offset
 {
   if ([container respondsToSelector: @selector(selectedPaths)]) {
     NSArray *selectedPaths = [container selectedPaths];
@@ -1105,7 +1149,6 @@ static NSImage *branchImage;
 
     if ([pb setPropertyList: selectedPaths forType: NSFilenamesPboardType]) {
       NSImage *dragIcon;
-      NSPoint dragPoint;
 
       if ([selectedPaths count] == 1) {
         dragIcon = icon;
@@ -1113,12 +1156,9 @@ static NSImage *branchImage;
         dragIcon = [fsnodeRep multipleSelectionIconOfSize: iconSize];
       }     
 
-      dragPoint = [event locationInWindow];      
-      dragPoint = [self convertPoint: dragPoint fromView: nil];
-
       [self dragImage: dragIcon
-                   at: dragPoint 
-               offset: NSZeroSize
+                   at: icnPoint
+               offset: offset
                 event: event
            pasteboard: pb
                source: self
@@ -1155,7 +1195,7 @@ static NSImage *branchImage;
 
 @implementation FSNIcon (DraggingDestination)
 
-- (unsigned int)draggingEntered:(id <NSDraggingInfo>)sender
+- (NSDragOperation)draggingEntered:(id <NSDraggingInfo>)sender
 {
 	NSPasteboard *pb;
   NSDragOperation sourceDragMask;
@@ -1163,34 +1203,55 @@ static NSImage *branchImage;
 	NSString *fromPath;
   NSString *nodePath;
   NSString *prePath;
-	int count;
-
+	unsigned i, count;
+  
   isDragTarget = NO;
   onSelf = NO;
 	
   if (selection || isLocked || ([node isDirectory] == NO) 
-                    || [node isPackage] || ([node isWritable] == NO)) {
+            || (([node isWritable] == NO) && ([node isApplication] == NO))) {
     return NSDragOperationNone;
   }
-  	
+  
+  if ([node isDirectory]) {
+    if ([node isSubnodeOfPath: [desktopApp trashPath]]) { 
+      return NSDragOperationNone;
+    }
+  }	
+  
+  if ([node isPackage] && ([node isApplication] == NO)) {
+    if ([container respondsToSelector: @selector(baseNode)]) { 
+      if ([node isEqual: [container baseNode]] == NO) {
+        return NSDragOperationNone;
+      }
+    } else {
+      return NSDragOperationNone;
+    }
+  }
+  
 	pb = [sender draggingPasteboard];
-
+  sourcePaths = nil;
+  
   if ([[pb types] containsObject: NSFilenamesPboardType]) {
     sourcePaths = [pb propertyListForType: NSFilenamesPboardType]; 
        
   } else if ([[pb types] containsObject: @"GWRemoteFilenamesPboardType"]) {
-    NSData *pbData = [pb dataForType: @"GWRemoteFilenamesPboardType"]; 
-    NSDictionary *pbDict = [NSUnarchiver unarchiveObjectWithData: pbData];
-    
-    sourcePaths = [pbDict objectForKey: @"paths"];
+    if ([node isPackage] == NO) {
+      NSData *pbData = [pb dataForType: @"GWRemoteFilenamesPboardType"]; 
+      NSDictionary *pbDict = [NSUnarchiver unarchiveObjectWithData: pbData];
 
+      sourcePaths = [pbDict objectForKey: @"paths"];
+    }
   } else if ([[pb types] containsObject: @"GWLSFolderPboardType"]) {
-    NSData *pbData = [pb dataForType: @"GWLSFolderPboardType"]; 
-    NSDictionary *pbDict = [NSUnarchiver unarchiveObjectWithData: pbData];
+    if ([node isPackage] == NO) {
+      NSData *pbData = [pb dataForType: @"GWLSFolderPboardType"]; 
+      NSDictionary *pbDict = [NSUnarchiver unarchiveObjectWithData: pbData];
     
-    sourcePaths = [pbDict objectForKey: @"paths"];
-
-  } else {
+      sourcePaths = [pbDict objectForKey: @"paths"];
+    }
+  }
+  
+  if (sourcePaths == nil) {
     return NSDragOperationNone;
   }
   
@@ -1229,28 +1290,86 @@ static NSImage *branchImage;
   prePath = [NSString stringWithString: nodePath];
 
   while (1) {
+    CREATE_AUTORELEASE_POOL(arp);
+    
     if ([sourcePaths containsObject: prePath]) {
+      RELEASE (arp);
       return NSDragOperationNone;
     }
     if ([prePath isEqual: path_separator()]) {
+      RELEASE (arp);
       break;
     }            
     prePath = [prePath stringByDeletingLastPathComponent];
   }
 
+  if ([node isDirectory] && [node isParentOfPath: fromPath]) {
+    NSArray *subNodes = [node subNodes];
+    
+    for (i = 0; i < [subNodes count]; i++) {
+      FSNode *nd = [subNodes objectAtIndex: i];
+      
+      if ([nd isDirectory]) {
+        int j;
+        
+        for (j = 0; j < count; j++) {
+          NSString *fname = [[sourcePaths objectAtIndex: j] lastPathComponent];
+          
+          if ([[nd name] isEqual: fname]) {
+            return NSDragOperationNone;
+          }
+        }
+      }
+    }
+  }	
+
+  if ([node isApplication]) {
+    if (([container respondsToSelector: @selector(baseNode)] == NO)
+                        || ([node isEqual: [container baseNode]] == NO)) { 
+      for (i = 0; i < count; i++) {
+        CREATE_AUTORELEASE_POOL(arp);
+        FSNode *nd = [FSNode nodeWithPath: [sourcePaths objectAtIndex: i]];
+
+        if (([nd isPlain] == NO) && ([nd isPackage] == NO)) {
+          RELEASE (arp);
+          return NSDragOperationNone;
+        }
+        RELEASE (arp);
+      }
+      
+    } else if ([node isEqual: [container baseNode]] == NO) {
+      return NSDragOperationNone;
+    }
+  }
+
   isDragTarget = YES;
   forceCopy = NO;
   
+  onApplication = ([node isApplication]
+                      && [container respondsToSelector: @selector(baseNode)]
+                                      && [node isEqual: [container baseNode]]);   
+  
 	sourceDragMask = [sender draggingSourceOperationMask];
 
 	if (sourceDragMask == NSDragOperationCopy) {
-		return NSDragOperationCopy;
-	} else if (sourceDragMask == NSDragOperationLink) {
-		return NSDragOperationLink;
-	} else {  
-    if ([[NSFileManager defaultManager] isWritableFileAtPath: fromPath]) {
-      return NSDragOperationAll;			
+    if ([node isApplication]) {
+      return (onApplication ? NSDragOperationCopy : NSDragOperationMove);
     } else {
+      return NSDragOperationCopy;
+    }
+    
+	} else if (sourceDragMask == NSDragOperationLink) {
+    if ([node isApplication]) {
+      return (onApplication ? NSDragOperationLink : NSDragOperationMove);
+    } else {
+      return NSDragOperationLink;
+    }
+  
+	} else {  
+    if ([[NSFileManager defaultManager] isWritableFileAtPath: fromPath]
+                          || ([node isApplication] && (onApplication == NO))) {
+      return NSDragOperationAll;			
+    } else if (([node isApplication] == NO) || onApplication) {
       forceCopy = YES;
 			return NSDragOperationCopy;			
     }
@@ -1259,25 +1378,25 @@ static NSImage *branchImage;
   return NSDragOperationNone;
 }
 
-- (unsigned int)draggingUpdated:(id <NSDraggingInfo>)sender
+- (NSDragOperation)draggingUpdated:(id <NSDraggingInfo>)sender
 {
   NSDragOperation sourceDragMask = [sender draggingSourceOperationMask];
   NSPoint p = [self convertPoint: [sender draggingLocation] fromView: nil];
 
   if ([self mouse: p inRect: icnBounds] == NO) {
-    if (drawicon == openicon) {
+    if (drawicon == selectedicon) {
       drawicon = icon;
       [self setNeedsDisplay: YES];
     }
     return [container draggingUpdated: sender];
     
   } else {
-    if ((openicon == nil) && isDragTarget && (onSelf == NO)) {
-      ASSIGN (openicon, [fsnodeRep openFolderIconOfSize: iconSize forNode: node]);
+    if ((selectedicon == nil) && isDragTarget && (onSelf == NO)) {
+      ASSIGN (selectedicon, [fsnodeRep openFolderIconOfSize: iconSize forNode: node]);
     }
   
-    if ((drawicon == icon) && isDragTarget && (onSelf == NO)) {
-      drawicon = openicon;
+    if (selectedicon && (drawicon == icon) && isDragTarget && (onSelf == NO)) {
+      drawicon = selectedicon;
       [self setNeedsDisplay: YES];
     }
   }
@@ -1285,9 +1404,19 @@ static NSImage *branchImage;
   if (isDragTarget == NO) {
     return NSDragOperationNone;
   } else if (sourceDragMask == NSDragOperationCopy) {
-		return NSDragOperationCopy;
+    if ([node isApplication]) {
+      return (onApplication ? NSDragOperationCopy : NSDragOperationMove);
+    } else {
+      return NSDragOperationCopy;
+    }
+    
 	} else if (sourceDragMask == NSDragOperationLink) {
-		return NSDragOperationLink;
+    if ([node isApplication]) {
+      return (onApplication ? NSDragOperationLink : NSDragOperationMove);
+    } else {
+      return NSDragOperationLink;
+    }
+
 	} else {
 		return forceCopy ? NSDragOperationCopy : NSDragOperationAll;
 	}
@@ -1327,7 +1456,7 @@ static NSImage *branchImage;
   NSMutableArray *files;
 	NSMutableDictionary *opDict;
 	NSString *trashPath;
-  int i;
+  unsigned i;
 
 	isDragTarget = NO;  
 
@@ -1346,56 +1475,80 @@ static NSImage *branchImage;
 
 	sourceDragMask = [sender draggingSourceOperationMask];
   pb = [sender draggingPasteboard];
-    
-  if ([[pb types] containsObject: @"GWRemoteFilenamesPboardType"]) {  
-    NSData *pbData = [pb dataForType: @"GWRemoteFilenamesPboardType"]; 
 
-    [desktopApp concludeRemoteFilesDragOperation: pbData
-                                     atLocalPath: [node path]];
-    return;
-    
-  } else if ([[pb types] containsObject: @"GWLSFolderPboardType"]) {  
-    NSData *pbData = [pb dataForType: @"GWLSFolderPboardType"]; 
+  if ([node isPackage] == NO) {    
+    if ([[pb types] containsObject: @"GWRemoteFilenamesPboardType"]) {  
+      NSData *pbData = [pb dataForType: @"GWRemoteFilenamesPboardType"]; 
 
-    [desktopApp lsfolderDragOperation: pbData
-                      concludedAtPath: [node path]];
-    return;
+      [desktopApp concludeRemoteFilesDragOperation: pbData
+                                       atLocalPath: [node path]];
+      return;
+
+    } else if ([[pb types] containsObject: @"GWLSFolderPboardType"]) {  
+      NSData *pbData = [pb dataForType: @"GWLSFolderPboardType"]; 
+
+      [desktopApp lsfolderDragOperation: pbData
+                        concludedAtPath: [node path]];
+      return;
+    }
   }
     
   sourcePaths = [pb propertyListForType: NSFilenamesPboardType];
 
-  source = [[sourcePaths objectAtIndex: 0] stringByDeletingLastPathComponent];
-  
-  trashPath = [desktopApp trashPath];
+  if (([node isApplication] == NO) || onApplication) {
+    source = [[sourcePaths objectAtIndex: 0] stringByDeletingLastPathComponent];
+    trashPath = [desktopApp trashPath];
 
-  if ([source isEqual: trashPath]) {
-    operation = @"GWorkspaceRecycleOutOperation";
-	} else {	
-		if (sourceDragMask == NSDragOperationCopy) {
-			operation = NSWorkspaceCopyOperation;
-		} else if (sourceDragMask == NSDragOperationLink) {
-			operation = NSWorkspaceLinkOperation;
-		} else {
-      if ([[NSFileManager defaultManager] isWritableFileAtPath: source]) {
-			  operation = NSWorkspaceMoveOperation;
-      } else {
+    if ([source isEqual: trashPath]) {
+      operation = @"GWorkspaceRecycleOutOperation";
+	  } else {	
+		  if (sourceDragMask == NSDragOperationCopy) {
 			  operation = NSWorkspaceCopyOperation;
-      }
-		}
+		  } else if (sourceDragMask == NSDragOperationLink) {
+			  operation = NSWorkspaceLinkOperation;
+		  } else {
+        if ([[NSFileManager defaultManager] isWritableFileAtPath: source]) {
+			    operation = NSWorkspaceMoveOperation;
+        } else {
+			    operation = NSWorkspaceCopyOperation;
+        }
+		  }
+    }
+
+    files = [NSMutableArray arrayWithCapacity: 1];    
+    for(i = 0; i < [sourcePaths count]; i++) {    
+      [files addObject: [[sourcePaths objectAtIndex: i] lastPathComponent]];
+    }  
+
+	  opDict = [NSMutableDictionary dictionaryWithCapacity: 4];
+	  [opDict setObject: operation forKey: @"operation"];
+	  [opDict setObject: source forKey: @"source"];
+	  [opDict setObject: [node path] forKey: @"destination"];
+	  [opDict setObject: files forKey: @"files"];
+
+    [desktopApp performFileOperation: opDict];
+
+  } else {
+    for (i = 0; i < [sourcePaths count]; i++) {  
+      NSString *path = [sourcePaths objectAtIndex: i];
+    
+      NS_DURING
+        {
+      [[NSWorkspace sharedWorkspace] openFile: path 
+                              withApplication: [node name]];
+        }
+      NS_HANDLER
+        {
+      NSRunAlertPanel(NSLocalizedString(@"error", @""), 
+          [NSString stringWithFormat: @"%@ %@!", 
+                    NSLocalizedString(@"Can't open ", @""), [node name]],
+                                        NSLocalizedString(@"OK", @""), 
+                                        nil, 
+                                        nil);                                     
+        }
+      NS_ENDHANDLER  
+    }
   }
-  
-  files = [NSMutableArray arrayWithCapacity: 1];    
-  for(i = 0; i < [sourcePaths count]; i++) {    
-    [files addObject: [[sourcePaths objectAtIndex: i] lastPathComponent]];
-  }  
-
-	opDict = [NSMutableDictionary dictionaryWithCapacity: 4];
-	[opDict setObject: operation forKey: @"operation"];
-	[opDict setObject: source forKey: @"source"];
-	[opDict setObject: [node path] forKey: @"destination"];
-	[opDict setObject: files forKey: @"files"];
-
-  [desktopApp performFileOperation: opDict];
 }
 
 @end
@@ -1405,7 +1558,7 @@ static NSImage *branchImage;
 
 - (void)dealloc
 {
-  TEST_RELEASE (node);
+  RELEASE (node);
   [super dealloc];
 }
 
@@ -1445,6 +1598,7 @@ static NSImage *branchImage;
       [self setAlignment: NSLeftTextAlignment];  
       [self setSelectable: YES];  
       [self setEditable: YES];  
+      [[self window] makeFirstResponder: self]; 
     }
     
   } else {
