@@ -1,6 +1,6 @@
 /* FSNIconsView.m
  *  
- * Copyright (C) 2004 Free Software Foundation, Inc.
+ * Copyright (C) 2004-2010 Free Software Foundation, Inc.
  *
  * Author: Enrico Sersale <enrico@imago.ro>
  * Date: March 2004
@@ -19,15 +19,19 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
  */
 
-#include <Foundation/Foundation.h>
-#include <AppKit/AppKit.h>
 #include <math.h>
-#include "FSNIconsView.h"
-#include "FSNIcon.h"
-#include "FSNFunctions.h"
+#include <unistd.h>
+#include <sys/types.h>
+
+#import <Foundation/Foundation.h>
+#import <AppKit/AppKit.h>
+
+#import "FSNIconsView.h"
+#import "FSNIcon.h"
+#import "FSNFunctions.h"
 
 #define DEF_ICN_SIZE 48
 #define DEF_TEXT_SIZE 12
@@ -64,15 +68,15 @@ if (rct.size.height < 0) rct.size.height = 0; \
 
 - (void)dealloc
 {
-  TEST_RELEASE (node);
-  TEST_RELEASE (extInfoType);
+  RELEASE (node);
+  RELEASE (extInfoType);
   RELEASE (icons);
   RELEASE (labelFont);
   RELEASE (nameEditor);
   RELEASE (horizontalImage);
   RELEASE (verticalImage);
-  TEST_RELEASE (lastSelection);
-  TEST_RELEASE (charBuffer);
+  RELEASE (lastSelection);
+  RELEASE (charBuffer);
   RELEASE (backColor);
   RELEASE (textColor);
   RELEASE (disabledTextColor);
@@ -184,7 +188,7 @@ if (rct.size.height < 0) rct.size.height = 0; \
     highlightSize.height = iconSize + 4;
   }
 
-  labelSize.height = floor([labelFont defaultLineHeightForFont]);
+  labelSize.height = floor([fsnodeRep heighOfFont: labelFont]);
   labelSize.width = [fsnodeRep labelWFactor] * labelTextSize;
 
   gridSize.height = highlightSize.height;
@@ -448,7 +452,9 @@ if (rct.size.height < 0) rct.size.height = 0; \
   NSRect selrect;
   float x, y, w, h;
   int i;
-  
+
+  pp = NSMakePoint(0,0);
+
 #define scrollPointToVisible(p) \
 { \
 NSRect sr; \
@@ -892,7 +898,8 @@ pp.y = NSMaxY(br) + 1; \
   NSString *prefsname = [NSString stringWithFormat: @"viewer_at_%@", [node path]];
   NSDictionary *nodeDict = nil;
 
-  if ([node isWritable]) {
+  if ([node isWritable]
+          && ([[fsnodeRep volumes] containsObject: [node path]] == NO)) {
     NSString *infoPath = [[node path] stringByAppendingPathComponent: @".gwdir"];
   
     if ([[NSFileManager defaultManager] fileExistsAtPath: infoPath]) {
@@ -949,15 +956,18 @@ pp.y = NSMaxY(br) + 1; \
   return nodeDict;
 }
 
-- (void)updateNodeInfo
+- (NSMutableDictionary *)updateNodeInfo:(BOOL)ondisk
 {
+  CREATE_AUTORELEASE_POOL(arp);
+  NSMutableDictionary *updatedInfo = nil;
+
   if ([node isValid]) {
     NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];	      
     NSString *prefsname = [NSString stringWithFormat: @"viewer_at_%@", [node path]];
     NSString *infoPath = [[node path] stringByAppendingPathComponent: @".gwdir"];
-    NSMutableDictionary *updatedInfo = nil;
-
-    if ([node isWritable]) {
+    BOOL writable = ([node isWritable] && ([[fsnodeRep volumes] containsObject: [node path]] == NO));
+    
+    if (writable) {
       if ([[NSFileManager defaultManager] fileExistsAtPath: infoPath]) {
         NSDictionary *dict = [NSDictionary dictionaryWithContentsOfFile: infoPath];
 
@@ -993,15 +1003,19 @@ pp.y = NSMaxY(br) + 1; \
     if (infoType == FSNInfoExtendedType) {
       [updatedInfo setObject: extInfoType forKey: @"ext_info_type"];
     }
-    
-    if ([node isWritable]) {
-      [updatedInfo writeToFile: infoPath atomically: YES];
-    } else {
-      [defaults setObject: updatedInfo forKey: prefsname];
+
+    if (ondisk) {
+      if (writable) {
+        [updatedInfo writeToFile: infoPath atomically: YES];
+      } else {
+        [defaults setObject: updatedInfo forKey: prefsname];
+      }
     }
-    
-    RELEASE (updatedInfo);
   }
+
+  RELEASE (arp);
+  
+  return (AUTORELEASE (updatedInfo));
 }
 
 - (void)reloadContents
@@ -1141,10 +1155,6 @@ pp.y = NSMaxY(br) + 1; \
     source = [source stringByDeletingLastPathComponent]; 
   }
 
-  if ([operation isEqual: @"NSWorkspaceRecycleOperation"]) {
-		files = [info objectForKey: @"origfiles"];
-  }	
-
   if (([ndpath isEqual: source] == NO) && ([ndpath isEqual: destination] == NO)) {
     [self reloadContents];
     return;
@@ -1154,16 +1164,19 @@ pp.y = NSMaxY(br) + 1; \
     if ([operation isEqual: @"NSWorkspaceMoveOperation"]
               || [operation isEqual: @"NSWorkspaceDestroyOperation"]
               || [operation isEqual: @"GWorkspaceRenameOperation"]
+              || [operation isEqual: @"NSWorkspaceRecycleOperation"]
 			        || [operation isEqual: @"GWorkspaceRecycleOutOperation"]) {
+      
+      if ([operation isEqual: @"NSWorkspaceRecycleOperation"]) {
+		    files = [info objectForKey: @"origfiles"];
+      }	
+              
       for (i = 0; i < [files count]; i++) {
         NSString *fname = [files objectAtIndex: i];
         FSNode *subnode = [FSNode nodeWithRelativePath: fname parent: node];
         [self removeRepOfSubnode: subnode];
       }
-    } else if ([operation isEqual: @"NSWorkspaceRecycleOperation"]) {
-      [self reloadContents];
-      return;
-    }
+    } 
   }
   
   if ([operation isEqual: @"GWorkspaceRenameOperation"]) {
@@ -1178,8 +1191,14 @@ pp.y = NSMaxY(br) + 1; \
               || [operation isEqual: @"NSWorkspaceDuplicateOperation"]
               || [operation isEqual: @"GWorkspaceCreateDirOperation"]
               || [operation isEqual: @"GWorkspaceCreateFileOperation"]
+              || [operation isEqual: @"NSWorkspaceRecycleOperation"]
               || [operation isEqual: @"GWorkspaceRenameOperation"]
 				      || [operation isEqual: @"GWorkspaceRecycleOutOperation"])) { 
+    
+    if ([operation isEqual: @"NSWorkspaceRecycleOperation"]) {
+		  files = [info objectForKey: @"files"];
+    }	
+
     for (i = 0; i < [files count]; i++) {
       NSString *fname = [files objectAtIndex: i];
       FSNode *subnode = [FSNode nodeWithRelativePath: fname parent: node];
@@ -1557,7 +1576,12 @@ pp.y = NSMaxY(br) + 1; \
   selectionMask |= FSNCreatingSelectionMask;
   
 	for (i = 0; i < [icons count]; i++) {
-    [[icons objectAtIndex: i] select];
+    FSNIcon *icon = [icons objectAtIndex: i];
+    FSNode *inode = [icon node];
+    
+    if ([inode isReserved] == NO) {
+      [icon select];
+    }
 	}
   
   selectionMask = NSSingleSelectionMask;
@@ -1647,10 +1671,10 @@ pp.y = NSMaxY(br) + 1; \
 - (void)selectionDidChange
 {
 	if (!(selectionMask & FSNCreatingSelectionMask)) {
-    NSArray *selection = [self selectedPaths];
+    NSArray *selection = [self selectedNodes];
 		
     if ([selection count] == 0) {
-      selection = [NSArray arrayWithObject: [node path]];
+      selection = [NSArray arrayWithObject: node];
     }
 
     if ((lastSelection == nil) || ([selection isEqual: lastSelection] == NO)) {
@@ -1689,7 +1713,7 @@ pp.y = NSMaxY(br) + 1; \
 - (void)restoreLastSelection
 {
   if (lastSelection) {
-    [self selectRepsOfPaths: lastSelection];
+    [self selectRepsOfSubnodes: lastSelection];
   }
 }
 
@@ -1784,7 +1808,7 @@ pp.y = NSMaxY(br) + 1; \
 
 @implementation FSNIconsView (DraggingDestination)
 
-- (unsigned int)draggingEntered:(id <NSDraggingInfo>)sender
+- (NSDragOperation)draggingEntered:(id <NSDraggingInfo>)sender
 {
 	NSPasteboard *pb;
   NSDragOperation sourceDragMask;
@@ -1849,6 +1873,27 @@ pp.y = NSMaxY(br) + 1; \
     prePath = [prePath stringByDeletingLastPathComponent];
   }
 
+  if ([node isDirectory] && [node isParentOfPath: basePath]) {
+    NSArray *subNodes = [node subNodes];
+    int i;
+    
+    for (i = 0; i < [subNodes count]; i++) {
+      FSNode *nd = [subNodes objectAtIndex: i];
+      
+      if ([nd isDirectory]) {
+        int j;
+        
+        for (j = 0; j < count; j++) {
+          NSString *fname = [[sourcePaths objectAtIndex: j] lastPathComponent];
+          
+          if ([[nd name] isEqual: fname]) {
+            return NSDragOperationNone;
+          }
+        }
+      }
+    }
+  }	
+
   isDragTarget = YES;	
   forceCopy = NO;
     
@@ -1871,7 +1916,7 @@ pp.y = NSMaxY(br) + 1; \
   return NSDragOperationNone;
 }
 
-- (unsigned int)draggingUpdated:(id <NSDraggingInfo>)sender
+- (NSDragOperation)draggingUpdated:(id <NSDraggingInfo>)sender
 {
   NSDragOperation sourceDragMask = [sender draggingSourceOperationMask];
   NSRect vr = [self visibleRect];
@@ -2036,7 +2081,7 @@ pp.y = NSMaxY(br) + 1; \
   [self stopRepNameEditing];
   
   if (lastSelection && ([lastSelection count] == 1)) {
-    editIcon = [self repOfSubnodePath: [lastSelection objectAtIndex: 0]]; 
+    editIcon = [self repOfSubnode: [lastSelection objectAtIndex: 0]]; 
   } 
   
   if (editIcon) {
@@ -2121,7 +2166,6 @@ pp.y = NSMaxY(br) + 1; \
     [nameEditor setNode: nil stringValue: @"" index: -1];
     [nameEditor removeFromSuperview];
     [self setNeedsDisplayInRect: edrect];
-    [[NSCursor arrowCursor] set];
   }
 
   for (i = 0; i < [icons count]; i++) {
@@ -2182,96 +2226,106 @@ pp.y = NSMaxY(br) + 1; \
 
 - (void)controlTextDidEndEditing:(NSNotification *)aNotification
 {
-  NSFileManager *fm = [NSFileManager defaultManager];
   FSNode *ednode = [nameEditor node];
-
+  BOOL writable = [ednode isWritable];
+    
 #define CLEAREDITING \
   [self stopRepNameEditing]; \
   return 
  
-  if ([ednode isWritable] == NO) {
-    NSRunAlertPanel(NSLocalizedString(@"Error", @""), 
-          [NSString stringWithFormat: @"%@\"%@\"!\n", 
-              NSLocalizedString(@"You have not write permission for ", @""), 
-                    [ednode name]], NSLocalizedString(@"Continue", @""), nil, nil);   
-    CLEAREDITING;
+  if (writable == NO) {
+    /* check for broken symlink */ 
+    if ([ednode isLink] && ([ednode hasValidPath] == NO)) { 
+      BOOL iamRoot;
+      
+      #ifdef __WIN32__
+		    iamRoot = YES;
+	    #else
+		    iamRoot = (geteuid() == 0);
+	    #endif
+      
+      writable = (iamRoot || [[ednode owner] isEqual: NSUserName()]);          
+    }
     
-  } else if ([fm isWritableFileAtPath: [ednode parentPath]] == NO) {
-    NSRunAlertPanel(NSLocalizedString(@"Error", @""), 
-          [NSString stringWithFormat: @"%@\"%@\"!\n", 
-              NSLocalizedString(@"You have not write permission for ", @""), 
-                  [ednode parentName]], NSLocalizedString(@"Continue", @""), nil, nil);   
-    CLEAREDITING;
-    
-  } else {
-    NSString *newname = [nameEditor stringValue];
-    NSString *newpath = [[ednode parentPath] stringByAppendingPathComponent: newname];
-    NSCharacterSet *notAllowSet = [NSCharacterSet characterSetWithCharactersInString: @"/\\*:?"];
-    NSRange range = [newname rangeOfCharacterFromSet: notAllowSet];
-    NSFileManager *fm = [NSFileManager defaultManager];
-    NSArray *dirContents = [fm directoryContentsAtPath: [ednode parentPath]];
-    NSMutableDictionary *userInfo = [NSMutableDictionary dictionary];
-    
-    if (range.length > 0) {
+    if (writable == NO) {
       NSRunAlertPanel(NSLocalizedString(@"Error", @""), 
-                NSLocalizedString(@"Invalid char in name", @""), 
-                          NSLocalizedString(@"Continue", @""), nil, nil);   
+            [NSString stringWithFormat: @"%@\"%@\"!\n", 
+                NSLocalizedString(@"You do not have write permission for ", @""), 
+                      [ednode name]], NSLocalizedString(@"Continue", @""), nil, nil);   
+      CLEAREDITING;
+    }
+  }
+  
+  if (writable) {
+    if ([ednode isParentWritable] == NO) {
+      NSRunAlertPanel(NSLocalizedString(@"Error", @""), 
+            [NSString stringWithFormat: @"%@\"%@\"!\n", 
+                NSLocalizedString(@"You do not have write permission for ", @""), 
+                    [ednode parentName]], NSLocalizedString(@"Continue", @""), nil, nil);   
       CLEAREDITING;
-    }	
 
-    if ([dirContents containsObject: newname]) {
-      if ([newname isEqual: [ednode name]]) {
-        CLEAREDITING;
-      } else {
+    } else if ([ednode isSubnodeOfPath: [desktopApp trashPath]]) {
+      NSRunAlertPanel(NSLocalizedString(@"Error", @""), 
+              NSLocalizedString(@"You can't rename an object that is in the Recycler", @""), 
+              NSLocalizedString(@"Continue", @""), nil, nil);   
+      CLEAREDITING;
+
+    } else {
+      NSString *newname = [nameEditor stringValue];
+      NSString *newpath = [[ednode parentPath] stringByAppendingPathComponent: newname];
+      NSString *extension = [newpath pathExtension];
+      NSCharacterSet *notAllowSet = [NSCharacterSet characterSetWithCharactersInString: @"/\\*:?\33"];
+      NSRange range = [newname rangeOfCharacterFromSet: notAllowSet];
+      NSArray *dirContents = [ednode subNodeNamesOfParent];
+      NSMutableDictionary *opinfo = [NSMutableDictionary dictionary];
+
+      if (([newname length] == 0) || (range.length > 0)) {
         NSRunAlertPanel(NSLocalizedString(@"Error", @""), 
-          [NSString stringWithFormat: @"%@\"%@\" %@ ", 
-              NSLocalizedString(@"The name ", @""), 
-              newname, NSLocalizedString(@" is already in use!", @"")], 
+                  NSLocalizedString(@"Invalid name", @""), 
                             NSLocalizedString(@"Continue", @""), nil, nil);   
         CLEAREDITING;
+      }	
+
+      if (([extension length] 
+              && ([ednode isDirectory] && ([ednode isPackage] == NO)))) {
+        NSString *msg = NSLocalizedString(@"Are you sure you want to add the extension ", @"");
+
+        msg = [msg stringByAppendingFormat: @"\"%@\" ", extension];
+        msg = [msg stringByAppendingString: NSLocalizedString(@"to the end of the name?", @"")];
+        msg = [msg stringByAppendingString: NSLocalizedString(@"\nif you make this change, your folder may appear as a single file.", @"")];
+
+        if (NSRunAlertPanel(@"", msg, 
+                            NSLocalizedString(@"Cancel", @""), 
+				                    NSLocalizedString(@"OK", @""), 
+                            nil) == NSAlertDefaultReturn) {
+          CLEAREDITING;
+        }
       }
+
+      if ([dirContents containsObject: newname]) {
+        if ([newname isEqual: [ednode name]]) {
+          CLEAREDITING;
+        } else {
+          NSRunAlertPanel(NSLocalizedString(@"Error", @""), 
+            [NSString stringWithFormat: @"%@\"%@\" %@ ", 
+                NSLocalizedString(@"The name ", @""), 
+                newname, NSLocalizedString(@" is already in use!", @"")], 
+                              NSLocalizedString(@"Continue", @""), nil, nil);   
+          CLEAREDITING;
+        }
+      }
+
+	    [opinfo setObject: @"GWorkspaceRenameOperation" forKey: @"operation"];	
+      [opinfo setObject: [ednode path] forKey: @"source"];	
+      [opinfo setObject: newpath forKey: @"destination"];	
+      [opinfo setObject: [NSArray arrayWithObject: @""] forKey: @"files"];	
+
+      [self stopRepNameEditing];
+      [desktopApp performFileOperation: opinfo];
     }
-        
-	  [userInfo setObject: @"GWorkspaceRenameOperation" forKey: @"operation"];	
-    [userInfo setObject: [ednode path] forKey: @"source"];	
-    [userInfo setObject: newpath forKey: @"destination"];	
-    [userInfo setObject: [NSArray arrayWithObject: @""] forKey: @"files"];	
-    
-    [desktopApp removeWatcherForPath: [node path]];
-  
-//    [[NSDistributedNotificationCenter defaultCenter]
-// 				postNotificationName: @"GWFileSystemWillChangeNotification"
-//	 								    object: nil 
-//                    userInfo: userInfo];
-
-    [fm movePath: [ednode path] toPath: newpath handler: self];
-
-    [[NSDistributedNotificationCenter defaultCenter]
- 				postNotificationName: @"GWFileSystemDidChangeNotification"
-	 								    object: nil 
-                    userInfo: userInfo];
-    
-    [desktopApp addWatcherForPath: [node path]];
   }
 }
 
-- (BOOL)fileManager:(NSFileManager *)manager 
-              shouldProceedAfterError:(NSDictionary *)errorDict
-{
-	NSString *title = NSLocalizedString(@"Error", @"");
-	NSString *msg1 = NSLocalizedString(@"Cannot rename ", @"");
-  NSString *name = [[nameEditor node] name];
-	NSString *msg2 = NSLocalizedString(@"Continue", @"");
-
-  NSRunAlertPanel(title, [NSString stringWithFormat: @"%@'%@'!", msg1, name], msg2, nil, nil);   
-
-	return NO;
-}
-
-- (void)fileManager:(NSFileManager *)manager willProcessPath:(NSString *)path
-{
-}
-
 @end
 
 
