@@ -1,6 +1,6 @@
 /* FSNListView.m
  *  
- * Copyright (C) 2004 Free Software Foundation, Inc.
+ * Copyright (C) 2004-2010 Free Software Foundation, Inc.
  *
  * Author: Enrico Sersale <enrico@imago.ro>
  * Date: December 2004
@@ -19,15 +19,17 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
  */
 
-#include <Foundation/Foundation.h>
-#include <AppKit/AppKit.h>
 #include <math.h>
-#include "FSNListView.h"
-#include "FSNTextCell.h"
-#include "FSNFunctions.h"
+#include <unistd.h>
+
+#import <Foundation/Foundation.h>
+#import <AppKit/AppKit.h>
+#import "FSNListView.h"
+#import "FSNTextCell.h"
+#import "FSNFunctions.h"
 
 #define ICNSIZE (24)
 #define CELLS_HEIGHT (28.0)
@@ -68,11 +70,11 @@ static NSString *defaultColumns = @"{ \
 
 - (void)dealloc
 {
-  TEST_RELEASE (node);
-  TEST_RELEASE (extInfoType);
+  RELEASE (node);
+  RELEASE (extInfoType);
   RELEASE (nodeReps);
   RELEASE (nameEditor);
-  TEST_RELEASE (lastSelection);
+  RELEASE (lastSelection);
 
   [super dealloc];
 }
@@ -115,9 +117,8 @@ static NSString *defaultColumns = @"{ \
 
     nameEditor = [FSNListViewNameEditor new];
     [nameEditor setDelegate: self];  
-    [nameEditor setEditable: YES];
-    [nameEditor setSelectable: YES];	   
-//	  [nameEditor setFont: [cellPrototype font]];
+    [nameEditor setEditable: NO];
+    [nameEditor setSelectable: NO];	   
 		[nameEditor setBezeled: NO];
 		[nameEditor setAlignment: NSLeftTextAlignment];
     
@@ -174,7 +175,7 @@ static NSString *defaultColumns = @"{ \
       [[column headerCell] setStringValue: NSLocalizedString(@"Name", @"")];
       break;
     case FSNInfoKindType:
-      [[column headerCell] setStringValue: NSLocalizedString(@"Kind", @"")];
+      [[column headerCell] setStringValue: NSLocalizedString(@"Type", @"")];
       break;
     case FSNInfoDateType:
       [[column headerCell] setStringValue: NSLocalizedString(@"Date Modified", @"")];
@@ -632,7 +633,8 @@ static NSString *defaultColumns = @"{ \
   NSString *prefsname = [NSString stringWithFormat: @"viewer_at_%@", [infoNode path]];
   NSDictionary *nodeDict = nil;
 
-  if ([infoNode isWritable]) {
+  if ([infoNode isWritable]
+          && ([[fsnodeRep volumes] containsObject: [node path]] == NO)) {
     NSString *infoPath = [[infoNode path] stringByAppendingPathComponent: @".gwdir"];
   
     if ([[NSFileManager defaultManager] fileExistsAtPath: infoPath]) {
@@ -670,17 +672,19 @@ static NSString *defaultColumns = @"{ \
   return nodeDict;
 }
 
-- (void)updateNodeInfo
+- (NSMutableDictionary *)updateNodeInfo:(BOOL)ondisk
 {
+  CREATE_AUTORELEASE_POOL(arp);
   FSNode *infoNode = [self infoNode];
+  NSMutableDictionary *updatedInfo = nil;
 
   if ([infoNode isValid]) {
     NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];	      
     NSString *prefsname = [NSString stringWithFormat: @"viewer_at_%@", [infoNode path]];
     NSString *infoPath = [[infoNode path] stringByAppendingPathComponent: @".gwdir"];
-    NSMutableDictionary *updatedInfo = nil;
-
-    if ([infoNode isWritable]) {
+    BOOL writable = ([infoNode isWritable] && ([[fsnodeRep volumes] containsObject: [node path]] == NO));
+    
+    if (writable) {
       if ([[NSFileManager defaultManager] fileExistsAtPath: infoPath]) {
         NSDictionary *dict = [NSDictionary dictionaryWithContentsOfFile: infoPath];
 
@@ -711,9 +715,18 @@ static NSString *defaultColumns = @"{ \
       [updatedInfo setObject: extInfoType forKey: @"ext_info_type"];
     }
     
-    [updatedInfo writeToFile: infoPath atomically: YES];
-    RELEASE (updatedInfo);
+    if (ondisk) {
+      if (writable) {
+        [updatedInfo writeToFile: infoPath atomically: YES];
+      } else {
+        [defaults setObject: updatedInfo forKey: prefsname];
+      }
+    }
   }
+  
+  RELEASE (arp);
+  
+  return (AUTORELEASE (updatedInfo));
 }
 
 - (void)reloadContents
@@ -843,10 +856,6 @@ static NSString *defaultColumns = @"{ \
     source = [source stringByDeletingLastPathComponent]; 
   }
 
-  if ([operation isEqual: @"NSWorkspaceRecycleOperation"]) {
-		files = [info objectForKey: @"origfiles"];
-  }	
-
   if (([ndpath isEqual: source] == NO) && ([ndpath isEqual: destination] == NO)) {    
     [self reloadContents];
     return;
@@ -855,18 +864,21 @@ static NSString *defaultColumns = @"{ \
   if ([ndpath isEqual: source]) {
     if ([operation isEqual: @"NSWorkspaceMoveOperation"]
               || [operation isEqual: @"NSWorkspaceDestroyOperation"]
+              || [operation isEqual: @"NSWorkspaceRecycleOperation"]
               || [operation isEqual: @"GWorkspaceRenameOperation"]
 			        || [operation isEqual: @"GWorkspaceRecycleOutOperation"]) {
+
+      if ([operation isEqual: @"NSWorkspaceRecycleOperation"]) {
+		    files = [info objectForKey: @"origfiles"];
+      }	
+
       for (i = 0; i < [files count]; i++) {
         NSString *fname = [files objectAtIndex: i];
         FSNode *subnode = [FSNode nodeWithRelativePath: fname parent: node];
         [self removeRepOfSubnode: subnode];
       }
       needsreload = YES;
-    } else if ([operation isEqual: @"NSWorkspaceRecycleOperation"]) {
-      [self reloadContents];
-      return;
-    }
+    } 
   }
 
   if ([operation isEqual: @"GWorkspaceRenameOperation"]) {
@@ -881,8 +893,14 @@ static NSString *defaultColumns = @"{ \
               || [operation isEqual: @"NSWorkspaceDuplicateOperation"]
               || [operation isEqual: @"GWorkspaceCreateDirOperation"]
               || [operation isEqual: @"GWorkspaceCreateFileOperation"]
+              || [operation isEqual: @"NSWorkspaceRecycleOperation"]
               || [operation isEqual: @"GWorkspaceRenameOperation"]
 				      || [operation isEqual: @"GWorkspaceRecycleOutOperation"])) { 
+
+    if ([operation isEqual: @"NSWorkspaceRecycleOperation"]) {
+		  files = [info objectForKey: @"files"];
+    }	
+
     for (i = 0; i < [files count]; i++) {
       NSString *fname = [files objectAtIndex: i];
       FSNode *subnode = [FSNode nodeWithRelativePath: fname parent: node];
@@ -1198,8 +1216,22 @@ static NSString *defaultColumns = @"{ \
 
 - (void)selectAll
 {
-  [listView selectAll: self];
-  [listView setNeedsDisplay: YES];
+  NSMutableIndexSet *set = [NSMutableIndexSet indexSet];
+  unsigned int i;
+
+  for (i = 0; i < [nodeReps count]; i++) {
+    FSNListViewNodeRep *rep = [nodeReps objectAtIndex: i];
+  
+    if ([[rep node] isReserved] == NO) {
+      [set addIndex: i];
+    }
+  }
+
+  if ([set count]) {
+    [listView deselectAll: self];
+    [listView selectRowIndexes: set byExtendingSelection: NO];
+    [listView setNeedsDisplay: YES];
+  }
 }
 
 - (void)scrollSelectionToVisible
@@ -1288,10 +1320,10 @@ static NSString *defaultColumns = @"{ \
 
 - (void)selectionDidChange
 {
-  NSArray *selection = [self selectedPaths];
+  NSArray *selection = [self selectedNodes];
 
   if ([selection count] == 0) {
-    selection = [NSArray arrayWithObject: [node path]];
+    selection = [NSArray arrayWithObject: node];
   }
 
   if ((lastSelection == nil) || ([selection isEqual: lastSelection] == NO)) {
@@ -1367,14 +1399,13 @@ static NSString *defaultColumns = @"{ \
 
 - (void)stopRepNameEditing
 {
-  if (nameEditor && [[listView subviews] containsObject: nameEditor]) {
+  if ([[listView subviews] containsObject: nameEditor]) {
     [nameEditor abortEditing];
     [nameEditor setEditable: NO];
     [nameEditor setSelectable: NO];
     [nameEditor setNode: nil stringValue: @"" index: -1];
     [nameEditor removeFromSuperview];
     [listView setNeedsDisplayInRect: [nameEditor frame]];
-    [[NSCursor arrowCursor] set];
   }
 }
 
@@ -1385,19 +1416,20 @@ static NSString *defaultColumns = @"{ \
 
 - (void)setEditorAtRow:(int)row
 {
+  [self stopRepNameEditing];
+
   if ([[listView selectedRowIndexes] count] == 1) {
     FSNListViewNodeRep *rep = [nodeReps objectAtIndex: row];  
     FSNode *nd = [rep node];
     BOOL canedit = (([rep isLocked] == NO) && ([nd isMountPoint] == NO));
     
-    [self stopRepNameEditing];
-  
     if (canedit) {   
       NSNumber *num = [NSNumber numberWithInt: FSNInfoNameType];
       unsigned col = [listView columnWithIdentifier: num];
       NSRect r = [listView frameOfCellAtColumn: col row: row];
       NSFont *edfont = [nameEditor font];
-      float fnheight = [edfont defaultLineHeightForFont];  
+      float fnheight = [fsnodeRep heighOfFont: edfont]; 
+      
       float xshift = [[rep icon] size].width + 4;
   
       r.origin.y += ((r.size.height - fnheight) / 2);
@@ -1409,8 +1441,6 @@ static NSString *defaultColumns = @"{ \
 
       [nameEditor setNode: nd stringValue: [nd name] index: 0];
 
-      [nameEditor setEditable: YES];
-      [nameEditor setSelectable: YES];	
       [listView addSubview: nameEditor];
     }
   }
@@ -1422,99 +1452,106 @@ static NSString *defaultColumns = @"{ \
 
 - (void)controlTextDidEndEditing:(NSNotification *)aNotification
 {
-  NSFileManager *fm = [NSFileManager defaultManager];
   FSNode *ednode = [nameEditor node];
+  BOOL writable = [ednode isWritable];
 
 #define CLEAREDITING \
   [self stopRepNameEditing]; \
   return 
     
-  if ([ednode isWritable] == NO) {
-    NSRunAlertPanel(NSLocalizedString(@"Error", @""), 
-          [NSString stringWithFormat: @"%@\"%@\"!\n", 
-              NSLocalizedString(@"You have not write permission for ", @""), 
-                    [ednode name]], NSLocalizedString(@"Continue", @""), nil, nil);   
-    CLEAREDITING;
+  if (writable == NO) {
+    /* check for broken symlink */     
+    if ([ednode isLink] && ([ednode hasValidPath] == NO)) { 
+      BOOL iamRoot;
+      
+      #ifdef __WIN32__
+		    iamRoot = YES;
+	    #else
+		    iamRoot = (geteuid() == 0);
+	    #endif
+      
+      writable = (iamRoot || [[ednode owner] isEqual: NSUserName()]);          
+    }
     
-  } else if ([fm isWritableFileAtPath: [ednode parentPath]] == NO) {
-    NSRunAlertPanel(NSLocalizedString(@"Error", @""), 
-          [NSString stringWithFormat: @"%@\"%@\"!\n", 
-              NSLocalizedString(@"You have not write permission for ", @""), 
-                  [ednode parentName]], NSLocalizedString(@"Continue", @""), nil, nil);   
-    CLEAREDITING;
-    
-  } else {
-    NSString *newname = [nameEditor stringValue];
-    NSString *newpath = [[ednode parentPath] stringByAppendingPathComponent: newname];
-    NSCharacterSet *notAllowSet = [NSCharacterSet characterSetWithCharactersInString: @"/\\*:?"];
-    NSRange range = [newname rangeOfCharacterFromSet: notAllowSet];
-    NSFileManager *fm = [NSFileManager defaultManager];    
-    NSArray *dirContents = [fm directoryContentsAtPath: [ednode parentPath]];
-    NSMutableDictionary *userInfo = [NSMutableDictionary dictionary];
-    
-    if (range.length > 0) {
+    if (writable == NO) {
       NSRunAlertPanel(NSLocalizedString(@"Error", @""), 
-                NSLocalizedString(@"Invalid char in name", @""), 
-                          NSLocalizedString(@"Continue", @""), nil, nil);   
+            [NSString stringWithFormat: @"%@\"%@\"!\n", 
+                NSLocalizedString(@"You do not have write permission for ", @""), 
+                      [ednode name]], NSLocalizedString(@"Continue", @""), nil, nil);   
       CLEAREDITING;
-    }	
+    }
+  } 
 
-    if ([dirContents containsObject: newname]) {
-      if ([newname isEqual: [ednode name]]) {
-        CLEAREDITING;
-      } else {
+  if (writable) {    
+    if ([ednode isParentWritable] == NO) {
+      NSRunAlertPanel(NSLocalizedString(@"Error", @""), 
+            [NSString stringWithFormat: @"%@\"%@\"!\n", 
+                NSLocalizedString(@"You do not have write permission for ", @""), 
+                    [ednode parentName]], NSLocalizedString(@"Continue", @""), nil, nil);   
+      CLEAREDITING;
+
+    } else if ([ednode isSubnodeOfPath: [desktopApp trashPath]]) {
+      NSRunAlertPanel(NSLocalizedString(@"Error", @""), 
+              NSLocalizedString(@"You can't rename an object that is in the Recycler", @""), 
+              NSLocalizedString(@"Continue", @""), nil, nil);   
+      CLEAREDITING;
+
+    } else {
+      NSString *newname = [nameEditor stringValue];
+      NSString *newpath = [[ednode parentPath] stringByAppendingPathComponent: newname];
+      NSString *extension = [newpath pathExtension];
+      NSCharacterSet *notAllowSet = [NSCharacterSet characterSetWithCharactersInString: @"/\\*:?\33"];
+      NSRange range = [newname rangeOfCharacterFromSet: notAllowSet];
+      NSArray *dirContents = [ednode subNodeNamesOfParent];
+      NSMutableDictionary *opinfo = [NSMutableDictionary dictionary];
+
+      if (([newname length] == 0) || (range.length > 0)) {
         NSRunAlertPanel(NSLocalizedString(@"Error", @""), 
-          [NSString stringWithFormat: @"%@\"%@\" %@ ", 
-              NSLocalizedString(@"The name ", @""), 
-              newname, NSLocalizedString(@" is already in use!", @"")], 
+                  NSLocalizedString(@"Invalid name", @""), 
                             NSLocalizedString(@"Continue", @""), nil, nil);   
         CLEAREDITING;
+      }	
+
+      if (([extension length] 
+              && ([ednode isDirectory] && ([ednode isPackage] == NO)))) {
+        NSString *msg = NSLocalizedString(@"Are you sure you want to add the extension ", @"");
+
+        msg = [msg stringByAppendingFormat: @"\"%@\" ", extension];
+        msg = [msg stringByAppendingString: NSLocalizedString(@"to the end of the name?", @"")];
+        msg = [msg stringByAppendingString: NSLocalizedString(@"\nif you make this change, your folder may appear as a single file.", @"")];
+
+        if (NSRunAlertPanel(@"", msg, 
+                            NSLocalizedString(@"Cancel", @""), 
+				                    NSLocalizedString(@"OK", @""), 
+                            nil) == NSAlertDefaultReturn) {
+          CLEAREDITING;
+        }
       }
+
+      if ([dirContents containsObject: newname]) {
+        if ([newname isEqual: [ednode name]]) {
+          CLEAREDITING;
+        } else {
+          NSRunAlertPanel(NSLocalizedString(@"Error", @""), 
+            [NSString stringWithFormat: @"%@\"%@\" %@ ", 
+                NSLocalizedString(@"The name ", @""), 
+                newname, NSLocalizedString(@" is already in use!", @"")], 
+                              NSLocalizedString(@"Continue", @""), nil, nil);   
+          CLEAREDITING;
+        }
+      }
+
+	    [opinfo setObject: @"GWorkspaceRenameOperation" forKey: @"operation"];	
+      [opinfo setObject: [ednode path] forKey: @"source"];	
+      [opinfo setObject: newpath forKey: @"destination"];	
+      [opinfo setObject: [NSArray arrayWithObject: @""] forKey: @"files"];	
+
+      [self stopRepNameEditing];
+      [desktopApp performFileOperation: opinfo];         
     }
-
-	  [userInfo setObject: @"GWorkspaceRenameOperation" forKey: @"operation"];	
-    [userInfo setObject: [ednode path] forKey: @"source"];	
-    [userInfo setObject: newpath forKey: @"destination"];	
-    [userInfo setObject: [NSArray arrayWithObject: @""] forKey: @"files"];	
-
-    [desktopApp removeWatcherForPath: [node path]];
-
-//    [[NSDistributedNotificationCenter defaultCenter]
-// 				postNotificationName: @"GWFileSystemWillChangeNotification"
-//	 								    object: nil 
-//                    userInfo: userInfo];
-    
-    [fm movePath: [ednode path] toPath: newpath handler: self];
-    
-    [[NSDistributedNotificationCenter defaultCenter]
- 				postNotificationName: @"GWFileSystemDidChangeNotification"
-	 								    object: nil 
-                    userInfo: userInfo];
-                    
-    [desktopApp addWatcherForPath: [node path]];
-
-    CLEAREDITING;
   }
 }
 
-- (BOOL)fileManager:(NSFileManager *)manager 
-              shouldProceedAfterError:(NSDictionary *)errorDict
-{
-	NSString *title = NSLocalizedString(@"Error", @"");
-	NSString *msg1 = NSLocalizedString(@"Cannot rename ", @"");
-  NSString *name = [[nameEditor node] name];
-	NSString *msg2 = NSLocalizedString(@"Continue", @"");
-
-  NSRunAlertPanel(title, [NSString stringWithFormat: @"%@'%@'!", msg1, name], msg2, nil, nil);   
-
-	return NO;
-}
-
-- (void)fileManager:(NSFileManager *)manager 
-    willProcessPath:(NSString *)path
-{
-}
-
 @end
 
 
@@ -1532,8 +1569,8 @@ static NSString *defaultColumns = @"{ \
   return NSPointInRect(loc, dndValidRect);
 }
 
-- (unsigned int)checkReturnValueForRep:(FSNListViewNodeRep *)arep
-                      withDraggingInfo:(id <NSDraggingInfo>)sender
+- (NSDragOperation)checkReturnValueForRep:(FSNListViewNodeRep *)arep
+                         withDraggingInfo:(id <NSDraggingInfo>)sender
 {
   if (dndTarget != arep) {
     dndTarget = arep;
@@ -1549,7 +1586,7 @@ static NSString *defaultColumns = @"{ \
   return dragOperation;
 }
 
-- (unsigned int)listViewDraggingEntered:(id <NSDraggingInfo>)sender
+- (NSDragOperation)listViewDraggingEntered:(id <NSDraggingInfo>)sender
 {
   NSPoint location;
   int row;
@@ -1644,6 +1681,27 @@ static NSString *defaultColumns = @"{ \
       prePath = [prePath stringByDeletingLastPathComponent];
     }
 
+    if ([node isDirectory] && [node isParentOfPath: basePath]) {
+      NSArray *subNodes = [node subNodes];
+      int i;
+
+      for (i = 0; i < [subNodes count]; i++) {
+        FSNode *nd = [subNodes objectAtIndex: i];
+
+        if ([nd isDirectory]) {
+          int j;
+
+          for (j = 0; j < count; j++) {
+            NSString *fname = [[sourcePaths objectAtIndex: j] lastPathComponent];
+
+            if ([[nd name] isEqual: fname]) {
+              return NSDragOperationNone;
+            }
+          }
+        }
+      }
+    }	
+
     isDragTarget = YES;	
     forceCopy = NO;
 
@@ -1666,7 +1724,7 @@ static NSString *defaultColumns = @"{ \
   return dragOperation;
 }
 
-- (unsigned int)listViewDraggingUpdated:(id <NSDraggingInfo>)sender
+- (NSDragOperation)listViewDraggingUpdated:(id <NSDraggingInfo>)sender
 {
   NSPoint location;
   int row;
@@ -1824,9 +1882,9 @@ static NSString *defaultColumns = @"{ \
 - (void)dealloc
 {
   RELEASE (icon); 
-  TEST_RELEASE (openicon); 
-  TEST_RELEASE (lockedicon); 
-  TEST_RELEASE (spopenicon); 
+  RELEASE (openicon); 
+  RELEASE (lockedicon); 
+  RELEASE (spopenicon); 
   RELEASE (extInfoStr); 
   [super dealloc];
 }
@@ -2146,7 +2204,7 @@ static NSString *defaultColumns = @"{ \
 
 @implementation FSNListViewNodeRep (DraggingDestination)
 
-- (unsigned int)repDraggingEntered:(id <NSDraggingInfo>)sender
+- (NSDragOperation)repDraggingEntered:(id <NSDraggingInfo>)sender
 {
 	NSPasteboard *pb;
   NSDragOperation sourceDragMask;
@@ -2162,6 +2220,14 @@ static NSString *defaultColumns = @"{ \
                     || [node isPackage] || ([node isWritable] == NO)) {
     return NSDragOperationNone;
   }
+
+  if ([node isDirectory]) {
+    id <DesktopApplication> desktopApp = [dataSource desktopApp];
+
+    if ([node isSubnodeOfPath: [desktopApp trashPath]]) { 
+      return NSDragOperationNone;
+    }
+  }	
   	
 	pb = [sender draggingPasteboard];
 
@@ -2213,6 +2279,27 @@ static NSString *defaultColumns = @"{ \
     prePath = [prePath stringByDeletingLastPathComponent];
   }
 
+  if ([node isDirectory] && [node isParentOfPath: fromPath]) {
+    NSArray *subNodes = [node subNodes];
+    int i;
+    
+    for (i = 0; i < [subNodes count]; i++) {
+      FSNode *nd = [subNodes objectAtIndex: i];
+      
+      if ([nd isDirectory]) {
+        int j;
+        
+        for (j = 0; j < count; j++) {
+          NSString *fname = [[sourcePaths objectAtIndex: j] lastPathComponent];
+          
+          if ([[nd name] isEqual: fname]) {
+            return NSDragOperationNone;
+          }
+        }
+      }
+    }
+  }	
+
   isDragTarget = YES;
   forceCopy = NO;
 
@@ -2302,9 +2389,17 @@ static NSString *defaultColumns = @"{ \
 
 @implementation FSNListViewNameEditor
 
+int sortSubviews(id view1, id view2, void *context)
+{
+  if ([view1 isMemberOfClass: [FSNListViewNameEditor class]]) {
+    return NSOrderedAscending;
+  }  
+  return NSOrderedDescending;
+}
+
 - (void)dealloc
 {
-  TEST_RELEASE (node);
+  RELEASE (node);
   [super dealloc];
 }
 
@@ -2330,12 +2425,20 @@ static NSString *defaultColumns = @"{ \
   return index;
 }
 
-- (void)mouseDown:(NSEvent*)theEvent
+- (void)mouseDown:(NSEvent *)theEvent
 {
-  if ([self isEditable]) {
+  NSView *view = [self superview];
+  
+  if ([self isEditable] == NO) {
+    [self setSelectable: YES];  
+    [self setEditable: YES];       
     [[self window] makeFirstResponder: self];
+  } else {  
+    [super mouseDown: theEvent];
   }
-  [super mouseDown: theEvent];
+
+  [view sortSubviewsUsingFunction: (int (*)(id, id, void *))sortSubviews context: nil];  
+  [view setNeedsDisplayInRect: [self frame]];
 }
 
 @end
@@ -2345,7 +2448,7 @@ static NSString *defaultColumns = @"{ \
 
 - (void)dealloc
 {
-  TEST_RELEASE (charBuffer);  
+  RELEASE (charBuffer);  
   RELEASE (dsource);  
   [super dealloc];
 }
@@ -2372,9 +2475,6 @@ static NSString *defaultColumns = @"{ \
     [self setTarget: dsource];
     [self setDoubleAction: @selector(doubleClickOnListView:)];
   
-    editstamp = 0.0;
-    editindex = -1;
-
 		lastKeyPressed = 0.;
     charBuffer = nil;
   
@@ -2391,12 +2491,12 @@ static NSString *defaultColumns = @"{ \
 - (void)checkSize
 {
   id sview = [self superview];
-  if (sview && ([self frame].size.width < [sview frame].size.width)) {
+  if (sview && ([self bounds].size.width < [sview bounds].size.width)) {
     [self sizeLastColumnToFit];  
   }
 }
 
-- (void)mouseDown:(NSEvent*)theEvent
+- (void)mouseDown:(NSEvent *)theEvent
 {
   NSPoint location;
   int clickCount;
@@ -2411,12 +2511,10 @@ static NSString *defaultColumns = @"{ \
   clickCount = [theEvent clickCount];
   
   if (clickCount >= 2) {
-    editindex = -1;
     return;
   }
 
   if ([theEvent modifierFlags] & NSShiftKeyMask) {
-    editindex = -1;
     return;
   }
   
@@ -2424,20 +2522,8 @@ static NSString *defaultColumns = @"{ \
   location = [self convertPoint: location fromView: nil];
   row = [self rowAtPoint: location];
   
-  if (row != -1) {
-    if (editindex != row) {
-      editindex = row;
-            
-    } else {
-      NSTimeInterval interval = ([theEvent timestamp] - editstamp);
-          
-      if ((interval > DOUBLE_CLICK_LIMIT)
-                                && (interval < EDIT_CLICK_LIMIT)) {
-        [dsource setEditorAtRow: row];
-      }
-    }
-    
-    editstamp = [theEvent timestamp];   
+  if (row != -1) {  
+    [dsource setEditorAtRow: row];
   }
 }
 
@@ -2455,6 +2541,7 @@ static NSString *defaultColumns = @"{ \
 
   switch (character) {
     case NSPageUpFunctionKey:
+      [dsource stopRepNameEditing];
 		  vRect = [self visibleRect];
 		  p = vRect.origin;
 		  x = p.x;
@@ -2466,6 +2553,7 @@ static NSString *defaultColumns = @"{ \
 	    return;
 
     case NSPageDownFunctionKey:
+      [dsource stopRepNameEditing];
 		  vRect = [self visibleRect];
 		  p = vRect.origin;    
 		  x = p.x;
@@ -2477,10 +2565,12 @@ static NSString *defaultColumns = @"{ \
 	    return;
 
     case NSUpArrowFunctionKey:
+      [dsource stopRepNameEditing];
 	    [dsource selectRepInPrevRow];
       return;
 
     case NSDownArrowFunctionKey:
+      [dsource stopRepNameEditing];
 	    [dsource selectRepInNextRow];
       return;
   
@@ -2570,9 +2660,9 @@ static NSString *defaultColumns = @"{ \
   return [dsource readNodeInfo];
 }
 
-- (void)updateNodeInfo
+- (NSMutableDictionary *)updateNodeInfo:(BOOL)ondisk
 {
-  [dsource updateNodeInfo];
+  return [dsource updateNodeInfo: ondisk];
 }
 
 - (void)reloadContents
@@ -2771,12 +2861,12 @@ static NSString *defaultColumns = @"{ \
 
 @implementation FSNListView (DraggingDestination)
 
-- (unsigned int)draggingEntered:(id <NSDraggingInfo>)sender
+- (NSDragOperation)draggingEntered:(id <NSDraggingInfo>)sender
 {
   return [dsource listViewDraggingEntered: sender];
 }
 
-- (unsigned int)draggingUpdated:(id <NSDraggingInfo>)sender
+- (NSDragOperation)draggingUpdated:(id <NSDraggingInfo>)sender
 {
   return [dsource listViewDraggingUpdated: sender];
 }
@@ -2814,3 +2904,5 @@ static NSString *defaultColumns = @"{ \
 }
 
 @end
+
+
