@@ -1,6 +1,6 @@
 /* FSNode.m
  *  
- * Copyright (C) 2004 Free Software Foundation, Inc.
+ * Copyright (C) 2004-2010 Free Software Foundation, Inc.
  *
  * Author: Enrico Sersale <enrico@imago.ro>
  * Date: March 2004
@@ -19,34 +19,34 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
  */
 
-#include <Foundation/Foundation.h>
-#include <AppKit/AppKit.h>
-#include "FSNode.h"
-#include "FSNodeRep.h"
-#include "FSNFunctions.h"
-#include "GNUstep.h"
+#import <Foundation/Foundation.h>
+#import <AppKit/AppKit.h>
+#import <GNUstepBase/GNUstep.h>
+#import "FSNode.h"
+#import "FSNodeRep.h"
+#import "FSNFunctions.h"
 
 @implementation FSNode
 
 - (void)dealloc
 {
-  TEST_RELEASE (path);
-  TEST_RELEASE (relativePath);  
-  TEST_RELEASE (name);  
-  TEST_RELEASE (attributes);  
-  TEST_RELEASE (fileType);
-  TEST_RELEASE (typeDescription);
-  TEST_RELEASE (crDate);
-  TEST_RELEASE (crDateDescription);
-  TEST_RELEASE (modDate);
-  TEST_RELEASE (modDateDescription);
-  TEST_RELEASE (owner);
-  TEST_RELEASE (ownerId);
-  TEST_RELEASE (group);
-  TEST_RELEASE (groupId);
+  RELEASE (path);
+  RELEASE (relativePath);  
+  RELEASE (name);  
+  RELEASE (attributes);  
+  RELEASE (fileType);
+  RELEASE (typeDescription);
+  RELEASE (crDate);
+  RELEASE (crDateDescription);
+  RELEASE (modDate);
+  RELEASE (modDateDescription);
+  RELEASE (owner);
+  RELEASE (ownerId);
+  RELEASE (group);
+  RELEASE (groupId);
 
   [super dealloc];
 }
@@ -116,12 +116,11 @@
     
     fileType = nil;    
     typeDescription = nil;
+    
+    application = nil;
                                       
     attributes = [fm fileAttributesAtPath: path traverseLink: NO];
-                                              
-    if (attributes) {
-      RETAIN (attributes);
-    }
+    RETAIN (attributes);
   }
     
   return self;
@@ -159,7 +158,7 @@
   int i;
   
   for (i = 0; i < [fnames count]; i++) {
-    NSString *fname = [fnames objectAtIndex: i] ;
+    NSString *fname = [fnames objectAtIndex: i];
     FSNode *node = [[FSNode alloc] initWithRelativePath: fname parent: self];
 
     [nodes addObject: node];
@@ -172,6 +171,42 @@
   return [[nodes autorelease] makeImmutableCopyOnFail: NO];
 }
 
+- (NSArray *)subNodeNames 
+{
+  return [fsnodeRep directoryContentsAtPath: path];
+}
+
+- (NSArray *)subNodesOfParent
+{
+  CREATE_AUTORELEASE_POOL(arp);
+  NSMutableArray *nodes = [NSMutableArray array];
+  NSArray *fnames = [fsnodeRep directoryContentsAtPath: [self parentPath]];
+  FSNode *pnd = nil;
+  int i;
+  
+  if (parent != nil) {
+    pnd = [parent parent];
+  }
+  
+  for (i = 0; i < [fnames count]; i++) {
+    NSString *fname = [fnames objectAtIndex: i];
+    FSNode *node = [[FSNode alloc] initWithRelativePath: fname parent: pnd];
+
+    [nodes addObject: node];
+    RELEASE (node);
+  }
+  
+  RETAIN (nodes);
+  RELEASE (arp);
+    
+  return [[nodes autorelease] makeImmutableCopyOnFail: NO];
+}
+
+- (NSArray *)subNodeNamesOfParent
+{
+  return [fsnodeRep directoryContentsAtPath: [self parentPath]];
+}
+
 + (NSArray *)nodeComponentsToNode:(FSNode *)anode
 {
   CREATE_AUTORELEASE_POOL(arp);
@@ -225,7 +260,7 @@
     RETAIN (components);
     RELEASE (arp);
     
-    return [components makeImmutableCopyOnFail: NO];
+    return [[components autorelease] makeImmutableCopyOnFail: NO];
     
   } else if ([secondNode isEqual: firstNode]) {
     return [NSArray arrayWithObject: firstNode];
@@ -351,6 +386,11 @@
   return NO;
 }
 
+- (FSNode *)parent
+{
+  return parent;
+}
+
 - (NSString *)parentPath
 {
   return [path stringByDeletingLastPathComponent];
@@ -401,7 +441,15 @@
   if (attributes && (fileType == nil)) {
     ASSIGN (fileType, [attributes fileType]);
   }
-  return (fileType ? fileType : [NSString string]);
+  return (fileType ? fileType : (NSString *)[NSString string]);
+}
+
+- (NSString *)application
+{
+  if ([self isApplication] == NO) {
+    return application;
+  }
+  return nil;
 }
 
 - (void)setTypeFlags
@@ -426,10 +474,14 @@
       flags.plain = 1;
 
     } else if (fileType == NSFileTypeDirectory) {
-	    NSString *defApp, *type;
+	    NSString *defApp = nil, *type = nil;
 
 	    [ws getInfoForFile: path application: &defApp type: &type]; 
-
+      
+      if (defApp) {
+        ASSIGN (application, defApp);
+      }
+      
       flags.directory = 1;
 
 	    if (type == NSApplicationFileType) {
@@ -471,10 +523,14 @@
     flags.plain = 1;
 
   } else if (ftype == NSFileTypeDirectory) {
-	  NSString *defApp, *type;
+	  NSString *defApp = nil, *type = nil;
 
 	  [ws getInfoForFile: path application: &defApp type: &type]; 
-
+      
+    if (defApp) {
+      ASSIGN (application, defApp);
+    }
+    
     flags.directory = 1;
 
 	  if (type == NSApplicationFileType) {
@@ -500,6 +556,8 @@
   } else {
     flags.unknown = 1;
   } 
+
+  ASSIGN (typeDescription, NSLocalizedString(@"symbolic link", @""));
 }
 
 - (NSString *)typeDescription
@@ -538,7 +596,7 @@
   if (attributes && (crDate == nil)) {
     ASSIGN (crDate, [attributes fileCreationDate]);
   }
-  return (crDate ? crDate : [NSDate date]);
+  return (crDate ? crDate : (NSDate *)[NSDate date]);
 }
 
 - (NSString *)crDateDescription
@@ -564,7 +622,7 @@
   if (attributes && (modDate == nil)) {
     ASSIGN (modDate, [attributes fileModificationDate]);
   }
-  return (modDate ? modDate : [NSDate date]);
+  return (modDate ? modDate : (NSDate *)[NSDate date]);
 }
 
 - (NSString *)modDateDescription
@@ -611,7 +669,7 @@
 	if (fsize == 0) {
 		sizeStr = @"0 bytes";
 	} else if (fsize < (10 * ONE_KB)) {
-		sizeStr = [NSString stringWithFormat: @"%s%d bytes", sign, fsize];
+		sizeStr = [NSString stringWithFormat: @"%s%d bytes", sign, (long)fsize];
 	} else if(fsize < (100 * ONE_KB)) {
  		sizeStr = [NSString stringWithFormat: @"%s%3.2f KB", sign,
                           					  ((double)fsize / (double)(ONE_KB))];
@@ -631,7 +689,7 @@
   if (attributes && (owner == nil)) {
     ASSIGN (owner, [attributes fileOwnerAccountName]);
   }
-  return (owner ? owner : [NSString string]);
+  return (owner ? owner : (NSString *)[NSString string]);
 }
 
 - (NSNumber *)ownerId
@@ -647,7 +705,7 @@
   if (attributes && (group == nil)) {
     ASSIGN (group, [attributes fileGroupOwnerAccountName]);
   }
-  return (group ? group : [NSString string]);
+  return (group ? group : (NSString *)[NSString string]);
 }
 
 - (NSNumber *)groupId
@@ -671,7 +729,7 @@
   if (flags.plain == -1) {
     [self setTypeFlags];
   }
-  return flags.plain;
+  return (flags.plain ? YES : NO);  
 }
 
 - (BOOL)isDirectory 
@@ -679,7 +737,7 @@
   if (flags.directory == -1) {
     [self setTypeFlags];
   }
-  return flags.directory;
+  return (flags.directory ? YES : NO);
 }
 
 - (BOOL)isLink 
@@ -687,7 +745,7 @@
   if (flags.link == -1) {
     [self setTypeFlags];
   }
-  return flags.link;
+  return (flags.link ? YES : NO);
 }
 
 - (BOOL)isSocket
@@ -695,7 +753,7 @@
   if (flags.socket == -1) {
     [self setTypeFlags];
   }
-  return flags.socket;
+  return (flags.socket ? YES : NO);
 }
 
 - (BOOL)isCharspecial
@@ -703,7 +761,7 @@
   if (flags.charspecial == -1) {
     [self setTypeFlags];
   }
-  return flags.charspecial;
+  return (flags.charspecial ? YES : NO);
 }
 
 - (BOOL)isBlockspecial
@@ -711,7 +769,7 @@
   if (flags.blockspecial == -1) {
     [self setTypeFlags];
   }
-  return flags.blockspecial;
+  return (flags.blockspecial ? YES : NO);
 }
 
 - (BOOL)isMountPoint
@@ -719,7 +777,7 @@
   if (flags.mountpoint == -1) {
     [self setTypeFlags];
   }
-  return flags.mountpoint;
+  return (flags.mountpoint ? YES : NO);
 }
 
 - (void)setMountPoint:(BOOL)value
@@ -732,7 +790,7 @@
   if (flags.application == -1) {
     [self setTypeFlags];
   }
-  return flags.application;
+  return (flags.application ? YES : NO);
 }
 
 - (BOOL)isPackage
@@ -740,7 +798,7 @@
   if (flags.package == -1) {
     [self setTypeFlags];
   }
-  return flags.package;
+  return (flags.package ? YES : NO);
 }
 
 - (BOOL)isReadable 
@@ -748,7 +806,7 @@
   if (flags.readable == -1) {
     flags.readable = [fm isReadableFileAtPath: path];
   }
-  return flags.readable;
+  return (flags.readable ? YES : NO);
 }
 
 - (BOOL)isWritable 
@@ -756,7 +814,17 @@
   if (flags.writable == -1) {
     flags.writable = [fm isWritableFileAtPath: path];
   }
-  return flags.writable;
+  return (flags.writable ? YES : NO);
+}
+
+- (void)checkWritable
+{
+  flags.writable = [fm isWritableFileAtPath: path];
+}
+
+- (BOOL)isParentWritable
+{
+  return [fm isWritableFileAtPath: [self parentPath]];
 }
 
 - (BOOL)isExecutable
@@ -764,7 +832,7 @@
   if (flags.executable == -1) {
     flags.executable = [fm isExecutableFileAtPath: path];
   }
-  return flags.executable;
+  return (flags.executable ? YES : NO);
 }
 
 - (BOOL)isDeletable
@@ -772,7 +840,7 @@
   if (flags.deletable == -1) {
     flags.deletable = [fm isDeletableFileAtPath: path];
   }
-  return flags.deletable;
+  return (flags.deletable ? YES : NO);
 }
 
 - (BOOL)isLocked
@@ -781,10 +849,30 @@
 }
 
 - (BOOL)isValid
+{
+  BOOL valid = (attributes != nil);
+
+  if (valid) {
+    valid = [fm fileExistsAtPath: path];
+
+    if ((valid == NO) && flags.link) {
+      valid = ([fm fileAttributesAtPath: path traverseLink: NO] != nil);
+    }
+  }
+  
+  return valid;
+}
+
+- (BOOL)hasValidPath
 {
   return [fm fileExistsAtPath: path];
 }
 
+- (BOOL)isReserved
+{
+  return [fsnodeRep isReservedName: name];
+}
+
 - (BOOL)willBeValidAfterFileOperation:(NSDictionary *)opinfo
 {
   NSString *operation = [opinfo objectForKey: @"operation"];
@@ -887,9 +975,11 @@
       }
     }
   }
-
-  if ([operation isEqual: @"GWorkspaceRenameOperation"]) { 
+    
+  if ([operation isEqual: @"GWorkspaceRenameOperation"]) {
+    destination = [opinfo objectForKey: @"destination"];	 
     files = [NSArray arrayWithObject: [destination lastPathComponent]]; 
+    destination = [destination stringByDeletingLastPathComponent];  
   } 
   
   if (isSubpathOfPath(destination, path)) {
@@ -911,6 +1001,11 @@
 
 @implementation FSNode (Comparing)
 
+- (NSComparisonResult)compareAccordingToPath:(FSNode *)aNode
+{
+  return [path compare: [aNode path]];
+}
+
 - (NSComparisonResult)compareAccordingToName:(FSNode *)aNode
 {
   NSString *n1 = [self name];
