@@ -1,6 +1,6 @@
 /* Dock.m
  *  
- * Copyright (C) 2005 Free Software Foundation, Inc.
+ * Copyright (C) 2005-2010 Free Software Foundation, Inc.
  *
  * Author: Enrico Sersale <enrico@imago.ro>
  * Date: January 2005
@@ -19,7 +19,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
  */
 
 #include <Foundation/Foundation.h>
@@ -28,6 +28,7 @@
 #include "Dock.h"
 #include "DockIcon.h"
 #include "GWDesktopView.h"
+#include "GWorkspace.h"
 
 #define MAX_ICN_SIZE 48
 #define MIN_ICN_SIZE 16
@@ -37,12 +38,11 @@
 
 - (void)dealloc
 {
-  [wsnc removeObserver: self];
   RELEASE (icons);
   RELEASE (backColor);
-  TEST_RELEASE (backImage);
+  RELEASE (backImage);
   
-	[super dealloc];
+  [super dealloc];
 }
 
 - (id)initForManager:(id)mngr
@@ -51,53 +51,20 @@
   
   if (self) {
 	  NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];	
-    NSArray *launched;
-    id appsdict;
+    NSDictionary *appsdict;
     NSArray *pbTypes;
     int i;
 
     manager = mngr;
     position = [manager dockPosition];
     
+    gw = [GWorkspace gworkspace];
+    fm = [NSFileManager defaultManager];    
     ws = [NSWorkspace sharedWorkspace];
-    wsnc = [ws notificationCenter];   
-    launched = [ws launchedApplications];
 
     icons = [NSMutableArray new];
     iconSize = MAX_ICN_SIZE;
-    
-    [self createWorkspaceIcon];
-          
-    appsdict = [defaults objectForKey: @"applications"];
-      
-    if (appsdict) {
-      NSArray *indexes = [appsdict allKeys];
-    
-      indexes = [indexes sortedArrayUsingSelector: @selector(compare:)];
-    
-      for (i = 0; i < [indexes count]; i++) {
-        NSNumber *index = [indexes objectAtIndex: i];
-        NSString *appname = [appsdict objectForKey: index];
-        DockIcon *icon = [self addIconForApplicationWithName: appname 
-                                                     atIndex: [index intValue]];
-        [icon setIsDocked: YES];
-      }
-    }
-
-    [self createTrashIcon];
-          
-    for (i = 0; i < [launched count]; i++) {
-    //  NSDictionary *dict = [launched objectAtIndex: i];
-    //  NSString *appname = [dict objectForKey: @"NSApplicationName"];
-    //  DockIcon *icon = [self iconForApplicationName: appname];
-      
-    //  if (icon) {
-    //    [icon setIsLaunched: YES];
-    //  }
-    }       
-          
-    [self setBackColor: [[manager desktopView] currentColor]];
-        
+                                
     dndSourceIcon = nil;
     isDragTarget = NO;  
     dragdelay = 0;
@@ -108,16 +75,33 @@
                                          @"DockIconPboardType", 
                                          nil];
     [self registerForDraggedTypes: pbTypes];    
+
+    [self setBackColor: [[manager desktopView] currentColor]];
     
-    [wsnc addObserver: self
-	           selector: @selector(applicationWillLaunch:)
-		             name: NSWorkspaceWillLaunchApplicationNotification
-		           object: nil];
+    [self createWorkspaceIcon];
+
+    appsdict = [defaults objectForKey: @"applications"];
+      
+    if (appsdict) {
+      NSArray *indexes = [appsdict allKeys];
     
-    [wsnc addObserver: self
-	           selector: @selector(applicationLaunched:)
-		             name: NSWorkspaceDidLaunchApplicationNotification
-		           object: nil];    
+      indexes = [indexes sortedArrayUsingSelector: @selector(compare:)];
+    
+      for (i = 0; i < [indexes count]; i++) {
+        NSNumber *index = [indexes objectAtIndex: i];
+        NSString *name = [[appsdict objectForKey: index] stringByDeletingPathExtension];
+        NSString *path = [ws fullPathForApplication: name];
+        
+        if (path) {
+          DockIcon *icon = [self addIconForApplicationAtPath: path
+                                                    withName: name 
+                                                     atIndex: [index intValue]];
+          [icon setDocked: YES];
+        }
+      }
+    }
+
+    [self createTrashIcon];
   }
   
   return self;  
@@ -132,15 +116,19 @@
   DockIcon *icon;
   
   if (wsname == nil) {
-    wsname = @"GWorkspace";
+    wsname = [gw gworkspaceProcessName];
   }
 
   path = [ws fullPathForApplication: wsname];
   node = [FSNode nodeWithPath: path];
   
-  icon = [[DockIcon alloc] initForNode: node iconSize: iconSize];
+  icon = [[DockIcon alloc] initForNode: node 
+                               appName: wsname
+                              iconSize: iconSize];
+  [icon setHighlightColor: backColor];
+  [icon setUseHlightImage: useBackImage];
   [icon setWsIcon: YES];   
-  [icon setIsDocked: YES];                        
+  [icon setDocked: YES];   
   [icons insertObject: icon atIndex: 0];
   [self addSubview: icon];
   RELEASE (icon);
@@ -150,10 +138,14 @@
 {
   NSString *path = [manager trashPath];
   FSNode *node = [FSNode nodeWithPath: path];
-  DockIcon *icon = [[DockIcon alloc] initForNode: node iconSize: iconSize];
-  
+  DockIcon *icon = [[DockIcon alloc] initForNode: node 
+                                         appName: nil
+                                        iconSize: iconSize];
+
+  [icon setHighlightColor: backColor];
+  [icon setUseHlightImage: useBackImage];  
   [icon setTrashIcon: YES];  
-  [icon setIsDocked: YES];                         
+  [icon setDocked: YES];                         
   [icons insertObject: icon atIndex: [icons count]];
   [self addSubview: icon];
   RELEASE (icon);
@@ -161,17 +153,18 @@
   [manager addWatcherForPath: path];
 }
 
-- (DockIcon *)addIconForApplicationWithName:(NSString *)name
-                                    atIndex:(int)index
+- (DockIcon *)addIconForApplicationAtPath:(NSString *)path
+                                 withName:(NSString *)name
+                                  atIndex:(int)index
 {
-  NSString *path = [ws fullPathForApplication: name];
-
-  if (path) {
+  if ([fm fileExistsAtPath: path]) {
     FSNode *node = [FSNode nodeWithPath: path];
     
     if ([node isApplication]) {
-      DockIcon *icon = [[DockIcon alloc] initForNode: node iconSize: iconSize];
       int icnindex;
+      DockIcon *icon = [[DockIcon alloc] initForNode: node 
+                                             appName: name
+                                            iconSize: iconSize];
 
       if (index == -1) {
         icnindex = ([icons count]) ? ([icons count] - 1) : 0;
@@ -198,11 +191,15 @@
                atIndex:(int)index
 {
   NSDictionary *dict = [NSUnarchiver unarchiveObjectWithData: icondata];
-  NSString *appname = [dict objectForKey: @"name"];
-  DockIcon *icon = [self addIconForApplicationWithName: appname atIndex: index];
+  NSString *name = [dict objectForKey: @"name"];
+  NSString *path = [dict objectForKey: @"path"];
+  DockIcon *icon = [self addIconForApplicationAtPath: path 
+                                            withName: name 
+                                             atIndex: index];
 
-  [icon setIsDocked: [[dict objectForKey: @"docked"] boolValue]];
-  [icon setIsLaunched: [[dict objectForKey: @"launched"] boolValue]];
+  [icon setDocked: [[dict objectForKey: @"docked"] boolValue]];
+  [icon setLaunched: [[dict objectForKey: @"launched"] boolValue]];
+  [icon setHidden: [[dict objectForKey: @"hidden"] boolValue]];
 }
 
 - (void)removeIcon:(DockIcon *)icon
@@ -213,7 +210,7 @@
     [icon removeFromSuperview];
   }
   if ([icon isLaunched]) {
-    [icon setIsLaunched: NO];
+    [icon setLaunched: NO];
   }
   [icons removeObject: icon];
   [self tile];
@@ -225,9 +222,8 @@
   
   for (i = 0; i < [icons count]; i++) {
     DockIcon *icon = [icons objectAtIndex: i];
-    NSString *nodename = [[icon node] name];
     
-    if ([[nodename stringByDeletingPathExtension] isEqual: name]) {
+    if ([[icon appName] isEqual: name]) {
       return icon;
     }
   }
@@ -286,33 +282,103 @@
   dndSourceIcon = icon;
 }
 
-- (void)applicationWillLaunch:(NSNotification *)notif
+- (void)appWillLaunch:(NSString *)appPath
+              appName:(NSString *)appName
 {
-  NSString *appname = [[notif userInfo] objectForKey: @"NSApplicationName"];
-  DockIcon *icon = [self iconForApplicationName: appname];
+  if ([appName isEqual: [gw gworkspaceProcessName]] == NO) {
+    DockIcon *icon = [self iconForApplicationName: appName];
   
-  if (icon == nil) {
-    icon = [self addIconForApplicationWithName: appname atIndex: -1];
-  } 
+    if (icon == nil) {
+      icon = [self addIconForApplicationAtPath: appPath
+                                      withName: appName
+                                       atIndex: -1];
+    } 
   
-  if (icon) {
     [self tile];
     [icon animateLaunch];
   }
 }
 
-- (void)applicationLaunched:(NSNotification *)notif
+- (void)appDidLaunch:(NSString *)appPath
+             appName:(NSString *)appName
 {
-  NSString *appname = [[notif userInfo] objectForKey: @"NSApplicationName"];
-  DockIcon *icon = [self iconForApplicationName: appname];
+  if ([appName isEqual: [gw gworkspaceProcessName]] == NO) {
+    DockIcon *icon = [self iconForApplicationName: appName];
 
-  if ((icon == nil) && ([appname isEqual: @"Desktop"] == NO)) {
-    icon = [self addIconForApplicationWithName: appname atIndex: -1];
-    [self tile];
-  } 
+    if (icon == nil) {
+      icon = [self addIconForApplicationAtPath: appPath
+                                      withName: appName
+                                       atIndex: -1];
+      [self tile];
+    } 
   
-  if (icon) {
-    [icon setIsLaunched: YES];
+    [icon setLaunched: YES];
+  }
+}
+
+- (void)appTerminated:(NSString *)appName
+{
+  if ([appName isEqual: [gw gworkspaceProcessName]] == NO) {
+    DockIcon *icon = [self iconForApplicationName: appName];
+
+    if (icon) {
+      if (([icon isDocked] == NO) && ([icon isSpecialIcon] == NO)) {
+        [self removeIcon: icon];
+      } else {
+        [icon setAppHidden: NO];
+        [icon setLaunched: NO];
+      }
+    }
+  }
+}
+
+- (void)appDidHide:(NSString *)appName
+{
+  if ([appName isEqual: [gw gworkspaceProcessName]] == NO) {
+    DockIcon *icon = [self iconForApplicationName: appName];
+
+    if (icon) {
+      [icon setAppHidden: YES];
+    }
+  }
+}
+
+- (void)appDidUnhide:(NSString *)appName
+{
+  if ([appName isEqual: [gw gworkspaceProcessName]] == NO) {
+    DockIcon *icon = [self iconForApplicationName: appName];
+
+    if (icon) {
+      [icon setAppHidden: NO];
+    }
+  }
+}
+
+- (void)iconMenuAction:(id)sender
+{
+  NSString *title = [(NSMenuItem *)sender title];
+  
+  if ([title isEqual: NSLocalizedString(@"Show In File Viewer", @"")]) {
+    NSString *path = [(NSMenuItem *)sender representedObject];
+    NSString *basePath = [path stringByDeletingLastPathComponent];
+  
+    [gw selectFile: path inFileViewerRootedAtPath: basePath];
+  
+  } else {
+    GWLaunchedApp *app = (GWLaunchedApp *)[(NSMenuItem *)sender representedObject];
+  
+    if ([app isRunning] == NO) {
+      /* terminated while the icon menu is open */
+      return;
+    }
+  
+    if ([title isEqual: NSLocalizedString(@"Hide", @"")]) {
+      [app hideApplication];
+    } else if ([title isEqual: NSLocalizedString(@"Unhide", @"")]) {
+      [app unhideApplication];
+    } else if ([title isEqual: NSLocalizedString(@"Quit", @"")]) {
+      [app terminateApplication];
+    }  
   }
 }
 
@@ -345,12 +411,12 @@
   DESTROY (backImage);
 
   if (image) {
-    NSSize size = [self frame].size;
+    NSRect r = [self bounds];
     
-    backImage = [[NSImage alloc] initWithSize: size];
+    backImage = [[NSImage alloc] initWithSize: r.size];
     [backImage lockFocus]; 
     [image compositeToPoint: NSZeroPoint 
-                   fromRect: [self frame]
+                   fromRect: r
                   operation: NSCompositeCopy];
     [backImage unlockFocus];
     [self setNeedsDisplay: YES];
@@ -457,7 +523,7 @@
     DockIcon *icon = [icons objectAtIndex: i];    
 
     if (([icon isSpecialIcon] == NO) && [icon isDocked]) {
-      [dict setObject: [[icon node] name] forKey: [NSNumber numberWithInt: i]];
+      [dict setObject: [icon appName] forKey: [NSNumber numberWithInt: i]];
       [manager removeWatcherForPath: [[icon node] path]];
     }
   }
@@ -467,6 +533,15 @@
   [manager removeWatcherForPath: [manager trashPath]];
 }
 
+- (void)checkRemovedApp:(id)sender
+{
+  DockIcon *icon = (DockIcon *)[sender userInfo];
+  
+  if ([[icon node] isValid] == NO) {
+    [self removeIcon: icon];
+  }
+}
+
 - (void)drawRect:(NSRect)rect
 {  
   [super drawRect: rect];
@@ -546,9 +621,7 @@
       int count = [subNodes count];
     
       for (i = 0; i < [subNodes count]; i++) {
-        FSNode *subNode = [subNodes objectAtIndex: i];
-        
-        if ([[subNode name] hasPrefix: @"."]) {
+        if ([[subNodes objectAtIndex: i] isReserved]) {
           count --;
         }
       }
@@ -562,28 +635,55 @@
 
 - (void)watchedPathChanged:(NSDictionary *)info
 {
+  CREATE_AUTORELEASE_POOL(arp);
   NSString *event = [info objectForKey: @"event"];
   NSString *path = [info objectForKey: @"path"];
     
-  if ([event isEqual: @"GWWatchedDirectoryDeleted"]) {
-    int count = [icons count];
+  if ([event isEqual: @"GWWatchedPathDeleted"]) {
     int i;
 
-    for (i = 0; i < count; i++) {
+    for (i = 0; i < [icons count]; i++) {
       DockIcon *icon = [icons objectAtIndex: i];
       
       if ([icon isSpecialIcon] == NO) {
         FSNode *node = [icon node];
         
         if ([path isEqual: [node path]]) {
-          [self removeIcon: icon];
-          count--;
-          i--;
+          [NSTimer scheduledTimerWithTimeInterval: 1.0 
+												                   target: self 
+                                         selector: @selector(checkRemovedApp:) 
+										                     userInfo: icon 
+                                          repeats: NO];
         }
       }
     }
     
   } else if ([event isEqual: @"GWFileDeletedInWatchedDirectory"]) {
+    NSArray *files = [info objectForKey: @"files"];
+    int i;
+    
+    for (i = 0; i < [files count]; i++) {
+      NSString *fname = [files objectAtIndex: i];
+      NSString *fullpath = [path stringByAppendingPathComponent: fname];
+      int j;
+      
+      for (j = 0; j < [icons count]; j++) {
+        DockIcon *icon = [icons objectAtIndex:j];
+
+        if ([icon isSpecialIcon] == NO) {
+          FSNode *node = [icon node];
+
+          if ([fullpath isEqual: [node path]]) {
+            [NSTimer scheduledTimerWithTimeInterval: 1.0 
+												                     target: self 
+                                           selector: @selector(checkRemovedApp:) 
+										                       userInfo: icon 
+                                            repeats: NO];
+          }
+        }
+      }
+    }
+    
     if ([path isEqual: [manager trashPath]]) {
       DockIcon *icon = [self trashIcon];  
       FSNode *node = [icon node];
@@ -592,9 +692,7 @@
       int i;
 
       for (i = 0; i < [subNodes count]; i++) {
-        FSNode *subNode = [subNodes objectAtIndex: i];
-        
-        if ([[subNode name] hasPrefix: @"."]) {
+        if ([[subNodes objectAtIndex: i] isReserved]) {
           count --;
         }
       }
@@ -612,15 +710,15 @@
       int i;
 
       for (i = 0; i < [subNodes count]; i++) {
-        FSNode *subNode = [subNodes objectAtIndex: i];
-        
-        if ([[subNode name] hasPrefix: @"."] == NO) {
+        if ([[subNodes objectAtIndex: i] isReserved] == NO) {
           [icon setTrashFull: YES];
           break;
         }
       }
     }
   }
+  
+  RELEASE (arp);
 }
 
 - (void)unselectOtherReps:(id)arep
@@ -676,7 +774,7 @@
 
 @implementation Dock (DraggingDestination)
 
-- (unsigned int)draggingEntered:(id <NSDraggingInfo>)sender
+- (NSDragOperation)draggingEntered:(id <NSDraggingInfo>)sender
 {
   NSPoint location = [sender draggingLocation];
   DockIcon *icon;
@@ -744,7 +842,7 @@
   return NSDragOperationNone;
 }
 
-- (unsigned int)draggingUpdated:(id <NSDraggingInfo>)sender
+- (NSDragOperation)draggingUpdated:(id <NSDraggingInfo>)sender
 {
   NSPoint location;
   DockIcon *icon;
@@ -885,7 +983,8 @@
       if ([sourcePaths count] == 1) {
         NSString *path = [sourcePaths objectAtIndex: 0];
         FSNode *node = [FSNode nodeWithPath: path];
-
+        NSString *appName = [[node name] stringByDeletingPathExtension];
+        
         if ([node isApplication]) {
           if ((icon == nil) || (icon && ([icon isTrashIcon] == NO))) {
             BOOL duplicate = NO;
@@ -894,7 +993,8 @@
             for (i = 0; i < [icons count]; i++) {
               DockIcon *icon = [icons objectAtIndex: i];
 
-              if ([[icon node] isEqualToNode: node]) {
+              if ([[icon node] isEqual: node] 
+                          && [[icon appName] isEqual: appName]) {
                 RETAIN (icon);
                 [icons removeObject: icon];
                 [icons insertObject: icon atIndex: targetIndex];
@@ -905,9 +1005,10 @@
             }
 
             if (duplicate == NO) {
-              DockIcon *icon = [self addIconForApplicationWithName: [node name] 
-                                                           atIndex: targetIndex];
-              [icon setIsDocked: YES];
+              DockIcon *icon = [self addIconForApplicationAtPath: path
+                                                        withName: appName 
+                                                         atIndex: targetIndex];
+              [icon setDocked: YES];
             }
 
             concluded = YES;
