@@ -1,6 +1,6 @@
 /* DockIcon.m
  *  
- * Copyright (C) 2005 Free Software Foundation, Inc.
+ * Copyright (C) 2005-2010 Free Software Foundation, Inc.
  *
  * Author: Enrico Sersale <enrico@imago.ro>
  * Date: January 2005
@@ -19,42 +19,34 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
  */
 
 #include <Foundation/Foundation.h>
 #include <AppKit/AppKit.h>
 #include <math.h>
+#include <unistd.h>
 #include "DockIcon.h"
 #include "Dock.h"
 #include "GWDesktopManager.h"
+#include "GWorkspace.h"
 
 @implementation DockIcon
 
 - (void)dealloc
 {
-  if (application) {
-    NSConnection *conn = [(NSDistantObject *)application connectionForProxy];
-  
-    if (conn && [conn isValid]) {
-      [nc removeObserver: self
-	                  name: NSConnectionDidDieNotification
-	                object: conn];
-      DESTROY (application);
-    }
-  }
-  
   RELEASE (appName);
-  TEST_RELEASE (highlightColor);
-  TEST_RELEASE (darkerColor);
-  TEST_RELEASE (highlightImage);
-  TEST_RELEASE (trashFullIcon);
+  RELEASE (highlightColor);
+  RELEASE (darkerColor);
+  RELEASE (highlightImage);
+  RELEASE (trashFullIcon);
   RELEASE (dragIcon);
   
   [super dealloc];
 }
 
 - (id)initForNode:(FSNode *)anode
+          appName:(NSString *)aname
          iconSize:(int)isize
 {
   self = [super initForNode: anode
@@ -70,21 +62,39 @@
                   slideBack: NO];
 
   if (self) {
-    ASSIGN (appName, [[node name] stringByDeletingPathExtension]);
-    isDocked = NO;
-    isLaunched = NO;
+    if (aname != nil) {
+      ASSIGN (appName, aname);
+    } else {
+      ASSIGN (appName, [[node name] stringByDeletingPathExtension]);
+    }
+        
+    dragIcon = [icon copy];
+    
+    docked = NO;
+    launched = NO;
+    apphidden = NO;
+    
     nc = [NSNotificationCenter defaultCenter];
     fm = [NSFileManager defaultManager];
     ws = [NSWorkspace sharedWorkspace];
-    dragIcon = [icon copy];
+
+    [self setToolTip: appName];  
   }
 
   return self;
 }
 
+- (NSString *)appName
+{
+  return appName;
+}
+
 - (void)setWsIcon:(BOOL)value
 {
   isWsIcon = value;
+  if (isWsIcon) {
+    [self removeAllToolTips];
+  }
 }
 
 - (BOOL)isWsIcon
@@ -108,9 +118,7 @@
       count = [subNodes count];
       
       for (i = 0; i < [subNodes count]; i++) {
-        FSNode *subNode = [subNodes objectAtIndex: i];
-        
-        if ([[subNode name] hasPrefix: @"."]) {
+        if ([[subNodes objectAtIndex: i] isReserved]) {
           count --;
         }
       }
@@ -122,6 +130,10 @@
                                   forNode: node]);
     }
   }
+  
+  if (isTrashIcon) {
+    [self removeAllToolTips];
+  }  
 }
 
 - (void)setTrashFull:(BOOL)value
@@ -140,114 +152,37 @@
   return (isWsIcon || isTrashIcon);
 }
 
-- (void)setIsDocked:(BOOL)value
+- (void)setDocked:(BOOL)value
 {
-  isDocked = value;
+  docked = value;
 }
 
 - (BOOL)isDocked
 {
-  return isDocked;
+  return docked;
 }
 
-- (void)setIsLaunched:(BOOL)value
+- (void)setLaunched:(BOOL)value
 {
-  isLaunched = value;
-
-  if (isLaunched) {
-    if (application == nil) {
-      [self connectApplication];
-    }
-    
-    if (application) {
-      [self setNeedsDisplay: YES];
-    }
-        
-  } else {
-    if (application) {
-      NSConnection *conn = [(NSDistantObject *)application connectionForProxy];
-
-      if (conn && [conn isValid]) {
-        [nc removeObserver: self
-	                    name: NSConnectionDidDieNotification
-	                  object: conn];
-        DESTROY (application);
-      }
-    }
-  
-    [self setNeedsDisplay: YES];
-    
-    if ((isDocked == NO) && ([self isSpecialIcon] == NO)) {
-      [(Dock *)container removeIcon: self];
-    }
-  }
-}
-
-- (void)connectApplication
-{
-  if (application == nil) {
-    id app = [NSConnection rootProxyForConnectionWithRegisteredName: appName
-                                                               host: @""];
-
-    if (app) {
-      NSConnection *conn = [app connectionForProxy];
-
-	    [nc addObserver: self
-	           selector: @selector(applicationConnectionDidDie:)
-		             name: NSConnectionDidDieNotification
-		           object: conn];
-      
-      application = app;
-      RETAIN (application);
-      
-	  } else {
-	    static BOOL recursion = NO;
-	  
-      if (recursion == NO) {
-        int i;
-        
-        for (i = 1; i <= 80; i++) {
-	        [[NSRunLoop currentRunLoop] runUntilDate:
-		                       [NSDate dateWithTimeIntervalSinceNow: 0.1]];
-                           
-          app = [NSConnection rootProxyForConnectionWithRegisteredName: appName
-                                                                  host: @""];                  
-          if (app) {
-            break;
-          }
-        }
-        
-	      recursion = YES;
-	      [self connectApplication];
-	      recursion = NO;
-        
-	    } else { 
-	      recursion = NO;
-        NSLog(@"unable to contact %@!", appName);  
-      }
-	  }
-  }
-}
-
-- (void)applicationConnectionDidDie:(NSNotification *)notif
-{
-  id conn = [notif object];
-
-  [nc removeObserver: self
-	              name: NSConnectionDidDieNotification
-	            object: conn];
-
-  NSAssert(conn == [application connectionForProxy],
-		                                  NSInternalInconsistencyException);
-  RELEASE (application);
-  application = nil;
-  
-  [self setIsLaunched: NO];
+  launched = value;
+  [self setNeedsDisplay: YES];
 }
 
 - (BOOL)isLaunched
 {
-  return isLaunched;
+  return launched;
+}
+
+- (void)setAppHidden:(BOOL)value
+{
+  apphidden = value;
+  [self setNeedsDisplay: YES];
+  [container setNeedsDisplayInRect: [self frame]];
+}
+
+- (BOOL)isAppHidden
+{
+  return apphidden;
 }
 
 - (void)animateLaunch
@@ -329,16 +264,20 @@
 - (void)mouseUp:(NSEvent *)theEvent
 {
   if ([theEvent clickCount] > 1) {  
-    if ((isLaunched == NO) && ([self isSpecialIcon] == NO)) {
-      [ws launchApplication: appName];
-      
+    if ([self isSpecialIcon] == NO) {
+      if (launched == NO) {
+        [ws launchApplication: appName];
+      } else if (apphidden) {
+        [[GWorkspace gworkspace] unhideAppWithPath: [node path] andName: appName];
+      } else {
+        [[GWorkspace gworkspace] activateAppWithPath: [node path] andName: appName];
+      }
     } else if (isWsIcon) {
       [[GWDesktopManager desktopManager] showRootViewer];
     
     } else if (isTrashIcon) {
       NSString *path = [node path];
-      [[GWDesktopManager desktopManager] selectFile: path 
-                           inFileViewerRootedAtPath: path];
+      [[GWDesktopManager desktopManager] selectFile: path inFileViewerRootedAtPath: path];
     }
   }  
 }
@@ -375,34 +314,98 @@
     }
 
     if (startdnd == YES) {  
-      [self startExternalDragOnEvent: nextEvent];    
+      [self startExternalDragOnEvent: theEvent withMouseOffset: NSZeroSize];
     } 
 	} 
 }
 
+- (NSMenu *)menuForEvent:(NSEvent *)theEvent
+{
+  if ([self isSpecialIcon] == NO) {
+    NSString *appPath = [ws fullPathForApplication: appName];
+    
+    if (appPath) {
+      CREATE_AUTORELEASE_POOL(arp);
+      NSMenu *menu = [[NSMenu alloc] initWithTitle: appName];
+      NSMenuItem *item;
+      GWLaunchedApp *app;
+      
+      item = [NSMenuItem new];  
+      [item setTitle: NSLocalizedString(@"Show In File Viewer", @"")];
+      [item setTarget: (Dock *)container];  
+      [item setAction: @selector(iconMenuAction:)]; 
+      [item setRepresentedObject: appPath];            
+      [menu addItem: item];
+      RELEASE (item);
+
+      app = [[GWorkspace gworkspace] launchedAppWithPath: appPath
+                                                 andName: appName];      
+      if (app && [app isRunning]) {
+        item = [NSMenuItem new];  
+        [item setTarget: (Dock *)container];  
+        [item setAction: @selector(iconMenuAction:)]; 
+        [item setRepresentedObject: app];            
+      
+        if ([app isHidden]) {
+          [item setTitle: NSLocalizedString(@"Unhide", @"")];
+        } else {
+          [item setTitle: NSLocalizedString(@"Hide", @"")];
+        }
+        
+        [menu addItem: item];
+        RELEASE (item);      
+
+        item = [NSMenuItem new];  
+        [item setTitle: NSLocalizedString(@"Quit", @"")];
+        [item setTarget: (Dock *)container];  
+        [item setAction: @selector(iconMenuAction:)]; 
+        [item setRepresentedObject: app];            
+        [menu addItem: item];
+        RELEASE (item);
+      } 
+      
+      RELEASE (arp);
+
+      return AUTORELEASE (menu);
+    }
+  }
+  
+  return [super menuForEvent: theEvent];
+}
+
 - (void)startExternalDragOnEvent:(NSEvent *)event
+                 withMouseOffset:(NSSize)offset
 {
   NSPasteboard *pb = [NSPasteboard pasteboardWithName: NSDragPboard];	
   NSMutableDictionary *dict = [NSMutableDictionary dictionary];
 
-  [dict setObject: [node name] forKey: @"name"];
-  [dict setObject: [NSNumber numberWithBool: isDocked] 
+  [dict setObject: appName forKey: @"name"];
+  [dict setObject: [node path] forKey: @"path"];
+  [dict setObject: [NSNumber numberWithBool: docked] 
            forKey: @"docked"];
-  [dict setObject: [NSNumber numberWithBool: isLaunched] 
+  [dict setObject: [NSNumber numberWithBool: launched] 
            forKey: @"launched"];
+  [dict setObject: [NSNumber numberWithBool: apphidden] 
+           forKey: @"hidden"];
   
   [pb declareTypes: [NSArray arrayWithObject: @"DockIconPboardType"] 
              owner: nil];
     
   if ([pb setData: [NSArchiver archivedDataWithRootObject: dict] 
           forType: @"DockIconPboardType"]) {
+    NSPoint dragPoint = [event locationInWindow]; 
+    NSSize fs = [self frame].size; 
+ 
+    dragPoint.x -= ((fs.width - icnPoint.x) / 2);
+    dragPoint.y -= ((fs.height - icnPoint.y) / 2);
+      
     [self unselect];  
     [self setIsDndSourceIcon: YES];
     [(Dock *)container setDndSourceIcon: self];
     [(Dock *)container tile];
     
     [[self window] dragImage: dragIcon
-                          at: NSZeroPoint 
+                          at: dragPoint 
                       offset: NSZeroSize
                        event: event
                   pasteboard: pb
@@ -422,6 +425,15 @@
 
 - (void)drawRect:(NSRect)rect
 {   
+#define DRAWDOT(c1, c2, p) \
+{ \
+[c1 set]; \
+NSRectFill(NSMakeRect(p.x, p.y, 3, 2)); \
+[c2 set]; \
+NSRectFill(NSMakeRect(p.x + 1, p.y, 2, 1)); \
+NSRectFill(NSMakeRect(p.x + 2, p.y + 1, 1, 1)); \
+}
+
 #define DRAWDOTS(c1, c2, p) \
 { \
 int i, x = p.x, y = p.y; \
@@ -434,7 +446,7 @@ NSRectFill(NSMakeRect(x + 2, y + 1, 1, 1)); \
 x += 6; \
 } \
 }
- 	 
+ 	
   if (isSelected || launching) {
     [highlightColor set];
     NSRectFill(rect);
@@ -460,17 +472,21 @@ x += 6; \
       }
     }
     
-    if ((isWsIcon == NO) && (isTrashIcon == NO) && (isLaunched == NO)) {
-      DRAWDOTS (darkerColor, [NSColor whiteColor], NSMakePoint(4, 2));
+    if ((isWsIcon == NO) && (isTrashIcon == NO)) { 
+      if (apphidden) {
+        DRAWDOT (darkerColor, [NSColor whiteColor], NSMakePoint(4, 2));
+      } else if (launched == NO) {
+        DRAWDOTS (darkerColor, [NSColor whiteColor], NSMakePoint(4, 2));
+      }
     }
   }
 }
 
 - (BOOL)acceptsDraggedPaths:(NSArray *)paths
 {
-  if ([self isSpecialIcon] == NO) {
-    int i;
+  unsigned i;
 
+  if ([self isSpecialIcon] == NO) {
     for (i = 0; i < [paths count]; i++) {
       NSString *path = [paths objectAtIndex: i];
       FSNode *nod = [FSNode nodeWithPath: path];
@@ -484,12 +500,30 @@ x += 6; \
     return YES;
     
   } else if (isTrashIcon) {
-	  NSString *fromPath = [[paths objectAtIndex: 0] stringByDeletingLastPathComponent];
-  
-    if ([fm isWritableFileAtPath: fromPath]) {
+    NSString *fromPath = [[paths objectAtIndex: 0] stringByDeletingLastPathComponent];
+    BOOL accept = YES;
+    
+    if ([fromPath isEqual: [[GWDesktopManager desktopManager] trashPath]] == NO) {
+      NSArray *vpaths = [ws mountedLocalVolumePaths];
+    
+      for (i = 0; i < [paths count]; i++) {
+        NSString *path = [paths objectAtIndex: i];
+
+        if (([vpaths containsObject: path] == NO)
+                          && ([fm isWritableFileAtPath: path] == NO)) {
+          accept = NO;
+          break;
+        }
+      }
+    } else {
+      accept = NO;
+    }
+      
+    if (accept) {
       [self select];
-      return YES;
-    } 
+    }
+  
+    return accept;
   }
 
   return NO;
@@ -507,7 +541,20 @@ x += 6; \
       FSNode *nod = [FSNode nodeWithPath: path];
 
       if ([nod isPlain] || ([nod isPackage] && ([nod isApplication] == NO))) {
-        [ws openFile: [paths objectAtIndex: i] withApplication: appName];
+        NS_DURING
+          {
+        [ws openFile: path withApplication: appName];
+          }
+        NS_HANDLER
+          {
+        NSRunAlertPanel(NSLocalizedString(@"error", @""), 
+            [NSString stringWithFormat: @"%@ %@!", 
+                NSLocalizedString(@"Can't open ", @""), [path lastPathComponent]],
+                                          NSLocalizedString(@"OK", @""), 
+                                          nil, 
+                                          nil);                                     
+          }
+        NS_ENDHANDLER  
       }
     }
 
@@ -516,7 +563,15 @@ x += 6; \
     NSMutableArray *files = [NSMutableArray array];
     NSMutableArray *umountPaths = [NSMutableArray array];
     NSMutableDictionary *opinfo = [NSMutableDictionary dictionary];
+    NSString *username = NSUserName();
+    BOOL iamRoot;
 
+	  #ifdef __WIN32__
+		  iamRoot = YES;
+	  #else
+		  iamRoot = (geteuid() == 0);
+	  #endif
+    
     for (i = 0; i < [paths count]; i++) {
       NSString *srcpath = [paths objectAtIndex: i];
 
@@ -528,13 +583,34 @@ x += 6; \
     }
 
     for (i = 0; i < [umountPaths count]; i++) {
-      [ws unmountAndEjectDeviceAtPath: [umountPaths objectAtIndex: i]];
+      NSString *umpath = [umountPaths objectAtIndex: i];
+	    NSDictionary *attrs = [fm fileAttributesAtPath: umpath traverseLink: NO];
+      NSString *usr = [attrs objectForKey: NSFileOwnerAccountName];
+      BOOL isMyFile = ([username isEqual: usr]);
+      
+      if (iamRoot || isMyFile) {
+        [ws unmountAndEjectDeviceAtPath: umpath];
+      } else {
+		    NSString *err = NSLocalizedString(@"Error", @"");
+		    NSString *msg = NSLocalizedString(@"You must be root to umount\n", @"");
+		    NSString *buttstr = NSLocalizedString(@"Continue", @"");
+        NSRunAlertPanel(err, [NSString stringWithFormat: @"%@ \"%@\"!\n", msg, umpath], buttstr, nil, nil);         
+      }
     }
 
     if ([files count]) {
+	    NSString *fromPath = [[paths objectAtIndex: 0] stringByDeletingLastPathComponent];
+    
+	    if ([fm isWritableFileAtPath: fromPath] == NO) {
+		    NSString *err = NSLocalizedString(@"Error", @"");
+		    NSString *msg = NSLocalizedString(@"You do not have write permission\nfor", @"");
+		    NSString *buttstr = NSLocalizedString(@"Continue", @"");
+        NSRunAlertPanel(err, [NSString stringWithFormat: @"%@ \"%@\"!\n", msg, fromPath], buttstr, nil, nil);   
+		    return;
+	    }
+    
       [opinfo setObject: @"NSWorkspaceRecycleOperation" forKey: @"operation"];
-      [opinfo setObject: [[paths objectAtIndex: 0] stringByDeletingLastPathComponent]
-                 forKey: @"source"];
+      [opinfo setObject: fromPath forKey: @"source"];
       [opinfo setObject: [node path] forKey: @"destination"];
       [opinfo setObject: files forKey: @"files"];
 
@@ -545,4 +621,3 @@ x += 6; \
 
 @end
 
-
