@@ -19,7 +19,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
  */
 
 #ifndef GW_DESKTOP_MANAGER
@@ -36,22 +36,28 @@ typedef enum DockPosition {
 @class GWorkspace;
 @class GWDesktopView;
 @class Dock;
+@class MPointWatcher;
 
 @interface GWDesktopManager : NSObject
 {
   FSNode *dskNode;
-  BOOL dskWatcherSuspended;
   id win;
   BOOL usexbundle;
   
+  GWDesktopView *desktopView;
+  
   Dock *dock;
   BOOL hidedock;
   DockPosition dockPosition;
   
   NSRect dockReservedFrame;
+  NSRect macmenuReservedFrame;
   NSRect tshelfReservedFrame;
+  NSRect tshelfActivateFrame;
   
   GWorkspace *gworkspace;
+  FSNodeRep *fsnodeRep;
+  MPointWatcher *mpointWatcher;
   id ws;
   NSFileManager *fm;
   NSNotificationCenter *nc;      
@@ -89,12 +95,20 @@ typedef enum DockPosition {
 
 - (void)setReservedFrames;
 
+- (NSRect)macmenuReservedFrame;
+
 - (NSRect)dockReservedFrame;
 
 - (NSRect)tshelfReservedFrame;
 
+- (NSRect)tshelfActivateFrame;
+
 - (NSImage *)tabbedShelfBackground;
 
+- (void)mouseEnteredTShelfActivateFrame;
+
+- (void)mouseExitedTShelfActiveFrame;
+
 - (void)deselectAllIcons;
 
 - (void)deselectInSpatialViewers;
@@ -124,16 +138,24 @@ typedef enum DockPosition {
 
 - (void)thumbnailsDidChangeInPaths:(NSArray *)paths;
 
+- (void)removableMediaPathsDidChange;
+
+- (void)hideDotsFileDidChange:(BOOL)hide;
+
+- (void)hiddenFilesDidChange:(NSArray *)paths;
+
 - (void)newVolumeMounted:(NSNotification *)notif;
 
 - (void)mountedVolumeWillUnmount:(NSNotification *)notif;
 
 - (void)mountedVolumeDidUnmount:(NSNotification *)notif;
 
-- (void)mountThreadWillExit:(NSNotification *)notif;
+- (void)mountedVolumesDidChange;
 
 - (void)updateDefaults;
 
+- (void)setContextHelp;
+
 @end
 
 
@@ -145,11 +167,13 @@ typedef enum DockPosition {
 - (BOOL)validateItem:(id)menuItem;
 - (void)openSelectionInNewViewer:(BOOL)newv;
 - (void)openSelectionAsFolder;
+- (void)openSelectionWith;
 - (void)newFolder;
 - (void)newFile;
 - (void)duplicateFiles;
-- (void)deleteFiles;
+- (void)recycleFiles;
 - (void)emptyTrash;
+- (void)deleteFiles;
 - (void)setShownType:(id)sender;
 - (void)setExtendedShownType:(id)sender;
 - (void)setIconsSize:(id)sender;
@@ -161,34 +185,32 @@ typedef enum DockPosition {
 @end
 
 
+@interface MPointWatcher : NSObject
+{
+  NSMutableArray *volinfo;
+  NSTimer *timer;
+  BOOL active;
+  GWDesktopManager *manager;
+  NSFileManager *fm;
+}
+
+- (id)initForManager:(GWDesktopManager *)mngr;
+
+- (void)startWatching;
+
+- (void)stopWatching;
+
+- (void)watchMountPoints:(id)sender;
+
+@end
+
+
 @interface GWMounter : NSObject
 {
 }
 
 + (void)mountRemovableMedia;
 
-- (void)mountRemovableMedia;
-
-@end
-
-
-@interface NSWorkspace (mounting)
-
-- (BOOL)getFileSystemInfoForPath:(NSString *)fullPath
-		                 isRemovable:(BOOL *)removableFlag
-		                  isWritable:(BOOL *)writableFlag
-		               isUnmountable:(BOOL *)unmountableFlag
-		                 description:(NSString **)description
-			                      type:(NSString **)fileSystemType;
-                            
-- (NSArray *)mountedLocalVolumePaths;
-
-- (NSArray *)mountedRemovableMedia;
-
-- (NSArray *)mountNewRemovableMedia;
-
-- (BOOL)unmountAndEjectDeviceAtPath:(NSString *)path;
-
 @end
 
 #endif // GW_DESKTOP_MANAGER
