@@ -1,6 +1,6 @@
 /* GWDesktopManager.m
  *  
- * Copyright (C) 2005 Free Software Foundation, Inc.
+ * Copyright (C) 2005-2010 Free Software Foundation, Inc.
  *
  * Author: Enrico Sersale <enrico@imago.ro>
  * Date: January 2005
@@ -19,7 +19,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
  */
 
 #include <AppKit/AppKit.h>
@@ -30,6 +30,9 @@
 #include "FSNFunctions.h"
 #include "GWorkspace.h"
 #include "GWViewersManager.h"
+#include "TShelf/TShelfWin.h"
+
+#define RESV_MARGIN 10
 
 static GWDesktopManager *desktopManager = nil;
 
@@ -47,11 +50,12 @@ static GWDesktopManager *desktopManager = nil;
 {
   [[ws notificationCenter] removeObserver: self];
   [nc removeObserver: self];
-  TEST_RELEASE (dskNode);
-  TEST_RELEASE (win);
-  TEST_RELEASE (dock);
+  RELEASE (dskNode);
+  RELEASE (win);
+  RELEASE (dock);
+  RELEASE (mpointWatcher);
     
-	[super dealloc];
+  [super dealloc];
 }
 
 - (id)init
@@ -63,12 +67,13 @@ static GWDesktopManager *desktopManager = nil;
     id defentry;
     NSString *path;
     id window = nil;
-    GWDesktopView *desktopView;
 
     fm = [NSFileManager defaultManager];
     nc = [NSNotificationCenter defaultCenter];
     ws = [NSWorkspace sharedWorkspace];
     gworkspace = [GWorkspace gworkspace];
+    fsnodeRep = [FSNodeRep sharedInstance];
+    mpointWatcher = [[MPointWatcher alloc] initForManager: self];
     
     [self checkDesktopDirs];
 
@@ -135,32 +140,32 @@ static GWDesktopManager *desktopManager = nil;
                 					    name: NSWorkspaceDidUnmountNotification
                 					  object: nil];
 
-    [nc addObserver: self
-           selector: @selector(mountThreadWillExit:)
-               name: NSThreadWillExitNotification
-             object: nil];
+    [self setContextHelp];
   }
   
   return self;
 }
 
 - (void)activateDesktop
-{  
+{
   [win activate];
-  [[win desktopView] showMountedVolumes];
-  [[win desktopView] showContentsOfNode: dskNode];
+  [desktopView showMountedVolumes];
+  [desktopView showContentsOfNode: dskNode];
   [self addWatcherForPath: [dskNode path]];
     
   if ((hidedock == NO) && ([dock superview] == nil)) {
-    [[win desktopView] addSubview: dock];
+    [desktopView addSubview: dock];
     [dock tile];
   }
+  
+  [mpointWatcher startWatching];  
 }
 
 - (void)deactivateDesktop
 {
   [win deactivate];
-  [self removeWatcherForPath: [dskNode path]];
+  [self removeWatcherForPath: [dskNode path]];  
+  [mpointWatcher stopWatching];
 }
 
 - (BOOL)isActive
@@ -189,17 +194,7 @@ static GWDesktopManager *desktopManager = nil;
                   atomically: YES];
   }
 
-  path = [NSSearchPathForDirectoriesInDomains(NSLibraryDirectory, NSUserDomainMask, YES) lastObject];
-  path = [path stringByAppendingPathComponent: @"Desktop"];
-
-  if (([fm fileExistsAtPath: path isDirectory: &isdir] && isdir) == NO) {
-    if ([fm createDirectoryAtPath: path attributes: nil] == NO) {
-      NSLog(@"Can't create the Recycler directory! Quitting now.");
-      [NSApp terminate: self];
-    }
-  }
-
-	path = [path stringByAppendingPathComponent: @".Trash"];
+  path = [NSHomeDirectory() stringByAppendingPathComponent: @".Trash"]; 
 
 	if ([fm fileExistsAtPath: path isDirectory: &isdir] == NO) {
     if ([fm createDirectoryAtPath: path attributes: nil] == NO) {
@@ -214,7 +209,6 @@ static GWDesktopManager *desktopManager = nil;
   usexbundle = value;
   
   if ([self isActive]) { 
-    GWDesktopView *desktopView = [win desktopView];
     id window = nil;  
     BOOL changed = NO;
     
@@ -256,18 +250,23 @@ static GWDesktopManager *desktopManager = nil;
 
 - (id)loadXWinBundle
 {
+  NSEnumerator	*enumerator;
   NSString *bpath;
   NSBundle *bundle;
   
-  bpath = [NSSearchPathForDirectoriesInDomains(NSLibraryDirectory, NSSystemDomainMask, YES) lastObject];
-  bpath = [bpath stringByAppendingPathComponent: @"Bundles"];
-  bpath = [bpath stringByAppendingPathComponent: @"XDesktopWindow.bundle"];
+  enumerator = [NSSearchPathForDirectoriesInDomains
+    (NSLibraryDirectory, NSAllDomainsMask, YES) objectEnumerator];
+  while ((bpath = [enumerator nextObject]) != nil)
+    {
+      bpath = [bpath stringByAppendingPathComponent: @"Bundles"];
+      bpath = [bpath stringByAppendingPathComponent: @"XDesktopWindow.bundle"];
 
-  bundle = [NSBundle bundleWithPath: bpath];
+      bundle = [NSBundle bundleWithPath: bpath];
   
-  if (bundle) {
-    return [[[bundle principalClass] alloc] init];
-  }
+      if (bundle) {
+	return [[[bundle principalClass] alloc] init];
+      }
+    }
 
   return nil;
 }
@@ -279,7 +278,7 @@ static GWDesktopManager *desktopManager = nil;
 
 - (id)desktopView
 {
-  return [win desktopView];
+  return desktopView;
 }
 
 - (Dock *)dock
@@ -297,7 +296,7 @@ static GWDesktopManager *desktopManager = nil;
   dockPosition = pos;
   [dock setPosition: pos];
   [self setReservedFrames];
-  [[win desktopView] dockPositionDidChange];
+  [desktopView dockPositionDidChange];
 }
 
 - (void)setDockActive:(BOOL)value
@@ -306,11 +305,12 @@ static GWDesktopManager *desktopManager = nil;
   
   if (hidedock && [dock superview]) {
     [dock removeFromSuperview];
-    [[win desktopView] setNeedsDisplayInRect: dockReservedFrame];
+    [desktopView setNeedsDisplayInRect: dockReservedFrame];
     
   } else if ([dock superview] == nil) {
-    [[win desktopView] addSubview: dock];
-    [[win desktopView] setNeedsDisplayInRect: dockReservedFrame];
+    [desktopView addSubview: dock];
+    [dock tile];
+    [desktopView setNeedsDisplayInRect: dockReservedFrame];
   }
 }
 
@@ -322,17 +322,34 @@ static GWDesktopManager *desktopManager = nil;
 - (void)setReservedFrames
 {
   NSRect screenFrame = [[NSScreen mainScreen] frame];
+  NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];	
+  NSString *menuStyle = [defaults objectForKey: @"NSMenuInterfaceStyle"];
+  
+  macmenuReservedFrame = NSZeroRect;
+
+  if (menuStyle && [menuStyle isEqual: @"NSMacintoshInterfaceStyle"]) {
+    macmenuReservedFrame.size.width = screenFrame.size.width;
+    macmenuReservedFrame.size.height = 25;
+    macmenuReservedFrame.origin.x = 0;
+    macmenuReservedFrame.origin.y = screenFrame.size.height - 25;    
+  }
 
   dockReservedFrame.size.height = screenFrame.size.height;
-  dockReservedFrame.size.width = 64 + 10;
+  dockReservedFrame.size.width = 64 + RESV_MARGIN;
   dockReservedFrame.origin.x = 0;
   dockReservedFrame.origin.y = 0;
   
   if (dockPosition == DockPositionRight) {
-    dockReservedFrame.origin.x = screenFrame.size.width - 64 - 10;
+    dockReservedFrame.origin.x = screenFrame.size.width - 64 - RESV_MARGIN;
   }
   
-  tshelfReservedFrame = NSMakeRect(0, 0, screenFrame.size.width, 106 + 10);
+  tshelfReservedFrame = NSMakeRect(0, 0, screenFrame.size.width, 106 + RESV_MARGIN);
+  tshelfActivateFrame = NSMakeRect(0, 0, screenFrame.size.width, 20);
+}
+
+- (NSRect)macmenuReservedFrame
+{
+  return macmenuReservedFrame;
 }
 
 - (NSRect)dockReservedFrame
@@ -345,15 +362,31 @@ static GWDesktopManager *desktopManager = nil;
   return tshelfReservedFrame;
 }
 
+- (NSRect)tshelfActivateFrame
+{
+  return tshelfActivateFrame;
+}
+
 - (NSImage *)tabbedShelfBackground
 {
-  return [[win desktopView] tshelfBackground];
+  return [desktopView tshelfBackground];
+}
+
+- (void)mouseEnteredTShelfActivateFrame
+{
+  [[gworkspace tabbedShelf] animateShowing];
+}
+
+- (void)mouseExitedTShelfActiveFrame
+{
+  [[gworkspace tabbedShelf] animateHiding];
 }
 
 - (void)deselectAllIcons
 {
-  [[win desktopView] unselectOtherReps: nil];
-  [[win desktopView] stopRepNameEditing];
+  [desktopView unselectOtherReps: nil];
+  [desktopView selectionDidChange];
+  [desktopView stopRepNameEditing];
 }
 
 - (void)deselectInSpatialViewers
@@ -417,7 +450,6 @@ static GWDesktopManager *desktopManager = nil;
   NSDictionary *opinfo = (NSDictionary *)[notif object];  
 
   if ([dskNode involvedByFileOperation: opinfo]) {
-    dskWatcherSuspended = YES;
     [[self desktopView] nodeContentsWillChange: opinfo];
   }
 }
@@ -440,7 +472,6 @@ static GWDesktopManager *desktopManager = nil;
   }
   
   [dock nodeContentsDidChange: opinfo];  
-  dskWatcherSuspended = NO;
 }
 
 - (void)watcherNotification:(NSNotification *)notif
@@ -449,22 +480,19 @@ static GWDesktopManager *desktopManager = nil;
   NSString *path = [info objectForKey: @"path"];
   NSString *event = [info objectForKey: @"event"];
   
-  if ([event isEqual: @"GWWatchedDirectoryDeleted"]) {
-    if ([path isEqual: [dskNode path]]) {
+  if ([path isEqual: [dskNode path]]) {
+    if ([event isEqual: @"GWWatchedPathDeleted"]) {
       NSRunAlertPanel(nil, 
                       NSLocalizedString(@"The Desktop directory has been deleted! Quiting now!", @""), 
                       NSLocalizedString(@"OK", @""), 
                       nil, 
                       nil);                                     
       [NSApp terminate: self];
+
+    } else {
+      [[self desktopView] watchedPathChanged: info];
     }
-    
-  } else if ([event isEqual: @"GWWatchedFileModified"]) {
-    [[self desktopView] watchedPathChanged: info];
-    
-  } else if ([path isEqual: [dskNode path]] && (dskWatcherSuspended == NO)) {
-    [[self desktopView] watchedPathChanged: info];
-  }    
+  }
 
   [dock watchedPathChanged: info];  
 }
@@ -474,6 +502,22 @@ static GWDesktopManager *desktopManager = nil;
   [[self desktopView] updateIcons];
 }
 
+- (void)removableMediaPathsDidChange
+{
+  [[self desktopView] showMountedVolumes];
+  [mpointWatcher startWatching];
+}
+
+- (void)hideDotsFileDidChange:(BOOL)hide
+{
+  [[self desktopView] reloadFromNode: dskNode];
+}
+
+- (void)hiddenFilesDidChange:(NSArray *)paths
+{
+  [[self desktopView] reloadFromNode: dskNode];
+}
+
 - (void)newVolumeMounted:(NSNotification *)notif
 {
   if (win && [win isVisible]) {
@@ -490,7 +534,7 @@ static GWDesktopManager *desktopManager = nil;
     NSDictionary *dict = [notif userInfo];  
     NSString *volpath = [dict objectForKey: @"NSDevicePath"];
 
-    [[FSNodeRep sharedInstance] lockPaths: [NSArray arrayWithObject: volpath]];
+    [fsnodeRep lockPaths: [NSArray arrayWithObject: volpath]];
     [[self desktopView] workspaceWillUnmountVolumeAtPath: volpath];
   }
 }
@@ -501,14 +545,14 @@ static GWDesktopManager *desktopManager = nil;
     NSDictionary *dict = [notif userInfo];  
     NSString *volpath = [dict objectForKey: @"NSDevicePath"];
 
-    [[FSNodeRep sharedInstance] unlockPaths: [NSArray arrayWithObject: volpath]];
+    [fsnodeRep unlockPaths: [NSArray arrayWithObject: volpath]];
     [[self desktopView] workspaceDidUnmountVolumeAtPath: volpath];
   }
 }
 
-- (void)mountThreadWillExit:(NSNotification *)notif
+- (void)mountedVolumesDidChange
 {
-  NSLog(@"mount thread will exit");
+  [[self desktopView] showMountedVolumes];
 }
 
 - (void)updateDefaults
@@ -522,7 +566,24 @@ static GWDesktopManager *desktopManager = nil;
   [defaults setBool: hidedock forKey: @"hidedock"];
   
   [dock updateDefaults];
-  [[win desktopView] updateDefaults];
+  [desktopView updateDefaults];
+}
+
+- (void)setContextHelp
+{
+  NSHelpManager *manager = [NSHelpManager sharedHelpManager];
+  NSString *help;
+
+  help = @"Desktop.rtfd";
+  [manager setContextHelp: (NSAttributedString *)help 
+               withObject: [self desktopView]];
+
+  help = @"Dock.rtfd";
+  [manager setContextHelp: (NSAttributedString *)help withObject: dock];
+  
+  help = @"Recycler.rtfd";
+  [manager setContextHelp: (NSAttributedString *)help 
+               withObject: [dock trashIcon]];
 }
 
 @end
@@ -536,14 +597,51 @@ static GWDesktopManager *desktopManager = nil;
 - (BOOL)validateItem:(id)menuItem
 {
   if ([self isActive]) {
-    NSString *itemTitle = [menuItem title];
-    GWDesktopView *desktopView = [win desktopView];
+    SEL action = [menuItem action];
 
-    if ([itemTitle isEqual: NSLocalizedString(@"Duplicate", @"")]
-        || [itemTitle isEqual: NSLocalizedString(@"Move to Recycler", @"")]) {
+    if (sel_eq(action, @selector(duplicateFiles:))
+                || sel_eq(action, @selector(recycleFiles:))
+                      || sel_eq(action, @selector(deleteFiles:))) {
       return ([[desktopView selectedNodes] count] > 0);
-    }
 
+    } else if (sel_eq(action, @selector(openSelection:))) {
+      NSArray *selection = [desktopView selectedNodes];
+     
+      return (selection && [selection count] 
+            && ([selection isEqual: [NSArray arrayWithObject: dskNode]] == NO));
+    
+    } else if (sel_eq(action, @selector(openWith:))) {
+      NSArray *selection = [desktopView selectedNodes];
+      BOOL canopen = YES;
+      int i;
+
+      if (selection && [selection count]
+            && ([selection isEqual: [NSArray arrayWithObject: dskNode]] == NO)) {
+        for (i = 0; i < [selection count]; i++) {
+          FSNode *node = [selection objectAtIndex: i];
+
+          if (([node isPlain] == NO) 
+                && (([node isPackage] == NO) || [node isApplication])) {
+            canopen = NO;
+            break;
+          }
+        }
+      } else {
+        canopen = NO;
+      }
+
+      return canopen;
+      
+    } else if (sel_eq(action, @selector(openSelectionAsFolder:))) {
+      NSArray *selection = [desktopView selectedNodes];
+    
+      if (selection && ([selection count] == 1)) {  
+        return [[selection objectAtIndex: 0] isPackage];
+      }
+    
+      return NO;
+    }
+         
     return YES;
   }
   
@@ -552,32 +650,54 @@ static GWDesktopManager *desktopManager = nil;
 
 - (void)openSelectionInNewViewer:(BOOL)newv
 {
-  NSArray *selreps = [[win desktopView] selectedReps];
+  NSArray *selreps = [desktopView selectedReps];
   int i;
     
   for (i = 0; i < [selreps count]; i++) {
     FSNode *node = [[selreps objectAtIndex: i] node];
-    NSString *path = [node path];
         
-    if ([node isDirectory]) {
-      if ([node isPackage]) {    
-        if ([node isApplication] == NO) {
-          [gworkspace openFile: path];
+    if ([node hasValidPath]) {           
+      NS_DURING
+        {
+      if ([node isDirectory]) {
+        if ([node isPackage]) {    
+          if ([node isApplication] == NO) {
+            [gworkspace openFile: [node path]];
+          } else {
+            [ws launchApplication: [node path]];
+          }
         } else {
-          [ws launchApplication: path];
+          [gworkspace newViewerAtPath: [node path]];
+        } 
+      } else if ([node isPlain]) {        
+        [gworkspace openFile: [node path]];
+      }
         }
-      } else {
-        [gworkspace newViewerAtPath: [node path]];
-      } 
-    } else if ([node isPlain]) {        
-      [gworkspace openFile: path];
+      NS_HANDLER
+        {
+          NSRunAlertPanel(NSLocalizedString(@"error", @""), 
+              [NSString stringWithFormat: @"%@ %@!", 
+                        NSLocalizedString(@"Can't open ", @""), [node name]],
+                                            NSLocalizedString(@"OK", @""), 
+                                            nil, 
+                                            nil);                                     
+        }
+      NS_ENDHANDLER
+      
+    } else {
+      NSRunAlertPanel(NSLocalizedString(@"error", @""), 
+          [NSString stringWithFormat: @"%@ %@!", 
+                    NSLocalizedString(@"Can't open ", @""), [node name]],
+                                        NSLocalizedString(@"OK", @""), 
+                                        nil, 
+                                        nil);                                     
     }
   }
 }
 
 - (void)openSelectionAsFolder
 {
-  NSArray *selnodes = [[win desktopView] selectedNodes];
+  NSArray *selnodes = [desktopView selectedNodes];
   int i;
     
   for (i = 0; i < [selnodes count]; i++) {
@@ -591,6 +711,11 @@ static GWDesktopManager *desktopManager = nil;
   }
 }
 
+- (void)openSelectionWith
+{
+  [gworkspace openSelectedPathsWith];
+}
+
 - (void)newFolder
 {
   [gworkspace newObjectAtPath: [dskNode path] isDirectory: YES];
@@ -603,14 +728,14 @@ static GWDesktopManager *desktopManager = nil;
 
 - (void)duplicateFiles
 {
-  if ([[[win desktopView] selectedNodes] count]) {
+  if ([[desktopView selectedNodes] count]) {
     [gworkspace duplicateFiles];
   }
 }
 
-- (void)deleteFiles
+- (void)recycleFiles
 {
-  if ([[[win desktopView] selectedNodes] count]) {
+  if ([[desktopView selectedNodes] count]) {
     [gworkspace moveToTrash];
   }
 }
@@ -620,6 +745,13 @@ static GWDesktopManager *desktopManager = nil;
   [gworkspace emptyRecycler: nil];
 }
 
+- (void)deleteFiles
+{
+  if ([[desktopView selectedNodes] count]) {
+    [gworkspace deleteFiles];
+  }
+}
+
 - (void)setShownType:(id)sender
 {
   NSString *title = [sender title];
@@ -627,7 +759,7 @@ static GWDesktopManager *desktopManager = nil;
 
   if ([title isEqual: NSLocalizedString(@"Name", @"")]) {
     type = FSNInfoNameType;
-  } else if ([title isEqual: NSLocalizedString(@"Kind", @"")]) {
+  } else if ([title isEqual: NSLocalizedString(@"Type", @"")]) {
     type = FSNInfoKindType;
   } else if ([title isEqual: NSLocalizedString(@"Size", @"")]) {
     type = FSNInfoSizeType;
@@ -639,17 +771,17 @@ static GWDesktopManager *desktopManager = nil;
     type = FSNInfoNameType;
   } 
 
-  [(id <FSNodeRepContainer>)[win desktopView] setShowType: type];  
+  [desktopView setShowType: type];  
 }
 
 - (void)setExtendedShownType:(id)sender
 {
-  [(id <FSNodeRepContainer>)[win desktopView] setExtendedShowType: [sender title]]; 
+  [desktopView setExtendedShowType: [sender title]]; 
 }
 
 - (void)setIconsSize:(id)sender
 {
-  [(id <FSNodeRepContainer>)[win desktopView] setIconSize: [[sender title] intValue]];
+  [desktopView setIconSize: [[sender title] intValue]];
 }
 
 - (void)setIconsPosition:(id)sender
@@ -657,20 +789,20 @@ static GWDesktopManager *desktopManager = nil;
   NSString *title = [sender title];
 
   if ([title isEqual: NSLocalizedString(@"Left", @"")]) {
-    [(id <FSNodeRepContainer>)[win desktopView] setIconPosition: NSImageLeft];
+    [desktopView setIconPosition: NSImageLeft];
   } else {
-    [(id <FSNodeRepContainer>)[win desktopView] setIconPosition: NSImageAbove];
+    [desktopView setIconPosition: NSImageAbove];
   }
 }
 
 - (void)setLabelSize:(id)sender
 {
-  [[win desktopView] setLabelTextSize: [[sender title] intValue]];
+  [desktopView setLabelTextSize: [[sender title] intValue]];
 }
 
 - (void)selectAllInViewer
 {
-	[[win desktopView] selectAll];
+	[desktopView selectAll];
 }
 
 - (void)showTerminal
@@ -681,241 +813,116 @@ static GWDesktopManager *desktopManager = nil;
 @end
 
 
+@implementation MPointWatcher
+
+- (void)dealloc
+{
+	if (timer && [timer isValid]) {
+		[timer invalidate];
+	}  
+  RELEASE (volinfo);  
+	[super dealloc];
+}
+
+- (id)initForManager:(GWDesktopManager *)mngr
+{
+  self = [super init];
+  
+  if (self) {
+    manager = mngr;    
+    volinfo = [NSMutableArray new];
+    active = NO;
+    fm = [NSFileManager defaultManager];
+    
+    timer = [NSTimer scheduledTimerWithTimeInterval: 1.0 
+												                     target: self 
+                                           selector: @selector(watchMountPoints:) 
+										                       userInfo: nil 
+                                            repeats: YES];
+  }
+  
+  return self;
+}
+
+- (void)startWatching
+{
+  NSSet *volumes = [[FSNodeRep sharedInstance] volumes];
+  NSEnumerator *enumerator = [volumes objectEnumerator];
+  NSString *path;
+  
+  [volinfo removeAllObjects];
+
+  while ((path = [enumerator nextObject])) {
+    NSDictionary *attributes = [fm fileAttributesAtPath: path traverseLink: NO];
+    
+    if (attributes) {
+      NSDate *moddate = [attributes fileModificationDate];
+      NSMutableDictionary *dict = [NSMutableDictionary dictionary];
+      
+      [dict setObject: path forKey: @"path"];
+      [dict setObject: moddate forKey: @"moddate"];
+      
+      [volinfo addObject: dict];  
+    }
+  }
+
+  active = YES;
+}
+
+- (void)stopWatching
+{
+  active = NO;
+  [volinfo removeAllObjects];
+}
+
+- (void)watchMountPoints:(id)sender
+{
+  if (active) {
+    int count = [volinfo count];
+    BOOL changed = NO;
+    int i;
+    
+    for (i = 0; i < count; i++) {
+      NSMutableDictionary *dict = [volinfo objectAtIndex: i];
+      NSString *path = [dict objectForKey: @"path"];
+      NSDate *moddate = [dict objectForKey: @"moddate"];
+      NSDictionary *attributes = [fm fileAttributesAtPath: path traverseLink: NO];      
+      
+      if (attributes) {
+        NSDate *lastmod = [attributes fileModificationDate];
+      
+        if ([moddate isEqualToDate: lastmod] == NO) {
+          [dict setObject: lastmod forKey: @"moddate"];
+          changed = YES;
+        }
+        
+      } else {
+        [volinfo removeObjectAtIndex: i]; 
+        count--;
+        i--;
+        changed = YES;       
+      }      
+    }
+    
+    if (changed) {
+      [manager mountedVolumesDidChange]; 
+    }
+  }
+}
+
+@end
+
+
 @implementation GWMounter
 
 + (void)mountRemovableMedia
 {
-  NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
-	GWMounter *mounter = [GWMounter new];  
-  
-  [mounter mountRemovableMedia];
-  
-  [[NSRunLoop currentRunLoop] run];
-  [pool release];
-}
-
-- (void)mountRemovableMedia
-{
+  CREATE_AUTORELEASE_POOL(pool);
   [[NSWorkspace sharedWorkspace] mountNewRemovableMedia];
-  [NSThread exit];
+  RELEASE (pool);  
 }
 
 @end
 
 
-@implementation NSWorkspace (mounting)
-
-- (BOOL)getFileSystemInfoForPath:(NSString *)fullPath
-		                 isRemovable:(BOOL *)removableFlag
-		                  isWritable:(BOOL *)writableFlag
-		               isUnmountable:(BOOL *)unmountableFlag
-		                 description:(NSString **)description
-			                      type:(NSString **)fileSystemType
-{
-  NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
-  NSString *mtabpath = [defaults stringForKey: @"GSMtabPath"];
-  NSArray *removables = [defaults arrayForKey: @"GSRemovableMediaPaths"];
-  NSString *mtab;
-  NSArray *mounts;
-  int i;
-   
-  if (mtabpath == nil) {
-    mtabpath = @"/etc/mtab";
-  }
-
-  if (removables == nil) {
-    removables = [NSArray arrayWithObjects: @"/mnt/floppy", @"/mnt/cdrom", nil];
-  }
-  
-  mtab = [NSString stringWithContentsOfFile: mtabpath];
-  mounts = [mtab componentsSeparatedByString: @"\n"];
-
-  for (i = 0; i < [mounts count]; i++) {
-    NSString *mount = [mounts objectAtIndex: i];
-    
-    if ([mount length]) {
-      NSArray	*parts = [mount componentsSeparatedByString: @" "];
-      
-      if ([parts count] == 6) {
-     //   NSString *device = [parts objectAtIndex: 0];
-        NSString *mountPoint = [parts objectAtIndex: 1];
-        NSString *fsType = [parts objectAtIndex: 2];
-        NSString *fsOptions = [parts objectAtIndex: 3];
-    //    NSString *fsDump = [parts objectAtIndex: 4];      
-    //    NSString *fsPass = [parts objectAtIndex: 5];      
-
-        if ([mountPoint isEqual: fullPath]) {
-          NSScanner *scanner = [NSScanner scannerWithString: fsOptions];
-          
-          *removableFlag = [removables containsObject: mountPoint];
-          *writableFlag = [scanner scanString: @"rw" intoString: NULL];
-          *unmountableFlag = YES;
-          *description = fsType;
-          *fileSystemType = fsType;
-          
-          return YES;
-        }
-      }
-    }
-  }
-
-  return NO;
-}
-
-- (NSArray *)mountedLocalVolumePaths
-{
-  NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
-  NSString *mtabpath = [defaults stringForKey: @"GSMtabPath"];
-  NSArray *reserved = [defaults arrayForKey: @"GSReservedMountPoints"];
-  NSString *mtab;
-  NSArray *mounts;
-  NSMutableArray *names;
-  int i;
-   
-  if (mtabpath == nil) {
-    mtabpath = @"/etc/mtab";
-  }
-
-  if (reserved == nil) {
-    reserved = [NSArray arrayWithObjects: @"proc", @"devpts", @"shm", 
-                                    @"usbdevfs", @"devpts", 
-                                    @"sysfs", @"tmpfs", nil];
-  }
-
-  mtab = [NSString stringWithContentsOfFile: mtabpath];
-  mounts = [mtab componentsSeparatedByString: @"\n"];
-  names = [NSMutableArray array];
-
-  for (i = 0; i < [mounts count]; i++) {
-    NSString *mount = [mounts objectAtIndex: i];
-    
-    if ([mount length]) {
-      NSArray	*parts = [mount componentsSeparatedByString: @" "];
-        
-      if ([parts count] >= 2) {
-        NSString *type = [parts objectAtIndex: 2];
-        
-        if ([reserved containsObject: type] == NO) {
-	        [names addObject: [parts objectAtIndex: 1]];
-	      }
-      }
-    } 
-  }
-
-  return names;
-}
-
-- (NSArray *)mountedRemovableMedia
-{
-  NSArray	*volumes = [self mountedLocalVolumePaths];
-  NSMutableArray *names = [NSMutableArray array];
-  unsigned	i;
-
-  for (i = 0; i < [volumes count]; i++) {
-    BOOL removableFlag;
-    BOOL writableFlag;
-    BOOL unmountableFlag;
-    NSString *description;
-    NSString *fileSystemType;
-    NSString *name = [volumes objectAtIndex: i];
-
-    if ([self getFileSystemInfoForPath: name
-		              isRemovable: &removableFlag
-		              isWritable: &writableFlag
-		              isUnmountable: &unmountableFlag
-		              description: &description
-		              type: &fileSystemType] && removableFlag) {
-	    [names addObject: name];
-	  }
-  }
-
-  return names;
-}
-
-- (NSArray *)mountNewRemovableMedia
-{
-  NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
-  NSArray *removables = [defaults arrayForKey: @"GSRemovableMediaPaths"];
-  NSArray *mountedMedia = [self mountedRemovableMedia]; 
-  NSMutableArray *willMountMedia = [NSMutableArray array];
-  NSMutableArray *newlyMountedMedia = [NSMutableArray array];
-  int i;
-
-  if (removables == nil) {
-    removables = [NSArray arrayWithObjects: @"/mnt/floppy", @"/mnt/cdrom", nil];
-  }
-
-  for (i = 0; i < [removables count]; i++) {
-    NSString *removable = [removables objectAtIndex: i];
-    
-    if ([mountedMedia containsObject: removable] == NO) {
-      [willMountMedia addObject: removable];
-    }
-  }  
-  
-  for (i = 0; i < [willMountMedia count]; i++) {
-    NSString *media = [willMountMedia objectAtIndex: i];
-    NSTask *task = [NSTask launchedTaskWithLaunchPath: @"mount"
-                                arguments: [NSArray arrayWithObject: media]];
-      
-    if (task) {
-      [task waitUntilExit];
-      
-      if ([task terminationStatus] == 0) {
-        NSDictionary *userinfo = [NSDictionary dictionaryWithObject: media 
-                                                      forKey: @"NSDevicePath"];
-
-        [[self notificationCenter] postNotificationName: NSWorkspaceDidMountNotification
-                                  object: self
-                                userInfo: userinfo];
-
-        [newlyMountedMedia addObject: media];
-      }
-    }
-  }
-
-  return newlyMountedMedia;
-}
-
-- (BOOL)unmountAndEjectDeviceAtPath:(NSString *)path
-{
-  NSArray	*volumes = [self mountedLocalVolumePaths];
-
-  if ([volumes containsObject: path]) {
-    NSDictionary *userinfo;
-    NSTask *task;
-
-    userinfo = [NSDictionary dictionaryWithObject: path forKey: @"NSDevicePath"];
-
-    [[self notificationCenter] postNotificationName: NSWorkspaceWillUnmountNotification
-				                object: self
-				              userInfo: userinfo];
-
-    task = [NSTask launchedTaskWithLaunchPath: @"umount"
-				                            arguments: [NSArray arrayWithObject: path]];
-
-    if (task) {
-      [task waitUntilExit];
-      if ([task terminationStatus] != 0) {
-	      return NO;
-	    } 
-    } else {
-      return NO;
-    }
-
-    [[self notificationCenter] postNotificationName: NSWorkspaceDidUnmountNotification
-				                object: self
-				              userInfo: userinfo];
-
-    task = [NSTask launchedTaskWithLaunchPath: @"eject"
-				                            arguments: [NSArray arrayWithObject: path]];
-
-    return YES;
-  }
-  
-  return NO;
-}
-
-@end
-
 
