@@ -1,6 +1,6 @@
 /* GWDesktopView.m
  *  
- * Copyright (C) 2005 Free Software Foundation, Inc.
+ * Copyright (C) 2005-2010 Free Software Foundation, Inc.
  *
  * Author: Enrico Sersale <enrico@imago.ro>
  * Date: January 2005
@@ -19,7 +19,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
  */
  
 #include <Foundation/Foundation.h>
@@ -59,10 +59,11 @@
 	if (grid != NULL) {
 		NSZoneFree (NSDefaultMallocZone(), grid);
 	}
+  RELEASE (mountedVolumes);
   RELEASE (desktopInfo);
-  TEST_RELEASE (backImage);
-  TEST_RELEASE (imagePath);
-  TEST_RELEASE (dragIcon);
+  RELEASE (backImage);
+  RELEASE (imagePath);
+  RELEASE (dragIcon);
 
   [super dealloc];
 }
@@ -109,6 +110,7 @@
     ASSIGN (backColor, DEF_COLOR);
 
     backImageStyle = BackImageCenterStyle;
+    mountedVolumes = [NSMutableArray new];
     [self getDesktopInfo];
     [self makeIconsGrid];
     dragIcon = nil;
@@ -144,42 +146,38 @@
 
 - (void)showMountedVolumes
 {
-  NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
-  NSString *mtabpath = [defaults stringForKey: @"GSMtabPath"];
   NSArray *rvpaths = [[NSWorkspace sharedWorkspace] mountedRemovableMedia];
-  int count = [icons count];
-  int i;
 
-  if (mtabpath == nil) {
-    mtabpath = @"/etc/mtab";
-  }
+  if ([mountedVolumes isEqual: rvpaths] == NO) {
+    int count = [icons count];
+    int i;
+        
+    [mountedVolumes removeAllObjects];
+    [mountedVolumes addObjectsFromArray: rvpaths];
 
-  [desktopApp removeWatcherForPath: mtabpath];
+    for (i = 0; i < count; i++) {
+      FSNIcon *icon = [icons objectAtIndex: i];
 
-  for (i = 0; i < count; i++) {
-    FSNIcon *icon = [icons objectAtIndex: i];
-  
-    if ([[icon node] isMountPoint]) {
-      [self removeRep: icon];
-      count--;
-      i--;
+      if ([[icon node] isMountPoint]) {
+        [self removeRep: icon];
+        count--;
+        i--;
+      }
     }
-  }
 
-  for (i = 0; i < [rvpaths count]; i++) {
-    NSString *vpath = [rvpaths objectAtIndex: i];
-    
-    if ([vpath isEqual: path_separator()] == NO) {
-      FSNode *vnode = [FSNode nodeWithPath: vpath];
-  
-      [vnode setMountPoint: YES];
-      [self addRepForSubnode: vnode];
+    for (i = 0; i < [mountedVolumes count]; i++) {
+      NSString *vpath = [mountedVolumes objectAtIndex: i];
+
+      if ([vpath isEqual: path_separator()] == NO) {
+        FSNode *vnode = [FSNode nodeWithPath: vpath];
+
+        [vnode setMountPoint: YES];
+        [self addRepForSubnode: vnode];
+      }
     }
+
+    [self tile];  
   }
-  
-  [self tile];  
-  
-  [desktopApp addWatcherForPath: mtabpath];
 }
 
 - (void)dockPositionDidChange
@@ -294,7 +292,11 @@
 		}
   }
   
-  return [icns count] ? icns : nil;
+  if ([icns count]) {
+    return icns;
+  }
+  
+  return nil;
 }
 
 - (NSArray *)iconsWithGridOriginY:(float)y
@@ -310,8 +312,12 @@
       [icns addObject: icon];
 		}
   }
+
+  if ([icns count]) {
+    return icns;
+  }
   
-  return [icns count] ? icns : nil;
+  return nil;
 }
 
 - (int)indexOfGridRectContainingPoint:(NSPoint)p
@@ -341,9 +347,9 @@
   if (iconPosition == NSImageAbove) {  
     hlightRect.origin.x = ceil((gridSize.width - hlightRect.size.width) / 2);   
     if (infoType != FSNInfoNameType) {
-      hlightRect.origin.y = floor([labelFont defaultLineHeightForFont] * 2 - 2);
-    } else {
-      hlightRect.origin.y = floor([labelFont defaultLineHeightForFont]);
+      hlightRect.origin.y = floor([fsnodeRep heighOfFont: labelFont] * 2 - 2);
+    } else { 
+      hlightRect.origin.y = floor([fsnodeRep heighOfFont: labelFont]);
     }
   } else {
     hlightRect.origin.x = 0;
@@ -360,6 +366,7 @@
 {
   NSRect dckr = [manager dockReservedFrame];
   NSRect tshfr = [manager tshelfReservedFrame];
+  NSRect mmfr = [manager macmenuReservedFrame];
   NSRect gridrect = screenFrame;
   int ymargin;
   NSPoint gpnt;
@@ -374,6 +381,9 @@
   gridrect.origin.y += tshfr.size.height;
   gridrect.size.height -= tshfr.size.height;
   gridrect.size.width -= dckr.size.width;
+  
+  gridrect.size.height -= mmfr.size.height;
+  
   if ([manager dockPosition] == DockPositionLeft) {
     gridrect.origin.x += dckr.size.width;
   }
@@ -466,17 +476,17 @@
 - (void)getDesktopInfo
 {
   NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];	
-  id dskinfo = [defaults objectForKey: @"desktopinfo"];
+  NSDictionary *dskinfo = [defaults objectForKey: @"desktopinfo"];
 
   if (dskinfo) {
     id entry = [dskinfo objectForKey: @"backcolor"];
     FSNInfoType itype;
 
     if (entry) {
-      float red = [[entry objectForKey: @"red"] floatValue];
-      float green = [[entry objectForKey: @"green"] floatValue];
-      float blue = [[entry objectForKey: @"blue"] floatValue];
-      float alpha = [[entry objectForKey: @"alpha"] floatValue];
+      float red = [[(NSDictionary *)entry objectForKey: @"red"] floatValue];
+      float green = [[(NSDictionary *)entry objectForKey: @"green"] floatValue];
+      float blue = [[(NSDictionary *)entry objectForKey: @"blue"] floatValue];
+      float alpha = [[(NSDictionary *)entry objectForKey: @"alpha"] floatValue];
 
       ASSIGN (backColor, [NSColor colorWithCalibratedRed: red 
                                                    green: green 
@@ -484,22 +494,6 @@
                                                    alpha: alpha]);
     }
 
-    entry = [dskinfo objectForKey: @"textcolor"];
-
-    if (entry) {
-      float red = [[entry objectForKey: @"red"] floatValue];
-      float green = [[entry objectForKey: @"green"] floatValue];
-      float blue = [[entry objectForKey: @"blue"] floatValue];
-      float alpha = [[entry objectForKey: @"alpha"] floatValue];
-
-      ASSIGN (textColor, [NSColor colorWithCalibratedRed: red 
-                                                   green: green 
-                                                    blue: blue 
-                                                   alpha: alpha]);
-
-      ASSIGN (disabledTextColor, [textColor highlightWithLevel: NSDarkGray]);      
-    }
-
     entry = [dskinfo objectForKey: @"imagestyle"];
     backImageStyle = entry ? [entry intValue] : backImageStyle;
 
@@ -570,7 +564,6 @@
   NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];	
   NSMutableDictionary *indexes = [NSMutableDictionary dictionary];
   NSMutableDictionary *backColorDict = [NSMutableDictionary dictionary];
-  NSMutableDictionary *txtColorDict = [NSMutableDictionary dictionary];
   float red, green, blue, alpha;
   int i;
 
@@ -582,14 +575,6 @@
 
   [desktopInfo setObject: backColorDict forKey: @"backcolor"];
 
-  [textColor getRed: &red green: &green blue: &blue alpha: &alpha];
-  [txtColorDict setObject: [NSNumber numberWithFloat: red] forKey: @"red"];
-  [txtColorDict setObject: [NSNumber numberWithFloat: green] forKey: @"green"];
-  [txtColorDict setObject: [NSNumber numberWithFloat: blue] forKey: @"blue"];
-  [txtColorDict setObject: [NSNumber numberWithFloat: alpha] forKey: @"alpha"];
-
-  [desktopInfo setObject: txtColorDict forKey: @"textcolor"];
-
   [desktopInfo setObject: [NSNumber numberWithBool: useBackImage] 
                   forKey: @"usebackimage"];
 
@@ -749,7 +734,10 @@
 
 - (void)mouseDown:(NSEvent *)theEvent
 {
-  [[self window] makeKeyWindow];
+  NSWindow *win = [self window];
+  
+  [win makeMainWindow];
+  [win makeKeyWindow];
   
   if ([theEvent modifierFlags] != NSShiftKeyMask) {
     selectionMask = NSSingleSelectionMask;
@@ -930,6 +918,19 @@
   [super keyDown: theEvent];
 }
 
+- (void)mouseMoved:(NSEvent *)theEvent
+{
+  NSPoint p = [theEvent locationInWindow];
+  
+  if (NSPointInRect(p, [manager tshelfActivateFrame])) { 
+    [manager mouseEnteredTShelfActivateFrame];
+  } else if (NSPointInRect(p, [manager tshelfReservedFrame]) == NO) {
+    [manager mouseExitedTShelfActiveFrame];
+  }
+  
+  [super mouseMoved: theEvent];
+}
+
 - (void)drawRect:(NSRect)rect
 {  
   [super drawRect: rect];
@@ -1083,7 +1084,7 @@
       NSString *fname = [files objectAtIndex: i];
       FSNode *subnode = [FSNode nodeWithRelativePath: fname parent: node];
       FSNIcon *icon = [self repOfSubnode: subnode];
-      int index;
+      int index = 0;
 
       if (i == 0) {
         if (insertIndex != -1) {
@@ -1153,21 +1154,6 @@
         needupdate = YES;
       }
     }
-    
-  } else if ([event isEqual: @"GWWatchedFileModified"]) {
-    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
-    NSString *mtabpath = [defaults stringForKey: @"GSMtabPath"];
-
-    if (mtabpath == nil) {
-      mtabpath = @"/etc/mtab";
-    }
-  
-    fpath = [info objectForKey: @"path"];
-  
-    if ([fpath isEqual: mtabpath]) {
-      [self showMountedVolumes];
-      needupdate = YES;
-    }
   }
   
   if (needupdate) {
@@ -1306,13 +1292,39 @@
   }
 }
 
+- (void)selectAll
+{
+	int i;
+
+  selectionMask = NSSingleSelectionMask;
+  selectionMask |= FSNCreatingSelectionMask;
+  
+	[self unselectOtherReps: nil];
+  
+  selectionMask = FSNMultipleSelectionMask;
+  selectionMask |= FSNCreatingSelectionMask;
+  
+	for (i = 0; i < [icons count]; i++) {
+    FSNIcon *icon = [icons objectAtIndex: i];
+    FSNode *inode = [icon node];
+    
+    if (([inode isReserved] == NO) && ([inode isMountPoint] == NO)) {
+      [icon select];
+    }
+	}
+  
+  selectionMask = NSSingleSelectionMask;
+
+  [self selectionDidChange];
+}
+
 - (void)selectionDidChange
 {
 	if (!(selectionMask & FSNCreatingSelectionMask)) {
-    NSArray *selection = [self selectedPaths];
+    NSArray *selection = [self selectedNodes];
 		
     if ([selection count] == 0) {
-      selection = [NSArray arrayWithObject: [node path]];
+      selection = [NSArray arrayWithObject: node];
     } else {
       [manager deselectInSpatialViewers];
     }
@@ -1402,7 +1414,7 @@
 
 @implementation GWDesktopView (DraggingDestination)
 
-- (unsigned int)draggingEntered:(id <NSDraggingInfo>)sender
+- (NSDragOperation)draggingEntered:(id <NSDraggingInfo>)sender
 {
 	NSPasteboard *pb;
   NSDragOperation sourceDragMask;
@@ -1486,6 +1498,27 @@
     prePath = [prePath stringByDeletingLastPathComponent];
   }
 
+  if ([node isDirectory] && [node isParentOfPath: basePath]) {
+    NSArray *subNodes = [node subNodes];
+    int i;
+    
+    for (i = 0; i < [subNodes count]; i++) {
+      FSNode *nd = [subNodes objectAtIndex: i];
+      
+      if ([nd isDirectory]) {
+        int j;
+        
+        for (j = 0; j < count; j++) {
+          NSString *fname = [[sourcePaths objectAtIndex: j] lastPathComponent];
+          
+          if ([[nd name] isEqual: fname]) {
+            return NSDragOperationNone;
+          }
+        }
+      }
+    }
+  }	
+
   isDragTarget = YES;	
   forceCopy = NO;
   dragPoint = NSZeroPoint;
@@ -1511,18 +1544,23 @@
   return NSDragOperationNone;
 }
 
-- (unsigned int)draggingUpdated:(id <NSDraggingInfo>)sender
+- (NSDragOperation)draggingUpdated:(id <NSDraggingInfo>)sender
 {
-  NSDragOperation sourceDragMask;
-  NSPoint dpoint;
+  NSDragOperation sourceDragMask = [sender draggingSourceOperationMask];
+  NSPoint dpoint = [sender draggingLocation];
   int index;
+  
+  if (NSPointInRect(dpoint, [manager tshelfActivateFrame])) { 
+    [manager mouseEnteredTShelfActivateFrame];
+    return NSDragOperationNone;
+  } else if (NSPointInRect(dpoint, [manager tshelfReservedFrame]) == NO) {
+    [manager mouseExitedTShelfActiveFrame];
+  }
 
 	if (isDragTarget == NO) {
 		return NSDragOperationNone;
 	}
   
-  sourceDragMask = [sender draggingSourceOperationMask];
-  dpoint = [sender draggingLocation];
   index = [self indexOfGridRectContainingPoint: dpoint];
   
   if ((index != -1) && ([self isFreeGridIndex: index])) {
@@ -1569,11 +1607,17 @@
 
 - (void)draggingExited:(id <NSDraggingInfo>)sender
 {
+  NSPoint dpoint = [sender draggingLocation];
+
   DESTROY (dragIcon);
   if (insertIndex != -1) {
     [self setNeedsDisplayInRect: grid[insertIndex]];
   }
 	isDragTarget = NO;
+
+  if (NSPointInRect(dpoint, [manager tshelfReservedFrame]) == NO) {
+    [manager mouseExitedTShelfActiveFrame];
+  }
 }
 
 - (BOOL)prepareForDragOperation:(id <NSDraggingInfo>)sender
@@ -1677,8 +1721,8 @@ int sortDragged(id icn1, id icn2, void *context)
     for (i = 0; i < [sorted count]; i++) {
       FSNIcon *icon = [sorted objectAtIndex: i];
       int oldindex = [icon gridIndex];
-      int index;
-      int shift;
+      int index = 0;
+      int shift = 0;
     
       if (i == 0) {
         index = insertIndex;
