@@ -1,6 +1,6 @@
 /* Fiend.m
  *  
- * Copyright (C) 2003 Free Software Foundation, Inc.
+ * Copyright (C) 2003-2010 Free Software Foundation, Inc.
  *
  * Author: Enrico Sersale <enrico@imago.ro>
  * Date: August 2001
@@ -19,24 +19,36 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
  */
 
-#include <Foundation/Foundation.h>
-#include <AppKit/AppKit.h>
-#include <AppKit/AppKit.h>
-#include "FSNode.h"
-#include "GWFunctions.h"
-#include "Fiend.h"
-#include "FiendLeaf.h"
-#include "Dialogs/Dialogs.h"
-#include "GWorkspace.h"
-#include "GNUstep.h"
+#import <Foundation/Foundation.h>
+#import <AppKit/AppKit.h>
+#import <GNUstepBase/GNUstep.h>
+
+#import "FSNode.h"
+#import "FSNFunctions.h"
+#import "GWFunctions.h"
+#import "Fiend.h"
+#import "FiendLeaf.h"
+#import "Dialogs/Dialogs.h"
+#import "GWorkspace.h"
+
 
 @implementation Fiend
 
 - (void)dealloc
 {
+  NSEnumerator *enumerator = [watchedPaths objectEnumerator];  
+  NSString *wpath;
+            
+  while ((wpath = [enumerator nextObject])) {
+    [gw removeWatcherForPath: wpath];
+  }
+	RELEASE (watchedPaths);
+  
+  [[NSNotificationCenter defaultCenter] removeObserver: self];	
+
   RELEASE (layers);
   RELEASE (namelabel);
   RELEASE (ffButt);
@@ -44,10 +56,10 @@
   RELEASE (leftArr);
   RELEASE (rightArr);
   RELEASE (currentName);
-  TEST_RELEASE (freePositions);
+  RELEASE (freePositions);
   RELEASE (tile);  
   RELEASE (myWin);
-	[super dealloc];
+  [super dealloc];
 }
 
 - (id)init
@@ -74,10 +86,12 @@
     [myWin setFrame: r display: NO];
     
     [myWin setReleasedWhenClosed: NO]; 
+    [myWin setExcludedFromWindowsMenu: YES];
     
     defaults = [NSUserDefaults standardUserDefaults];	
 
     layers = [[NSMutableDictionary alloc] initWithCapacity: 1];
+		watchedPaths = [[NSCountedSet alloc] initWithCapacity: 1];
     
     myPrefs = [defaults dictionaryForKey: @"fiendlayers"];
     if (myPrefs != nil) {
@@ -91,19 +105,31 @@
         
         for (j = 0; j < [paths count]; j++) {
           NSString *path = [paths objectAtIndex: j];
+	        NSString *watched = [path stringByDeletingLastPathComponent];	
           
-          if ([[NSFileManager defaultManager] fileExistsAtPath: path] == YES) { 
+          if ([[NSFileManager defaultManager] fileExistsAtPath: path]) { 
             NSDictionary *dict = [pathsAndRects objectForKey: path];
             int posx = [[dict objectForKey: @"posx"] intValue];
             int posy = [[dict objectForKey: @"posy"] intValue];            
 
-            leaf = [[FiendLeaf alloc] initWithPosX: posx posY: posy
-                                relativeToPoint: r.origin forPath: path 
-                                             inFiend: self  ghostImage: nil];
+            leaf = [[FiendLeaf alloc] initWithPosX: posx 
+                                              posY: posy
+                                   relativeToPoint: r.origin 
+                                           forPath: path 
+                                           inFiend: self  
+                                         layerName: layername
+                                        ghostImage: nil];
             [leaves addObject: leaf];
             RELEASE (leaf);
+            
+	          if ([watchedPaths containsObject: watched] == NO) {
+              [gw addWatcherForPath: watched];
+	          }
+            
+            [watchedPaths addObject: watched];
           }                 
-        }        
+        }
+                
         [layers setObject: leaves forKey: layername];        
       }    
       currentName = [defaults stringForKey: @"fiendcurrentlayer"];
@@ -153,7 +179,19 @@
     leaveshidden = NO;
     isDragTarget = NO;
 		
-		[myWin setContentView: self];		
+		[myWin setContentView: self];	
+
+		[[NSNotificationCenter defaultCenter] 
+               addObserver: self 
+                	selector: @selector(fileSystemDidChange:) 
+                			name: @"GWFileSystemDidChangeNotification"
+                		object: nil];                     
+    
+		[[NSNotificationCenter defaultCenter] 
+               addObserver: self 
+                  selector: @selector(watcherNotification:) 
+                		  name: @"GWFileWatcherFileDidChangeNotification"
+                	  object: nil];
   }
   
   return self;
@@ -263,7 +301,17 @@
   leaves = [layers objectForKey: currentName];  
   for (i = 0; i < [leaves count]; i++) {
     id leaf = [leaves objectAtIndex: i];    
-    [[leaf window] close];
+    NSString *watched = [[[leaf node] path] stringByDeletingLastPathComponent];    
+
+	  if ([watchedPaths containsObject: watched]) {
+		  [watchedPaths removeObject: watched];
+      
+      if ([watchedPaths containsObject: watched] == NO) {
+        [gw removeWatcherForPath: watched];
+      }
+	  }
+    
+    [[leaf window] close];    
   }
 
   [layers removeObjectForKey: currentName];     
@@ -505,8 +553,12 @@
       if ([lfpos containsPoint: location]) {
         if (hlightleaf == nil) {        
           hlightleaf = [[FiendLeaf alloc] initWithPosX: [lfpos posx] 
-                  posY: [lfpos posy] relativeToPoint: r.origin forPath: nil 
-                                       inFiend: self ghostImage: [leaf icon]];
+                                                  posY: [lfpos posy] 
+                                       relativeToPoint: r.origin 
+                                               forPath: nil 
+                                               inFiend: self 
+                                             layerName: nil
+                                            ghostImage: [leaf icon]];
           [[hlightleaf window] display];                           
           [[hlightleaf window] orderBack: self];                                 
         } else {        
@@ -544,6 +596,16 @@
     }
       
     if (newpos == NO) {    
+      NSString *watched = [[[leaf node] path] stringByDeletingLastPathComponent];    
+
+	    if ([watchedPaths containsObject: watched]) {
+		    [watchedPaths removeObject: watched];
+        
+        if ([watchedPaths containsObject: watched] == NO) {
+          [gw removeWatcherForPath: watched];
+        }
+	    }
+    
       [[leaf window] close];
       [leaves removeObject: leaf];
     }
@@ -690,6 +752,235 @@
   }
 }
 
+- (void)removeInvalidLeaf:(FiendLeaf *)leaf
+{
+  NSString *layerName = [leaf layerName];
+  NSMutableArray *leaves = [layers objectForKey: layerName];
+  NSString *watched = [[[leaf node] path] stringByDeletingLastPathComponent];    
+
+  if ([watchedPaths containsObject: watched]) {
+    [watchedPaths removeObject: watched];
+    
+    if ([watchedPaths containsObject: watched] == NO) {
+      [gw removeWatcherForPath: watched];
+    }
+  }
+  
+  [[leaf window] close];
+  [leaves removeObject: leaf]; 
+}
+
+- (void)checkIconsAfterDotsFilesChange
+{
+  NSArray *names = [layers allKeys];
+  int i;
+
+  for (i = 0; i < [names count]; i++) {
+    NSString *lname = [names objectAtIndex: i];
+    NSMutableArray *leaves = [layers objectForKey: lname];
+    int count = [leaves count];
+    BOOL modified = NO;  
+    int j;
+
+    for (j = 0; j < count; j++) {
+      id leaf = [leaves objectAtIndex: j];
+      NSString *leafpath = [[leaf node] path];
+      
+      if ([leafpath rangeOfString: @"."].location != NSNotFound) {
+        [self removeInvalidLeaf: leaf];
+        modified = YES;   
+        count--;
+        j--;
+      }
+    }
+    
+    if (modified && ([lname isEqual: currentName])) {
+      [self orderFrontLeaves];
+      [self findFreePositions];
+    }
+  }
+}
+
+- (void)checkIconsAfterHidingOfPaths:(NSArray *)paths
+{
+  NSArray *names = [layers allKeys];
+  int i;
+
+  for (i = 0; i < [names count]; i++) {
+    NSString *lname = [names objectAtIndex: i];
+    NSMutableArray *leaves = [layers objectForKey: lname];
+    int count = [leaves count];
+    BOOL modified = NO;  
+    int j, m;
+
+    for (j = 0; j < count; j++) {
+      id leaf = [leaves objectAtIndex: j];
+      NSString *leafpath = [[leaf node] path];
+      
+      for (m = 0; m < [paths count]; m++) {
+        NSString *path = [paths objectAtIndex: m]; 
+      
+        if (isSubpathOfPath(path, leafpath) || [path isEqual: leafpath]) {
+          [self removeInvalidLeaf: leaf];          
+          modified = YES;   
+          count--;
+          j--;
+          break;
+        }
+      }
+    }
+    
+    if (modified && ([lname isEqual: currentName])) {
+      [self orderFrontLeaves];
+      [self findFreePositions];
+    }
+  }
+}
+
+- (void)fileSystemDidChange:(NSNotification *)notification
+{
+  CREATE_AUTORELEASE_POOL(arp);
+  NSDictionary *dict = [notification object];
+  NSString *operation = [dict objectForKey: @"operation"];
+  NSString *source = [dict objectForKey: @"source"];
+  NSArray *files = [dict objectForKey: @"files"];
+  
+  if ([operation isEqual: @"GWorkspaceRenameOperation"]) {
+		files = [NSArray arrayWithObject: [source lastPathComponent]];
+    source = [source stringByDeletingLastPathComponent];
+  }	
+
+  if ([operation isEqual: NSWorkspaceMoveOperation] 
+        || [operation isEqual: NSWorkspaceDestroyOperation]
+				|| [operation isEqual: @"GWorkspaceRenameOperation"]
+				|| [operation isEqual: NSWorkspaceRecycleOperation]
+				|| [operation isEqual: @"GWorkspaceRecycleOutOperation"]
+				|| [operation isEqual: @"GWorkspaceEmptyRecyclerOperation"]) {
+    NSMutableArray *paths = [NSMutableArray arrayWithCapacity: 1];
+    NSArray *names = [layers allKeys];    
+    int i;
+    
+    for (i = 0; i < [files count]; i++) {
+      NSString *s = [source stringByAppendingPathComponent: [files objectAtIndex: i]];
+      [paths addObject: s];
+    }
+
+    for (i = 0; i < [names count]; i++) {
+      NSString *lname = [names objectAtIndex: i];
+      NSMutableArray *leaves = [layers objectForKey: lname];
+      int count = [leaves count];
+      BOOL modified = NO;  
+      int j, m;
+
+      for (j = 0; j < count; j++) {
+        id leaf = [leaves objectAtIndex: j];
+        NSString *leafpath = [[leaf node] path];
+
+        for (m = 0; m < [paths count]; m++) {
+          NSString *path = [paths objectAtIndex: m]; 
+
+          if (isSubpathOfPath(path, leafpath) || [path isEqual: leafpath]) {
+            [self removeInvalidLeaf: leaf];          
+            modified = YES;   
+            count--;
+            j--;
+            break;
+          }
+        }
+      }
+      
+      if (modified && ([lname isEqual: currentName])) {
+        [self orderFrontLeaves];
+        [self findFreePositions];
+      }
+    }
+  }
+  
+  RELEASE (arp);
+}
+
+- (void)watcherNotification:(NSNotification *)notification
+{
+  CREATE_AUTORELEASE_POOL(arp);
+	NSDictionary *notifdict = (NSDictionary *)[notification object];
+  NSString *path = [notifdict objectForKey: @"path"];
+	NSString *event = [notifdict objectForKey: @"event"];
+  NSEnumerator *enumerator;
+  NSString *wpath;
+	BOOL contained = NO;
+	
+	if ([event isEqual: @"GWFileCreatedInWatchedDirectory"]) {
+    RELEASE (arp);
+		return;
+	}
+  
+  enumerator = [watchedPaths objectEnumerator];  
+            
+  while ((wpath = [enumerator nextObject])) {
+		if (([wpath isEqual: path]) || (isSubpathOfPath(path, wpath))) {
+			contained = YES;
+			break;
+		}
+  }
+
+  if (contained) {
+    NSArray *names = [layers allKeys];
+    int i;
+
+    for (i = 0; i < [names count]; i++) {
+      NSString *lname = [names objectAtIndex: i];
+      NSMutableArray *leaves = [layers objectForKey: lname];
+      int count = [leaves count];
+      BOOL modified = NO;  
+      int j;
+      
+      if ([event isEqual: @"GWWatchedPathDeleted"]) {
+        for (j = 0; j < count; j++) {
+          id leaf = [leaves objectAtIndex: j];
+          NSString *leafpath = [[leaf node] path];
+
+				  if (isSubpathOfPath(path, leafpath)) {
+					  [self removeInvalidLeaf: leaf];
+            modified = YES; 
+					  count--;
+					  j--;
+				  }
+        }
+
+      } else if ([event isEqual: @"GWFileDeletedInWatchedDirectory"]) {
+			  NSArray *files = [notifdict objectForKey: @"files"];
+      
+        for (j = 0; j < count; j++) {
+          id leaf = [leaves objectAtIndex: j];
+          NSString *leafpath = [[leaf node] path];
+          int m;
+          
+          for (m = 0; m < [files count]; m++) {
+            NSString *fname = [files objectAtIndex: m];
+            NSString *fullPath = [path stringByAppendingPathComponent: fname];
+
+            if ((isSubpathOfPath(fullPath, leafpath))
+															      || ([fullPath isEqual: leafpath])) {
+					    [self removeInvalidLeaf: leaf];
+              modified = YES; 
+					    count--;
+					    j--;
+              break;
+            }
+          }
+        }
+      }
+      
+      if (modified && ([lname isEqual: currentName])) {
+        [self orderFrontLeaves];
+        [self findFreePositions];
+      }
+    }
+  }
+  
+  RELEASE (arp);
+}
+
 - (void)updateDefaults
 {
 	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];		
@@ -735,7 +1026,7 @@
 
 @implementation Fiend (DraggingDestination)
 
-- (unsigned int)draggingEntered:(id <NSDraggingInfo>)sender
+- (NSDragOperation)draggingEntered:(id <NSDraggingInfo>)sender
 {
 	NSPasteboard *pb = [sender draggingPasteboard];
 	
@@ -754,7 +1045,7 @@
   return NSDragOperationNone;
 }
 
-- (unsigned int)draggingUpdated:(id <NSDraggingInfo>)sender
+- (NSDragOperation)draggingUpdated:(id <NSDraggingInfo>)sender
 {
   NSDragOperation sourceDragMask;
 	
@@ -792,6 +1083,7 @@
   NSPasteboard *pb;
 	NSArray *sourcePaths;
   NSString *path;
+  NSString *basepath;
   NSMutableArray *leaves;
   id leaf;
   NSRect r;
@@ -812,7 +1104,13 @@
   leaves = [layers objectForKey: currentName];  
 
   path = [sourcePaths objectAtIndex: 0];
-  
+  basepath = [path stringByDeletingLastPathComponent];
+
+  if ([basepath isEqual: [gw trashPath]]) {
+    isDragTarget = NO;
+    return;
+  }
+    
   for (i = 0; i < [leaves count]; i++) {
     leaf = [leaves objectAtIndex: i];    
     if ([[[leaf node] path] isEqual: path] == YES) {
@@ -838,12 +1136,22 @@
   }
   posy--;
                   
-  leaf = [[FiendLeaf alloc] initWithPosX: posx posY: posy
-                            relativeToPoint: r.origin forPath: path 
-                                          inFiend: self  ghostImage: nil];                          
+  leaf = [[FiendLeaf alloc] initWithPosX: posx 
+                                    posY: posy
+                         relativeToPoint: r.origin 
+                                 forPath: path 
+                                 inFiend: self 
+                               layerName: currentName 
+                              ghostImage: nil];                          
   [leaves addObject: leaf];
   RELEASE (leaf);
   
+	if ([watchedPaths containsObject: basepath] == NO) {
+    [gw addWatcherForPath: basepath];
+	}
+  
+  [watchedPaths addObject: basepath];
+  
   leaf = [leaves objectAtIndex: [leaves count] -1];
   [[leaf window] display]; 
   [self findFreePositions];  
