@@ -1,6 +1,6 @@
 /* GWSpatialViewer.m
  *  
- * Copyright (C) 2004 Free Software Foundation, Inc.
+ * Copyright (C) 2004-2010 Free Software Foundation, Inc.
  *
  * Author: Enrico Sersale <enrico@imago.ro>
  * Date: June 2004
@@ -19,28 +19,30 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
  */
 
-#include <AppKit/AppKit.h>
 #include <math.h>
-#include "GWSpatialViewer.h"
-#include "GWViewersManager.h"
-#include "GWViewerWindow.h"
-#include "GWViewerScrollView.h"
-#include "GWViewerBrowser.h"
-#include "GWViewerIconsView.h"
-#include "GWViewerListView.h"
-#include "GWViewerPathsPopUp.h"
-#include "GWorkspace.h"
-#include "GWFunctions.h"
-#include "FSNodeRep.h"
-#include "FSNIcon.h"
-#include "FSNFunctions.h"
+
+#import <AppKit/AppKit.h>
+#import "GWSpatialViewer.h"
+#import "GWViewersManager.h"
+#import "GWViewerWindow.h"
+#import "GWViewerScrollView.h"
+#import "GWViewerBrowser.h"
+#import "GWViewerIconsView.h"
+#import "GWViewerListView.h"
+#import "GWViewerPathsPopUp.h"
+#import "GWorkspace.h"
+#import "GWFunctions.h"
+#import "FSNodeRep.h"
+#import "FSNIcon.h"
+#import "FSNFunctions.h"
  
 #define DEFAULT_INCR 150
 #define MIN_W_HEIGHT 180
 
+
 @implementation GWSpatialViewer
 
 - (void)dealloc
@@ -48,31 +50,34 @@
   [nc removeObserver: self];
   
   RELEASE (baseNode);
-  TEST_RELEASE (lastSelection);
+  RELEASE (baseNodeArray);
+  RELEASE (lastSelection);
+  RELEASE (rootViewerKey);
   RELEASE (watchedNodes);
-  RELEASE (watchedSuspended);
   RELEASE (vwrwin);
   RELEASE (viewType);
+  RELEASE (viewerPrefs);
   
-	[super dealloc];
+  [super dealloc];
 }
 
 - (id)initForNode:(FSNode *)node
          inWindow:(GWViewerWindow *)win
+         showType:(NSString *)stype
     showSelection:(BOOL)showsel
 {
   self = [super init];
   
   if (self) {
-    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];	      
-    NSString *prefsname = [NSString stringWithFormat: @"viewer_at_%@", [node path]];
-    NSDictionary *viewerPrefs = nil;
+    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];	
+    NSString *prefsname;
     id defEntry;
 
     ASSIGN (baseNode, [FSNode nodeWithPath: [node path]]);
+    ASSIGN (baseNodeArray, [NSArray arrayWithObject: baseNode]);
+    fsnodeRep = [FSNodeRep sharedInstance];
     lastSelection = nil;
     watchedNodes = [NSMutableArray new];
-    watchedSuspended = [NSMutableArray new];
     manager = [GWViewersManager viewersManager];
     gworkspace = [GWorkspace gworkspace];
     nc = [NSNotificationCenter defaultCenter];
@@ -87,7 +92,25 @@
     rootviewer = ([[baseNode path] isEqual: path_separator()]
                 && ([[manager viewersForBaseNode: baseNode] count] == 0));
 
-    if ([baseNode isWritable] && (rootviewer == NO)) {
+    if ((rootviewer == NO) && [[baseNode path] isEqual: path_separator()]) {
+      rootViewerKey = [manager nextRootViewerKey];
+      
+      if (rootViewerKey == nil) {
+        ASSIGN (rootViewerKey, [NSNumber numberWithUnsignedLong: (unsigned long)self]);
+      } else {
+        RETAIN (rootViewerKey);
+      }
+      
+      prefsname = [NSString stringWithFormat: @"viewer_at_%@_%i", 
+                            [node path], [rootViewerKey unsignedLongValue]];
+
+    } else {
+      rootViewerKey = nil;
+      prefsname = [NSString stringWithFormat: @"viewer_at_%@", [node path]];
+    }
+
+    if ([baseNode isWritable] && (rootviewer == NO) && (rootViewerKey == nil) 
+                 && ([[fsnodeRep volumes] containsObject: [node path]] == NO)) {
 		  NSString *dictPath = [[baseNode path] stringByAppendingPathComponent: @".gwdir"];
 
       if ([[NSFileManager defaultManager] fileExistsAtPath: dictPath]) {
@@ -142,7 +165,11 @@
     if (rootviewer) {
       [vwrwin setTitle: NSLocalizedString(@"File Viewer", @"")];
     } else {
-      [vwrwin setTitle: [NSString stringWithFormat: @"%@ - %@", [node name], [node parentPath]]];   
+      if (rootViewerKey == nil) {   
+        [vwrwin setTitle: [NSString stringWithFormat: @"%@ - %@", [node name], [node parentPath]]];   
+      } else {
+        [vwrwin setTitle: [NSString stringWithFormat: @"%@", [node name]]];   
+      }
     }
 
     [self createSubviews];
@@ -151,7 +178,7 @@
       nodeView = [[GWViewerIconsView alloc] initForViewer: self];
 
     } else if ([viewType isEqual: @"List"]) { 
-      NSRect r = [[scroll contentView] frame];
+      NSRect r = [[scroll contentView] bounds];
       
       nodeView = [[GWViewerListView alloc] initWithFrame: r forViewer: self];
 
@@ -202,13 +229,14 @@
     [self updeateInfoLabels];
 
     [self scrollToBeginning];
-    
-    RELEASE (viewerPrefs);
-    
+        
     [nc addObserver: self 
            selector: @selector(columnsWidthChanged:) 
                name: @"GWBrowserColumnWidthChangedNotification"
              object: nil];
+    
+    invalidated = NO;
+    closing = NO;    
   }
   
   return self;
@@ -216,7 +244,7 @@
 
 - (void)createSubviews
 {
-  NSRect r = [[vwrwin contentView] frame];
+  NSRect r = [[vwrwin contentView] bounds];
   float w = r.size.width;
 	float h = r.size.height; 
   int boxh = 32;  
@@ -317,6 +345,7 @@
 {
   if ([nodeView isShowingNode: anode]) {
     [nodeView reloadFromNode: anode];
+    [self updeateInfoLabels];
   }
 }
 
@@ -354,6 +383,11 @@
   return rootviewer;
 }
 
+- (NSNumber *)rootViewerKey
+{
+  return rootViewerKey;
+}
+
 - (BOOL)isSpatial
 {
   return YES;
@@ -364,7 +398,6 @@
   return SPATIAL;
 }
 
-
 - (void)activate
 {
   [vwrwin makeKeyAndOrderFront: nil];
@@ -393,6 +426,10 @@
   return invalidated;
 }
 
+- (BOOL)isClosing
+{
+  return closing;
+}
 
 - (void)setOpened:(BOOL)opened 
         repOfNode:(FSNode *)anode
@@ -410,6 +447,7 @@
 
 - (void)unselectAllReps
 {
+  [nodeView stopRepNameEditing];
   [nodeView unselectOtherReps: nil];
   [nodeView selectionDidChange];
 }
@@ -418,8 +456,11 @@
 {
   FSNode *node;
   NSArray *components;
-  int i, j, count;
 
+  if (closing) {
+    return;
+  }
+  
   [manager selectionChanged: newsel];
 
   if (lastSelection && [newsel isEqual: lastSelection]) {
@@ -429,7 +470,7 @@
   ASSIGN (lastSelection, newsel);
   [self updeateInfoLabels]; 
     
-  node = [FSNode nodeWithPath: [newsel objectAtIndex: 0]];   
+  node = [newsel objectAtIndex: 0];   
     
   if ([nodeView isSingleNode]) {    
     if ([node isEqual: baseNode] == NO) {
@@ -444,34 +485,43 @@
   } 
     
   components = [FSNode nodeComponentsFromNode: baseNode toNode: node];
-  count = [components count];
 
-  for (i = 0; i < [watchedNodes count]; i++) {
-    FSNode *n1 = [watchedNodes objectAtIndex: i];
-    FSNode *n2 = nil;
+  if ([components isEqual: watchedNodes] == NO) {
+    unsigned count = [components count];
+    unsigned pos = 0;
+    unsigned i;
+  
+    for (i = 0; i < [watchedNodes count]; i++) { 
+      FSNode *nd = [watchedNodes objectAtIndex: i];
+      
+      if (i < count) {
+        FSNode *ndcomp = [components objectAtIndex: i];
 
-    if (count > i) {
-      n2 = [components objectAtIndex: i];  
-    } else {
-      i = count;
-      break;
+        if ([nd isEqual: ndcomp] == NO) {
+          [gworkspace removeWatcherForPath: [nd path]];
+        } else {
+          pos = i + 1;
+        }
+
+      } else {
+        [gworkspace removeWatcherForPath: [nd path]];
+      }
     }
 
-    if ([n1 isEqual: n2] == NO) {
-      break;
-    }    
-  }
+    for (i = pos; i < count; i++) {   
+      [gworkspace addWatcherForPath: [[components objectAtIndex: i] path]];
+    }
 
-  for (j = i; j < [watchedNodes count]; j++) {  
-    [gworkspace removeWatcherForPath: [[watchedNodes objectAtIndex: j] path]];
-  }
+    [watchedNodes removeAllObjects];
+    [watchedNodes addObjectsFromArray: components];
+  }  
+}
 
-  for (j = i; j < [components count]; j++) { 
-    [gworkspace addWatcherForPath: [[components objectAtIndex: j] path]];
+- (void)multipleNodeViewDidSelectSubNode:(FSNode *)node
+{
+  if ([node isDirectory] && ([node isPackage] == NO)) {
+    [nodeView setLastShownNode: node];
   }
-  
-  [watchedNodes removeAllObjects];
-  [watchedNodes addObjectsFromArray: components];
 }
 
 - (void)setSelectableNodesRange:(NSRange)range
@@ -498,9 +548,22 @@
 	if (freefs == nil) {  
 		labelstr = NSLocalizedString(@"unknown volume size", @"");    
 	} else {
+    unsigned long long freeSize = [freefs unsignedLongLongValue];
+    unsigned systemType = [fsnodeRep systemType];
+    
+    switch (systemType) {
+
+      case NSMACHOperatingSystem:
+        freeSize = (freeSize >> 8);
+        break;
+        
+      default:
+        break;
+    }
+    
 		labelstr = [NSString stringWithFormat: @"%@ %@", 
-                   sizeDescription([freefs unsignedLongLongValue]),
-                                        NSLocalizedString(@"free", @"")];
+                                      sizeDescription(freeSize),
+                                            NSLocalizedString(@"free", @"")];
 	}
 
   [spaceLabel setStringValue: labelstr];
@@ -513,15 +576,18 @@
   if ([path isEqual: [baseNode path]] == NO) {
     FSNode *node = [FSNode nodeWithPath: path];
     BOOL close = [sender closeViewer];
-  
+    id vwr = nil;
+    
     if (close) {
       [pathsPopUp setTarget: nil];
+      vwr = self;
     }
   
     [manager newViewerOfType: SPATIAL
+                    showType: nil
                      forNode: node 
                showSelection: NO
-              closeOldViewer: (close ? self : nil)
+              closeOldViewer: vwr
                     forceNew: NO];
   } else {
     [nodeView showContentsOfNode: baseNode]; 
@@ -539,149 +605,12 @@
 
 - (void)nodeContentsWillChange:(NSDictionary *)info
 {
-  NSString *operation = [info objectForKey: @"operation"];
-  NSString *source = [info objectForKey: @"source"];
-  NSString *destination = [info objectForKey: @"destination"];
-  NSArray *files = [info objectForKey: @"files"];
-  
-  if ([operation isEqual: @"GWorkspaceRenameOperation"]) {
-    files = [NSArray arrayWithObject: [destination lastPathComponent]];
-    destination = [destination stringByDeletingLastPathComponent]; 
-  }
-
-  if ([operation isEqual: @"NSWorkspaceMoveOperation"] 
-        || [operation isEqual: @"NSWorkspaceCopyOperation"]
-        || [operation isEqual: @"NSWorkspaceLinkOperation"]
-        || [operation isEqual: @"NSWorkspaceDuplicateOperation"]
-        || [operation isEqual: @"GWorkspaceCreateDirOperation"]
-        || [operation isEqual: @"GWorkspaceCreateFileOperation"]
-        || [operation isEqual: @"GWorkspaceRenameOperation"]
-			  || [operation isEqual: @"GWorkspaceRecycleOutOperation"]) { 
-    if ([nodeView isShowingPath: destination]
-                    || [baseNode isSubnodeOfPath: destination]) {
-      [self suspendWatchersFromPath: destination];
-    }
-  }
-
-  if ([operation isEqual: @"NSWorkspaceMoveOperation"]
-        || [operation isEqual: @"NSWorkspaceDestroyOperation"]
-				|| [operation isEqual: @"NSWorkspaceRecycleOperation"]
-				|| [operation isEqual: @"GWorkspaceRecycleOutOperation"]
-				|| [operation isEqual: @"GWorkspaceEmptyRecyclerOperation"]) {
-    if ([nodeView isShowingPath: source]
-                      || [baseNode isSubnodeOfPath: source]) {
-      [self suspendWatchersFromPath: source]; 
-    }
-  }
-
   [nodeView nodeContentsWillChange: info];
 }
-
+ 
 - (void)nodeContentsDidChange:(NSDictionary *)info
 {
-  NSString *operation = [info objectForKey: @"operation"];
-  NSString *source = [info objectForKey: @"source"];
-  NSString *destination = [info objectForKey: @"destination"];
-  NSArray *files = [info objectForKey: @"files"];
-
   [nodeView nodeContentsDidChange: info];
-    
-  if ([operation isEqual: @"GWorkspaceRenameOperation"]) {
-    files = [NSArray arrayWithObject: [destination lastPathComponent]];
-    destination = [destination stringByDeletingLastPathComponent]; 
-  }
-
-  if ([operation isEqual: @"NSWorkspaceRecycleOperation"]) {
-		files = [info objectForKey: @"origfiles"];
-  }	
-
-  if ([operation isEqual: @"NSWorkspaceMoveOperation"] 
-        || [operation isEqual: @"NSWorkspaceCopyOperation"]
-        || [operation isEqual: @"NSWorkspaceLinkOperation"]
-        || [operation isEqual: @"NSWorkspaceDuplicateOperation"]
-        || [operation isEqual: @"GWorkspaceCreateDirOperation"]
-        || [operation isEqual: @"GWorkspaceCreateFileOperation"]
-        || [operation isEqual: @"GWorkspaceRenameOperation"]
-			  || [operation isEqual: @"GWorkspaceRecycleOutOperation"]) { 
-    if ([nodeView isShowingPath: destination]
-                        || [baseNode isSubnodeOfPath: destination]) {
-      [self reactivateWatchersFromPath: destination];
-    } 
-    
-    [self clearSuspendedWatchersFromPath: destination];
-  }
-
-  if ([operation isEqual: @"NSWorkspaceMoveOperation"]
-        || [operation isEqual: @"NSWorkspaceDestroyOperation"]
-				|| [operation isEqual: @"NSWorkspaceRecycleOperation"]
-				|| [operation isEqual: @"GWorkspaceRecycleOutOperation"]
-				|| [operation isEqual: @"GWorkspaceEmptyRecyclerOperation"]) {
-    if ([nodeView isShowingPath: source]
-                        || [baseNode isSubnodeOfPath: source]) {
-      [self reactivateWatchersFromPath: source];
-    } 
-    
-    [self clearSuspendedWatchersFromPath: source];
-  }
-}
-
-- (void)suspendWatchersFromPath:(NSString *)path
-{
-  NSString *start = [baseNode isSubnodeOfPath: path] ? [baseNode path] : path;
-  unsigned index = [FSNode indexOfNodeWithPath: start 
-                                  inComponents: watchedNodes];
-
-  if (index != NSNotFound) {
-    int i;
-
-    for (i = index; i < [watchedNodes count]; i++) {  
-      NSString *path = [[watchedNodes objectAtIndex: i] path];
-        
-      [gworkspace removeWatcherForPath: path];
-      [watchedSuspended addObject: path];
-    }
-  } 
-}
-
-- (void)reactivateWatchersFromPath:(NSString *)path
-{
-  NSString *start = [baseNode isSubnodeOfPath: path] ? [baseNode path] : path;
-  unsigned index = [FSNode indexOfNodeWithPath: start 
-                                  inComponents: watchedNodes];
-
-  if (index != NSNotFound) {
-    int count = [watchedNodes count];
-    int i;
-    
-    for (i = index; i < count; i++) {
-      FSNode *node = [watchedNodes objectAtIndex: i];
-      NSString *path = [node path];
-    
-      if ([node isValid] && [node isDirectory]) {
-        [gworkspace addWatcherForPath: path];
-      } else {
-        [watchedNodes removeObjectAtIndex: i];
-        count--;
-        i--;
-      }
-    }
-  }
-}
-
-- (void)clearSuspendedWatchersFromPath:(NSString *)path
-{
-  int count = [watchedSuspended count];
-  int i;
-
-  for (i = 0; i < count; i++) {
-    NSString *suspended = [watchedSuspended objectAtIndex: i];
-
-    if ([suspended isEqual: path] || isSubpathOfPath(path, suspended)) {
-      [watchedSuspended removeObjectAtIndex: i];
-      count--;
-      i--;
-    }
-  }
 }
 
 - (void)watchedPathChanged:(NSDictionary *)info
@@ -689,10 +618,9 @@
   if (invalidated == NO) {
     NSString *path = [info objectForKey: @"path"];
   
-    if ([watchedSuspended containsObject: path] == NO) {
-      if ([nodeView isShowingPath: path]) {
-        [nodeView watchedPathChanged: info];
-      }
+    if ([nodeView isShowingPath: path]) {
+      [nodeView watchedPathChanged: info];
+      [self updeateInfoLabels];
     }
   }
 }
@@ -702,6 +630,16 @@
   return watchedNodes;
 }
 
+- (void)hideDotsFileChanged:(BOOL)hide
+{
+  [self reloadFromNode: baseNode];
+}
+
+- (void)hiddenFilesChanged:(NSArray *)paths
+{
+  [self reloadFromNode: baseNode];
+}
+
 - (void)columnsWidthChanged:(NSNotification *)notification
 {
   NSRect r = [vwrwin frame];
@@ -725,33 +663,11 @@
 - (void)updateDefaults
 {
   if ([baseNode isValid]) {
-    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];	      
-    NSString *prefsname = [NSString stringWithFormat: @"viewer_at_%@", [baseNode path]];
-    NSString *dictPath = [[baseNode path] stringByAppendingPathComponent: @".gwdir"];
-    NSMutableDictionary *updatedprefs = nil;
+    NSMutableDictionary *updatedprefs = [nodeView updateNodeInfo: NO];
     id defEntry;
     
-    [nodeView updateNodeInfo];
-    
-    if ([baseNode isWritable] && (rootviewer == NO)) {
-      if ([[NSFileManager defaultManager] fileExistsAtPath: dictPath]) {
-        NSDictionary *dict = [NSDictionary dictionaryWithContentsOfFile: dictPath];
-
-        if (dict) {
-          updatedprefs = [dict mutableCopy];
-        }   
-      }
-  
-    } else { 
-      NSDictionary *prefs = [defaults dictionaryForKey: prefsname];
-  
-      if (prefs) {
-        updatedprefs = [prefs mutableCopy];
-      }
-    }
-
     if (updatedprefs == nil) {
-      updatedprefs = [NSMutableDictionary new];
+      updatedprefs = [NSMutableDictionary dictionary];
     }
 
     [updatedprefs setObject: [NSNumber numberWithBool: YES]
@@ -762,6 +678,16 @@
 
     [updatedprefs setObject: viewType forKey: @"viewtype"];
 
+    defEntry = [viewerPrefs objectForKey: @"shelfheight"];
+    if (defEntry) {
+      [updatedprefs setObject: defEntry forKey: @"shelfheight"];
+    }
+
+    defEntry = [viewerPrefs objectForKey: @"shelfdicts"];
+    if (defEntry) {
+      [updatedprefs setObject: defEntry forKey: @"shelfdicts"];
+    }
+
     defEntry = [nodeView selectedPaths];
     if (defEntry) {
       [updatedprefs setObject: defEntry forKey: @"lastselection"];
@@ -770,13 +696,28 @@
     [updatedprefs setObject: [vwrwin stringWithSavedFrame] 
                      forKey: @"geometry"];
 
-    if ([baseNode isWritable] && (rootviewer == NO)) {
+    [baseNode checkWritable];
+
+    if ([baseNode isWritable] && (rootviewer == NO) && (rootViewerKey == nil)
+            && ([[fsnodeRep volumes] containsObject: [baseNode path]] == NO)) {
+      NSString *dictPath = [[baseNode path] stringByAppendingPathComponent: @".gwdir"];
+            
       [updatedprefs writeToFile: dictPath atomically: YES];
     } else {
+      NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];	
+      NSString *prefsname;
+    
+      if (rootViewerKey != nil) {
+        prefsname = [NSString stringWithFormat: @"viewer_at_%@_%i", 
+                            [baseNode path], [rootViewerKey unsignedLongValue]];
+      } else {
+        prefsname = [NSString stringWithFormat: @"viewer_at_%@", [baseNode path]];
+      }    
+    
       [defaults setObject: updatedprefs forKey: prefsname];
     }
-  
-    RELEASE (updatedprefs);
+    
+    ASSIGN (viewerPrefs, [updatedprefs makeImmutableCopyOnFail: NO]);
   }
 }
 
@@ -790,7 +731,7 @@
 
 - (void)windowDidBecomeKey:(NSNotification *)aNotification
 {
-  NSArray *selection = [nodeView selectedPaths];  
+  NSArray *selection = [nodeView selectedNodes];  
   int count = [selection count];
   
   [vwrwin makeFirstResponder: nodeView]; 
@@ -798,12 +739,12 @@
   [manager selectedSpatialViewerChanged: self];
 
   if (count == 0) {
-    selection = [NSArray arrayWithObject: [[nodeView shownNode] path]];
+    selection = [NSArray arrayWithObject: [nodeView shownNode]];
     [manager synchronizeSelectionInParentOfViewer: self];
   
   } else if (count == 1) {
     if (([nodeView isSingleNode] == NO)
-             || ([[selection objectAtIndex: 0] isEqual: [baseNode path]])) {
+             || ([[selection objectAtIndex: 0] isEqual: baseNode])) {
       [manager synchronizeSelectionInParentOfViewer: self];
     }
     
@@ -821,17 +762,28 @@
 
 - (BOOL)windowShouldClose:(id)sender
 {
+  [manager updateDesktop];
 	return YES;
 }
 
 - (void)windowWillClose:(NSNotification *)aNotification
 {
   if (invalidated == NO) {
+    closing = YES;
     [self updateDefaults];
+    [vwrwin setDelegate: nil];
     [manager viewerWillClose: self]; 
   }
 }
 
+- (void)windowWillMiniaturize:(NSNotification *)aNotification
+{
+  NSImage *image = [fsnodeRep iconOfSize: 48 forNode: baseNode];
+
+  [vwrwin setMiniwindowImage: image];
+  [vwrwin setMiniwindowTitle: [baseNode name]];
+}
+
 - (void)windowDidResize:(NSNotification *)aNotification
 {
   if (nodeView) {
@@ -845,57 +797,125 @@
 
 - (void)openSelectionInNewViewer:(BOOL)newv
 {
-  [manager openSelectionInViewer: self closeSender: newv];
+  if ([[baseNode path] isEqual: [gworkspace trashPath]] == NO) {
+    [manager openSelectionInViewer: self closeSender: newv];
+  } else {
+    NSRunAlertPanel(nil, 
+                  NSLocalizedString(@"You can't open a document that is in the Recycler!", @""),
+					        NSLocalizedString(@"OK", @""), 
+                  nil, 
+                  nil);  
+  }
 }
 
 - (void)openSelectionAsFolder
 {
-  [manager openAsFolderSelectionInViewer: self];
+  if ([[baseNode path] isEqual: [gworkspace trashPath]] == NO) {
+    [manager openAsFolderSelectionInViewer: self];
+  } else {
+    NSRunAlertPanel(nil, 
+                  NSLocalizedString(@"You can't do this in the Recycler!", @""),
+					        NSLocalizedString(@"OK", @""), 
+                  nil, 
+                  nil);  
+  }
+}
+
+- (void)openSelectionWith
+{
+  if ([[baseNode path] isEqual: [gworkspace trashPath]] == NO) {
+    [manager openWithSelectionInViewer: self];
+  } else {
+    NSRunAlertPanel(nil, 
+                  NSLocalizedString(@"You can't do this in the Recycler!", @""),
+					        NSLocalizedString(@"OK", @""), 
+                  nil, 
+                  nil);  
+  }
 }
 
 - (void)newFolder
 {
-  [gworkspace newObjectAtPath: [[nodeView shownNode] path] 
-                  isDirectory: YES];
+  if ([[baseNode path] isEqual: [gworkspace trashPath]] == NO) {
+    [gworkspace newObjectAtPath: [[nodeView shownNode] path] 
+                    isDirectory: YES];
+  } else {
+    NSRunAlertPanel(nil, 
+                  NSLocalizedString(@"You can't create a new folder in the Recycler!", @""),
+					        NSLocalizedString(@"OK", @""), 
+                  nil, 
+                  nil);  
+  }
 }
 
 - (void)newFile
 {
-  [gworkspace newObjectAtPath: [[nodeView shownNode] path] 
-                  isDirectory: NO];
+  if ([[baseNode path] isEqual: [gworkspace trashPath]] == NO) {
+    [gworkspace newObjectAtPath: [[nodeView shownNode] path] 
+                    isDirectory: NO];
+  } else {
+    NSRunAlertPanel(nil, 
+                  NSLocalizedString(@"You can't create a new file in the Recycler!", @""),
+					        NSLocalizedString(@"OK", @""), 
+                  nil, 
+                  nil);  
+  }
 }
 
 - (void)duplicateFiles
 {
-  NSArray *selection = [nodeView selectedNodes];
+  if ([[baseNode path] isEqual: [gworkspace trashPath]] == NO) {
+    NSArray *selection = [nodeView selectedNodes];
 
-  if (selection && [selection count]) {
-    if ([nodeView isSingleNode]) {
-      [gworkspace duplicateFiles];
-    } else if ([selection isEqual: [NSArray arrayWithObject: baseNode]] == NO) {
-      [gworkspace duplicateFiles];
+    if (selection && [selection count]) {
+      if ([nodeView isSingleNode]) {
+        [gworkspace duplicateFiles];
+      } else if ([selection isEqual: baseNodeArray] == NO) {
+        [gworkspace duplicateFiles];
+      }
+    }
+  } else {
+    NSRunAlertPanel(nil, 
+                  NSLocalizedString(@"You can't duplicate files in the Recycler!", @""),
+					        NSLocalizedString(@"OK", @""), 
+                  nil, 
+                  nil);  
+  }
+}
+
+- (void)recycleFiles
+{
+  if ([[baseNode path] isEqual: [gworkspace trashPath]] == NO) {
+    NSArray *selection = [nodeView selectedNodes];
+
+    if (selection && [selection count]) {
+      if ([nodeView isSingleNode]) {
+        [gworkspace moveToTrash];
+      } else if ([selection isEqual: baseNodeArray] == NO) {
+        [gworkspace moveToTrash];
+      }
     }
   }
 }
 
+- (void)emptyTrash
+{
+  [gworkspace emptyRecycler: nil];
+}
+
 - (void)deleteFiles
 {
   NSArray *selection = [nodeView selectedNodes];
 
   if (selection && [selection count]) {
     if ([nodeView isSingleNode]) {
-      [gworkspace moveToTrash];
-    } else if ([selection isEqual: [NSArray arrayWithObject: baseNode]] == NO) {
-      [gworkspace moveToTrash];
+      [gworkspace deleteFiles];
+    } else if ([selection isEqual: baseNodeArray] == NO) {
+      [gworkspace deleteFiles];
     }
   }
 }
 
-- (void)emptyTrash
-{
-  [gworkspace emptyRecycler: nil];
-}
-
 - (void)goBackwardInHistory
 {
   [manager goBackwardInHistoryOfViewer: self];
@@ -916,14 +936,27 @@
   NSString *title = [sender title];
   
 	if ([title isEqual: NSLocalizedString(viewType, @"")] == NO) {
-    NSArray *selection = [nodeView selectedPaths];
+    NSArray *selection = [nodeView selectedNodes];
+    NSArray *reps = [nodeView reps];
+    NSMutableArray *opennodes = [NSMutableArray array];
     int i;
     
-    [nodeView updateNodeInfo];
+    [nodeView updateNodeInfo: YES];
     if ([nodeView isSingleNode] && ([selection count] == 0)) {
-      selection = [NSArray arrayWithObject: [[nodeView shownNode] path]];
+      selection = [NSArray arrayWithObject: [nodeView shownNode]];
     }    
     RETAIN (selection);
+    
+    for (i = 0; i < [reps count]; i++) {
+      id rep = [reps objectAtIndex: i];
+
+      if ([rep isOpened]) {
+        [opennodes addObject: [rep node]];
+      }
+    }
+
+    RETAIN (opennodes);
+    
     [scroll setDocumentView: nil];	
     
     if ([title isEqual: NSLocalizedString(@"Browser", @"")]) {
@@ -944,7 +977,7 @@
       ASSIGN (viewType, @"Icon");
       
     } else if ([title isEqual: NSLocalizedString(@"List", @"")]) {
-      NSRect r = [[scroll contentView] frame];
+      NSRect r = [[scroll contentView] bounds];
 
       nodeView = [[GWViewerListView alloc] initWithFrame: r forViewer: self];
       [scroll setHasVerticalScroller: YES];
@@ -954,9 +987,19 @@
     [scroll setDocumentView: nodeView];	
     RELEASE (nodeView);                 
     [nodeView showContentsOfNode: baseNode]; 
+        
+    for (i = 0; i < [opennodes count]; i++) {
+      id rep = [nodeView repOfSubnode: [opennodes objectAtIndex: i]];
+
+      if (rep) {
+        [rep setOpened: YES];
+      }
+    }
+    
+    RELEASE (opennodes);
     
     if ([selection count]) {
-      [nodeView selectRepsOfPaths: selection];
+      [nodeView selectRepsOfSubnodes: selection];
     }
     
     DESTROY (selection);
@@ -971,10 +1014,10 @@
     [watchedNodes removeAllObjects];
     
     DESTROY (lastSelection);
-    selection = [nodeView selectedPaths];
+    selection = [nodeView selectedNodes];
     
     if ([selection count] == 0) {
-      selection = [NSArray arrayWithObject: [[nodeView shownNode] path]];
+      selection = [NSArray arrayWithObject: [nodeView shownNode]];
     }
     
     [self selectionChanged: selection];
@@ -990,7 +1033,7 @@
 
   if ([title isEqual: NSLocalizedString(@"Name", @"")]) {
     type = FSNInfoNameType;
-  } else if ([title isEqual: NSLocalizedString(@"Kind", @"")]) {
+  } else if ([title isEqual: NSLocalizedString(@"Type", @"")]) {
     type = FSNInfoKindType;
   } else if ([title isEqual: NSLocalizedString(@"Size", @"")]) {
     type = FSNInfoSizeType;
@@ -1004,12 +1047,14 @@
 
   [(id <FSNodeRepContainer>)nodeView setShowType: type];  
   [self scrollToBeginning];
+  [nodeView updateNodeInfo: YES];
 }
 
 - (void)setExtendedShownType:(id)sender
 {
   [(id <FSNodeRepContainer>)nodeView setExtendedShowType: [sender title]]; 
   [self scrollToBeginning]; 
+  [nodeView updateNodeInfo: YES];
 }
 
 - (void)setIconsSize:(id)sender
@@ -1017,6 +1062,7 @@
   if ([nodeView respondsToSelector: @selector(setIconSize:)]) {
     [(id <FSNodeRepContainer>)nodeView setIconSize: [[sender title] intValue]];
     [self scrollToBeginning];
+    [nodeView updateNodeInfo: YES];
   }
 }
 
@@ -1032,6 +1078,7 @@
     }
     
     [self scrollToBeginning];
+    [nodeView updateNodeInfo: YES];
   }
 }
 
@@ -1040,6 +1087,7 @@
   if ([nodeView respondsToSelector: @selector(setLabelTextSize:)]) {
     [nodeView setLabelTextSize: [[sender title] intValue]];
     [self scrollToBeginning];
+    [nodeView updateNodeInfo: YES];
   }
 }
 
@@ -1094,36 +1142,81 @@
 
 - (BOOL)validateItem:(id)menuItem
 {
-  NSString *itemTitle = [menuItem title];
-  NSString *menuTitle = [[menuItem menu] title];
+  if ([NSApp keyWindow] == vwrwin) {
+    SEL action = [menuItem action];
+    NSString *itemTitle = [menuItem title];
+    NSString *menuTitle = [[menuItem menu] title];
 
-  if ([menuTitle isEqual: NSLocalizedString(@"Icon Size", @"")]) {
-    return [nodeView respondsToSelector: @selector(setIconSize:)];
-  } else if ([menuTitle isEqual: NSLocalizedString(@"Icon Position", @"")]) {
-    return [nodeView respondsToSelector: @selector(setIconPosition:)];
-  } else if ([menuTitle isEqual: NSLocalizedString(@"Label Size", @"")]) {
-    return [nodeView respondsToSelector: @selector(setLabelTextSize:)];
-  } else if ([itemTitle isEqual: NSLocalizedString(@"Label Color...", @"")]) {
-    return [nodeView respondsToSelector: @selector(setTextColor:)];
-  } else if ([itemTitle isEqual: NSLocalizedString(@"Background Color...", @"")]) {
-    return [nodeView respondsToSelector: @selector(setBackgroundColor:)];
-  
-  } else if ([itemTitle isEqual: NSLocalizedString(@"Duplicate", @"")]
-       || [itemTitle isEqual: NSLocalizedString(@"Move to Recycler", @"")]) {
-    NSArray *selection = [nodeView selectedNodes];
+    if ([menuTitle isEqual: NSLocalizedString(@"Icon Size", @"")]) {
+      return [nodeView respondsToSelector: @selector(setIconSize:)];
+    } else if ([menuTitle isEqual: NSLocalizedString(@"Icon Position", @"")]) {
+      return [nodeView respondsToSelector: @selector(setIconPosition:)];
+    } else if ([menuTitle isEqual: NSLocalizedString(@"Label Size", @"")]) {
+      return [nodeView respondsToSelector: @selector(setLabelTextSize:)];
+    } else if ([itemTitle isEqual: NSLocalizedString(@"Label Color...", @"")]) {
+      return [nodeView respondsToSelector: @selector(setTextColor:)];
+    } else if ([itemTitle isEqual: NSLocalizedString(@"Background Color...", @"")]) {
+      return [nodeView respondsToSelector: @selector(setBackgroundColor:)];
 
-    if (selection && [selection count]) {
-      if ([nodeView isSingleNode]) {
-        return YES;
-      } else if ([selection isEqual: [NSArray arrayWithObject: baseNode]] == NO) {
-        return YES;
+    } else if (sel_eq(action, @selector(duplicateFiles:))
+                    || sel_eq(action, @selector(recycleFiles:))
+                        || sel_eq(action, @selector(deleteFiles:))) {
+      if (lastSelection && [lastSelection count]
+              && ([lastSelection isEqual: baseNodeArray] == NO)) {
+        return ([[baseNode path] isEqual: [gworkspace trashPath]] == NO);
       }
-    }
-    
-    return NO;
-  } 
 
-  return YES;
+      return NO;
+
+    } else if (sel_eq(action, @selector(openSelection:))) {
+      if ([[baseNode path] isEqual: [gworkspace trashPath]] == NO) {
+        return (lastSelection && [lastSelection count] 
+                && ([lastSelection isEqual: baseNodeArray] == NO));
+      } 
+
+      return NO;
+
+    } else if (sel_eq(action, @selector(openSelectionAsFolder:))) {
+      if (lastSelection && ([lastSelection count] == 1)) {  
+        return [[lastSelection objectAtIndex: 0] isPackage];
+      }
+
+      return NO;
+
+    } else if (sel_eq(action, @selector(openWith:))) {
+      BOOL canopen = YES;
+      int i;
+
+      if (lastSelection && [lastSelection count]
+            && ([lastSelection isEqual: baseNodeArray] == NO)) {
+        for (i = 0; i < [lastSelection count]; i++) {
+          FSNode *node = [lastSelection objectAtIndex: i];
+
+          if (([node isPlain] == NO) 
+                && (([node isPackage] == NO) || [node isApplication])) {
+            canopen = NO;
+            break;
+          }
+        }
+      } else {
+        canopen = NO;
+      }
+
+      return canopen;
+
+    } else if (sel_eq(action, @selector(newFolder:))
+                                || sel_eq(action, @selector(newFile:))) {
+      if ([[baseNode path] isEqual: [gworkspace trashPath]] == NO) {
+        return [[nodeView shownNode] isWritable];
+      }
+
+      return NO;
+    } 
+    
+    return YES;
+  }
+  
+  return NO;
 }
 
 @end
@@ -1140,4 +1233,3 @@
 
 
 
-
