@@ -1,6 +1,6 @@
 /* GWViewerIconsPath.m
  *  
- * Copyright (C) 2004 Free Software Foundation, Inc.
+ * Copyright (C) 2004-2010 Free Software Foundation, Inc.
  *
  * Author: Enrico Sersale <enrico@imago.ro>
  * Date: July 2004
@@ -19,15 +19,18 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
  */
 
 #include <AppKit/AppKit.h>
 #include <math.h>
+#include <sys/types.h>
+#include <unistd.h>
 #include "FSNIcon.h"
 #include "FSNFunctions.h"
 #include "GWViewerIconsPath.h"
 #include "GWViewer.h"
+#include "GWorkspace.h"
 
 #define DEF_ICN_SIZE 48
 #define DEF_TEXT_SIZE 12
@@ -44,7 +47,7 @@
 - (void)dealloc
 {
   RELEASE (icons);
-  TEST_RELEASE (extInfoType);
+  RELEASE (extInfoType);
   RELEASE (labelFont);
   RELEASE (backColor);
   RELEASE (textColor);
@@ -76,10 +79,10 @@
    
     defentry = [defaults dictionaryForKey: @"backcolor"];
     if (defentry) {
-      float red = [[defentry objectForKey: @"red"] floatValue];
-      float green = [[defentry objectForKey: @"green"] floatValue];
-      float blue = [[defentry objectForKey: @"blue"] floatValue];
-      float alpha = [[defentry objectForKey: @"alpha"] floatValue];
+      float red = [[(NSDictionary *)defentry objectForKey: @"red"] floatValue];
+      float green = [[(NSDictionary *)defentry objectForKey: @"green"] floatValue];
+      float blue = [[(NSDictionary *)defentry objectForKey: @"blue"] floatValue];
+      float alpha = [[(NSDictionary *)defentry objectForKey: @"alpha"] floatValue];
     
       ASSIGN (backColor, [NSColor colorWithCalibratedRed: red 
                                                    green: green 
@@ -91,10 +94,10 @@
 
     defentry = [defaults dictionaryForKey: @"textcolor"];
     if (defentry) {
-      float red = [[defentry objectForKey: @"red"] floatValue];
-      float green = [[defentry objectForKey: @"green"] floatValue];
-      float blue = [[defentry objectForKey: @"blue"] floatValue];
-      float alpha = [[defentry objectForKey: @"alpha"] floatValue];
+      float red = [[(NSDictionary *)defentry objectForKey: @"red"] floatValue];
+      float green = [[(NSDictionary *)defentry objectForKey: @"green"] floatValue];
+      float blue = [[(NSDictionary *)defentry objectForKey: @"blue"] floatValue];
+      float alpha = [[(NSDictionary *)defentry objectForKey: @"alpha"] floatValue];
     
       ASSIGN (textColor, [NSColor colorWithCalibratedRed: red 
                                                    green: green 
@@ -156,15 +159,14 @@
 - (void)setOwnsScroller:(BOOL)ownscr
 {
   ownScroller = ownscr;
-  [self setFrame: [[self superview] frame]];
+  [self setFrame: [[self superview] bounds]];
   [self tile];
 }
 
 - (void)showPathComponents:(NSArray *)components
                  selection:(NSArray *)selection
 {
-  NSString *firstsel = [selection objectAtIndex: 0];
-  FSNode *node = [FSNode nodeWithPath: firstsel];
+  FSNode *node = [selection objectAtIndex: 0];
   int count = [components count];
   FSNIcon *icon;
   int icncount;
@@ -178,7 +180,7 @@
       [self removeRep: icon];
     }
   }
-
+  
   icncount = [icons count];
 
   for (i = 0; i < [components count]; i++) {
@@ -203,8 +205,7 @@
       NSMutableArray *selnodes = [NSMutableArray array];
     
       for (i = 0; i < [selection count]; i++) {
-        NSString *selpath = [selection objectAtIndex: i];
-        FSNode *selnode = [FSNode nodeWithPath: selpath];
+        FSNode *selnode = [selection objectAtIndex: i];
         [selnodes addObject: selnode];
       }
       
@@ -218,7 +219,7 @@
   
   editIcon = nil;
   
-  [self tile];
+  [self tile];  
 }
 
 - (void)setSelectableIconsRange:(NSRange)range
@@ -226,7 +227,7 @@
   int cols = range.length;
 
   if (cols != visibleIcons) {
-    [self setFrame: [[self superview] frame]];
+    [self setFrame: [[self superview] bounds]];
     visibleIcons = cols;  
   }
 
@@ -283,16 +284,16 @@
     highlightSize.height = iconSize + 4;
   }
 
-  labelSize.height = myrintf([labelFont defaultLineHeightForFont]);
+  labelSize.height = myrintf([fsnodeRep heighOfFont: labelFont]);
   gridSize.height = highlightSize.height + labelSize.height;
 }
 
 - (void)tile
 {
-  NSClipView *clip = [self superview];
+  NSClipView *clip = (NSClipView *)[self superview];
   float vwidth = [clip visibleRect].size.width;
 	int count = [icons count];
-  int i;
+  int i;    
     
   if (ownScroller) {
     NSRect fr = [self frame];
@@ -311,7 +312,7 @@
       r.origin.x = posx;
       
       [[icons objectAtIndex: i] setFrame: r];
-
+      
       posx += gridSize.width;
     }
     
@@ -321,7 +322,7 @@
 
     if (count > visibleIcons) {    
       x += gridSize.width * count;
-      [clip scrollToPoint: NSMakePoint(x, y)];
+      [clip scrollToPoint: NSMakePoint(x, y)];      
     }
 
   } else {
@@ -356,7 +357,7 @@
   }
 
   [self updateNameEditor]; 
-    
+       
   [self setNeedsDisplay: YES];
 }
 
@@ -365,6 +366,78 @@
   [self tile];
 }
 
+- (NSMenu *)menuForEvent:(NSEvent *)theEvent
+{
+  NSPoint location = [theEvent locationInWindow];
+  NSPoint selfloc = [self convertPoint: location fromView: nil];
+
+  if (editIcon && [self mouse: selfloc inRect: [editIcon frame]]) {
+    NSArray *selnodes;
+    NSMenu *menu;
+    NSMenuItem *menuItem;
+    NSString *firstext; 
+    NSDictionary *apps;
+    NSEnumerator *app_enum;
+    id key; 
+    int i;
+
+    if ([theEvent modifierFlags] == NSControlKeyMask) {
+      return [super menuForEvent: theEvent];
+    } 
+
+    selnodes = [self selectedNodes];
+
+    if ([selnodes count]) {
+      NSAutoreleasePool *pool;
+
+      firstext = [[[selnodes objectAtIndex: 0] path] pathExtension];
+
+      for (i = 0; i < [selnodes count]; i++) {
+        FSNode *snode = [selnodes objectAtIndex: i];
+        NSString *selpath = [snode path];
+        NSString *ext = [selpath pathExtension];   
+
+        if ([ext isEqual: firstext] == NO) {
+          return [super menuForEvent: theEvent];  
+        }
+
+        if ([snode isDirectory] == NO) {
+          if ([snode isPlain] == NO) {
+            return [super menuForEvent: theEvent];
+          }
+        } else {
+          if (([snode isPackage] == NO) || [snode isApplication]) {
+            return [super menuForEvent: theEvent];
+          } 
+        }
+      }
+
+      menu = [[NSMenu alloc] initWithTitle: NSLocalizedString(@"Open with", @"")];
+      apps = [[NSWorkspace sharedWorkspace] infoForExtension: firstext];
+      app_enum = [[apps allKeys] objectEnumerator];
+
+      pool = [NSAutoreleasePool new];
+
+      while ((key = [app_enum nextObject])) {
+        menuItem = [NSMenuItem new];    
+        key = [key stringByDeletingPathExtension];
+        [menuItem setTitle: key];
+        [menuItem setTarget: [GWorkspace gworkspace]];      
+        [menuItem setAction: @selector(openSelectionWithApp:)];      
+        [menuItem setRepresentedObject: key];            
+        [menu addItem: menuItem];
+        RELEASE (menuItem);
+      }
+
+      RELEASE (pool);
+
+      return [menu autorelease];
+    }
+  }
+     
+  return [super menuForEvent: theEvent]; 
+}
+
 //
 // scrollview delegate
 //
@@ -372,26 +445,28 @@
          scrollViewScrolled:(NSClipView *)clip
                     hitPart:(NSScrollerPart)hitpart
 {
-  int x = (int)[clip bounds].origin.x;
-  int y = (int)[clip bounds].origin.y;
-  int rem = x % (int)(myrintf(gridSize.width));
+  if (hitpart != NSScrollerNoPart) {
+    int x = (int)[clip bounds].origin.x;
+    int y = (int)[clip bounds].origin.y;
+    int rem = x % (int)(myrintf(gridSize.width));
 
-  [self stopRepNameEditing]; 
+    [self stopRepNameEditing]; 
 
-  if (rem != 0) {
-    if (rem <= gridSize.width / 2) {
-      x -= rem;
-    } else {
-      x += myrintf(gridSize.width) - rem;
+    if (rem != 0) {
+      if (rem <= gridSize.width / 2) {
+        x -= rem;
+      } else {
+        x += myrintf(gridSize.width) - rem;
+      }
+
+      [clip scrollToPoint: NSMakePoint(x, y)];      
+      [self setNeedsDisplay: YES];
+    }
+
+    editIcon = [self lastIcon];
+    if (editIcon && NSContainsRect([editIcon visibleRect], [editIcon iconBounds])) {
+      [self updateNameEditor]; 
     }
-    
-    [clip scrollToPoint: NSMakePoint(x, y)];      
-    [self setNeedsDisplay: YES];
-  }
-  
-  editIcon = [self lastIcon];
-  if (editIcon && NSContainsRect([editIcon visibleRect], [editIcon iconBounds])) {
-    [self updateNameEditor]; 
   }
 }
 
@@ -400,6 +475,11 @@
 
 @implementation GWViewerIconsPath (NodeRepContainer)
 
+- (FSNode *)baseNode
+{
+  return [viewer baseNode];  
+}
+
 - (id)repOfSubnode:(FSNode *)anode
 {
   int i;
@@ -485,6 +565,28 @@
   }
 }
 
+- (NSArray *)selectedNodes
+{
+  NSMutableArray *selectedNodes = [NSMutableArray array];
+  int i;
+  
+  for (i = 0; i < [icons count]; i++) {
+    FSNIcon *icon = [icons objectAtIndex: i];
+
+    if ([icon isSelected]) {
+      NSArray *selection = [icon selection];
+      
+      if (selection) {
+        [selectedNodes addObjectsFromArray: selection];
+      } else {
+        [selectedNodes addObject: [icon node]];
+      }
+    }
+  }
+
+  return [selectedNodes makeImmutableCopyOnFail: NO];
+}
+
 - (NSArray *)selectedPaths
 {
   NSMutableArray *selectedPaths = [NSMutableArray array];
@@ -506,7 +608,7 @@
     }
   }
 
-  return [NSArray arrayWithArray: selectedPaths];
+  return [selectedPaths makeImmutableCopyOnFail: NO];
 }
 
 - (void)checkLockedReps
@@ -524,7 +626,7 @@
 }
 
 - (void)openSelectionInNewViewer:(BOOL)newv
-{
+{  
   [viewer openSelectionInNewViewer: newv];
 }
 
@@ -549,7 +651,7 @@
   return [NSColor disabledControlTextColor];
 }
 
-- (unsigned int)draggingUpdated:(id <NSDraggingInfo>)sender
+- (NSDragOperation)draggingUpdated:(id <NSDraggingInfo>)sender
 {
   return NSDragOperationNone;
 }
@@ -624,7 +726,6 @@
     [nameEditor setNode: nil stringValue: @"" index: -1];
     [nameEditor removeFromSuperview];
     [self setNeedsDisplayInRect: edrect];
-    [[NSCursor arrowCursor] set];
   }
 
   for (i = 0; i < [icons count]; i++) {
@@ -676,93 +777,106 @@
 
 - (void)controlTextDidEndEditing:(NSNotification *)aNotification
 {
-  NSFileManager *fm = [NSFileManager defaultManager];
   FSNode *ednode = [nameEditor node];
+  BOOL writable = [ednode isWritable];
 
 #define CLEAREDITING \
   [self stopRepNameEditing]; \
   return 
   
-  if ([ednode isWritable] == NO) {
-    NSRunAlertPanel(NSLocalizedString(@"Error", @""), 
-          [NSString stringWithFormat: @"%@\"%@\"!\n", 
-              NSLocalizedString(@"You have not write permission for ", @""), 
-                    [ednode name]], NSLocalizedString(@"Continue", @""), nil, nil);   
-    CLEAREDITING;
+  if (writable == NO) {
+    /* check for broken symlink */     
+    if ([ednode isLink] && ([ednode hasValidPath] == NO)) { 
+      BOOL iamRoot;
+      
+      #ifdef __WIN32__
+		    iamRoot = YES;
+	    #else
+		    iamRoot = (geteuid() == 0);
+	    #endif
+      
+      writable = (iamRoot || [[ednode owner] isEqual: NSUserName()]);          
+    }
     
-  } else if ([fm isWritableFileAtPath: [ednode parentPath]] == NO) {
-    NSRunAlertPanel(NSLocalizedString(@"Error", @""), 
-          [NSString stringWithFormat: @"%@\"%@\"!\n", 
-              NSLocalizedString(@"You have not write permission for ", @""), 
-                  [ednode parentName]], NSLocalizedString(@"Continue", @""), nil, nil);   
-    CLEAREDITING;
-    
-  } else {
-    NSString *newname = [nameEditor stringValue];
-    NSString *newpath = [[ednode parentPath] stringByAppendingPathComponent: newname];
-    NSCharacterSet *notAllowSet = [NSCharacterSet characterSetWithCharactersInString: @"/\\*:?"];
-    NSRange range = [newname rangeOfCharacterFromSet: notAllowSet];
-    NSArray *dirContents = [fm directoryContentsAtPath: [ednode parentPath]];
-    NSMutableDictionary *userInfo = [NSMutableDictionary dictionary];
-    
-    if (range.length > 0) {
+    if (writable == NO) {
       NSRunAlertPanel(NSLocalizedString(@"Error", @""), 
-                NSLocalizedString(@"Invalid char in name", @""), 
-                          NSLocalizedString(@"Continue", @""), nil, nil);   
+            [NSString stringWithFormat: @"%@\"%@\"!\n", 
+                NSLocalizedString(@"You do not have write permission for ", @""), 
+                      [ednode name]], NSLocalizedString(@"Continue", @""), nil, nil);   
+      CLEAREDITING;
+    }
+  }
+  
+  if (writable) {    
+    if ([ednode isParentWritable] == NO) {
+      NSRunAlertPanel(NSLocalizedString(@"Error", @""), 
+            [NSString stringWithFormat: @"%@\"%@\"!\n", 
+                NSLocalizedString(@"You do not have write permission for ", @""), 
+                    [ednode parentName]], NSLocalizedString(@"Continue", @""), nil, nil);   
       CLEAREDITING;
-    }	
 
-    if ([dirContents containsObject: newname]) {
-      if ([newname isEqual: [ednode name]]) {
-        CLEAREDITING;
-      } else {
+    } else if ([ednode isSubnodeOfPath: [[GWorkspace gworkspace] trashPath]]) {
+      NSRunAlertPanel(NSLocalizedString(@"Error", @""), 
+              NSLocalizedString(@"You can't rename an object that is in the Recycler", @""), 
+              NSLocalizedString(@"Continue", @""), nil, nil);   
+      CLEAREDITING;
+
+    } else {
+      NSString *newname = [nameEditor stringValue];
+      NSString *newpath = [[ednode parentPath] stringByAppendingPathComponent: newname];
+      NSString *extension = [newpath pathExtension];
+      NSCharacterSet *notAllowSet = [NSCharacterSet characterSetWithCharactersInString: @"/\\*:?\33"];
+      NSRange range = [newname rangeOfCharacterFromSet: notAllowSet];
+      NSArray *dirContents = [ednode subNodeNamesOfParent];
+      NSMutableDictionary *opinfo = [NSMutableDictionary dictionary];
+
+      if (([newname length] == 0) || (range.length > 0)) {
         NSRunAlertPanel(NSLocalizedString(@"Error", @""), 
-          [NSString stringWithFormat: @"%@\"%@\" %@ ", 
-              NSLocalizedString(@"The name ", @""), 
-              newname, NSLocalizedString(@" is already in use!", @"")], 
+                  NSLocalizedString(@"Invalid name", @""), 
                             NSLocalizedString(@"Continue", @""), nil, nil);   
         CLEAREDITING;
+      }	
+
+      if (([extension length] 
+              && ([ednode isDirectory] && ([ednode isPackage] == NO)))) {
+        NSString *msg = NSLocalizedString(@"Are you sure you want to add the extension ", @"");
+
+        msg = [msg stringByAppendingFormat: @"\"%@\" ", extension];
+        msg = [msg stringByAppendingString: NSLocalizedString(@"to the end of the name?", @"")];
+        msg = [msg stringByAppendingString: NSLocalizedString(@"\nif you make this change, your folder may appear as a single file.", @"")];
+
+        if (NSRunAlertPanel(@"", msg, 
+                            NSLocalizedString(@"Cancel", @""), 
+				                    NSLocalizedString(@"OK", @""), 
+                            nil) == NSAlertDefaultReturn) {
+          CLEAREDITING;
+        }
       }
+
+      if ([dirContents containsObject: newname]) {
+        if ([newname isEqual: [ednode name]]) {
+          CLEAREDITING;
+        } else {
+          NSRunAlertPanel(NSLocalizedString(@"Error", @""), 
+            [NSString stringWithFormat: @"%@\"%@\" %@ ", 
+                NSLocalizedString(@"The name ", @""), 
+                newname, NSLocalizedString(@" is already in use!", @"")], 
+                              NSLocalizedString(@"Continue", @""), nil, nil);   
+          CLEAREDITING;
+        }
+      }
+
+	    [opinfo setObject: @"GWorkspaceRenameOperation" forKey: @"operation"];	
+      [opinfo setObject: [ednode path] forKey: @"source"];	
+      [opinfo setObject: newpath forKey: @"destination"];	
+      [opinfo setObject: [NSArray arrayWithObject: @""] forKey: @"files"];	
+
+      [self stopRepNameEditing];
+      [[GWorkspace gworkspace] performFileOperation: opinfo];         
     }
-        
-	  [userInfo setObject: @"GWorkspaceRenameOperation" forKey: @"operation"];	
-    [userInfo setObject: [ednode path] forKey: @"source"];	
-    [userInfo setObject: newpath forKey: @"destination"];	
-    [userInfo setObject: [NSArray arrayWithObject: @""] forKey: @"files"];	
-
-    [self stopRepNameEditing];
-
-//    [[NSDistributedNotificationCenter defaultCenter]
-// 				postNotificationName: @"GWFileSystemWillChangeNotification"
-//	 								    object: nil 
-//                    userInfo: userInfo];
-
-    [fm movePath: [ednode path] toPath: newpath handler: self];
-
-    [[NSDistributedNotificationCenter defaultCenter]
- 				postNotificationName: @"GWFileSystemDidChangeNotification"
-	 								    object: nil 
-                    userInfo: userInfo];
   }
 }
 
-- (BOOL)fileManager:(NSFileManager *)manager 
-              shouldProceedAfterError:(NSDictionary *)errorDict
-{
-	NSString *title = NSLocalizedString(@"Error", @"");
-	NSString *msg1 = NSLocalizedString(@"Cannot rename ", @"");
-  NSString *name = [[nameEditor node] name];
-	NSString *msg2 = NSLocalizedString(@"Continue", @"");
-
-  NSRunAlertPanel(title, [NSString stringWithFormat: @"%@'%@'!", msg1, name], msg2, nil, nil);   
-
-	return NO;
-}
-
-- (void)fileManager:(NSFileManager *)manager willProcessPath:(NSString *)path
-{
-}
-
 @end
 
 
