@@ -1,6 +1,6 @@
 /* GWViewerShelf.h
  *  
- * Copyright (C) 2004 Free Software Foundation, Inc.
+ * Copyright (C) 2004-2010 Free Software Foundation, Inc.
  *
  * Author: Enrico Sersale <enrico@imago.ro>
  * Date: July 2004
@@ -19,7 +19,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
  */
 
 #include <AppKit/AppKit.h>
@@ -38,6 +38,7 @@
 
 #define DEF_GRID_WIDTH 90
 #define Y_MARGIN (4)
+#define EDIT_MARGIN (4)
 
 
 @implementation GWViewerShelf
@@ -47,11 +48,13 @@
 	[self unsetWatchers];
   RELEASE (watchedPaths);
   RELEASE (icons);
-  TEST_RELEASE (extInfoType);
-	if (grid != NULL) {
-		NSZoneFree (NSDefaultMallocZone(), grid);
-	}
-  TEST_RELEASE (dragIcon);
+  RELEASE (extInfoType);
+  if (grid != NULL)
+    {
+      NSZoneFree (NSDefaultMallocZone(), grid);
+    }
+  RELEASE (dragIcon);
+  RELEASE (focusedIconLabel);  
   RELEASE (backColor);
   RELEASE (textColor);
   RELEASE (disabledTextColor);
@@ -72,10 +75,10 @@
     
     defentry = [defaults dictionaryForKey: @"backcolor"];
     if (defentry) {
-      float red = [[defentry objectForKey: @"red"] floatValue];
-      float green = [[defentry objectForKey: @"green"] floatValue];
-      float blue = [[defentry objectForKey: @"blue"] floatValue];
-      float alpha = [[defentry objectForKey: @"alpha"] floatValue];
+      float red = [[(NSDictionary *)defentry objectForKey: @"red"] floatValue];
+      float green = [[(NSDictionary *)defentry objectForKey: @"green"] floatValue];
+      float blue = [[(NSDictionary *)defentry objectForKey: @"blue"] floatValue];
+      float alpha = [[(NSDictionary *)defentry objectForKey: @"alpha"] floatValue];
     
       ASSIGN (backColor, [NSColor colorWithCalibratedRed: red 
                                                    green: green 
@@ -87,10 +90,10 @@
 
     defentry = [defaults dictionaryForKey: @"textcolor"];
     if (defentry) {
-      float red = [[defentry objectForKey: @"red"] floatValue];
-      float green = [[defentry objectForKey: @"green"] floatValue];
-      float blue = [[defentry objectForKey: @"blue"] floatValue];
-      float alpha = [[defentry objectForKey: @"alpha"] floatValue];
+      float red = [[(NSDictionary *)defentry objectForKey: @"red"] floatValue];
+      float green = [[(NSDictionary *)defentry objectForKey: @"green"] floatValue];
+      float blue = [[(NSDictionary *)defentry objectForKey: @"blue"] floatValue];
+      float alpha = [[(NSDictionary *)defentry objectForKey: @"alpha"] floatValue];
     
       ASSIGN (textColor, [NSColor colorWithCalibratedRed: red 
                                                    green: green 
@@ -139,13 +142,24 @@
     gworkspace = [GWorkspace gworkspace];
 
     dragIcon = nil;
+    focusedIcon = nil;
 
     [self calculateGridSize];    
     [self makeIconsGrid];
     
   	[self registerForDraggedTypes: [NSArray arrayWithObject: NSFilenamesPboardType]];
 
-		watchedPaths = [NSMutableArray new];
+		watchedPaths = [[NSCountedSet alloc] initWithCapacity: 1];
+
+    focusedIconLabel = [NSTextField new];
+		[focusedIconLabel setFont: [NSFont systemFontOfSize: 12]];
+		[focusedIconLabel setBezeled: NO];
+		[focusedIconLabel setAlignment: NSCenterTextAlignment];
+    [focusedIconLabel setEditable: NO];
+    [focusedIconLabel setSelectable: NO];
+    [focusedIconLabel setBackgroundColor: backColor];
+	  [focusedIconLabel setTextColor: [NSColor controlTextColor]];
+    [focusedIconLabel setFrame: NSMakeRect(0, 0, 0, 14)];    
   }
   
   return self;
@@ -239,9 +253,9 @@
   	NSString *watched = [node parentPath];	
 
 	  if ([watchedPaths containsObject: watched] == NO) {
-		  [watchedPaths addObject: watched];
 		  [self setWatcherForPath: watched];
 	  }
+    [watchedPaths addObject: watched];
   }
   
   return icon;
@@ -338,14 +352,14 @@
     highlightSize.height = iconSize + 4;
   }
 
-  labelSize.height = myrintf([labelFont defaultLineHeightForFont]);
+  labelSize.height = myrintf([fsnodeRep heighOfFont: labelFont]);
   labelSize.width = gridSize.width;
   gridSize.height = highlightSize.height + labelSize.height;
 }
 
 - (void)makeIconsGrid
 {
-  NSRect gridrect = [self frame];
+  NSRect gridrect = [self bounds];
   NSPoint gpnt;
   int i;
 
@@ -463,7 +477,7 @@
   hlightRect.size.width = ceil(iconSize / 3 * 4);
   hlightRect.size.height = ceil(hlightRect.size.width * [fsnodeRep highlightHeightFactor]);
   hlightRect.origin.x = ceil((gridSize.width - hlightRect.size.width) / 2);   
-  hlightRect.origin.y = floor([labelFont defaultLineHeightForFont]);
+  hlightRect.origin.y = floor([fsnodeRep heighOfFont: labelFont]);
   
   icnBounds.origin.x += hlightRect.origin.x + ((hlightRect.size.width - iconSize) / 2);
   icnBounds.origin.y += hlightRect.origin.y + ((hlightRect.size.height - iconSize) / 2);
@@ -490,9 +504,61 @@
         [icon setFrame: grid[index]];
       }
     } else {
+      if (focusedIcon == icon) {
+        focusedIcon = nil;
+      }
+
       [icon removeFromSuperview];
     }
   }
+  
+  [self updateFocusedIconLabel];
+}
+
+- (void)updateFocusedIconLabel
+{
+  if ([[self subviews] containsObject: focusedIconLabel]) {
+    NSRect rect = [focusedIconLabel frame];
+
+    [focusedIconLabel removeFromSuperview];
+    [self setNeedsDisplayInRect: rect];
+  }
+
+  if (focusedIcon) {
+    NSRect icnr = [focusedIcon frame];
+    float centerx = icnr.origin.x + (icnr.size.width / 2);
+    NSRect edrect = [self convertRect: [focusedIcon labelRect] fromView: focusedIcon];
+    int margin = [fsnodeRep labelMargin];
+    float bw = [self bounds].size.width - EDIT_MARGIN;
+    NSString *nodeDescr = [focusedIcon shownInfo]; 
+    float edwidth = [[focusedIconLabel font] widthOfString: nodeDescr];
+    
+    edwidth += margin;
+
+    if ((centerx + (edwidth / 2)) >= bw) {
+      centerx -= (centerx + (edwidth / 2) - bw);
+    } else if ((centerx - (edwidth / 2)) < margin) {
+      centerx += fabs(centerx - (edwidth / 2)) + margin;
+    }    
+    
+    edrect.origin.x = centerx - (edwidth / 2);
+    edrect.size.width = edwidth;
+    edrect = NSIntegralRect(edrect);
+
+    [focusedIconLabel setFrame: edrect];
+    [focusedIconLabel setStringValue: nodeDescr];
+
+    if ([focusedIcon isLocked] == NO) {
+      [focusedIconLabel setTextColor: [NSColor controlTextColor]];    
+    } else {
+      [focusedIconLabel setTextColor: [NSColor disabledControlTextColor]];    
+    }
+
+    [focusedIcon setNameEdited: YES];
+    
+    [self addSubview: focusedIconLabel];  
+    [self setNeedsDisplayInRect: edrect];
+  }  
 }
 
 - (void)setWatcherForPath:(NSString *)path
@@ -507,16 +573,115 @@
 
 - (void)unsetWatchers
 {
-	int i;
-	
-  for (i = 0; i < [watchedPaths count]; i++) {
-    [self unsetWatcherForPath: [watchedPaths objectAtIndex: i]];  
+  NSEnumerator *enumerator = [watchedPaths objectEnumerator]; 
+  NSString *wpath;
+
+	while ((wpath = [enumerator nextObject])) {
+    [self unsetWatcherForPath: wpath]; 
   }
 }
 
 - (NSArray *)watchedPaths
 {
-  return watchedPaths;
+  return [watchedPaths allObjects];
+}
+
+- (void)checkIconsAfterDotsFilesChange
+{
+  int count = [icons count]; 
+  BOOL updated = NO;
+  int i;
+
+	for (i = 0; i < count; i++) {
+    FSNIcon *icon = [icons objectAtIndex: i]; 
+    int j;
+  
+    if ([icon isShowingSelection] == NO) {
+      if ([[[icon node] path] rangeOfString: @"."].location != NSNotFound) {
+        [self removeRep: icon];
+        updated = YES;
+        count--;
+        i--;
+      }
+      
+    } else {
+      NSArray *iconpaths = [icon pathsSelection];
+  
+      for (j = 0; j < [iconpaths count]; j++) {
+        NSString *iconpath = [iconpaths objectAtIndex: j];
+
+        if ([iconpath rangeOfString: @"."].location != NSNotFound) {
+          [self removeRep: icon];
+          updated = YES;
+          count--;
+          i--;
+          break;
+        }
+      }
+    }
+	}
+  
+  if (updated) {
+    [self tile];
+    [self setNeedsDisplay: YES];
+  }
+}
+
+- (void)checkIconsAfterHidingOfPaths:(NSArray *)hpaths
+{
+  int count = [icons count]; 
+  BOOL updated = NO;
+  int i;
+
+	for (i = 0; i < count; i++) {
+    FSNIcon *icon = [icons objectAtIndex: i]; 
+    int j, m;
+  
+    if ([icon isShowingSelection] == NO) {
+      NSString *iconpath = [[icon node] path];
+
+	    for (m = 0; m < [hpaths count]; m++) {
+        NSString *hpath = [hpaths objectAtIndex: m]; 
+      
+        if (isSubpathOfPath(hpath, iconpath) || [hpath isEqual: iconpath]) {
+          [self removeRep: icon];
+          updated = YES;
+          count--;
+          i--;
+          break;
+        }
+      }
+    } else {
+      NSArray *iconpaths = [icon pathsSelection];
+      BOOL removed = NO;
+  
+      for (j = 0; j < [iconpaths count]; j++) {
+        NSString *iconpath = [iconpaths objectAtIndex: j];
+
+	      for (m = 0; m < [hpaths count]; m++) {
+          NSString *hpath = [hpaths objectAtIndex: m]; 
+        
+          if (isSubpathOfPath(hpath, iconpath) || [hpath isEqual: iconpath]) {
+            [self removeRep: icon];
+            updated = YES;
+            count--;
+            i--;
+            removed = YES;
+            break;
+          }
+        }
+
+        if (removed) {
+          break;
+        } 
+      }
+    }
+	}
+  
+  if (updated) {
+    [self tile];
+    [self setNeedsDisplay: YES];
+  }
 }
 
 - (void)resizeWithOldSuperviewSize:(NSSize)oldFrameSize
@@ -550,13 +715,21 @@
 
 	if ([watchedPaths containsObject: watched]) {
     [watchedPaths removeObject: watched];
-    [self unsetWatcherForPath: watched];
-	}
+    
+    if ([watchedPaths containsObject: watched] == NO) {
+      [self unsetWatcherForPath: watched];
+	  }
+  }
 
   if ([[self subviews] containsObject: arep]) {
     [arep removeFromSuperviewWithoutNeedingDisplay];
   }
   
+  if (focusedIcon == arep) {
+    focusedIcon = nil;
+    [self updateFocusedIconLabel];
+  }
+  
   [icons removeObject: arep];
 }
 
@@ -713,27 +886,30 @@
 {
   NSString *path = [info objectForKey: @"path"];
 	NSString *event = [info objectForKey: @"event"];
-	BOOL contained = NO;
-	int i;
+	NSEnumerator *enumerator;
+  NSString *wpath;
+  BOOL contained = NO;
 
 	if ([event isEqual: @"GWFileCreatedInWatchedDirectory"]) {
 		return;
 	}
-
-	for (i = 0; i < [watchedPaths count]; i++) {
-		NSString *wpath = [watchedPaths objectAtIndex: i];
+  
+  enumerator = [watchedPaths objectEnumerator];
+  
+	while ((wpath = [enumerator nextObject])) {
 		if (([wpath isEqual: path]) || (isSubpathOfPath(path, wpath))) {
 			contained = YES;
 			break;
 		}
-	}
+  }
 
   if (contained) {
 		int count = [icons count];
     BOOL updated = NO;
     FSNIcon *icon;
+	  int i;
 
-		if ([event isEqual: @"GWWatchedDirectoryDeleted"]) {		
+		if ([event isEqual: @"GWWatchedPathDeleted"]) {		
 			for (i = 0; i < count; i++) {
         icon = [icons objectAtIndex: i];
         
@@ -842,6 +1018,18 @@
   [self unselectOtherReps: nil];
 }
 
+- (void)setFocusedRep:(id)arep
+{
+  if (arep == nil) {
+    if (focusedIcon) {
+      [focusedIcon setNameEdited: NO];
+    }
+  } 
+
+  focusedIcon = arep;  
+  [self updateFocusedIconLabel];
+}
+
 - (NSColor *)backgroundColor
 {
   return backColor;
@@ -862,7 +1050,7 @@
 
 @implementation GWViewerShelf (DraggingDestination)
 
-- (unsigned int)draggingEntered:(id <NSDraggingInfo>)sender
+- (NSDragOperation)draggingEntered:(id <NSDraggingInfo>)sender
 {
 	NSPasteboard *pb = [sender draggingPasteboard];
   NSDragOperation sourceDragMask = [sender draggingSourceOperationMask];
@@ -880,6 +1068,7 @@
     NSArray *sourcePaths = [pb propertyListForType: NSFilenamesPboardType]; 
     int count = [sourcePaths count];
     FSNode *baseNode = [viewer baseNode];
+    NSString *basePath;
     int i;
         
 	  if (count == 0) {
@@ -893,7 +1082,12 @@
         return NSDragOperationNone;
       } 
     }  
-  
+    
+    basePath = [[sourcePaths objectAtIndex: 0] stringByDeletingLastPathComponent];
+    if ([basePath isEqual: [gworkspace trashPath]]) {
+      return NSDragOperationNone;
+    }
+    
     if (count == 1) {
       dragLocalIcon = ([self iconForPath: [sourcePaths objectAtIndex: 0]] != nil);
     } else {
@@ -911,7 +1105,7 @@
   return NSDragOperationNone;
 }
 
-- (unsigned int)draggingUpdated:(id <NSDraggingInfo>)sender
+- (NSDragOperation)draggingUpdated:(id <NSDraggingInfo>)sender
 {
   NSDragOperation sourceDragMask;
   NSPoint dpoint;
