@@ -19,7 +19,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
  */
 
 #ifndef GWVIEWERS_MANAGER_H
@@ -37,13 +37,17 @@
 @interface GWViewersManager : NSObject
 {
   NSMutableArray *viewers;
+  NSMutableArray *rootViewersKeys;
   BOOL orderingViewers;
   GWorkspace *gworkspace;
   History *historyWindow;
   NSMutableArray *spatialViewersHistory;
   int spvHistoryPos;  
   BOOL settingHistoryPath;
-  NSNotificationCenter *nc;      
+  NSHelpManager *helpManager;
+  NSAttributedString *bviewerHelp;
+  NSAttributedString *sviewerHelp;
+  NSNotificationCenter *nc;  
 }
 
 + (GWViewersManager *)viewersManager;
@@ -58,6 +62,7 @@
             
 
 - (id)newViewerOfType:(unsigned)vtype
+             showType:(NSString *)stype
               forNode:(FSNode *)node
         showSelection:(BOOL)showsel
        closeOldViewer:(id)oldvwr
@@ -76,6 +81,8 @@
 
 - (id)rootViewer;
 
+- (NSNumber *)nextRootViewerKey;
+
 - (int)typeOfViewerForNode:(FSNode *)node;
 
 - (id)parentOfSpatialViewer:(id)aviewer;
@@ -100,6 +107,8 @@
                   
 - (void)openAsFolderSelectionInViewer:(id)viewer;
 
+- (void)openWithSelectionInViewer:(id)viewer;
+
 
 - (void)sortTypeDidChange:(NSNotification *)notif;
 
@@ -111,6 +120,10 @@
 
 - (void)thumbnailsDidChangeInPaths:(NSArray *)paths;
 
+- (void)hideDotsFileDidChange:(BOOL)hide;
+
+- (void)hiddenFilesDidChange:(NSArray *)paths;
+
 
 - (BOOL)hasViewerWithWindow:(id)awindow;
 
@@ -118,6 +131,9 @@
 
 - (NSArray *)viewerWindows;
 
+- (BOOL)orderingViewers;
+
+- (void)updateDesktop;
 
 - (void)updateDefaults;
 
