@@ -1,6 +1,6 @@
 /* GWViewersManager.h
  *  
- * Copyright (C) 2004 Free Software Foundation, Inc.
+ * Copyright (C) 2004-2010 Free Software Foundation, Inc.
  *
  * Author: Enrico Sersale <enrico@imago.ro>
  * Date: June 2004
@@ -19,18 +19,19 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
  */
 
-#include <AppKit/AppKit.h>
-#include "GWViewersManager.h"
-#include "GWViewer.h"
-#include "GWSpatialViewer.h"
-#include "GWViewerWindow.h"
-#include "History.h"
-#include "FSNFunctions.h"
-#include "GWorkspace.h"
-#include "GWDesktopManager.h"
+#import <AppKit/AppKit.h>
+#import "GWViewersManager.h"
+#import "GWViewer.h"
+#import "GWSpatialViewer.h"
+#import "GWViewerWindow.h"
+#import "History.h"
+#import "FSNFunctions.h"
+#import "GWorkspace.h"
+#import "GWDesktopManager.h"
+
 
 static GWViewersManager *vwrsmanager = nil;
 
@@ -49,9 +50,12 @@ static GWViewersManager *vwrsmanager = nil;
   [[NSDistributedNotificationCenter defaultCenter] removeObserver: self];
   [nc removeObserver: self];
   RELEASE (viewers);
+  RELEASE (rootViewersKeys);
   RELEASE (spatialViewersHistory);
+  RELEASE (bviewerHelp);
+  RELEASE (sviewerHelp);
     
-	[super dealloc];
+  [super dealloc];
 }
 
 - (id)init
@@ -59,8 +63,22 @@ static GWViewersManager *vwrsmanager = nil;
   self = [super init];
   
   if (self) {
+    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];  
+    id entry = [defaults objectForKey: @"root_viewers_keys"];
+    
     gworkspace = [GWorkspace gworkspace];
+    helpManager = [NSHelpManager sharedHelpManager];    
+    ASSIGN (bviewerHelp, [gworkspace contextHelpFromName: @"BViewer.rtfd"]);
+    ASSIGN (sviewerHelp, [gworkspace contextHelpFromName: @"SViewer.rtfd"]);
+    
     viewers = [NSMutableArray new];
+    orderingViewers = NO;
+    
+    rootViewersKeys = [NSMutableArray new];
+    if (entry) {
+      [rootViewersKeys addObjectsFromArray: entry];
+    }
+    
     spatialViewersHistory = [NSMutableArray new]; 
     spvHistoryPos = 0;  
     historyWindow = [gworkspace historyWindow]; 
@@ -86,7 +104,7 @@ static GWViewersManager *vwrsmanager = nil;
                 					  name: @"GWSortTypeDidChangeNotification"
                 					object: nil];
     
-    [[FSNodeRep sharedInstance] setLabelWFactor: 9.0];
+    [[FSNodeRep sharedInstance] setLabelWFactor: 9.0];    
   }
   
   return self;
@@ -109,10 +127,11 @@ static GWViewersManager *vwrsmanager = nil;
     
       if (node && [node isValid]) {
         [self newViewerOfType: type
-                forNode: node
-          showSelection: YES
-         closeOldViewer: NO
-               forceNew: NO];
+                     showType: nil
+                      forNode: node
+                showSelection: YES
+               closeOldViewer: NO
+                     forceNew: YES];
       }
     }
 
@@ -142,6 +161,7 @@ static GWViewersManager *vwrsmanager = nil;
     }
   
     viewer = [self newViewerOfType: type
+                          showType: nil
                            forNode: node
                      showSelection: YES
                     closeOldViewer: NO
@@ -158,8 +178,9 @@ static GWViewersManager *vwrsmanager = nil;
       }
 
       viewer = [self newViewerOfType: type
+                            showType: nil
                              forNode: node
-                       showSelection: NO
+                       showSelection: (type == BROWSING)
                       closeOldViewer: NO
                             forceNew: YES];
     }
@@ -192,6 +213,7 @@ static GWViewersManager *vwrsmanager = nil;
             && [[viewer nodeView] isSingleNode]
                               && (baseIsParent == NO)) { 
       viewer = [self newViewerOfType: BROWSING
+                            showType: nil
                              forNode: base
                        showSelection: NO
                       closeOldViewer: NO
@@ -203,6 +225,7 @@ static GWViewersManager *vwrsmanager = nil;
     int newtype = ((type == SPATIAL) && baseIsParent) ? SPATIAL : BROWSING;
 
     viewer = [self newViewerOfType: newtype
+                          showType: nil
                            forNode: base
                      showSelection: NO
                     closeOldViewer: NO
@@ -215,6 +238,7 @@ static GWViewersManager *vwrsmanager = nil;
 }
 
 - (id)newViewerOfType:(unsigned)vtype
+             showType:(NSString *)stype
               forNode:(FSNode *)node
         showSelection:(BOOL)showsel
        closeOldViewer:(id)oldvwr
@@ -231,6 +255,7 @@ static GWViewersManager *vwrsmanager = nil;
     
     viewer = [[c alloc] initForNode: node 
                            inWindow: win 
+                           showType: stype
                       showSelection: showsel]; 
                         
     [viewers addObject: viewer];
@@ -239,6 +264,7 @@ static GWViewersManager *vwrsmanager = nil;
   } 
 
   if (oldvwr) {
+    [helpManager removeContextHelpForObject: [[oldvwr win] contentView]];
     [[oldvwr win] close]; 
   }
   
@@ -254,6 +280,13 @@ static GWViewersManager *vwrsmanager = nil;
         [rep setOpened: YES];
       }
     }
+   
+    [helpManager setContextHelp: sviewerHelp 
+                      forObject: [[viewer win] contentView]];
+    
+  } else {
+    [helpManager setContextHelp: bviewerHelp
+                      forObject: [[viewer win] contentView]];
   }
        
   return viewer;
@@ -274,6 +307,7 @@ static GWViewersManager *vwrsmanager = nil;
     
     viewer = [[c alloc] initForNode: node 
                            inWindow: win 
+                           showType: nil
                       showSelection: YES];   
     [viewers addObject: viewer];
     RELEASE (viewer);
@@ -282,6 +316,9 @@ static GWViewersManager *vwrsmanager = nil;
     
     [viewer activate];
     [viewer windowDidBecomeKey: nil];
+
+    [helpManager setContextHelp: (vtype == SPATIAL) ? sviewerHelp : bviewerHelp
+                      forObject: [[viewer win] contentView]];
   }
 }
 
@@ -348,6 +385,19 @@ static GWViewersManager *vwrsmanager = nil;
   return nil;
 }
 
+- (NSNumber *)nextRootViewerKey
+{
+  NSNumber *key = nil;
+    
+  if ([rootViewersKeys count]) {
+    key = [rootViewersKeys objectAtIndex: 0];
+    RETAIN (key);
+    [rootViewersKeys removeObjectAtIndex: 0];
+  }
+  
+  return AUTORELEASE (key);
+}
+
 - (int)typeOfViewerForNode:(FSNode *)node
 {
   NSFileManager *fm = [NSFileManager defaultManager];
@@ -395,13 +445,9 @@ static GWViewersManager *vwrsmanager = nil;
   FSNode *node = [aviewer baseNode];
   NSString *path = [node path];
   NSArray *watchedNodes = [aviewer watchedNodes];
+  NSNumber *key = [aviewer rootViewerKey];
   id parentViewer = [self parentOfSpatialViewer: aviewer];
-  id desktopManager = [gworkspace desktopManager];  
   int i;
-
-  if ([desktopManager isActive]) {
-    [desktopManager deselectAllIcons];
-  }
   
   if (parentViewer && ([parentViewer invalidated] == NO)) {
     [parentViewer setOpened: NO repOfNode: node];
@@ -427,6 +473,11 @@ static GWViewersManager *vwrsmanager = nil;
     [self changeHistoryOwner: nil];
   }
       
+  if (key) {
+    [rootViewersKeys insertObject: key atIndex: 0];
+  }
+
+  [helpManager removeContextHelpForObject: [[aviewer win] contentView]];
   [viewers removeObject: aviewer];
 }
 
@@ -468,6 +519,7 @@ static GWViewersManager *vwrsmanager = nil;
 
     [viewer deactivate];
 	  [[NSRunLoop currentRunLoop] runUntilDate: limit];
+    [helpManager removeContextHelpForObject: [[viewer win] contentView]];
     [viewers removeObject: viewer];
   }
 }
@@ -513,7 +565,13 @@ static GWViewersManager *vwrsmanager = nil;
   id parentViewer = [self parentOfSpatialViewer: aviewer];
     
   if (parentViewer) {
+    id nodeView = [parentViewer nodeView];
+
     [parentViewer setOpened: YES repOfNode: [aviewer baseNode]];
+
+    if ([nodeView respondsToSelector: @selector(scrollSelectionToVisible)]) {
+      [nodeView scrollSelectionToVisible];
+    }  
   }
 }
 
@@ -538,7 +596,6 @@ static GWViewersManager *vwrsmanager = nil;
   }  
 }
 
-
 - (void)selectionChanged:(NSArray *)selection
 {
   if (orderingViewers == NO) {
@@ -550,28 +607,65 @@ static GWViewersManager *vwrsmanager = nil;
                   closeSender:(BOOL)close
 {
   NSArray *selreps = [[viewer nodeView] selectedReps];
+  int count = [selreps count];
   int i;
     
-  for (i = 0; i < [selreps count]; i++) {
+  if (count > OPEN_MAX) {
+    NSString *msg1 = NSLocalizedString(@"Are you sure you want to open", @"");
+    NSString *msg2 = NSLocalizedString(@"items?", @"");
+
+    if (NSRunAlertPanel(nil,
+                [NSString stringWithFormat: @"%@ %i %@", msg1, count, msg2],
+                NSLocalizedString(@"Cancel", @""),
+                NSLocalizedString(@"Yes", @""),
+                nil)) {
+      return;
+    }
+  }
+    
+  for (i = 0; i < count; i++) {
     FSNode *node = [[selreps objectAtIndex: i] node];
-    NSString *path = [node path];
-        
-    if ([node isDirectory]) {
-      if ([node isPackage]) {    
-        if ([node isApplication] == NO) {
-          [gworkspace openFile: path];
+    
+    if ([node hasValidPath]) {            
+      NS_DURING
+        {
+      if ([node isDirectory]) {
+        if ([node isPackage]) {    
+          if ([node isApplication] == NO) {
+            [gworkspace openFile: [node path]];
+          } else {
+            [[NSWorkspace sharedWorkspace] launchApplication: [node path]];
+          }
         } else {
-          [[NSWorkspace sharedWorkspace] launchApplication: path];
+          [self newViewerOfType: [viewer vtype] 
+                       showType: nil
+                        forNode: node 
+                  showSelection: NO
+                 closeOldViewer: nil
+                       forceNew: NO];
+        } 
+      } else if ([node isPlain]) {        
+        [gworkspace openFile: [node path]];
+      }
         }
-      } else {
-        [self newViewerOfType: [viewer vtype] 
-                      forNode: node 
-                showSelection: NO
-               closeOldViewer: nil
-                     forceNew: NO];
-      } 
-    } else if ([node isPlain]) {        
-      [gworkspace openFile: path];
+      NS_HANDLER
+        {
+          NSRunAlertPanel(NSLocalizedString(@"error", @""), 
+              [NSString stringWithFormat: @"%@ %@!", 
+                        NSLocalizedString(@"Can't open ", @""), [node name]],
+                                            NSLocalizedString(@"OK", @""), 
+                                            nil, 
+                                            nil);                                     
+        }
+      NS_ENDHANDLER
+    
+    } else {
+      NSRunAlertPanel(NSLocalizedString(@"error", @""), 
+          [NSString stringWithFormat: @"%@ %@!", 
+                    NSLocalizedString(@"Can't open ", @""), [node name]],
+                                        NSLocalizedString(@"OK", @""), 
+                                        nil, 
+                                        nil);                                     
     }
   }
 
@@ -583,23 +677,34 @@ static GWViewersManager *vwrsmanager = nil;
 - (void)openAsFolderSelectionInViewer:(id)viewer
 {
   NSArray *selnodes = [[viewer nodeView] selectedNodes];
+  BOOL force = NO;
   int i;
-    
+  
+  if ((selnodes == nil) || ([selnodes count] == 0)) {
+    selnodes = [NSArray arrayWithObject: [[viewer nodeView] shownNode]];
+    force = YES;
+  }
+  
   for (i = 0; i < [selnodes count]; i++) {
     FSNode *node = [selnodes objectAtIndex: i];
         
     if ([node isDirectory]) {
       [self newViewerOfType: [viewer vtype] 
+                   showType: [viewer viewType]
                     forNode: node
               showSelection: NO
              closeOldViewer: nil
-                   forceNew: NO];
+                   forceNew: force];
     } else if ([node isPlain]) {        
       [gworkspace openFile: [node path]];
     }
   }
 }
 
+- (void)openWithSelectionInViewer:(id)viewer
+{
+  [gworkspace openSelectedPathsWith];
+}
 
 - (void)sortTypeDidChange:(NSNotification *)notif
 {
@@ -687,7 +792,7 @@ static GWViewersManager *vwrsmanager = nil;
     FSNode *node = [viewer baseNode];
     NSArray *watchedNodes = [viewer watchedNodes];
     
-    if ([event isEqual: @"GWWatchedDirectoryDeleted"]) {  
+    if ([event isEqual: @"GWWatchedPathDeleted"]) {  
       if (([[node path] isEqual: path]) || [node isSubnodeOfPath: path]) { 
         if ([viewer invalidated] == NO) {
           [viewer invalidate];
@@ -746,6 +851,57 @@ static GWViewersManager *vwrsmanager = nil;
   }
 }
 
+- (void)hideDotsFileDidChange:(BOOL)hide
+{
+  NSMutableArray *viewersToClose = [NSMutableArray array];
+  int i;  
+
+  for (i = 0; i < [viewers count]; i++) {
+    id viewer = [viewers objectAtIndex: i];
+        
+    if ([viewer invalidated] == NO) {
+      if (hide) {
+        if ([[[viewer baseNode] path] rangeOfString: @"."].location != NSNotFound) {
+          [viewer invalidate];
+          [viewersToClose addObject: viewer];
+        }
+      }
+      
+      if ([viewersToClose containsObject: viewer] == NO) {
+        [viewer hideDotsFileChanged: hide];
+      }
+    }
+  }
+
+  [self closeInvalidViewers: viewersToClose]; 
+}
+
+- (void)hiddenFilesDidChange:(NSArray *)paths
+{
+  NSMutableArray *viewersToClose = [NSMutableArray array];
+  int i, j;  
+
+  for (i = 0; i < [viewers count]; i++) {
+    id viewer = [viewers objectAtIndex: i];
+    NSString *vwrpath = [[viewer baseNode] path];
+
+    for (j = 0; j < [paths count]; j++) {
+      NSString *path = [paths objectAtIndex: j];
+      
+      if (isSubpathOfPath(path, vwrpath) || [path isEqual: vwrpath]) {
+        [viewer invalidate];
+        [viewersToClose addObject: viewer];
+      }
+    }
+    
+    if ([viewersToClose containsObject: viewer] == NO) {
+      [viewer hiddenFilesChanged: paths];
+    }
+  }
+
+  [self closeInvalidViewers: viewersToClose]; 
+}
+
 
 - (BOOL)hasViewerWithWindow:(id)awindow
 {
@@ -793,6 +949,19 @@ static GWViewersManager *vwrsmanager = nil;
   return wins;
 }
 
+- (BOOL)orderingViewers
+{
+  return orderingViewers;
+}
+
+- (void)updateDesktop
+{
+  id desktopManager = [gworkspace desktopManager];  
+
+  if ([desktopManager isActive]) {
+    [desktopManager deselectAllIcons];
+  }
+}
 
 - (void)updateDefaults
 {
@@ -815,6 +984,8 @@ static GWViewersManager *vwrsmanager = nil;
   }
   
 	[defaults setObject: viewersInfo forKey: @"viewersinfo"];
+  
+  [defaults setObject: rootViewersKeys forKey: @"root_viewers_keys"];
 }
 
 @end
@@ -917,16 +1088,16 @@ if (*pos >= i) *pos -= n; \
 		}
 	}
   
-	if ([history count] > 4) {
+  count = [history count];
+  
+	if (count > 4) {
 		FSNode *na[2], *nb[2];
-	
-		count = [history count];
 		
 		for (i = 0; i < count; i++) {
-			if (i < ([history count] - 3)) {
+			if (i < (count - 3)) {
 				na[0] = [history objectAtIndex: i];
 				na[1] = [history objectAtIndex: i + 1];
-				nb[0] = [history objectAtIndex: i + 2];
+				nb[0] = [history objectAtIndex: i + 2]; 
 				nb[1] = [history objectAtIndex: i + 3];
 		
 				if (([na[0] isEqual: nb[0]]) && ([na[1] isEqual: nb[1]])) {
@@ -1027,6 +1198,7 @@ if (*pos >= i) *pos -= n; \
     }
   } else {
     [self newViewerOfType: SPATIAL
+                 showType: nil
                   forNode: node
             showSelection: YES
            closeOldViewer: nil
