@@ -1,6 +1,6 @@
 /* Finder.m
  *  
- * Copyright (C) 2005 Free Software Foundation, Inc.
+ * Copyright (C) 2005-2010 Free Software Foundation, Inc.
  *
  * Author: Enrico Sersale <enrico@imago.ro>
  * Date: January 2005
@@ -19,37 +19,31 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
  */
 
-#include <Foundation/Foundation.h>
-#include <AppKit/AppKit.h>
-#include "Finder.h"
-#include "FinderModulesProtocol.h"
-#include "FindModuleView.h"
-#include "SearchPlacesBox.h"
-#include "SearchPlacesCell.h"
-#include "SearchResults.h"
-#include "LSFolder.h"
-#include "FSNodeRep.h"
-#include "GWorkspace.h"
-#include "GWFunctions.h"
+#import <Foundation/Foundation.h>
+#import <AppKit/AppKit.h>
+#import "Finder.h"
+#import "FinderModulesProtocol.h"
+#import "FindModuleView.h"
+#import "SearchPlacesBox.h"
+#import "SearchPlacesCell.h"
+#import "SearchResults.h"
+#import "LSFolder.h"
+#import "FSNodeRep.h"
+#import "GWorkspace.h"
+#import "GWFunctions.h"
 
 #define WINH (262.0)
 #define FMVIEWH (34.0)
 #define BORDER (4.0)
 #define HMARGIN (12.0)
 
-#define ITMSLABY (3.0)
-#define PLACESBOXY (24.0)
-#define PLACESBOXH (116.0)
-#define PLACEBUTTY (33.0)
-
 #define SELECTION 0
 #define PLACES 1
 
 #define CELLS_HEIGHT (28.0)
-#define ICON_SIZE NSMakeSize(24.0, 24.0)
 
 #define CHECKSIZE(sz) \
 if (sz.width < 0) sz.width = 0; \
@@ -72,14 +66,14 @@ static Finder *finder = nil;
 - (void)dealloc
 {
   [nc removeObserver: self];
-  TEST_RELEASE (modules);
+  RELEASE (modules);
   RELEASE (fmviews);
-  TEST_RELEASE (currentSelection);
-  TEST_RELEASE (win);
+  RELEASE (currentSelection);
+  RELEASE (win);
   RELEASE (searchResults);
   RELEASE (lsFolders);
     
-	[super dealloc];
+  [super dealloc];
 }
 
 - (id)init
@@ -118,7 +112,7 @@ static Finder *finder = nil;
 
     [placesBox setFinder: self];
 
-    rect = [[(NSBox *)placesBox contentView] frame];
+    rect = [[(NSBox *)placesBox contentView] bounds];
     placesScroll = [[NSScrollView alloc] initWithFrame: rect];
     [placesScroll setBorderType: NSBezelBorder];
     [placesScroll setHasHorizontalScroller: NO];
@@ -142,7 +136,7 @@ static Finder *finder = nil;
     ms.width = cs.width;
     CHECKSIZE (ms);
     [placesMatrix setCellSize: ms];
-	  [placesScroll setDocumentView: placesMatrix];	
+    [placesScroll setDocumentView: placesMatrix];	
     RELEASE (placesMatrix);
   
     defentry = [defaults objectForKey: @"saved_places"];
@@ -173,11 +167,15 @@ static Finder *finder = nil;
         [fmview setAddEnabled: NO];    
       }
 
-      [[(NSBox *)modulesBox contentView] addSubview: [fmview mainBox]];
+      [[modulesBox contentView] addSubview: [fmview mainBox]];
       [fmviews insertObject: fmview atIndex: [fmviews count]];
       RELEASE (fmview);
     }
 
+    for (i = 0; i < [fmviews count]; i++) {
+      [[fmviews objectAtIndex: i] updateMenuForModules: modules];
+    }
+
     [recursiveSwitch setState: NSOnState];
 
     defentry = [defaults objectForKey: @"search_res_h"];
@@ -252,7 +250,8 @@ static Finder *finder = nil;
 - (void)loadModules
 {
   NSString *bundlesDir;
-  BOOL isdir;
+  NSEnumerator *enumerator;
+  NSString *path;
   NSMutableArray *bundlesPaths;
   NSMutableArray *unsortedModules;
   NSDictionary *lastUsedModules;
@@ -260,30 +259,28 @@ static Finder *finder = nil;
   int index;
   int i;
 
-  bundlesDir = [NSSearchPathForDirectoriesInDomains(NSLibraryDirectory, NSSystemDomainMask, YES) lastObject];
-  bundlesDir = [bundlesDir stringByAppendingPathComponent: @"Bundles"];
   bundlesPaths = [NSMutableArray array];
-  [bundlesPaths addObjectsFromArray: [self bundlesWithExtension: @"finder" 
-                                                         inPath: bundlesDir]];
+  enumerator = [NSSearchPathForDirectoriesInDomains
+    (NSLibraryDirectory, NSAllDomainsMask, YES) objectEnumerator];
+  while ((bundlesDir = [enumerator nextObject]) != nil)
+    {
+      NSEnumerator *enumerator;
 
-  bundlesDir = [NSSearchPathForDirectoriesInDomains(NSLibraryDirectory, NSUserDomainMask, YES) lastObject];
-  bundlesDir = [bundlesDir stringByAppendingPathComponent: @"GWorkspace"];
+      bundlesDir = [bundlesDir stringByAppendingPathComponent: @"Bundles"];
+      enumerator = [[fm directoryContentsAtPath: bundlesDir] objectEnumerator];
 
-  if (([fm fileExistsAtPath: bundlesDir isDirectory: &isdir] && isdir) == NO) {
-    if ([fm createDirectoryAtPath: bundlesDir attributes: nil] == NO) {
-      NSRunAlertPanel(NSLocalizedString(@"error", @""), 
-             NSLocalizedString(@"Can't create the GWorkspace directory! Quiting now.", @""), 
-                                    NSLocalizedString(@"OK", @""), nil, nil);                                     
-      [NSApp terminate: self];
-    }
+      while ((path = [enumerator nextObject])) {
+	if ([[path pathExtension] isEqual: @"finder"]) {
+	  [bundlesPaths addObject:
+	    [bundlesDir stringByAppendingPathComponent: path]];
+	}
+      }
   }
 
-  [bundlesPaths addObjectsFromArray: [self bundlesWithExtension: @"finder" 
-                                                         inPath: bundlesDir]];
-
   unsortedModules = [NSMutableArray array];
 
   for (i = 0; i < [bundlesPaths count]; i++) {
+    CREATE_AUTORELEASE_POOL(arp);
     NSString *bpath = [bundlesPaths objectAtIndex: i];
     NSBundle *bundle = [NSBundle bundleWithPath: bpath];
      
@@ -291,28 +288,14 @@ static Finder *finder = nil;
 			Class principalClass = [bundle principalClass];
 
 			if ([principalClass conformsToProtocol: @protocol(FinderModulesProtocol)]) {	
-	      CREATE_AUTORELEASE_POOL (pool);
         id module = [[principalClass alloc] initInterface];
-	  		NSString *name = [module moduleName];
-        BOOL exists = NO;	
-        int j;
-        			
-				for (j = 0; j < [unsortedModules count]; j++) {
-					if ([name isEqual: [[unsortedModules objectAtIndex: j] moduleName]]) {
-            NSLog(@"duplicate module \"%@\" at %@", name, bpath);
-						exists = YES;
-						break;
-					}
-				}
-
-				if (exists == NO) {
-          [unsortedModules addObject: module];
-        }
-
+        
+        [unsortedModules addObject: module];
 	  		RELEASE ((id)module);			
-        RELEASE (pool);		
 			}
     }
+    
+    RELEASE (arp);
   }
 
   if ([unsortedModules count] == 0) {  
@@ -359,28 +342,6 @@ static Finder *finder = nil;
   modules = [[unsortedModules sortedArrayUsingSelector: @selector(compareModule:)] mutableCopy];
 }
 
-- (NSArray *)bundlesWithExtension:(NSString *)extension 
-													 inPath:(NSString *)path
-{
-  NSMutableArray *bundleList = [NSMutableArray array];
-  NSEnumerator *enumerator;
-  NSString *dir;
-  BOOL isDir;
-  
-  if ((([fm fileExistsAtPath: path isDirectory: &isDir]) && isDir) == NO) {
-		return nil;
-  }
-	  
-  enumerator = [[fm directoryContentsAtPath: path] objectEnumerator];
-  while ((dir = [enumerator nextObject])) {
-    if ([[dir pathExtension] isEqualToString: extension]) {
-			[bundleList addObject: [path stringByAppendingPathComponent: dir]];
-		}
-  }
-  
-  return bundleList;
-}
-
 - (NSArray *)modules
 {
   return modules;
@@ -446,7 +407,7 @@ static Finder *finder = nil;
     [module setInUse: YES];
     [fmview setModule: module];
 
-    [[(NSBox *)modulesBox contentView] addSubview: [fmview mainBox]];
+    [[modulesBox contentView] addSubview: [fmview mainBox]];
     [fmviews insertObject: fmview atIndex: index + 1];
     RELEASE (fmview);
     
@@ -533,7 +494,7 @@ static Finder *finder = nil;
   [removePlaceButt setEnabled: NO];
 }
 
-- (unsigned int)draggingEnteredInSearchPlaces:(id <NSDraggingInfo>)sender
+- (NSDragOperation)draggingEnteredInSearchPlaces:(id <NSDraggingInfo>)sender
 {
   if (usesSearchPlaces) {
 	  NSPasteboard *pb = [sender draggingPasteboard];
@@ -567,7 +528,7 @@ static Finder *finder = nil;
   return NSDragOperationNone;
 }
 
-- (unsigned int)draggingUpdatedInSearchPlaces:(id <NSDraggingInfo>)sender
+- (NSDragOperation)draggingUpdatedInSearchPlaces:(id <NSDraggingInfo>)sender
 {
 	if (splacesDndTarget && usesSearchPlaces) {
 		return [sender draggingSourceOperationMask];
@@ -606,7 +567,7 @@ static Finder *finder = nil;
 	[openPanel setCanChooseFiles: YES];
 	[openPanel setCanChooseDirectories: YES];
 
-	result = [openPanel runModalForDirectory: fixPath(@"/", 0) 
+	result = [openPanel runModalForDirectory: systemRoot() 
                                       file: nil 
 							                       types: nil];
 	if (result != NSOKButton) {
@@ -876,8 +837,7 @@ static Finder *finder = nil;
       } else if ([node isPackage]) {
         [gworkspace openFile: [node path]];
       } else {
-        [gworkspace selectFile: [node path] 
-                inFileViewerRootedAtPath: [node parentPath]];
+        [gworkspace newViewerAtPath: [node path]];
       }        
     } else if ([node isPlain] || [node isExecutable]) {
       [gworkspace openFile: [node path]];    
@@ -885,7 +845,6 @@ static Finder *finder = nil;
       [ws launchApplication: [node path]];    
     }
   }
-
 }
 
 - (void)fileSystemWillChange:(NSNotification *)notif
@@ -1041,7 +1000,7 @@ static Finder *finder = nil;
   NSDictionary *info = [notif object];
   NSString *event = [info objectForKey: @"event"];
 
-  if ([event isEqual: @"GWWatchedFileDeleted"]) {
+  if ([event isEqual: @"GWWatchedPathDeleted"]) {
     LSFolder *folder = [self lsfolderWithPath: [info objectForKey: @"path"]];
     
     if (folder) {
@@ -1054,7 +1013,7 @@ static Finder *finder = nil;
 - (void)tile
 {
   NSRect wrect = [win frame];
-  NSRect mbrect = [modulesBox frame];
+  NSRect mbrect = [modulesBox bounds];
   int count = [fmviews count];
   float hspace = (count * FMVIEWH) + HMARGIN + BORDER;
   int i;
@@ -1067,7 +1026,7 @@ static Finder *finder = nil;
     [win setFrame: wrect display: NO];
   }
 
-  mbrect = [modulesBox frame];
+  mbrect = [modulesBox bounds];
   
   for (i = 0; i < count; i++) {  
     FindModuleView *fmview = [fmviews objectAtIndex: i];
