@@ -1,6 +1,6 @@
 /* LSFolder.m
  *  
- * Copyright (C) 2004 Free Software Foundation, Inc.
+ * Copyright (C) 2004-2010 Free Software Foundation, Inc.
  *
  * Author: Enrico Sersale <enrico@imago.ro>
  * Date: October 2004
@@ -19,21 +19,21 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
  */
 
-#include <Foundation/Foundation.h>
-#include <AppKit/AppKit.h>
-#include <math.h>
-#include "LSFolder.h"
-#include "ResultsTableView.h"
-#include "FSNTextCell.h"
-#include "ResultsPathsView.h"
-#include "LSFEditor.h"
-#include "Finder.h"
-#include "FinderModulesProtocol.h"
-#include "GWorkspace.h"
-#include "GWFunctions.h"
+#import <Foundation/Foundation.h>
+#import <AppKit/AppKit.h>
+#import <math.h>
+#import "LSFolder.h"
+#import "ResultsTableView.h"
+#import "FSNTextCell.h"
+#import "FSNPathComponentsViewer.h"
+#import "LSFEditor.h"
+#import "Finder.h"
+#import "FinderModulesProtocol.h"
+#import "GWorkspace.h"
+#import "GWFunctions.h"
 
 #define CELLS_HEIGHT (28.0)
 
@@ -65,11 +65,11 @@ BOOL isPathInResults(NSString *path, NSArray *results);
     [gworkspace removeWatcherForPath: [node path]];
   }
   
-  TEST_RELEASE (node);
-  TEST_RELEASE (lsfinfo);
-  TEST_RELEASE (win);
-  TEST_RELEASE (foundObjects);
-  TEST_RELEASE (editor);      
+  RELEASE (node);
+  RELEASE (lsfinfo);
+  RELEASE (win);
+  RELEASE (foundObjects);
+  RELEASE (editor);      
   RELEASE (elementsStr);
   DESTROY (conn);
    
@@ -202,7 +202,7 @@ BOOL isPathInResults(NSString *path, NSArray *results);
       needupdate = YES;
     }
     [win makeKeyAndOrderFront: nil];
-    visibleRows = (int)([resultsScroll frame].size.height / CELLS_HEIGHT + 1);
+    visibleRows = (int)([resultsScroll bounds].size.height / CELLS_HEIGHT + 1);
   } else {  
     needupdate = YES;
   }
@@ -229,7 +229,6 @@ BOOL isPathInResults(NSString *path, NSArray *results);
 - (void)startUpdater
 {
   NSString *cname;
-  NSArray *paths;
   NSString *cmd;
 
   cname = [NSString stringWithFormat: @"search_%i", (unsigned long)self];
@@ -271,8 +270,7 @@ BOOL isPathInResults(NSString *path, NSArray *results);
 																 userInfo: nil 
                                   repeats: NO];
 
-  paths = NSSearchPathForDirectoriesInDomains(GSToolsDirectory, NSSystemDomainMask, YES);
-  cmd = [[paths objectAtIndex: 0] stringByAppendingPathComponent: @"lsfupdater"];
+  cmd = [NSTask launchPathForTool: @"lsfupdater"];
 
   [NSTask launchedTaskWithLaunchPath: cmd 
                            arguments: [NSArray arrayWithObject: cname]];
@@ -368,7 +366,7 @@ BOOL isPathInResults(NSString *path, NSArray *results);
   [elementsLabel setStringValue: [NSString stringWithFormat: @"%i %@", 
                                         [foundObjects count], elementsStr]];
   [resultsView noteNumberOfRowsChanged];
-  [pathsView showComponentsOfSelection: [self selectedObjects]];
+  [pathViewer showComponentsOfSelection: [self selectedObjects]];
   RELEASE (pool);
 }
 
@@ -378,7 +376,7 @@ BOOL isPathInResults(NSString *path, NSArray *results);
   [elementsLabel setStringValue: [NSString stringWithFormat: @"%i %@", 
                                         [foundObjects count], elementsStr]];
   [resultsView noteNumberOfRowsChanged];
-  [pathsView showComponentsOfSelection: nil];
+  [pathViewer showComponentsOfSelection: nil];
 }
 
 - (void)endUpdate
@@ -453,7 +451,6 @@ BOOL isPathInResults(NSString *path, NSArray *results);
     NSArray *items;
     id entry;
     NSRect r;
-    int srh;
     int i;
     
     if (sizesDict) {
@@ -466,11 +463,12 @@ BOOL isPathInResults(NSString *path, NSArray *results);
     
     [win setTitle: [node name]];
     [win setReleasedWhenClosed: NO];
+    [win setAcceptsMouseMovedEvents: YES];
     [win setDelegate: self];
 
     progView = [[ProgrView alloc] initWithFrame: NSMakeRect(0, 0, 16, 16)
                                 refreshInterval: 0.1];
-    [(NSBox *)progBox setContentView: progView]; 
+    [progBox setContentView: progView]; 
     RELEASE (progView);
 
     [elementsLabel setStringValue: @""];
@@ -517,13 +515,11 @@ BOOL isPathInResults(NSString *path, NSArray *results);
 
     [updateButt setTitle: NSLocalizedString(@"Update now", @"")];
 
-    [splitView setDelegate: self];
-
     [resultsScroll setBorderType: NSBezelBorder];
     [resultsScroll setHasHorizontalScroller: YES];
     [resultsScroll setHasVerticalScroller: YES]; 
     
-    r = [[resultsScroll contentView] frame];
+    r = [[resultsScroll contentView] bounds];
 
     resultsView = [[ResultsTableView alloc] initWithFrame: r];
     [resultsView setDrawsGrid: NO];
@@ -584,7 +580,7 @@ BOOL isPathInResults(NSString *path, NSArray *results);
     [kindColumn setDataCell: AUTORELEASE ([[FSNTextCell alloc] init])];
     [kindColumn setEditable: NO];
     [kindColumn setResizable: YES];
-    [[kindColumn headerCell] setStringValue: NSLocalizedString(@"Kind", @"")];
+    [[kindColumn headerCell] setStringValue: NSLocalizedString(@"Type", @"")];
     [[kindColumn headerCell] setAlignment: NSLeftTextAlignment];
     [kindColumn setMinWidth: 80];
     [kindColumn setWidth: 80];
@@ -643,34 +639,10 @@ BOOL isPathInResults(NSString *path, NSArray *results);
       [self setCurrentOrder: FSNInfoNameType];
     }
     
-    r = [pathsScroll frame];
-    srh = 0;
-    
-    if (sizesDict) {
-      entry = [sizesDict objectForKey: @"paths_scr_h"];
-  
-      if (entry) {
-        srh = [entry intValue];
-      }
-    }
-    
-    if (srh == 0) {
-      srh = [finder searchResultsHeight];
-    }
-
-    if (srh != 0) {
-      r.size.height = srh;
-      [pathsScroll setFrame: r];
-    } 
-
-    [pathsScroll setBorderType: NSBezelBorder];
-    [pathsScroll setHasHorizontalScroller: NO];
-    [pathsScroll setHasVerticalScroller: YES]; 
-
-    r = [[pathsScroll contentView] frame];
-    pathsView = [[ResultsPathsView alloc] initWithFrame: r];
-    [pathsScroll setDocumentView: pathsView];
-    RELEASE (pathsView);
+    r = [[pathBox contentView] bounds];
+    pathViewer = [[FSNPathComponentsViewer alloc] initWithFrame: r];
+    [pathBox setContentView: pathViewer];
+    RELEASE (pathViewer);
 
     [[NSDistributedNotificationCenter defaultCenter] addObserver: self
                         selector: @selector(fileSystemDidChange:) 
@@ -728,9 +700,6 @@ BOOL isPathInResults(NSString *path, NSArray *results);
                     forKey: @"editor_win"];
     }
 
-    [sizesDict setObject: [NSNumber numberWithInt: ceil(NSHeight([pathsScroll frame]))] 
-                  forKey: @"paths_scr_h"];
-
     [sizesDict setObject: [NSNumber numberWithInt: currentOrder] 
                   forKey: @"sorting_order"];
 
@@ -789,7 +758,7 @@ BOOL isPathInResults(NSString *path, NSArray *results);
   [foundObjects sortUsingSelector: sortingSel];
   [resultsView setHighlightedTableColumn: column];
   [resultsView reloadData];
-  [pathsView showComponentsOfSelection: [self selectedObjects]];
+  [pathViewer showComponentsOfSelection: [self selectedObjects]];
 }
 
 - (void)setCurrentOrder:(FSNInfoType)order
@@ -949,7 +918,7 @@ BOOL isPathInResults(NSString *path, NSArray *results);
   NSArray *selected = [self selectedObjects];
 
   if ([selected count]) {
-    [finder foundSelectionChanged: [FSNode pathsOfNodes: selected]];
+    [finder foundSelectionChanged: selected];
   }
 }
 
@@ -970,73 +939,6 @@ BOOL isPathInResults(NSString *path, NSArray *results);
 }
 
 
-//
-// NSSplitView delegate methods
-//
-#define MIN_PATHS_H 40
-#define MIN_RESULTS_H 80
-
-- (void)splitView:(NSSplitView *)sender 
-                  resizeSubviewsWithOldSize:(NSSize)oldSize
-{
-  NSRect spBounds = [splitView bounds];
-  float dvt = [splitView dividerThickness];
-  float ptsHeight = NSHeight([pathsScroll frame]);
-  float	resHeight;
-  NSRect newFrame;
-  
-  if ((ptsHeight + dvt) > (NSHeight(spBounds) - MIN_RESULTS_H)) {
-    ptsHeight = MIN_PATHS_H;
-  }
-  
-  resHeight = NSHeight(spBounds) - ptsHeight - dvt;
-  
-  newFrame = NSMakeRect(0, 0, NSWidth(spBounds), resHeight);
-  [resultsScroll setFrame: newFrame];
-
-  newFrame = NSMakeRect(0, resHeight + dvt, NSWidth(spBounds), ptsHeight);
-  [pathsScroll setFrame: newFrame];
-  
-  [resultsView sizeLastColumnToFit];
-  
-  [finder setSearchResultsHeight: ceil(NSHeight([pathsScroll frame]))];  
-}
-
-- (float)splitView:(NSSplitView *)sender 
-            constrainMaxCoordinate:(float)proposedMax 
-                       ofSubviewAt:(int)offset
-{
-  float spHeight = NSHeight([splitView bounds]);
-  float dvt = [splitView dividerThickness];
-
-  if (proposedMax > (spHeight - dvt - MIN_PATHS_H)) {
-    return (spHeight - dvt - MIN_PATHS_H);
-  }
-  
-  return proposedMax;
-}
-
-- (float)splitView:(NSSplitView *)sender 
-            constrainMinCoordinate:(float)proposedMin 
-                       ofSubviewAt:(int)offset
-{
-  if (proposedMin < MIN_RESULTS_H) {
-    return MIN_RESULTS_H;
-  }
-
-  return proposedMin;
-}
-
-- (void)splitViewDidResizeSubviews:(NSNotification *)aNotification
-{
-  visibleRows = (int)([resultsScroll frame].size.height / CELLS_HEIGHT + 1);
-  if ([foundObjects count] <= visibleRows) {
-    [resultsView noteNumberOfRowsChanged];
-  }
-  [finder setSearchResultsHeight: ceil(NSHeight([pathsScroll frame]))]; 
-}
-
-
 //
 // NSTableDataSource protocol
 //
@@ -1106,10 +1008,10 @@ BOOL isPathInResults(NSString *path, NSArray *results);
 {
   NSArray *selected = [self selectedObjects];
 
-  [pathsView showComponentsOfSelection: selected];
+  [pathViewer showComponentsOfSelection: selected];
   
   if ([selected count]) {
-    [finder foundSelectionChanged: [FSNode pathsOfNodes: selected]];
+    [finder foundSelectionChanged: selected];
   }
 }
 
