@@ -1,6 +1,6 @@
 /* SearchResults.m
  *  
- * Copyright (C) 2004 Free Software Foundation, Inc.
+ * Copyright (C) 2004-2010 Free Software Foundation, Inc.
  *
  * Author: Enrico Sersale <enrico@imago.ro>
  * Date: March 2004
@@ -19,7 +19,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
  */
 
 #include <Foundation/Foundation.h>
@@ -28,11 +28,11 @@
 #include "SearchResults.h"
 #include "ResultsTableView.h"
 #include "FSNTextCell.h"
-#include "ResultsPathsView.h"
 #include "Finder.h"
 #include "FinderModulesProtocol.h"
 #include "FSNode.h"
 #include "FSNodeRep.h"
+#include "FSNPathComponentsViewer.h"
 #include "GWFunctions.h"
 #include "Dialogs/Dialogs.h"
 
@@ -58,10 +58,10 @@ static NSString *lsfname = @"LiveSearch.lsf";
     DESTROY (toolConn);
   }
 
-  TEST_RELEASE (win);
-  TEST_RELEASE (searchCriteria);
-  TEST_RELEASE (foundObjects);
-  TEST_RELEASE (searchPaths);
+  RELEASE (win);
+  RELEASE (searchCriteria);
+  RELEASE (foundObjects);
+  RELEASE (searchPaths);
   RELEASE (elementsStr);
   DESTROY (conn);
         
@@ -70,13 +70,12 @@ static NSString *lsfname = @"LiveSearch.lsf";
 
 - (id)init
 {
-	self = [super init];
+  self = [super init];
 
   if (self) {
     NSUserDefaults *defaults;
     id entry;
     NSRect r;
-    int srh;
       
 		if ([NSBundle loadNibNamed: nibName owner: self] == NO) {
       NSLog(@"failed to load %@!", nibName);
@@ -85,16 +84,17 @@ static NSString *lsfname = @"LiveSearch.lsf";
     }
         
     [win setFrameUsingName: @"search_results"];
+    [win setAcceptsMouseMovedEvents: YES];
     [win setDelegate: self];
     
     progView = [[ProgressView alloc] initWithFrame: NSMakeRect(0, 0, 16, 16)
                                    refreshInterval: 0.1];
-    [(NSBox *)progBox setContentView: progView]; 
+    [progBox setContentView: progView]; 
     RELEASE (progView);
     
-    r = [[(NSBox *)dragIconBox contentView] frame];
+    r = [[dragIconBox contentView] bounds];
     documentIcon = [[DocumentIcon alloc] initWithFrame: r searchResult: self];
-    [(NSBox *)dragIconBox setContentView: documentIcon]; 
+    [dragIconBox setContentView: documentIcon]; 
     RELEASE (documentIcon);
     
     [elementsLabel setStringValue: @""];
@@ -104,14 +104,12 @@ static NSString *lsfname = @"LiveSearch.lsf";
     [stopButt setEnabled: NO];
     [restartButt setImage: [NSImage imageNamed: @"magnify_small"]];
     [restartButt setEnabled: NO];
-    
-    [splitView setDelegate: self];
-    
+        
     [resultsScroll setBorderType: NSBezelBorder];
     [resultsScroll setHasHorizontalScroller: YES];
     [resultsScroll setHasVerticalScroller: YES]; 
     
-    r = [[resultsScroll contentView] frame];
+    r = [[resultsScroll contentView] bounds];
 
     resultsView = [[ResultsTableView alloc] initWithFrame: r];
     [resultsView setDrawsGrid: NO];
@@ -172,7 +170,7 @@ static NSString *lsfname = @"LiveSearch.lsf";
     [kindColumn setDataCell: AUTORELEASE ([[FSNTextCell alloc] init])];
     [kindColumn setEditable: NO];
     [kindColumn setResizable: YES];
-    [[kindColumn headerCell] setStringValue: NSLocalizedString(@"Kind", @"")];
+    [[kindColumn headerCell] setStringValue: NSLocalizedString(@"Type", @"")];
     [[kindColumn headerCell] setAlignment: NSLeftTextAlignment];
     [kindColumn setMinWidth: 80];
     [kindColumn setWidth: 80];
@@ -199,26 +197,13 @@ static NSString *lsfname = @"LiveSearch.lsf";
     } else {
       [self setCurrentOrder: FSNInfoNameType];
     }
+        
+    r = [[pathBox contentView] bounds];
+    pathViewer = [[FSNPathComponentsViewer alloc] initWithFrame: r];
+    [pathBox setContentView: pathViewer];
+    RELEASE (pathViewer);
 
-    finder = [Finder finder];
-        
-    r = [pathsScroll frame];
-    srh = [finder searchResultsHeight];
-    
-    if (srh != 0) {
-      r.size.height = srh;
-      [pathsScroll setFrame: r];
-    } 
-        
-    [pathsScroll setBorderType: NSBezelBorder];
-    [pathsScroll setHasHorizontalScroller: NO];
-    [pathsScroll setHasVerticalScroller: YES]; 
-
-    r = [[pathsScroll contentView] frame];
-    pathsView = [[ResultsPathsView alloc] initWithFrame: r];
-    [pathsScroll setDocumentView: pathsView];
-    RELEASE (pathsView);
-        
+    finder = [Finder finder];        
     fm = [NSFileManager defaultManager];
     ws = [NSWorkspace sharedWorkspace];
     nc = [NSNotificationCenter defaultCenter];
@@ -237,11 +222,10 @@ static NSString *lsfname = @"LiveSearch.lsf";
                    recursive:(BOOL)rec
 {
   NSString *cname;
-  NSArray *paths;
   NSString *cmd;
 
   [win makeKeyAndOrderFront: nil];
-  visibleRows = (int)([resultsScroll frame].size.height / CELLS_HEIGHT + 1);
+  visibleRows = (int)([resultsScroll bounds].size.height / CELLS_HEIGHT + 1);
   ASSIGN (searchPaths, selection);
   ASSIGN (searchCriteria, criteria);
   recursive = rec;
@@ -285,8 +269,7 @@ static NSString *lsfname = @"LiveSearch.lsf";
 																 userInfo: nil 
                                   repeats: NO];
 
-  paths = NSSearchPathForDirectoriesInDomains(GSToolsDirectory, NSSystemDomainMask, YES);
-  cmd = [[paths objectAtIndex: 0] stringByAppendingPathComponent: @"searchtool"];
+  cmd = [NSTask launchPathForTool: @"searchtool"];
                                           
   [NSTask launchedTaskWithLaunchPath: cmd 
                            arguments: [NSArray arrayWithObject: cname]];
@@ -413,6 +396,7 @@ static NSString *lsfname = @"LiveSearch.lsf";
 - (IBAction)restartSearch:(id)sender
 {
   if (searchtool == nil) {
+    [pathViewer showComponentsOfSelection: nil];
     [foundObjects removeAllObjects];
     [resultsView reloadData];
     [self activateForSelection: searchPaths
@@ -723,7 +707,7 @@ static NSString *lsfname = @"LiveSearch.lsf";
   NSArray *selected = [self selectedObjects];
 
   if ([selected count]) {
-    [finder foundSelectionChanged: [FSNode pathsOfNodes: selected]];
+    [finder foundSelectionChanged: selected];
   }
 }
 
@@ -742,73 +726,6 @@ static NSString *lsfname = @"LiveSearch.lsf";
 }
 
 
-//
-// NSSplitView delegate methods
-//
-#define MIN_PATHS_H 40
-#define MIN_RESULTS_H 80
-
-- (void)splitView:(NSSplitView *)sender 
-                  resizeSubviewsWithOldSize:(NSSize)oldSize
-{
-  NSRect spBounds = [splitView bounds];
-  float dvt = [splitView dividerThickness];
-  float ptsHeight = NSHeight([pathsScroll frame]);
-  float	resHeight;
-  NSRect newFrame;
-  
-  if ((ptsHeight + dvt) > (NSHeight(spBounds) - MIN_RESULTS_H)) {
-    ptsHeight = MIN_PATHS_H;
-  }
-  
-  resHeight = NSHeight(spBounds) - ptsHeight - dvt;
-  
-  newFrame = NSMakeRect(0, 0, NSWidth(spBounds), resHeight);
-  [resultsScroll setFrame: newFrame];
-
-  newFrame = NSMakeRect(0, resHeight + dvt, NSWidth(spBounds), ptsHeight);
-  [pathsScroll setFrame: newFrame];
-  
-  [resultsView sizeLastColumnToFit];
-  
-  [finder setSearchResultsHeight: ceil(NSHeight([pathsScroll frame]))];  
-}
-
-- (float)splitView:(NSSplitView *)sender 
-            constrainMaxCoordinate:(float)proposedMax 
-                       ofSubviewAt:(int)offset
-{
-  float spHeight = NSHeight([splitView bounds]);
-  float dvt = [splitView dividerThickness];
-
-  if (proposedMax > (spHeight - dvt - MIN_PATHS_H)) {
-    return (spHeight - dvt - MIN_PATHS_H);
-  }
-  
-  return proposedMax;
-}
-
-- (float)splitView:(NSSplitView *)sender 
-            constrainMinCoordinate:(float)proposedMin 
-                       ofSubviewAt:(int)offset
-{
-  if (proposedMin < MIN_RESULTS_H) {
-    return MIN_RESULTS_H;
-  }
-
-  return proposedMin;
-}
-
-- (void)splitViewDidResizeSubviews:(NSNotification *)aNotification
-{
-  visibleRows = (int)([resultsScroll frame].size.height / CELLS_HEIGHT + 1);
-  if ([foundObjects count] <= visibleRows) {
-    [resultsView noteNumberOfRowsChanged];
-  }
-  [finder setSearchResultsHeight: ceil(NSHeight([pathsScroll frame]))]; 
-}
-
-
 //
 // NSTableDataSource protocol
 //
@@ -878,10 +795,10 @@ static NSString *lsfname = @"LiveSearch.lsf";
 {
   NSArray *selected = [self selectedObjects];
 
-  [pathsView showComponentsOfSelection: selected];
+  [pathViewer showComponentsOfSelection: selected];
   
   if ([selected count]) {
-    [finder foundSelectionChanged: [FSNode pathsOfNodes: selected]];
+    [finder foundSelectionChanged: selected];
   }
 }
 
@@ -1070,7 +987,7 @@ static NSString *lsfname = @"LiveSearch.lsf";
     }
 
     if (startdnd == YES) {  
-      [self startExternalDragOnEvent: nextEvent];    
+      [self startExternalDragOnEvent: theEvent];    
     } 
   }              
 }
@@ -1084,7 +1001,6 @@ static NSString *lsfname = @"LiveSearch.lsf";
 
 - (void)startExternalDragOnEvent:(NSEvent *)event
 {
-  NSPoint dragPoint = [event locationInWindow];
   NSPasteboard *pb = [NSPasteboard pasteboardWithName: NSDragPboard];	
   NSArray *dndtypes = [NSArray arrayWithObject: @"GWLSFolderPboardType"];
   NSMutableDictionary *pbDict = [NSMutableDictionary dictionary];
@@ -1101,7 +1017,7 @@ static NSString *lsfname = @"LiveSearch.lsf";
   [pb setData: pbData forType: @"GWLSFolderPboardType"];
 
   [self dragImage: icon
-               at: [self convertPoint: dragPoint fromView: nil] 
+               at: NSZeroPoint 
            offset: NSZeroSize
             event: event
        pasteboard: pb
