@@ -1,6 +1,6 @@
 /* GWorkspace.h
  *  
- * Copyright (C) 2003 Free Software Foundation, Inc.
+ * Copyright (C) 2003-2010 Free Software Foundation, Inc.
  *
  * Author: Enrico Sersale <enrico@imago.ro>
  * Date: August 2001
@@ -19,25 +19,32 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
  */
 
 #ifndef GWORKSPACE_H
 #define GWORKSPACE_H
 
-#include <Foundation/Foundation.h>
-#include <AppKit/NSApplication.h>
-#include "GWProtocol.h"
+#import <Foundation/Foundation.h>
+#import <AppKit/NSApplication.h>
+#import <AppKit/NSWorkspace.h>
 
 #define NOEDIT 0
 #define NOXTERM 1
 
+#if !defined(OPEN_MAX) || OPEN_MAX > 255
+#define OPEN_MAX 10
+#endif
+
+
 @class NSWorkspace;
+@class FSNode;
 @class FSNodeRep;
 @class GWViewersManager;
 @class GWDesktopManager;
 @class Finder;
 @class Inspector;
+@class Operation;
 @class GWViewer;
 @class PrefController;
 @class Fiend;
@@ -46,17 +53,21 @@
 @class OpenWithController;
 @class RunExternalController;
 @class StartAppWin;
+@class GWLaunchedApp;
 
 @protocol	FSWClientProtocol
 
-- (void)watchedPathDidChange:(NSData *)dirinfo;
+- (oneway void)watchedPathDidChange:(NSData *)dirinfo;
+
+- (oneway void)globalWatchedPathDidChange:(NSDictionary *)dirinfo;
 
 @end
 
 
 @protocol	FSWatcherProtocol
 
-- (oneway void)registerClient:(id <FSWClientProtocol>)client;
+- (oneway void)registerClient:(id <FSWClientProtocol>)client
+              isGlobalWatcher:(BOOL)global;
 
 - (oneway void)unregisterClient:(id <FSWClientProtocol>)client;
 
@@ -76,21 +87,8 @@
 @end
 
 
-@protocol	OperationProtocol
-
-- (oneway void)performOperation:(NSData *)opinfo;
-
-- (oneway void)setFilenamesCutted:(BOOL)value;
-
-- (BOOL)filenamesWasCutted;
-
-@end
-
-
 @protocol	DDBdProtocol
 
-- (BOOL)dbactive;
-
 - (oneway void)insertPath:(NSString *)path;
 
 - (oneway void)removePath:(NSString *)path;
@@ -100,22 +98,38 @@
 - (oneway void)setAnnotations:(NSString *)annotations
                       forPath:(NSString *)path;
 
+- (oneway void)fileSystemDidChange:(NSData *)info;
+
 @end
 
 
-@interface GWorkspace : NSObject <GWProtocol, FSWClientProtocol>
+@protocol	MDExtractorProtocol
+
+@end
+
+/* The protocol of the remote dnd source */
+@protocol GWRemoteFilesDraggingInfo
+- (oneway void)remoteDraggingDestinationReply:(NSData *)reply;
+@end 
+
+
+@interface GWorkspace : NSObject <FSWClientProtocol>
 {	
   FSNodeRep *fsnodeRep;
   
 	NSArray *selectedPaths;
-
+  NSMutableArray *trashContents;
+  NSString *trashPath;
+  
   id fswatcher;
   BOOL fswnotifications;
-	
-  id operationsApp;
+	NSCountedSet *watchedPaths;
+  
   id recyclerApp;
+  BOOL recyclerCanQuit;
   
   id ddbd;
+  id mdextractor;
   
   PrefController *prefController;
   Fiend *fiend;
@@ -127,12 +141,8 @@
   GWDesktopManager *dtopManager;  
   Inspector *inspector;
   Finder *finder;
+  Operation *fileOpsManager;
   
-  BOOL animateChdir;
-  BOOL animateSlideBack;
-
-  BOOL contestualMenu;
-
   BOOL dontWarnOnQuit;
   BOOL terminating;
   
@@ -144,23 +154,40 @@
   RunExternalController *runExtController;
   
   StartAppWin *startAppWin;
-    	      
-  int shelfCellsWidth;
-
+  
+  NSString *gwProcessName;  	      
+  NSString *gwBundlePath;  	      
 	NSString *defEditor;
   NSString *defXterm;
   NSString *defXtermArgs;
   BOOL teminalService;
           
   NSFileManager *fm;
-  NSWorkspace *ws;
+
+  //
+  // WorkspaceApplication
+  //
+  NSWorkspace *ws;  
+  NSNotificationCenter *wsnc; 
+  
+  NSMutableArray *launchedApps;
+  GWLaunchedApp *activeApplication;
+  
+  NSString *storedAppinfoPath;
+  NSDistributedLock *storedAppinfoLock;
+  
+  NSTimer *logoutTimer;
+  BOOL loggingout;
+  int autoLogoutDelay;
+  int maxLogoutDelay;  
+  int logoutDelay;
 }
 
++ (GWorkspace *)gworkspace;
+
 + (void)registerForServices;
 
-- (void)applicationDidFinishLaunching:(NSNotification *)aNotification;
-
-- (BOOL)applicationShouldTerminate:(NSApplication *)app;
+- (void)createMenu;
 
 - (NSString *)defEditor;
 
@@ -184,6 +211,10 @@
 
 - (id)rootViewer;
 
+- (void)showRootViewer;
+
+- (void)rootViewerSelectFiles:(NSArray *)paths;
+
 - (void)newViewerAtPath:(NSString *)path;
 
 - (void)changeDefaultEditor:(NSNotification *)notif;
@@ -192,8 +223,14 @@
                  arguments:(NSString *)args;
              
 - (void)setUseTerminalService:(BOOL)value;             
-                             
+
+- (NSString *)gworkspaceProcessName;
+
 - (void)updateDefaults;
+
+- (void)setContextHelp;
+
+- (NSAttributedString *)contextHelpFromName:(NSString *)fileName;
 					 
 - (void)startXTermOnDirectory:(NSString *)dirPath;
 
@@ -201,17 +238,11 @@
 
 - (void)setDefaultSortType:(int)type;
 
-- (int)shelfCellsWidth; 
-
-- (int)defaultShelfCellsWidth; 
-
-- (void)setShelfCellsWidth:(int)w; 
-
 - (void)createTabbedShelf;
 
-- (void)checkViewersAfterHidingOfPaths:(NSArray *)paths;
+- (TShelfWin *)tabbedShelf;
 
-- (void)iconAnimationChanged:(NSNotification *)notif;
+- (StartAppWin *)startAppWin;
 
 - (void)fileSystemWillChange:(NSNotification *)notif;
 
@@ -223,6 +254,18 @@
 
 - (NSArray *)selectedPaths;
 
+- (void)openSelectedPaths:(NSArray *)paths 
+                newViewer:(BOOL)newv;
+
+- (void)openSelectedPathsWith;
+
+- (BOOL)openFile:(NSString *)fullPath;
+
+- (BOOL)application:(NSApplication *)theApplication 
+           openFile:(NSString *)filename;
+
+- (NSArray *)getSelectedPaths;
+
 - (void)showPasteboardData:(NSData *)data 
                     ofType:(NSString *)type
                   typeIcon:(NSImage *)icon;
@@ -242,6 +285,16 @@
 
 - (void)thumbnailsDidChange:(NSNotification *)notif;
 
+- (void)removableMediaPathsDidChange:(NSNotification *)notif;
+
+- (void)reservedMountNamesDidChange:(NSNotification *)notif;
+
+- (void)hideDotsFileDidChange:(NSNotification *)notif;
+
+- (void)hiddenFilesDidChange:(NSArray *)paths;
+
+- (void)customDirectoryIconDidChange:(NSNotification *)notif;
+
 - (void)applicationForExtensionsDidChange:(NSNotification *)notif;
 
 - (int)maxHistoryCache;
@@ -256,10 +309,6 @@
 
 - (void)recyclerConnectionDidDie:(NSNotification *)notif;
 
-- (void)connectOperation;
-
-- (void)operationConnectionDidDie:(NSNotification *)notif;
-
 - (void)connectDDBd;
 
 - (void)ddbdConnectionDidDie:(NSNotification *)notif;
@@ -275,7 +324,13 @@
 - (void)ddbdSetAnnotations:(NSString *)annotations
                    forPath:(NSString *)path;
 
-- (id)connectApplication:(NSString *)appName;
+- (void)connectMDExtractor;
+
+- (void)mdextractorConnectionDidDie:(NSNotification *)notif;
+
+- (void)slideImage:(NSImage *)image 
+							from:(NSPoint)fromPoint 
+								to:(NSPoint)toPoint;
 
 
 //
@@ -296,6 +351,8 @@
 //
 - (void)closeMainWin:(id)sender;
 
+- (void)logout:(id)sender;
+
 - (void)showInfo:(id)sender;
 
 - (void)showPreferences:(id)sender;
@@ -334,8 +391,6 @@
 
 - (void)hideTShelf:(id)sender;
 
-- (void)maximizeMinimizeTShelf:(id)sender;
-
 - (void)selectSpecialTShelfTab:(id)sender;
 
 - (void)addTShelfTab:(id)sender;
@@ -344,8 +399,6 @@
 
 - (void)renameTShelfTab:(id)sender;
 
-- (void)openWith:(id)sender;
-
 - (void)runCommand:(id)sender;
 
 - (void)checkRemovableMedia:(id)sender;
@@ -364,17 +417,21 @@
 
 - (void)performFileOperation:(NSDictionary *)opinfo;
 
+- (BOOL)filenamesWasCutted;
+
+- (void)setFilenamesCutted:(BOOL)value;
+
 - (void)lsfolderDragOperation:(NSData *)opinfo
               concludedAtPath:(NSString *)path;
                           
 - (void)concludeRemoteFilesDragOperation:(NSData *)opinfo
                              atLocalPath:(NSString *)localPath;
 
-// - (void)addWatcherForPath:(NSString *)path; // already in GWProtocol
+- (void)addWatcherForPath:(NSString *)path;
 
-// - (void)removeWatcherForPath:(NSString *)path; // already in GWProtocol
+- (void)removeWatcherForPath:(NSString *)path;
 
-// - (NSString *)trashPath; // already in GWProtocol
+- (NSString *)trashPath;
 
 - (id)workspaceApplication;
 
@@ -382,4 +439,174 @@
 
 @end
 
+
+@interface GWorkspace (SharedInspector)
+
+- (oneway void)showExternalSelection:(NSArray *)selection;
+
+@end
+
+
+@interface GWorkspace (WorkspaceApplication)
+
+- (BOOL)performFileOperation:(NSString *)operation 
+                      source:(NSString *)source 
+                 destination:(NSString *)destination 
+                       files:(NSArray *)files 
+                         tag:(int *)tag;
+
+- (BOOL)selectFile:(NSString *)fullPath
+							inFileViewerRootedAtPath:(NSString *)rootFullpath;
+
+- (int)extendPowerOffBy:(int)requested;
+
+- (NSArray *)launchedApplications;
+
+- (NSDictionary *)activeApplication;
+
+- (BOOL)openFile:(NSString *)fullPath
+          withApplication:(NSString *)appname
+            andDeactivate:(BOOL)flag;
+
+- (BOOL)launchApplication:(NSString *)appname
+		             showIcon:(BOOL)showIcon
+	             autolaunch:(BOOL)autolaunch;
+
+- (BOOL)openTempFile:(NSString *)fullPath;
+
+@end
+
+
+@interface GWorkspace (Applications)
+
+- (void)initializeWorkspace;
+
+- (void)applicationName:(NSString **)appName
+                andPath:(NSString **)appPath
+                forName:(NSString *)name;
+
+- (BOOL)launchApplication:(NSString *)appname
+		            arguments:(NSArray *)args;
+
+- (void)appWillLaunch:(NSNotification *)notif;
+
+- (void)appDidLaunch:(NSNotification *)notif;
+
+- (void)appDidTerminate:(NSNotification *)notif;
+
+- (void)appDidBecomeActive:(NSNotification *)notif;
+
+- (void)appDidResignActive:(NSNotification *)notif;
+
+- (void)activateAppWithPath:(NSString *)path
+                    andName:(NSString *)name;
+
+- (void)appDidHide:(NSNotification *)notif;
+
+- (void)appDidUnhide:(NSNotification *)notif;
+
+- (void)unhideAppWithPath:(NSString *)path
+                  andName:(NSString *)name;
+
+- (void)applicationTerminated:(GWLaunchedApp *)app;
+
+- (GWLaunchedApp *)launchedAppWithPath:(NSString *)path
+                               andName:(NSString *)name;
+
+- (NSArray *)storedAppInfo;
+
+- (void)updateStoredAppInfoWithLaunchedApps:(NSArray *)apps;
+
+- (void)checkLastRunningApps;
+
+- (void)startLogout;
+
+- (void)doLogout:(id)sender;
+
+- (void)terminateTasks:(id)sender;
+
+@end
+
+
+@interface GWLaunchedApp : NSObject
+{
+  NSTask *task;
+  NSString *name;
+  NSString *path;
+  NSNumber *identifier;
+  NSConnection *conn;
+  id application;
+  BOOL active;
+  BOOL hidden;
+  
+  GWorkspace *gw;   
+  NSNotificationCenter *nc;
+}
+
++ (id)appWithApplicationPath:(NSString *)apath
+             applicationName:(NSString *)aname
+                launchedTask:(NSTask *)atask;
+
++ (id)appWithApplicationPath:(NSString *)apath
+             applicationName:(NSString *)aname
+           processIdentifier:(NSNumber *)ident
+                checkRunning:(BOOL)check;
+            
+- (NSDictionary *)appInfo;
+
+- (void)setTask:(NSTask *)atask;
+
+- (NSTask *)task;
+
+- (void)setPath:(NSString *)apath;
+
+- (NSString *)path;
+
+- (void)setName:(NSString *)aname;
+
+- (NSString *)name;
+
+- (void)setIdentifier:(NSNumber *)ident;
+
+- (NSNumber *)identifier;
+
+- (id)application;
+
+- (void)setActive:(BOOL)value;
+
+- (BOOL)isActive;
+
+- (void)activateApplication;
+
+- (void)setHidden:(BOOL)value;
+
+- (BOOL)isHidden;
+
+- (void)hideApplication;
+
+- (void)unhideApplication;
+
+- (BOOL)isApplicationHidden;
+
+- (BOOL)gwlaunched;
+
+- (BOOL)isRunning;
+
+- (void)terminateApplication;
+
+- (void)terminateTask;
+
+- (void)connectApplication:(BOOL)showProgress;
+
+- (void)connectionDidDie:(NSNotification *)notif;
+
+@end
+
+
+@interface NSWorkspace (WorkspaceApplication)
+
+- (id)_workspaceApplication;
+
+@end
+
 #endif // GWORKSPACE_H
