@@ -1,6 +1,6 @@
 /* History.m
  *  
- * Copyright (C) 2003 Free Software Foundation, Inc.
+ * Copyright (C) 2003-2010 Free Software Foundation, Inc.
  *
  * Author: Enrico Sersale <enrico@imago.ro>
  * Date: August 2001
@@ -19,15 +19,18 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
  */
 
-#include <Foundation/Foundation.h>
-#include <AppKit/AppKit.h>
-#include "History.h"
-#include "GWViewersManager.h"
-#include "GWFunctions.h"
-#include "GNUstep.h"
+#import <Foundation/Foundation.h>
+#import <AppKit/AppKit.h>
+#import <GNUstepBase/GNUstep.h>
+
+#import "History.h"
+#import "GWViewersManager.h"
+#import "GWFunctions.h"
+#import "FSNodeRep.h"
+
 
 @implementation History
 
@@ -62,7 +65,7 @@
     [scrollView setHasHorizontalScroller: YES];
     [scrollView setHasVerticalScroller: YES]; 
   	[scrollView setAutoresizingMask: (NSViewWidthSizable | NSViewHeightSizable)];
-		[scrollView setFrame: [[win contentView] frame]];
+		[scrollView setFrame: [[win contentView] bounds]];
     [win setContentView: scrollView];
     RELEASE (scrollView);
     
@@ -74,8 +77,9 @@
     [matrix setTarget: self];
     [matrix setDoubleAction: @selector(matrixAction:)];
     [matrix setIntercellSpacing: NSZeroSize];
-    ms.width = [[scrollView contentView] frame].size.width;
-    ms.height = [[NSFont systemFontOfSize: 12] defaultLineHeightForFont];
+    ms.width = [[scrollView contentView] bounds].size.width;
+    ms.height = [[FSNodeRep sharedInstance] heighOfFont: [NSFont systemFontOfSize: 12]];
+    
     [matrix setCellSize: ms];
     [matrix setAutoscroll: YES];
 	  [matrix setAllowsEmptySelection: YES];
@@ -164,7 +168,7 @@
 	NSFont *font = [NSFont systemFontOfSize: 12];
 	NSArray *cells = [matrix cells];
 	float mh = [matrix cellSize].height;
-	float maxw = [[scrollView contentView] frame].size.width;
+	float maxw = [[scrollView contentView] bounds].size.width;
 	int i;
 	
 	for (i = 0; i < [cells count]; i++) {
