@@ -1,6 +1,6 @@
 /* HiddenFilesPref.m
  *  
- * Copyright (C) 2003 Free Software Foundation, Inc.
+ * Copyright (C) 2003-2010 Free Software Foundation, Inc.
  *
  * Author: Enrico Sersale <enrico@imago.ro>
  * Date: August 2001
@@ -19,16 +19,19 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
  */
 
-#include <Foundation/Foundation.h>
-#include <AppKit/AppKit.h>
-#include "FSNodeRep.h"
-#include "FSNFunctions.h"
-#include "HiddenFilesPref.h"
-#include "GWorkspace.h"
-#include "GNUstep.h"
+
+#import <Foundation/Foundation.h>
+#import <AppKit/AppKit.h>
+#import <GNUstepBase/GNUstep.h>
+
+#import "FSNodeRep.h"
+#import "FSNFunctions.h"
+#import "HiddenFilesPref.h"
+#import "GWorkspace.h"
+
 
 static NSString *nibName = @"HiddenFilesPref";
 
@@ -162,15 +165,28 @@ if (sz.height < 0) sz.height = 0
       [setDirButt setEnabled: NO];
                                                             
       /* Internationalization */
-      [setButt setTitle: NSLocalizedString(@"Activate changes", @"")];
-      [loadButt setTitle: NSLocalizedString(@"Load", @"")];
-      [hiddenlabel setStringValue: NSLocalizedString(@"Hidden files", @"")];
-      [shownlabel setStringValue: NSLocalizedString(@"Shown files", @"")];
-      [labelinfo setStringValue: NSLocalizedString(@"Select and move the files to hide or to show", @"")];
-      [hiddenDirslabel setStringValue: NSLocalizedString(@"Hidden directories", @"")];
-      [addDirButt setTitle: NSLocalizedString(@"add", @"")];
-      [removeDirButt setTitle: NSLocalizedString(@"remove", @"")];
-      [setDirButt setTitle: NSLocalizedString(@"Activate changes", @"")];
+      {
+        int tabindex;
+        NSTabViewItem *item;
+        
+        tabindex = [tabView indexOfTabViewItemWithIdentifier: @"Files"];
+        item = [tabView tabViewItemAtIndex: tabindex];
+        [item setLabel: NSLocalizedString(@"Files", @"")];
+
+        tabindex = [tabView indexOfTabViewItemWithIdentifier: @"Folders"];
+        item = [tabView tabViewItemAtIndex: tabindex];
+        [item setLabel: NSLocalizedString(@"Folders", @"")];
+
+        [setButt setTitle: NSLocalizedString(@"Activate changes", @"")];
+        [loadButt setTitle: NSLocalizedString(@"Load", @"")];
+        [hiddenlabel setStringValue: NSLocalizedString(@"Hidden files", @"")];
+        [shownlabel setStringValue: NSLocalizedString(@"Shown files", @"")];
+        [labelinfo setStringValue: NSLocalizedString(@"Select and move the files to hide or to show", @"")];
+        [hiddenDirslabel setStringValue: NSLocalizedString(@"Hidden directories", @"")];
+        [addDirButt setTitle: NSLocalizedString(@"add", @"")];
+        [removeDirButt setTitle: NSLocalizedString(@"remove", @"")];
+        [setDirButt setTitle: NSLocalizedString(@"Activate changes", @"")];
+      }
     }
   }
   
@@ -428,6 +444,8 @@ if (sz.height < 0) sz.height = 0
                         nil, nil);
     return;
   } else {
+    NSString *base = [currentNode path];
+    NSMutableArray *paths = [NSMutableArray array];
     NSArray *cells = [leftMatrix cells];
     
     if (cells) {	
@@ -438,14 +456,17 @@ if (sz.height < 0) sz.height = 0
      
       names = [NSMutableArray arrayWithCapacity: 1];    
       for (i = 0; i < [cells count]; i++) {
-        id cell = [cells objectAtIndex: i];      
-        [names addObject: [cell stringValue]];
+        id cell = [cells objectAtIndex: i]; 
+        NSString *name = [cell stringValue];
+        
+        [names addObject: name];
+        [paths addObject: [base stringByAppendingPathComponent: name]];
       }
       
       hconts = [names componentsJoinedByString: @"\n"];      
       h = [[currentNode path] stringByAppendingPathComponent: @".hidden"];
       [hconts writeToFile: h atomically: YES];
-      
+      [gw hiddenFilesDidChange: paths];
       [setButt setEnabled: NO];
     }
   }
@@ -537,7 +558,7 @@ if (sz.height < 0) sz.height = 0
 - (IBAction)activateDirChanges:(id)sender
 {
   [[FSNodeRep sharedInstance] setHiddenPaths: hiddenPaths];
-  [gw checkViewersAfterHidingOfPaths: hiddenPaths];
+  [gw hiddenFilesDidChange: hiddenPaths];
   [setDirButt setEnabled: NO];
 }
 
