@@ -19,7 +19,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
  */
 
 #ifndef TSHELF_ICON_H
@@ -36,6 +36,7 @@ inRect: NSMakeRect(((int)(s1).width - (int)(s2).width) >> 1,\
 @class NSPasteboard;
 @class NSTextField;
 @class NSImage;
+@class NSBezierPath;
 @class NSWorkspace;
 @class TShelfIconsView;
 @class FSNode;
@@ -54,17 +55,18 @@ inRect: NSMakeRect(((int)(s1).width - (int)(s2).width) >> 1,\
   BOOL isSelect;
   BOOL locked;
   
-	NSImage *icon, *highlight;
-	NSTextField *namelabel;
+	NSImage *icon;
+	NSTextField *namelabel;  
+  NSBezierPath *highlightPath;
 	NSPoint position;
 	int gridindex;
   int labelWidth;
-
+  NSTrackingRectTag trectTag;
+  
   TShelfIconsView *tview;
   
   FSNodeRep *fsnodeRep;
   NSFileManager *fm;
-	NSWorkspace *ws;
   GWorkspace *gw;
   
   int dragdelay;
@@ -106,6 +108,8 @@ inRect: NSMakeRect(((int)(s1).width - (int)(s2).width) >> 1,\
 
 - (NSTextField *)myLabel;
 
+- (NSString *)shownName;
+
 - (NSArray *)paths;
 
 - (BOOL)isSinglePath;
@@ -135,9 +139,9 @@ inRect: NSMakeRect(((int)(s1).width - (int)(s2).width) >> 1,\
 
 @interface TShelfIcon (DraggingDestination)
 
-- (unsigned int)draggingEntered:(id <NSDraggingInfo>)sender;
+- (NSDragOperation)draggingEntered:(id <NSDraggingInfo>)sender;
 
-- (unsigned int)draggingUpdated:(id <NSDraggingInfo>)sender;
+- (NSDragOperation)draggingUpdated:(id <NSDraggingInfo>)sender;
 
 - (void)draggingExited:(id <NSDraggingInfo>)sender;
 
