@@ -1,6 +1,6 @@
 /* TShelfIcon.m
  *  
- * Copyright (C) 2003 Free Software Foundation, Inc.
+ * Copyright (C) 2003-2010 Free Software Foundation, Inc.
  *
  * Author: Enrico Sersale <enrico@imago.ro>
  * Date: August 2001
@@ -19,18 +19,20 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
  */
 
-#include <Foundation/Foundation.h>
-#include <AppKit/AppKit.h>
-#include "FSNodeRep.h"
-#include "FSNFunctions.h"
-#include "GWFunctions.h"
-#include "TShelfIcon.h"
-#include "TShelfIconsView.h"
-#include "GWorkspace.h"
-#include "GNUstep.h"
+#import <Foundation/Foundation.h>
+#import <AppKit/AppKit.h>
+#import <GNUstepBase/GNUstep.h>
+
+#import "FSNodeRep.h"
+#import "FSNFunctions.h"
+#import "GWFunctions.h"
+#import "TShelfIcon.h"
+#import "TShelfIconsView.h"
+#import "GWorkspace.h"
+
 
 #define ICON_SIZE 48
 
@@ -41,13 +43,16 @@
 
 - (void)dealloc
 {
+  if (trectTag != -1) {
+    [self removeTrackingRect: trectTag];
+  }
   RELEASE (paths);
   RELEASE (name);
-	TEST_RELEASE (hostname);
-	TEST_RELEASE (node);
-	RELEASE (namelabel);
+  RELEASE (hostname);
+  RELEASE (node);
+  RELEASE (namelabel);
   RELEASE (icon);
-  RELEASE (highlight);
+  RELEASE (highlightPath);
   [super dealloc];
 }
 
@@ -56,17 +61,12 @@
 {
   self = [super init];
   if (self) {
-    NSArray *pbTypes = [NSArray arrayWithObjects: 
-                                              NSFilenamesPboardType, 
-                                              @"GWLSFolderPboardType", 
-                                              @"GWRemoteFilenamesPboardType", 
-                                              nil];
     NSFont *font;
+    NSRect hlightRect;
     int count;
 
     fsnodeRep = [FSNodeRep sharedInstance];
     fm = [NSFileManager defaultManager];
-		ws = [NSWorkspace sharedWorkspace];
     gw = [GWorkspace gworkspace];
 
     [self setFrame: NSMakeRect(0, 0, 64, 52)];
@@ -103,8 +103,15 @@
     } else {
       ASSIGN (icon, [fsnodeRep multipleSelectionIconOfSize: ICON_SIZE]);
     }
-    
-    ASSIGN (highlight, [NSImage imageNamed: @"CellHighlight.tiff"]);
+        
+    hlightRect = NSZeroRect;
+    hlightRect.size.width = (float)ICON_SIZE / 3 * 4;
+    hlightRect.size.height = hlightRect.size.width * [fsnodeRep highlightHeightFactor];
+    if ((hlightRect.size.height - ICON_SIZE) < 4) {
+      hlightRect.size.height = ICON_SIZE + 4;
+    }
+    hlightRect = NSIntegralRect(hlightRect);
+    ASSIGN (highlightPath, [fsnodeRep highlightPathOfSize: hlightRect.size]);
 
 		if (isRootIcon) {
 			NSHost *host = [NSHost currentHost];
@@ -125,17 +132,22 @@
 		[namelabel setEditable: NO];
 		[namelabel setSelectable: NO];
 		[namelabel setAlignment: NSCenterTextAlignment];
-	  [namelabel setBackgroundColor: [NSColor windowBackgroundColor]];
-	  [namelabel setTextColor: [NSColor blackColor]];
+    [namelabel setDrawsBackground: NO];
+	  [namelabel setTextColor: [NSColor controlTextColor]];
 		[self setLabelWidth]; 
     
-    [self registerForDraggedTypes: pbTypes];
+    [self registerForDraggedTypes: [NSArray arrayWithObjects: 
+                                              NSFilenamesPboardType, 
+                                              @"GWLSFolderPboardType", 
+                                              @"GWRemoteFilenamesPboardType", 
+                                              nil]];
     
 		position = NSMakePoint(0, 0);
 		gridindex = -1;
     dragdelay = 0;
     isDragTarget = NO;
     onSelf = NO;
+    trectTag = -1;
   }
   
   return self;
@@ -163,8 +175,8 @@
 {
   int count;
 
-	TEST_RELEASE (paths);
-	TEST_RELEASE (node);
+  RELEASE (paths);
+  RELEASE (node);
 	paths = [[NSMutableArray alloc] initWithCapacity: 1];
 	[paths addObjectsFromArray: fpaths];
   count = [paths count];                    
@@ -205,7 +217,7 @@
 		} 			
 		ASSIGN (hostname, hname);			
   } else {
-		TEST_RELEASE (hostname);
+		RELEASE (hostname);
 		hostname = nil;
 	}
 
@@ -295,6 +307,11 @@
   return namelabel;
 }
 
+- (NSString *)shownName
+{
+  return (isRootIcon ? hostname : name);
+}
+
 - (NSArray *)paths
 {
   return paths;
@@ -338,14 +355,14 @@
 		if (locked == NO) {				
 			[tview openCurrentSelection: paths];
 		} 
-													
     [self unselect];
-    return;
 	}  
 }
 
 - (void)mouseDown:(NSEvent *)theEvent
 {
+  unsigned eventmask = NSAlternateKeyMask | NSCommandKeyMask | NSControlKeyMask;
+
   CHECK_LOCK;
   
 	if ([theEvent clickCount] == 1) { 
@@ -365,16 +382,17 @@
     							              NSLeftMouseUpMask | NSLeftMouseDraggedMask];
 
       if ([nextEvent type] == NSLeftMouseUp) {
-        [tview setCurrentSelection: paths];
+        if ([theEvent modifierFlags] & eventmask) {
+          [tview setCurrentSelection: paths];
+        }
         [self unselect];
         break;
 
       } else if ([nextEvent type] == NSLeftMouseDragged) {
-	      if(dragdelay < 5) {
+	      if (dragdelay < 5) {
           dragdelay++;
         } else {      
           NSPoint p = [nextEvent locationInWindow];
-        
           offset = NSMakeSize(p.x - location.x, p.y - location.y); 
           startdnd = YES;        
           break;
@@ -383,11 +401,37 @@
     }
 
     if (startdnd) {  
-      [self startExternalDragOnEvent: nextEvent withMouseOffset: offset];    
+      [tview setFocusedIcon: nil];
+      [self startExternalDragOnEvent: theEvent withMouseOffset: offset];    
     }    
   }           
 }
 
+- (void)mouseEntered:(NSEvent *)theEvent
+{
+  [tview setFocusedIcon: self];
+}
+
+- (void)mouseExited:(NSEvent *)theEvent
+{
+  [tview setFocusedIcon: nil];
+}
+ 
+- (void)setFrame:(NSRect)rect
+{	
+  NSSize s = [icon size];
+  NSPoint ip = NSMakePoint((rect.size.width - s.width) / 2, (rect.size.height - s.height) / 2);	
+  NSRect ir = NSMakeRect(ip.x, ip.y, s.width, s.height);
+  
+  [super setFrame: rect];	
+	
+  if (trectTag != -1) {
+    [self removeTrackingRect: trectTag];
+  }
+  
+  trectTag = [self addTrackingRect: ir owner: self userData: nil assumeInside: NO]; 
+}
+
 - (NSMenu *)menuForEvent:(NSEvent *)theEvent
 {
   return [super menuForEvent: theEvent];
@@ -399,7 +443,8 @@
   NSSize s;
       	
 	if(isSelect) {
-		[highlight compositeToPoint: NSZeroPoint operation: NSCompositeSourceOver];
+    [[NSColor selectedControlColor] set];
+    [highlightPath fill];
 	}
 	
   s = [icon size];
@@ -481,38 +526,42 @@
 
 @implementation TShelfIcon (DraggingDestination)
 
-- (unsigned int)draggingEntered:(id <NSDraggingInfo>)sender
+- (NSDragOperation)draggingEntered:(id <NSDraggingInfo>)sender
 {
 	NSPasteboard *pb;
   NSDragOperation sourceDragMask;
 	NSArray *sourcePaths;
 	NSString *fromPath;
   NSString *buff;
-	NSString *iconPath;
-	int i, count;
+	unsigned i, count;
 
 	CHECK_LOCK_RET (NSDragOperationNone);
 	
 	isDragTarget = NO;
 
 	pb = [sender draggingPasteboard];
+  sourcePaths = nil;
   
   if ([[pb types] containsObject: NSFilenamesPboardType]) {
     sourcePaths = [pb propertyListForType: NSFilenamesPboardType]; 
        
   } else if ([[pb types] containsObject: @"GWRemoteFilenamesPboardType"]) {
-    NSData *pbData = [pb dataForType: @"GWRemoteFilenamesPboardType"]; 
-    NSDictionary *pbDict = [NSUnarchiver unarchiveObjectWithData: pbData];
+    if ([node isApplication] == NO) {
+      NSData *pbData = [pb dataForType: @"GWRemoteFilenamesPboardType"]; 
+      NSDictionary *pbDict = [NSUnarchiver unarchiveObjectWithData: pbData];
     
-    sourcePaths = [pbDict objectForKey: @"paths"];
-
+      sourcePaths = [pbDict objectForKey: @"paths"];
+    }
   } else if ([[pb types] containsObject: @"GWLSFolderPboardType"]) {
-    NSData *pbData = [pb dataForType: @"GWLSFolderPboardType"]; 
-    NSDictionary *pbDict = [NSUnarchiver unarchiveObjectWithData: pbData];
+    if ([node isApplication] == NO) {
+      NSData *pbData = [pb dataForType: @"GWLSFolderPboardType"]; 
+      NSDictionary *pbDict = [NSUnarchiver unarchiveObjectWithData: pbData];
     
-    sourcePaths = [pbDict objectForKey: @"paths"];
+      sourcePaths = [pbDict objectForKey: @"paths"];
+    }
+  }
 
-  } else {
+  if (sourcePaths == nil) {
     return NSDragOperationNone;
   }
 
@@ -526,7 +575,8 @@
     return NSDragOperationNone;
   }
   
-  if ((([node isDirectory] == NO) && ([node isMountPoint] == NO)) || [node isPackage]) {
+  if ((([node isDirectory] == NO) && ([node isMountPoint] == NO)) 
+                  || ([node isPackage] && ([node isApplication] == NO))) {
     return NSDragOperationNone;
   }
 
@@ -537,63 +587,85 @@
 		return NSDragOperationNone;
   } 
 
-	if ([node isWritable] == NO) {
-		return NSDragOperationNone;
+	if (([node isWritable] == NO) && ([node isApplication] == NO)) {
+    return NSDragOperationNone;
 	}
 
 	if ([[node path] isEqual: fromPath]) {
 		return NSDragOperationNone;
   }  
 
-	for (i = 0; i < count; i++) {
-		if ([[node path] isEqual: [sourcePaths objectAtIndex: i]]) {
-		  return NSDragOperationNone;
-		}
-	}
+  if ([sourcePaths containsObject: [node path]]) {
+    return NSDragOperationNone;
+  }
 
 	buff = [NSString stringWithString: [node path]];
 	while (1) {
+    CREATE_AUTORELEASE_POOL(arp);
+
 		for (i = 0; i < count; i++) {
 			if ([buff isEqual: [sourcePaths objectAtIndex: i]]) {
+        RELEASE (arp);
  		    return NSDragOperationNone;
 			}
 		}
     if ([buff isEqual: path_separator()]) {
+      RELEASE (arp);
       break;
     }            
 		buff = [buff stringByDeletingLastPathComponent];
-	}
+  }
 
+  if ([node isDirectory] && [node isParentOfPath: fromPath]) {
+    NSArray *subNodes = [node subNodes];
+    
+    for (i = 0; i < [subNodes count]; i++) {
+      FSNode *nd = [subNodes objectAtIndex: i];
+      
+      if ([nd isDirectory]) {
+        int j;
+        
+        for (j = 0; j < count; j++) {
+          NSString *fname = [[sourcePaths objectAtIndex: j] lastPathComponent];
+          
+          if ([[nd name] isEqual: fname]) {
+            return NSDragOperationNone;
+          }
+        }
+      }
+    }
+  }	
+
+  if ([node isApplication]) {
+    for (i = 0; i < count; i++) {
+      CREATE_AUTORELEASE_POOL(arp);
+      FSNode *nd = [FSNode nodeWithPath: [sourcePaths objectAtIndex: i]];
+      
+      if (([nd isPlain] == NO) && ([nd isPackage] == NO)) {
+        RELEASE (arp);
+        return NSDragOperationNone;
+      }
+      RELEASE (arp);
+    }
+  }
+  
   isDragTarget = YES;
   forceCopy = NO;
 
-  iconPath =  [[node path] stringByAppendingPathComponent: @".opendir.tiff"];
-
-  if ([fm isReadableFileAtPath: iconPath]) {
-    NSImage *img = [[NSImage alloc] initWithContentsOfFile: iconPath];
-
-    if (img) {
-      ASSIGN (icon, img);
-      RELEASE (img);
-    } else {
-      ASSIGN (icon, [NSImage imageNamed: @"FileIcon_Directory_Open.tiff"]);
-    }      
-  } else {
-	  ASSIGN (icon, [NSImage imageNamed: @"FileIcon_Directory_Open.tiff"]);    
-  }
-
+  ASSIGN (icon, [fsnodeRep openFolderIconOfSize: ICON_SIZE forNode: node]);
   [self setNeedsDisplay: YES];
 
 	sourceDragMask = [sender draggingSourceOperationMask];
 
 	if (sourceDragMask == NSDragOperationCopy) {
-		return NSDragOperationCopy;
+		return ([node isApplication] ? NSDragOperationMove : NSDragOperationCopy);
 	} else if (sourceDragMask == NSDragOperationLink) {
-		return NSDragOperationLink;
+		return ([node isApplication] ? NSDragOperationMove : NSDragOperationLink);
 	} else {
-    if ([[NSFileManager defaultManager] isWritableFileAtPath: fromPath]) {
+    if ([[NSFileManager defaultManager] isWritableFileAtPath: fromPath]
+                                                  || [node isApplication]) {
       return NSDragOperationAll;			
-    } else {
+    } else if ([node isApplication] == NO) {
       forceCopy = YES;
 			return NSDragOperationCopy;			
     }
@@ -602,7 +674,7 @@
   return NSDragOperationNone;
 }
 
-- (unsigned int)draggingUpdated:(id <NSDraggingInfo>)sender
+- (NSDragOperation)draggingUpdated:(id <NSDraggingInfo>)sender
 {
   NSDragOperation sourceDragMask;
 
@@ -610,16 +682,17 @@
     return NSDragOperationNone;
   }
 	
-	if (!isDragTarget || locked || [node isPackage]) {
-		return NSDragOperationNone;
+	if ((isDragTarget == NO) || locked 
+                  || ([node isPackage] && ([node isApplication] == NO))) {
+    return NSDragOperationNone;
 	}
 
 	sourceDragMask = [sender draggingSourceOperationMask];
 	
 	if (sourceDragMask == NSDragOperationCopy) {
-		return NSDragOperationCopy;
+		return ([node isApplication] ? NSDragOperationMove : NSDragOperationCopy);
 	} else if (sourceDragMask == NSDragOperationLink) {
-		return NSDragOperationLink;
+		return ([node isApplication] ? NSDragOperationMove : NSDragOperationLink);
 	} else {
 		return forceCopy ? NSDragOperationCopy : NSDragOperationAll;
 	}
@@ -676,52 +749,83 @@
 	sourceDragMask = [sender draggingSourceOperationMask];  
   pb = [sender draggingPasteboard];
   
-  if ([[pb types] containsObject: @"GWRemoteFilenamesPboardType"]) {  
-    NSData *pbData = [pb dataForType: @"GWRemoteFilenamesPboardType"]; 
-    
-    [gw concludeRemoteFilesDragOperation: pbData
-                             atLocalPath: [node path]];
-    return;
-  
-  } else if ([[pb types] containsObject: @"GWLSFolderPboardType"]) {  
-    NSData *pbData = [pb dataForType: @"GWLSFolderPboardType"]; 
-    
-    [gw lsfolderDragOperation: pbData
-              concludedAtPath: [node path]];
-    return;
-  }  
-  
-  sourcePaths = [pb propertyListForType: NSFilenamesPboardType];
-  source = [[sourcePaths objectAtIndex: 0] stringByDeletingLastPathComponent];
+  if ([node isApplication] == NO) {
+    if ([[pb types] containsObject: @"GWRemoteFilenamesPboardType"]) {  
+      NSData *pbData = [pb dataForType: @"GWRemoteFilenamesPboardType"]; 
 
-	if ([source isEqual: [gw trashPath]]) {
-		operation = @"GWorkspaceRecycleOutOperation";
-	} else {
-		if (sourceDragMask == NSDragOperationCopy) {
-			operation = NSWorkspaceCopyOperation;
-		} else if (sourceDragMask == NSDragOperationLink) {
-			operation = NSWorkspaceLinkOperation;
-		} else {
-      if ([fm isWritableFileAtPath: source]) {
-			  operation = NSWorkspaceMoveOperation;
-      } else {
-			  operation = NSWorkspaceCopyOperation;
-      }
-		}
+      [gw concludeRemoteFilesDragOperation: pbData
+                               atLocalPath: [node path]];
+      return;
+
+    } else if ([[pb types] containsObject: @"GWLSFolderPboardType"]) {  
+      NSData *pbData = [pb dataForType: @"GWLSFolderPboardType"]; 
+
+      [gw lsfolderDragOperation: pbData
+                concludedAtPath: [node path]];
+      return;
+    }  
   }
+   
+  sourcePaths = [pb propertyListForType: NSFilenamesPboardType];
   
-  files = [NSMutableArray arrayWithCapacity: 1];    
-  for(i = 0; i < [sourcePaths count]; i++) {    
-    [files addObject: [[sourcePaths objectAtIndex: i] lastPathComponent]];
-  }  
+  if ([node isApplication] == NO) {
+    source = [[sourcePaths objectAtIndex: 0] stringByDeletingLastPathComponent];
+	  
+    if ([source isEqual: [gw trashPath]]) {
+		  operation = @"GWorkspaceRecycleOutOperation";
+	  } else {
+		  if (sourceDragMask == NSDragOperationCopy) {
+			  operation = NSWorkspaceCopyOperation;
+		  } else if (sourceDragMask == NSDragOperationLink) {
+			  operation = NSWorkspaceLinkOperation;
+		  } else {
+        if ([fm isWritableFileAtPath: source]) {
+			    operation = NSWorkspaceMoveOperation;
+        } else {
+			    operation = NSWorkspaceCopyOperation;
+        }
+		  }
+    }
 
-	opDict = [NSMutableDictionary dictionaryWithCapacity: 4];
-	[opDict setObject: operation forKey: @"operation"];
-	[opDict setObject: source forKey: @"source"];
-	[opDict setObject: [node path] forKey: @"destination"];
-	[opDict setObject: files forKey: @"files"];
+    files = [NSMutableArray arrayWithCapacity: 1];    
+    for (i = 0; i < [sourcePaths count]; i++) {    
+      [files addObject: [[sourcePaths objectAtIndex: i] lastPathComponent]];
+    }  
+
+	  opDict = [NSMutableDictionary dictionaryWithCapacity: 4];
+	  [opDict setObject: operation forKey: @"operation"];
+	  [opDict setObject: source forKey: @"source"];
+	  [opDict setObject: [node path] forKey: @"destination"];
+	  [opDict setObject: files forKey: @"files"];
+
+    [gw performFileOperation: opDict];
+
+  } else {  
+    for (i = 0; i < [sourcePaths count]; i++) {    
+      NSString *path = [sourcePaths objectAtIndex: i];
     
-  [gw performFileOperationWithDictionary: opDict];
+      NS_DURING
+        {
+      [[NSWorkspace sharedWorkspace] openFile: path 
+                              withApplication: [node name]];
+        }
+      NS_HANDLER
+        {
+      NSRunAlertPanel(NSLocalizedString(@"error", @""), 
+          [NSString stringWithFormat: @"%@ %@!", 
+                    NSLocalizedString(@"Can't open ", @""), [node name]],
+                                        NSLocalizedString(@"OK", @""), 
+                                        nil, 
+                                        nil);                                     
+        }
+      NS_ENDHANDLER  
+    }
+  }
 }
 
 @end
+
+
+
+
+
