@@ -1,6 +1,6 @@
 /* TShelfIconsView.m
  *  
- * Copyright (C) 2003 Free Software Foundation, Inc.
+ * Copyright (C) 2003-2010 Free Software Foundation, Inc.
  *
  * Author: Enrico Sersale <enrico@imago.ro>
  * Date: August 2001
@@ -19,20 +19,26 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
  */
 
-#include <Foundation/Foundation.h>
-#include <AppKit/AppKit.h>
-#include "FSNodeRep.h"
-#include "FSNFunctions.h"
-#include "TShelfIconsView.h"
-#include "TShelfIcon.h"
-#include "TShelfPBIcon.h"
-#include "GWorkspace.h"
-#include "GNUstep.h"
 
-#define CELLS_WIDTH 80
+#include <math.h>
+
+#import <Foundation/Foundation.h>
+#import <AppKit/AppKit.h>
+#import <GNUstepBase/GNUstep.h>
+
+#import "FSNodeRep.h"
+#import "FSNFunctions.h"
+#import "TShelfIconsView.h"
+#import "TShelfIcon.h"
+#import "TShelfPBIcon.h"
+#import "GWorkspace.h"
+
+
+#define CELLS_WIDTH (80)
+#define EDIT_MARGIN (4)
 
 @interface TShelfIcon (TShelfIconsViewSorting)
 
@@ -85,9 +91,10 @@
 	if (gpoints != NULL) {
 		NSZoneFree (NSDefaultMallocZone(), gpoints);
 	}
-	RELEASE (icons);  
-	RELEASE (watchedPaths);
-	TEST_RELEASE (dragImage);
+  RELEASE (icons);  
+  RELEASE (watchedPaths);
+  RELEASE (dragImage);
+  RELEASE (focusedIconLabel);
   [super dealloc];
 }
 
@@ -112,8 +119,20 @@
 		
     cellsWidth = CELLS_WIDTH;
     		
-		watchedPaths = [[NSMutableArray alloc] initWithCapacity: 1];
-		
+		watchedPaths = [[NSCountedSet alloc] initWithCapacity: 1];
+		    
+    focusedIconLabel = [NSTextField new];
+		[focusedIconLabel setFont: [NSFont systemFontOfSize: 12]];
+		[focusedIconLabel setBezeled: NO];
+		[focusedIconLabel setAlignment: NSCenterTextAlignment];
+    [focusedIconLabel setEditable: NO];
+    [focusedIconLabel setSelectable: NO];
+    [focusedIconLabel setBackgroundColor: [NSColor windowBackgroundColor]];
+	  [focusedIconLabel setTextColor: [NSColor controlTextColor]];
+    [focusedIconLabel setFrame: NSMakeRect(0, 0, 0, 14)];
+    
+    focusedIcon = nil;
+    
 		icons = [[NSMutableArray alloc] initWithCapacity: 1];
         
     iconsType = itype;
@@ -245,9 +264,9 @@
 	[self resizeWithOldSuperviewSize: [self frame].size];  
 
 	if ([watchedPaths containsObject: watched] == NO) {
-		[watchedPaths addObject: watched];
 		[self setWatcherForPath: watched];
 	}
+  [watchedPaths addObject: watched];
 }
 
 - (TShelfPBIcon *)addPBIconForDataAtPath:(NSString *)dpath 
@@ -274,23 +293,39 @@
 
 - (void)removeIcon:(id)anIcon
 {
-  int index = [anIcon gridindex];	
+  if (anIcon) {
+    id label = [anIcon myLabel];
+    int index = [anIcon gridindex];	
 
-  if (iconsType == FILES_TAB) {
-	  NSString *watched = [[[anIcon paths] objectAtIndex: 0] stringByDeletingLastPathComponent];
+    if (iconsType == FILES_TAB) {
+	    NSString *watched = [[[anIcon paths] objectAtIndex: 0] stringByDeletingLastPathComponent];
 
-	  if ([watchedPaths containsObject: watched]) {
-		  [watchedPaths removeObject: watched];
-		  [self unsetWatcherForPath: watched];
-	  }
-  
-    [[anIcon myLabel] removeFromSuperview];
+	    if ([watchedPaths containsObject: watched]) {
+		    [watchedPaths removeObject: watched];
+        
+        if ([watchedPaths containsObject: watched] == NO) {
+		      [self unsetWatcherForPath: watched];
+        }
+	    }
+      
+      if (label && [[self subviews] containsObject: label]) {
+        [label removeFromSuperview];
+      }
+    }
+    
+    if (focusedIcon == anIcon) {
+      focusedIcon = nil;
+      [self updateFocusedIconLabel];
+    }
+    
+    if ([[self subviews] containsObject: anIcon]) {
+      [anIcon removeFromSuperview];
+    }
+    
+    [icons removeObject: anIcon];
+    gpoints[index].used = 0;
+    [self resizeWithOldSuperviewSize: [self frame].size];  
   }
-  
-  [anIcon removeFromSuperview];
-  [icons removeObject: anIcon];
-  gpoints[index].used = 0;
-  [self resizeWithOldSuperviewSize: [self frame].size];  
 }
 
 - (void)removePBIconsWithData:(NSData *)data ofType:(NSString *)type
@@ -327,7 +362,7 @@
 	iconwidth = [icon frame].size.width;
 	labwidth = [label frame].size.width;
 
-	if(iconwidth > labwidth) {
+	if (iconwidth > labwidth) {
 		labxpos = [icon frame].origin.x + ((iconwidth - labwidth) / 2);
 	} else {
 		labxpos = [icon frame].origin.x - ((labwidth - iconwidth) / 2);
@@ -363,6 +398,59 @@
   }  
 }
 
+- (void)setFocusedIcon:(id)anIcon
+{
+  if (anIcon == nil) {
+    if (focusedIcon) {
+      [self addSubview: [focusedIcon myLabel]];
+      [self setLabelRectOfIcon: focusedIcon];
+    }
+  } 
+
+  focusedIcon = anIcon;  
+  [self updateFocusedIconLabel];
+}
+
+- (void)updateFocusedIconLabel
+{
+  if ([[self subviews] containsObject: focusedIconLabel]) {
+    NSRect rect = [focusedIconLabel frame];
+
+    [focusedIconLabel removeFromSuperview];
+    [self setNeedsDisplayInRect: rect];
+  }
+  
+  if (focusedIcon) {
+    NSRect iconrect = [focusedIcon frame];
+    float centerx = iconrect.origin.x + (iconrect.size.width / 2);  
+    NSTextField *label = [focusedIcon myLabel];
+    NSRect labelrect = [label frame];
+    NSString *name = [focusedIcon shownName];  
+    float fwidth = [[label font] widthOfString: name];
+    float boundswidth = [self bounds].size.width - EDIT_MARGIN;
+    int margin = 8;
+      
+    fwidth += margin;  
+
+    if ((centerx + (fwidth / 2)) >= boundswidth) {
+      centerx -= (centerx + (fwidth / 2) - boundswidth);
+    } else if ((centerx - (fwidth / 2)) < margin) {
+      centerx += fabs(centerx - (fwidth / 2)) + margin;
+    }    
+
+    labelrect.origin.x = centerx - (fwidth / 2);
+    labelrect.size.width = fwidth;
+    labelrect = NSIntegralRect(labelrect);
+
+    [label removeFromSuperview];
+
+    [focusedIconLabel setFrame: labelrect];
+    [focusedIconLabel setStringValue: name];
+    [self addSubview: focusedIconLabel];  
+    [self setNeedsDisplayInRect: labelrect];
+  }  
+}
+
 - (void)sortIcons
 {
   SEL sel = (iconsType == FILES_TAB) ? @selector(iconCompare:) : @selector(pbiconCompare:);
@@ -413,7 +501,32 @@
 
 - (void)openCurrentSelection:(NSArray *)paths
 {
-  [gw openSelectedPaths: paths newViewer: NO];
+  [gw openSelectedPaths: paths newViewer: YES];
+}
+
+- (void)checkIconsAfterDotsFilesChange
+{
+  if (iconsType == FILES_TAB) {
+    int count = [icons count]; 
+    int i;
+
+	  for (i = 0; i < count; i++) {
+		  TShelfIcon *icon = [icons objectAtIndex: i];
+      NSArray *iconpaths = [icon paths];
+      int j;
+
+	    for (j = 0; j < [iconpaths count]; j++) {
+        NSString *op = [iconpaths objectAtIndex: j];
+
+        if ([op rangeOfString: @"."].location != NSNotFound) {
+          [self removeIcon: icon];
+          count--;
+          i--;
+          break;
+        }
+      }
+	  }
+  }
 }
 
 - (void)checkIconsAfterHidingOfPaths:(NSArray *)hpaths
@@ -458,6 +571,7 @@
 
 - (void)fileSystemWillChange:(NSNotification *)notification
 {
+  CREATE_AUTORELEASE_POOL(arp);
   NSDictionary *dict = [notification object];
   NSString *operation = [dict objectForKey: @"operation"];
 	NSString *source = [dict objectForKey: @"source"];	  
@@ -497,40 +611,18 @@
       }
 	  }
   }
+
+  RELEASE (arp);
 }
 
 - (void)fileSystemDidChange:(NSNotification *)notification
 {
-  NSDictionary *dict;
-  NSString *operation, *source, *destination;
-  NSArray *files;
-  NSMutableArray *paths;
-  TShelfIcon *icon;
-  NSArray *iconpaths;
-  int count;
-	int i, j, m;
+  CREATE_AUTORELEASE_POOL(arp);
+  NSDictionary *dict = [notification object];
+  NSString *operation = [dict objectForKey: @"operation"];
+  NSString *source = [dict objectForKey: @"source"];
+  NSArray *files = [dict objectForKey: @"files"];
   
-  dict = [notification object];
-  operation = [dict objectForKey: @"operation"];
-  source = [dict objectForKey: @"source"];
-  destination = [dict objectForKey: @"destination"];
-  files = [dict objectForKey: @"files"];
-
-/*		                    
-  if ([operation isEqual: @"GWorkspaceRenameOperation"]) {      
-    for (i = 0; i < [icons count]; i++) {
-      icon = [icons objectAtIndex: i];      
-      if ([icon isSinglePath] == YES) {      
-        if ([[[icon paths] objectAtIndex: 0] isEqual: source]) {     
-          [icon setPaths: [NSArray arrayWithObject: destination]];
-          [icon setNeedsDisplay: YES];
-					[self resizeWithOldSuperviewSize: [self frame].size];  
-          break;
-        }
-      }          
-    }        
-  }  
-*/
   if ([operation isEqual: @"GWorkspaceRenameOperation"]) {
 		files = [NSArray arrayWithObject: [source lastPathComponent]];
     source = [source stringByDeletingLastPathComponent];
@@ -542,8 +634,12 @@
 				|| [operation isEqual: NSWorkspaceRecycleOperation]
 				|| [operation isEqual: @"GWorkspaceRecycleOutOperation"]
 				|| [operation isEqual: @"GWorkspaceEmptyRecyclerOperation"]) {
+    NSMutableArray *paths = [NSMutableArray arrayWithCapacity: 1];
+    TShelfIcon *icon;
+    NSArray *iconpaths;
+    int count;
+	  int i, j, m;
 
-    paths = [NSMutableArray arrayWithCapacity: 1];
     for (i = 0; i < [files count]; i++) {
       NSString *s = [source stringByAppendingPathComponent: [files objectAtIndex: i]];
       [paths addObject: s];
@@ -580,14 +676,19 @@
       }
 	  }
   }
+  
+  RELEASE (arp);
 }
 
 - (void)watcherNotification:(NSNotification *)notification
 {
+  CREATE_AUTORELEASE_POOL(arp);
 	NSDictionary *notifdict = (NSDictionary *)[notification object];
   NSString *path = [notifdict objectForKey: @"path"];
 	NSString *event = [notifdict objectForKey: @"event"];
-	BOOL contained = NO;
+	NSEnumerator *enumerator;
+  NSString *wpath;
+  BOOL contained = NO;
 	int i;
   
   if (iconsType == DATA_TAB) {
@@ -615,16 +716,18 @@
     }
   } else {
 	  if ([event isEqual: @"GWFileCreatedInWatchedDirectory"]) {
+      RELEASE (arp);
 		  return;
 	  }
-
-	  for (i = 0; i < [watchedPaths count]; i++) {
-		  NSString *wpath = [watchedPaths objectAtIndex: i];
+    
+    enumerator = [watchedPaths objectEnumerator]; 
+    
+    while ((wpath = [enumerator nextObject])) {
 		  if (([wpath isEqual: path]) || (isSubpathOfPath(path, wpath))) {
 			  contained = YES;
 			  break;
 		  }
-	  }
+    }
 
     if (contained) {
 		  id icon;
@@ -632,7 +735,7 @@
 		  NSString *ipath;
 		  int count = [icons count];
 
-		  if ([event isEqual: @"GWWatchedDirectoryDeleted"]) {		
+		  if ([event isEqual: @"GWWatchedPathDeleted"]) {		
 			  for (i = 0; i < count; i++) {
 				  icon = [icons objectAtIndex: i];
 				  ipaths = [icon paths];
@@ -644,6 +747,8 @@
 					  i--;
 				  }
 			  }
+        
+        RELEASE (arp);
 			  return;
 		  }		
 
@@ -673,7 +778,6 @@
 					  }
 
 				  } else {
-
 					  for (j = 0; j < [files count]; j++) {
 						  NSString *fname = [files objectAtIndex: j];
 						  NSString *fullPath = [path stringByAppendingPathComponent: fname];
@@ -720,14 +824,17 @@
 		  }
 	  }
   }
+
+  RELEASE (arp);
 }
 
 - (void)setWatchers
 {
-	int i;
-	
-  for (i = 0; i < [watchedPaths count]; i++) {
-    [self setWatcherForPath: [watchedPaths objectAtIndex: i]];  
+  NSEnumerator *enumerator = [watchedPaths objectEnumerator]; 
+  NSString *wpath;
+
+	while ((wpath = [enumerator nextObject])) {
+    [self setWatcherForPath: wpath]; 
   }
 }
 
@@ -738,10 +845,11 @@
 
 - (void)unsetWatchers
 {
-	int i;
-	
-  for (i = 0; i < [watchedPaths count]; i++) {
-    [self unsetWatcherForPath: [watchedPaths objectAtIndex: i]];  
+  NSEnumerator *enumerator = [watchedPaths objectEnumerator]; 
+  NSString *wpath;
+
+	while ((wpath = [enumerator nextObject])) {
+    [self unsetWatcherForPath: wpath]; 
   }
 }
 
@@ -755,8 +863,8 @@
   float wdt, hgt, x, y;
 	int i;
   
-  wdt = [self frame].size.width;
-  hgt = [self frame].size.height;
+  wdt = [self bounds].size.width;
+  hgt = [self bounds].size.height;
 	
 	pcount = (int)((wdt - 16) / cellsWidth);
 		
@@ -786,8 +894,9 @@
 
 - (gridpoint *)gridPointNearestToPoint:(NSPoint)p
 {
-	float maxx = [self frame].size.width;
-	float maxy = [self frame].size.height;
+  NSRect r = [self bounds];
+	float maxx = r.size.width;
+	float maxy = r.size.height;
 	float px = p.x;
 	float py = p.y;	
 	float minx = maxx;
@@ -857,7 +966,7 @@
 		NSRect r = NSMakeRect(p.x, p.y, 64, 52);
 
 		[icon setPosition: p];
-		[icon setFrame: r];
+		[icon setFrame: NSIntegralRect(r)];
 		gpoints[index].used = 1;
     
     if (iconsType == FILES_TAB) {
@@ -1021,7 +1130,7 @@
 
 @implementation TShelfIconsView(DraggingDestination)
 
-- (unsigned int)draggingEntered:(id <NSDraggingInfo>)sender
+- (NSDragOperation)draggingEntered:(id <NSDraggingInfo>)sender
 {
   NSPasteboard *pb = [sender draggingPasteboard];
   NSDragOperation sourceDragMask = [sender draggingSourceOperationMask];
@@ -1038,7 +1147,22 @@
   }
   
   if (iconsType == FILES_TAB) {  
-    if ([[pb types] indexOfObject: NSFilenamesPboardType] == NSNotFound) {
+    if ([[pb types] indexOfObject: NSFilenamesPboardType] != NSNotFound) {
+      NSArray *sourcePaths = [pb propertyListForType: NSFilenamesPboardType]; 
+      
+      if ([sourcePaths count]) {
+        NSString *basePath = [sourcePaths objectAtIndex: 0];
+      
+        basePath = [basePath stringByDeletingLastPathComponent];
+        
+        if ([basePath isEqual: [gw trashPath]]) {
+          found = NO;
+        }
+      
+      } else {
+        found = NO;
+      }
+    } else {
       found = NO;
     }
   } else {
@@ -1078,7 +1202,7 @@
   return NSDragOperationNone;
 }
 
-- (unsigned int)draggingUpdated:(id <NSDraggingInfo>)sender
+- (NSDragOperation)draggingUpdated:(id <NSDraggingInfo>)sender
 {
   NSDragOperation sourceDragMask = [sender draggingSourceOperationMask];
 	NSPoint p = [sender draggedImageLocation];
