@@ -19,7 +19,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
  */
 
 #ifndef TSHELF_PB_ICON_H
@@ -28,6 +28,7 @@
 #include <AppKit/NSView.h>
 
 @class NSImage;
+@class NSBezierPath;
 @class TShelfIconsView;
 
 @interface TShelfPBIcon : NSView
@@ -36,7 +37,7 @@
   NSString *dataType;
 
 	NSImage *icon;
-  NSImage *highlight;
+  NSBezierPath *highlightPath;
 	NSPoint position;
 	int gridindex;
 
@@ -76,6 +77,8 @@
 
 - (int)gridindex;
 
+- (NSTextField *)myLabel;
+
 @end
 
 @interface TShelfPBIcon (DraggingSource)
