@@ -1,6 +1,6 @@
 /* TShelfPBIcon.m
  *  
- * Copyright (C) 2003 Free Software Foundation, Inc.
+ * Copyright (C) 2003-2010 Free Software Foundation, Inc.
  *
  * Author: Enrico Sersale <enrico@imago.ro>
  * Date: August 2001
@@ -19,16 +19,20 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
  */
 
-#include <Foundation/Foundation.h>
-#include <AppKit/AppKit.h>
-#include "GWFunctions.h"
-#include "TShelfPBIcon.h"
-#include "TShelfIconsView.h"
-#include "GWorkspace.h"
-#include "GNUstep.h"
+#import <Foundation/Foundation.h>
+#import <AppKit/AppKit.h>
+#import <GNUstepBase/GNUstep.h>
+
+#import "FSNodeRep.h"
+#import "GWFunctions.h"
+#import "TShelfPBIcon.h"
+#import "TShelfIconsView.h"
+#import "GWorkspace.h"
+
+#define ICON_SIZE 48
 
 @implementation TShelfPBIcon
 
@@ -48,7 +52,7 @@
 {
   RELEASE (dataPath);
   RELEASE (dataType);
-  RELEASE (highlight);
+  RELEASE (highlightPath);
   RELEASE (icon);
 
   [super dealloc];
@@ -61,12 +65,21 @@
 {
   self = [super init];
   if (self) {
+    NSRect hlightRect;
+
     [self setFrame: NSMakeRect(0, 0, 64, 52)];
 
     ASSIGN (dataPath, dpath);
     ASSIGN (dataType, type);
 
-    ASSIGN (highlight, [NSImage imageNamed: @"CellHighlight.tiff"]);
+    hlightRect = NSZeroRect;
+    hlightRect.size.width = (float)ICON_SIZE / 3 * 4;
+    hlightRect.size.height = hlightRect.size.width * [[FSNodeRep sharedInstance] highlightHeightFactor];
+    if ((hlightRect.size.height - ICON_SIZE) < 4) {
+      hlightRect.size.height = ICON_SIZE + 4;
+    }
+    hlightRect = NSIntegralRect(hlightRect);
+    ASSIGN (highlightPath, [[FSNodeRep sharedInstance] highlightPathOfSize: hlightRect.size]);
 
     if ([dataType isEqual: NSStringPboardType]) {
       ASSIGN (icon, [NSImage imageNamed: @"stringPboard.tiff"]);
@@ -155,6 +168,11 @@
   return gridindex;
 }
 
+- (NSTextField *)myLabel
+{
+  return nil;
+}
+
 - (BOOL)acceptsFirstMouse:(NSEvent *)theEvent 
 {
   return YES;
@@ -183,7 +201,6 @@
           dragdelay++;
         } else {      
           NSPoint p = [nextEvent locationInWindow];
-        
           offset = NSMakeSize(p.x - location.x, p.y - location.y); 
           startdnd = YES;        
           break;
@@ -192,7 +209,7 @@
     }
 
     if (startdnd == YES) {  
-      [self startExternalDragOnEvent: nextEvent withMouseOffset: offset];    
+      [self startExternalDragOnEvent: theEvent withMouseOffset: offset];    
     }    
   }           
 }
@@ -203,7 +220,8 @@
   NSSize s;
       	
 	if(isSelect) {
-		[highlight compositeToPoint: NSZeroPoint operation: NSCompositeSourceOver];
+    [[NSColor selectedControlColor] set];
+    [highlightPath fill];
 	}
 	
   s = [icon size];
