@@ -19,10 +19,9 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
  */
 
-
 #ifndef TSHELF_WIN_H
 #define TSHELF_WIN_H
 
@@ -39,6 +38,8 @@
 @interface TShelfWin : NSWindow
 {
   TShelfView *tView;
+  BOOL autohide;
+  BOOL autohidden;
 }
 
 - (TShelfView *)shelfView;
@@ -47,6 +48,14 @@
 
 - (void)deactivate;
 
+- (void)animateShowing;
+
+- (void)animateHiding;
+
+- (void)setAutohide:(BOOL)value;
+
+- (BOOL)autohide;
+
 - (void)addTab;
 
 - (void)removeTab;
@@ -55,6 +64,8 @@
 
 - (void)updateIcons;
 
+- (void)checkIconsAfterDotsFilesChange;
+
 - (void)checkIconsAfterHidingOfPaths:(NSArray *)hpaths;
 
 - (void)saveDefaults;
