@@ -1,6 +1,6 @@
 /* TShelfWin.m
  *  
- * Copyright (C) 2003 Free Software Foundation, Inc.
+ * Copyright (C) 2003-2010 Free Software Foundation, Inc.
  *
  * Author: Enrico Sersale <enrico@imago.ro>
  * Date: August 2001
@@ -19,17 +19,19 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
  */
 
-#include <Foundation/Foundation.h>
-#include <AppKit/AppKit.h>
-#include "TShelfWin.h"
-#include "TShelfView.h"
-#include "TShelfViewItem.h"
-#include "TShelfIconsView.h"
-#include "Dialogs/Dialogs.h"
-#include "GNUstep.h"
+#import <Foundation/Foundation.h>
+#import <AppKit/AppKit.h>
+#import <GNUstepBase/GNUstep.h>
+
+#import "TShelfWin.h"
+#import "TShelfView.h"
+#import "TShelfViewItem.h"
+#import "TShelfIconsView.h"
+#import "Dialogs/Dialogs.h"
+
 
 #define SHELF_HEIGHT 106
 
@@ -53,22 +55,27 @@
   if (self) {
     NSUserDefaults *defaults;
     NSDictionary *tshelfDict;
+    id entry;
     NSArray *tabsArr;    
 		TShelfViewItem *item;
     TShelfIconsView *view;
     int i;
     
 		[self setReleasedWhenClosed: NO];
+    [self setExcludedFromWindowsMenu: YES];
         
-    tView = [[TShelfView alloc] initWithFrame: [[self contentView] frame]];
+    tView = [[TShelfView alloc] initWithFrame: [[self contentView] bounds]];
     [self setContentView: tView];		
     
     defaults = [NSUserDefaults standardUserDefaults];	
-    
+        
     tshelfDict = [defaults objectForKey: @"tabshelf"];
     if (tshelfDict == nil) {
       tshelfDict = [NSDictionary dictionary];
     }
+    
+    entry = [tshelfDict objectForKey: @"auto_hide"];
+    autohide = (entry && [entry boolValue]);
         
     tabsArr = [tshelfDict objectForKey: @"tabs"];
     
@@ -154,14 +161,7 @@
 
 - (void)activate
 {
-  NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];	
-  NSDictionary *tshelfDict = [defaults objectForKey: @"tabbedshelf"];
-
   [self makeKeyAndOrderFront: nil];
-  
-  if (tshelfDict) {
-    [tView setHiddenTabs: [[tshelfDict objectForKey: @"hiddentabs"] boolValue]];
-  }
 }
 
 - (void)deactivate
@@ -169,6 +169,94 @@
   [self orderOut: nil];
 }
 
+- (void)animateShowing
+{
+  if (([self isVisible] == NO) || (autohide == NO)) {
+    return;
+  }
+
+  if (autohidden) {
+    CREATE_AUTORELEASE_POOL(arp);
+    int p = (int)(SHELF_HEIGHT / 10);
+    int h = -SHELF_HEIGHT;
+    
+    [self disableFlushWindow];
+    
+    while (1) {
+      NSDate *date;
+
+      h += p;
+      [self setFrameOrigin: NSMakePoint(0, h)];
+
+      if (h >= 0) {
+        break;
+      }
+      
+      date = [NSDate dateWithTimeIntervalSinceNow: 0.01];
+      [[NSRunLoop currentRunLoop] runUntilDate: date];  
+    }
+    
+    [self setFrameOrigin: NSMakePoint(0, 0)];
+    [self enableFlushWindow];
+    [self flushWindowIfNeeded];
+    
+    RELEASE (arp);
+  }
+  
+  autohidden = NO;
+}
+
+- (void)animateHiding
+{
+  if (([self isVisible] == NO) || (autohide == NO)) {
+    return;
+  }
+
+  if (autohidden == NO) {
+    CREATE_AUTORELEASE_POOL(arp);
+    int p = (int)(SHELF_HEIGHT / 10);
+    int h = 0;
+    
+    [self disableFlushWindow];
+    
+    while (1) {
+      NSDate *date;
+
+      h -= p;
+      [self setFrameOrigin: NSMakePoint(0, h)];
+
+      if (h <= -SHELF_HEIGHT) {
+        break;
+      }
+      
+      date = [NSDate dateWithTimeIntervalSinceNow: 0.01];
+      [[NSRunLoop currentRunLoop] runUntilDate: date];  
+    }
+    
+    [self setFrameOrigin: NSMakePoint(0, -SHELF_HEIGHT)];
+    [self enableFlushWindow];
+    [self flushWindowIfNeeded];
+    
+    RELEASE (arp);
+  }
+    
+  autohidden = YES;
+}
+
+- (void)setAutohide:(BOOL)value
+{
+  autohide = value;
+  if (autohide == NO) {
+    [self setFrameOrigin: NSMakePoint(0, 0)];
+    autohidden = NO;
+  }
+}
+
+- (BOOL)autohide
+{
+  return autohide;
+}
+
 - (void)addTab
 {
   SympleDialog *dialog;
@@ -272,7 +360,7 @@
 
 	title = NSLocalizedString(@"Remove Tab", @"");
 	msg = NSLocalizedString(@"Are you sure that you want to remove the selected tab?", @"");
-	buttstr = NSLocalizedString(@"Continue", @"");
+	buttstr = NSLocalizedString(@"Cancel", @"");
   result = NSRunAlertPanel(title, msg, 
                   NSLocalizedString(@"OK", @""), buttstr, NULL);
   if(result != NSAlertDefaultReturn) {
@@ -374,6 +462,19 @@
   }  
 }
 
+- (void)checkIconsAfterDotsFilesChange
+{
+  NSArray *items = [tView items];
+  int i;
+  
+  for (i = 0; i < [items count]; i++) {
+    TShelfViewItem *item = [items objectAtIndex: i];
+    TShelfIconsView *iview = (TShelfIconsView *)[item view];
+
+    [iview checkIconsAfterDotsFilesChange];
+  }  
+}
+
 - (void)checkIconsAfterHidingOfPaths:(NSArray *)hpaths
 {
   NSArray *items = [tView items];
@@ -395,9 +496,6 @@
   NSMutableArray *tabsArr = [NSMutableArray array];
   int i;
 
-  [tshelfDict setObject: [NSNumber numberWithBool: [tView hiddenTabs]] 
-                 forKey: @"hiddentabs"];
-
   for (i = 0; i < [items count]; i++) {
     TShelfViewItem *item = [items objectAtIndex: i];
     NSString *label = [item label];
@@ -414,8 +512,10 @@
 
   [tshelfDict setObject: tabsArr forKey: @"tabs"];
 
+  [tshelfDict setObject: [NSNumber numberWithBool: autohide]
+                 forKey: @"auto_hide"];
+
   [defaults setObject: tshelfDict forKey: @"tabshelf"];
-  [defaults synchronize];
 }
 
 - (BOOL)canBecomeKeyWindow
