@@ -1,6 +1,6 @@
 /* Attributes.m
  *  
- * Copyright (C) 2004 Free Software Foundation, Inc.
+ * Copyright (C) 2004-2010 Free Software Foundation, Inc.
  *
  * Author: Enrico Sersale <enrico@imago.ro>
  * Date: January 2004
@@ -19,38 +19,26 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
  */
 
-#include <Foundation/Foundation.h>
-#include <AppKit/AppKit.h>
 #include <math.h>
-#include "Attributes.h"
-#include "Inspector.h"
-#include "TimeDateView.h"
-#include "Functions.h"
-#include "FSNodeRep.h"
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 
+#import <Foundation/Foundation.h>
+#import <AppKit/AppKit.h>
+#import "Attributes.h"
+#import "Inspector.h"
+#import "IconView.h"
+#import "TimeDateView.h"
+#import "Functions.h"
+#import "FSNodeRep.h"
+
 #define SINGLE 0
 #define MULTIPLE 1
 
-#define SET_BUTTON_STATE(b, v) { \
-if ((perms & v) == v) [b setState: NSOnState]; \
-else [b setState: NSOffState]; \
-}
-
-#define GET_BUTTON_STATE(b, v) { \
-if ([b state] == NSOnState) { \
-perms |= v; \
-} else { \
-if ((oldperms & v) == v) { \
-if ([b tag] == MULTIPLE) perms |= v; \
-} } \
-}
-
 #ifdef __WIN32__
 	#define S_IRUSR _S_IRUSR
 	#define S_IWUSR _S_IWUSR
@@ -70,14 +58,14 @@ static BOOL sizeStop = NO;
   [nc removeObserver: self];  
   DESTROY (sizerConn);
   DESTROY (sizer);
-	TEST_RELEASE (mainBox);
-  TEST_RELEASE (calculateButt);
-	TEST_RELEASE (insppaths);
-	TEST_RELEASE (attributes);
-	TEST_RELEASE (currentPath);
-	TEST_RELEASE (onImage);
-	TEST_RELEASE (offImage);
-	TEST_RELEASE (multipleImage);  
+  RELEASE (mainBox);
+  RELEASE (calculateButt);
+  RELEASE (insppaths);
+  RELEASE (attributes);
+  RELEASE (currentPath);
+  RELEASE (onImage);
+  RELEASE (offImage);
+  RELEASE (multipleImage);  
   
   [super dealloc];
 }
@@ -100,6 +88,7 @@ static BOOL sizeStop = NO;
     RELEASE (win);
 
     inspector = insp;
+    [iconView setInspector: inspector];
 		insppaths = nil;
 		attributes = nil;    
     currentPath = nil;
@@ -239,7 +228,7 @@ static BOOL sizeStop = NO;
 
 		ftype = [attributes objectForKey: NSFileType];
 		if ([ftype isEqual: NSFileTypeDirectory] == NO) {	
-      NSString *fsize = fileSizeDescription([[attributes objectForKey: NSFileSize] unsignedLongLongValue]);
+      NSString *fsize = fsDescription([[attributes objectForKey: NSFileSize] unsignedLongLongValue]);
 		  [sizeField setStringValue: fsize]; 
       [calculateButt setEnabled: NO];
       [insideButt	setEnabled: NO];
@@ -263,7 +252,7 @@ static BOOL sizeStop = NO;
 
 		if ([ftype isEqual: NSFileTypeSymbolicLink]) {
 			s = [fm pathContentOfSymbolicLinkAtPath: currentPath];
-			s = relativePathFittingInContainer(linkToField, s);
+			s = relativePathFit(linkToField, s);
 			[linkToField setStringValue: s];
       [linkToLabel setTextColor: [NSColor blackColor]];		
 			[linkToField setTextColor: [NSColor blackColor]];		      
@@ -383,6 +372,11 @@ static BOOL sizeStop = NO;
 	[okButt setEnabled: YES];
 }
 
+- (IBAction)insideButtonAction:(id)sender
+{
+  [okButt setEnabled: YES];
+}
+
 - (IBAction)changePermissions:(id)sender
 {
 	NSMutableDictionary *attrs;
@@ -396,29 +390,36 @@ static BOOL sizeStop = NO;
   recursive = ([insideButt isEnabled] && ([insideButt state] == NSOnState));
   
 	if (pathscount == 1) {
+		oldperms = [[attributes objectForKey: NSFilePosixPermissions] unsignedLongValue];
+		newperms = [self getPermissions: oldperms];		
+		attrs = [attributes mutableCopy];
+		[attrs setObject: [NSNumber numberWithInt: newperms] forKey: NSFilePosixPermissions];
+		[fm changeFileAttributes: attrs atPath: currentPath];	
+    RELEASE (attrs);
+
 		[fm fileExistsAtPath: currentPath isDirectory: &isdir];
 
-		if (recursive && isdir) {
+		if (isdir && recursive) {
 			enumerator = [fm enumeratorAtPath: currentPath];
-			while ((fpath = [enumerator nextObject])) {
+      
+      while ((fpath = [enumerator nextObject])) {
+        CREATE_AUTORELEASE_POOL(arp);  
+      
 				fpath = [currentPath stringByAppendingPathComponent: fpath];
 				attrs = [[fm fileAttributesAtPath: fpath traverseLink: NO] mutableCopy];
-				if (attrs) {			
-					oldperms = [[attrs objectForKey: NSFilePosixPermissions] unsignedLongValue];	
-					newperms = [self getPermissions: oldperms];			
-					[attrs setObject: [NSNumber numberWithInt: newperms] forKey: NSFilePosixPermissions];
-					[fm changeFileAttributes: attrs atPath: fpath];
-				}
-			}
+				oldperms = [[attrs objectForKey: NSFilePosixPermissions] unsignedLongValue];	
+				newperms = [self getPermissions: oldperms];			
+				[attrs setObject: [NSNumber numberWithInt: newperms] forKey: NSFilePosixPermissions];
+				[fm changeFileAttributes: attrs atPath: fpath];
+        RELEASE (attrs);
+        
+        RELEASE (arp);
+      }
+                  
 			ASSIGN (attributes, [fm fileAttributesAtPath: currentPath traverseLink: NO]);	
 			[self setPermissions: 0 isActive: YES];
 
 		} else {
-			oldperms = [[attributes objectForKey: NSFilePosixPermissions] unsignedLongValue];
-			newperms = [self getPermissions: oldperms];		
-			attrs = [attributes mutableCopy];
-			[attrs setObject: [NSNumber numberWithInt: newperms] forKey: NSFilePosixPermissions];
-			[fm changeFileAttributes: attrs atPath: currentPath];	
 			ASSIGN (attributes, [fm fileAttributesAtPath: currentPath traverseLink: NO]);	
 			newperms = [[attributes objectForKey: NSFilePosixPermissions] unsignedLongValue];				
 			[self setPermissions: newperms isActive: YES];
@@ -426,28 +427,32 @@ static BOOL sizeStop = NO;
 	
 	} else {
 		for (i = 0; i < [insppaths count]; i++) {
-			path = [insppaths objectAtIndex: i];
+			path = [insppaths objectAtIndex: i];			
+ 			attrs = [[fm fileAttributesAtPath: path traverseLink: NO] mutableCopy];
+			oldperms = [[attrs objectForKey: NSFilePosixPermissions] unsignedLongValue];	
+			newperms = [self getPermissions: oldperms];			
+			[attrs setObject: [NSNumber numberWithInt: newperms] forKey: NSFilePosixPermissions];
+			[fm changeFileAttributes: attrs atPath: path];	
+      RELEASE (attrs);			
+
 			[fm fileExistsAtPath: path isDirectory: &isdir];
-			
-			if ((recursive == YES) && (isdir == YES)) {
+     
+			if (isdir && recursive) {
 				enumerator = [fm enumeratorAtPath: path];
-				while ((fpath = [enumerator nextObject])) {
+        
+        while ((fpath = [enumerator nextObject])) {
+          CREATE_AUTORELEASE_POOL(arp);  
+
 					fpath = [path stringByAppendingPathComponent: fpath];
 					attrs = [[fm fileAttributesAtPath: fpath traverseLink: NO] mutableCopy];
-					if (attrs) {			
-						oldperms = [[attrs objectForKey: NSFilePosixPermissions] unsignedLongValue];	
-						newperms = [self getPermissions: oldperms];			
-						[attrs setObject: [NSNumber numberWithInt: newperms] forKey: NSFilePosixPermissions];
-						[fm changeFileAttributes: attrs atPath: fpath];
-					}
-				}
-				
-			} else {
-				attrs = [[fm fileAttributesAtPath: path traverseLink: NO] mutableCopy];
-				oldperms = [[attrs objectForKey: NSFilePosixPermissions] unsignedLongValue];	
-				newperms = [self getPermissions: oldperms];			
-				[attrs setObject: [NSNumber numberWithInt: newperms] forKey: NSFilePosixPermissions];
-				[fm changeFileAttributes: attrs atPath: path];				
+					oldperms = [[attrs objectForKey: NSFilePosixPermissions] unsignedLongValue];	
+					newperms = [self getPermissions: oldperms];			
+					[attrs setObject: [NSNumber numberWithInt: newperms] forKey: NSFilePosixPermissions];
+					[fm changeFileAttributes: attrs atPath: fpath];
+					RELEASE (attrs);
+
+          RELEASE (arp);
+        }
 			}
 		}
 		
@@ -564,6 +569,11 @@ static BOOL sizeStop = NO;
 	#endif
 	}
 
+#define SET_BUTTON_STATE(b, v) { \
+if ((perms & v) == v) [b setState: NSOnState]; \
+else [b setState: NSOffState]; \
+}
+
 	SET_BUTTON_STATE (ureadbutt, S_IRUSR);				
 	SET_BUTTON_STATE (uwritebutt, S_IWUSR);
 	SET_BUTTON_STATE (uexebutt, S_IXUSR);
@@ -582,6 +592,15 @@ static BOOL sizeStop = NO;
 {
 	unsigned long perms = 0;
 
+#define GET_BUTTON_STATE(b, v) { \
+if ([b state] == NSOnState) { \
+perms |= v; \
+} else { \
+if ((oldperms & v) == v) { \
+if ([b tag] == MULTIPLE) perms |= v; \
+} } \
+}
+
 	GET_BUTTON_STATE (ureadbutt, S_IRUSR);
 	GET_BUTTON_STATE (uwritebutt, S_IWUSR);
 	GET_BUTTON_STATE (uexebutt, S_IXUSR);
@@ -773,42 +792,51 @@ static BOOL sizeStop = NO;
     }
     
 		path = [paths objectAtIndex: i];
-		[fm fileExistsAtPath: path isDirectory: &isdir];
 		 
+		fileAttrs = [fm fileAttributesAtPath: path traverseLink: NO];
+		if (fileAttrs) {
+			fsize = [[fileAttrs objectForKey: NSFileSize] unsignedLongLongValue];
+			dirsize += fsize;
+		}
+     
+		[fm fileExistsAtPath: path isDirectory: &isdir];
+          
 		if (isdir) {
 			NSDirectoryEnumerator *enumerator = [fm enumeratorAtPath: path];
 			
-			while((filePath = [enumerator nextObject])) {
+      while (1) {
         CREATE_AUTORELEASE_POOL (arp2);
         
-        if (sizeStop) {
-          RELEASE (arp2);
-          RELEASE (arp1);
-          return;
-        }
-				filePath = [path stringByAppendingPathComponent: filePath];
-				fileAttrs = [fm fileAttributesAtPath: filePath traverseLink: NO];
-				if (fileAttrs) {
-					fsize = [[fileAttrs objectForKey: NSFileSize] unsignedLongLongValue];
-					dirsize += fsize;
-				}
+        filePath = [enumerator nextObject];
+      
+        if (filePath) {
+          if (sizeStop) {
+            RELEASE (arp2);
+            RELEASE (arp1);
+            return;
+          }
         
+			    filePath = [path stringByAppendingPathComponent: filePath];
+			    fileAttrs = [fm fileAttributesAtPath: filePath traverseLink: NO];
+			    if (fileAttrs) {
+				    fsize = [[fileAttrs objectForKey: NSFileSize] unsignedLongLongValue];
+				    dirsize += fsize;
+			    }
+      
+        } else {
+          RELEASE (arp2);
+          break;   
+        } 
+      
         RELEASE (arp2);
-			}
-			
-		} else {
-			fileAttrs = [fm fileAttributesAtPath: path traverseLink: NO];
-			if (fileAttrs) {
-				fsize = [[fileAttrs objectForKey: NSFileSize] unsignedLongLongValue];
-				dirsize += fsize;
-			}
+      }
 		}
     
     RELEASE (arp1);
 	}	
 
   if (sizeStop == NO) {
-    [attributes sizeReady: fileSizeDescription(dirsize)]; 
+    [attributes sizeReady: fsDescription(dirsize)]; 
   }
 }
 
