@@ -1,6 +1,6 @@
 /* Recycler.m
  *  
- * Copyright (C) 2004 Free Software Foundation, Inc.
+ * Copyright (C) 2004-2010 Free Software Foundation, Inc.
  *
  * Author: Enrico Sersale <enrico@imago.ro>
  * Date: June 2004
@@ -19,19 +19,21 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
  */
 
-#include <Foundation/Foundation.h>
-#include <AppKit/AppKit.h>
-#include "Recycler.h"
-#include "RecyclerView.h"
-#include "Preferences/RecyclerPrefs.h"
-#include "Dialogs/StartAppWin.h"
-#include "FSNode.h"
-#include "FSNodeRep.h"
-#include "FSNFunctions.h"
-#include "GNUstep.h"
+#import <Foundation/Foundation.h>
+#import <AppKit/AppKit.h>
+#import <GNUstepBase/GNUstep.h>
+
+#import "Recycler.h"
+#import "RecyclerView.h"
+#import "Preferences/RecyclerPrefs.h"
+#import "Dialogs/StartAppWin.h"
+#import "FSNode.h"
+#import "FSNodeRep.h"
+#import "FSNFunctions.h"
+
 
 static Recycler *recycler = nil;
 
@@ -63,7 +65,6 @@ static Recycler *recycler = nil;
   }
   [[NSDistributedNotificationCenter defaultCenter] removeObserver: self];
   DESTROY (workspaceApplication);
-  DESTROY (operationsApp);
   RELEASE (trashPath);
   RELEASE (recview);
   RELEASE (preferences);
@@ -72,49 +73,38 @@ static Recycler *recycler = nil;
 	[super dealloc];
 }
 
-- (id)init
+- (void)applicationWillFinishLaunching:(NSNotification *)aNotification
 {
-  self = [super init];
-  
-  if (self) {
-    fm = [NSFileManager defaultManager];
-    ws = [NSWorkspace sharedWorkspace];
-    nc = [NSNotificationCenter defaultCenter];
-    workspaceApplication = nil;
+	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
+	id entry;
+
+  fm = [NSFileManager defaultManager];
+  ws = [NSWorkspace sharedWorkspace];
+  nc = [NSNotificationCenter defaultCenter];
+  fsnodeRep = [FSNodeRep sharedInstance];
+  workspaceApplication = nil;
+
+	entry = [defaults objectForKey: @"reserved_names"];
+	if (entry) {
+    [fsnodeRep setReservedNames: entry];
+	} else {
+    [fsnodeRep setReservedNames: [NSArray arrayWithObjects: @".gwdir", @".gwsort", nil]];
   }
-  
-  return self;
 }
 
 - (void)applicationDidFinishLaunching:(NSNotification *)aNotification
 {
-	NSString *basePath;
   NSString *tpath; 
 	BOOL isdir;
-  
-  basePath = [NSSearchPathForDirectoriesInDomains(NSLibraryDirectory, NSUserDomainMask, YES) lastObject];
-  basePath = [basePath stringByAppendingPathComponent: @"Desktop"];
 
-  if (([fm fileExistsAtPath: basePath isDirectory: &isdir] && isdir) == NO) {
-    if ([fm createDirectoryAtPath: basePath attributes: nil] == NO) {
-      NSLog(@"Can't create the Recycler directory! Quitting now.");
-      [NSApp terminate: self];
-    }
-  }
-  
-	tpath = [basePath stringByAppendingPathComponent: @".Trash"];
+  tpath = [NSHomeDirectory() stringByAppendingPathComponent: @".Trash"]; 
 
 	if ([fm fileExistsAtPath: tpath isDirectory: &isdir] == NO) {
     if ([fm createDirectoryAtPath: tpath attributes: nil] == NO) {
-      NSLog(@"Can't create the Recycler directory! Quiting now.");
+      NSLog(@"Can't create the Recycler directory! Quitting now.");
       [NSApp terminate: self];
     }
-	} else {
-		if (isdir == NO) {
-			NSLog (@"Warning - %@ is not a directory - quiting now!", tpath);			
-			[NSApp terminate: self];
-		}
-  }
+	}
   
   ASSIGN (trashPath, tpath);
 
@@ -143,10 +133,14 @@ static Recycler *recycler = nil;
                 				selector: @selector(fileSystemDidChange:) 
                 					  name: @"GWFileSystemDidChangeNotification"
                 					object: nil];
+  
+  terminating = NO;
 }
 
 - (BOOL)applicationShouldTerminate:(NSApplication *)app 
 {
+  terminating = YES;
+  
   [self removeWatcherForPath: trashPath];
 
   if (fswatcher) {
@@ -165,6 +159,11 @@ static Recycler *recycler = nil;
 	return YES;
 }
 
+- (oneway void)emptyTrash
+{
+  [self emptyTrashFromMenu: nil];
+}
+
 - (void)setDocked:(BOOL)value
 {
   docked = value;
@@ -176,6 +175,7 @@ static Recycler *recycler = nil;
     [[[NSApp iconWindow] contentView] addSubview: recview];
 
   } else {
+    [recview removeFromSuperview];
     DESTROY (recview);
     recview = [[RecyclerView alloc] initWithWindow];
     [recview activate];
@@ -211,8 +211,6 @@ static Recycler *recycler = nil;
 
 - (void)contactWorkspaceApp
 {
-  id app = nil;
-
   if (workspaceApplication == nil) {
     NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
     NSString *appName = [defaults stringForKey: @"GSWorkspaceApplication"];
@@ -220,55 +218,41 @@ static Recycler *recycler = nil;
     if (appName == nil) {
       appName = @"GWorkspace";
     }
+  
+    workspaceApplication = [NSConnection rootProxyForConnectionWithRegisteredName: appName
+                                                                             host: @""];
+    if (workspaceApplication == nil) {
+      int i;
+  
+      [ws launchApplication: appName];  
+  
+      for (i = 0; i < 80; i++) {
+	      [[NSRunLoop currentRunLoop] runUntilDate:
+		                     [NSDate dateWithTimeIntervalSinceNow: 0.1]];
 
-    app = [NSConnection rootProxyForConnectionWithRegisteredName: appName
-                                                            host: @""];
-
-    if (app) {
-      NSConnection *c = [app connectionForProxy];
-
+        workspaceApplication = [NSConnection rootProxyForConnectionWithRegisteredName: appName 
+                                                                                 host: @""];                  
+        if (workspaceApplication) {
+          break;
+        }
+      }
+    }  
+    
+    if (workspaceApplication) {
+      [workspaceApplication setProtocolForProxy: @protocol(workspaceAppProtocol)];
+      RETAIN (workspaceApplication);
+    
 	    [nc addObserver: self
 	           selector: @selector(workspaceAppConnectionDidDie:)
 		             name: NSConnectionDidDieNotification
-		           object: c];
-
-      workspaceApplication = app;
-      [workspaceApplication setProtocolForProxy: @protocol(workspaceAppProtocol)];
-      RETAIN (workspaceApplication);
-      
-	  } else {
-	    static BOOL recursion = NO;
-	  
-      if (recursion == NO) {
-        int i;
-        
-        [ws launchApplication: appName];
-
-        for (i = 1; i <= 80; i++) {
-          NSDate *limit = [[NSDate alloc] initWithTimeIntervalSinceNow: 0.1];
-          [[NSRunLoop currentRunLoop] runUntilDate: limit];
-          RELEASE(limit);
-        
-          app = [NSConnection rootProxyForConnectionWithRegisteredName: appName 
-                                                                   host: @""];                  
-          if (app) {
-            break;
-          }
-        }
-                
-	      recursion = YES;
-	      [self contactWorkspaceApp];
-	      recursion = NO;
-        
-	    } else { 
-	      recursion = NO;
-        NSRunAlertPanel(nil,
-                NSLocalizedString(@"unable to contact the workspace application!", @""),
-                NSLocalizedString(@"Ok", @""),
-                nil, 
-                nil);  
-      }
-	  }
+		           object: [workspaceApplication connectionForProxy]];
+    } else {
+      NSRunAlertPanel(nil,
+              NSLocalizedString(@"unable to contact the workspace application!", @""),
+              NSLocalizedString(@"Ok", @""),
+              nil, 
+              nil);  
+    }
   }
 }
 
@@ -288,74 +272,56 @@ static Recycler *recycler = nil;
 - (void)connectFSWatcher
 {
   if (fswatcher == nil) {
-    id fsw = [NSConnection rootProxyForConnectionWithRegisteredName: @"fswatcher" 
-                                                               host: @""];
+    fswatcher = [NSConnection rootProxyForConnectionWithRegisteredName: @"fswatcher" 
+                                                                  host: @""];
 
-    if (fsw) {
-      NSConnection *c = [fsw connectionForProxy];
+    if (fswatcher == nil) {
+	    NSString *cmd;
+      int i;
+    
+      cmd = [NSTask launchPathForTool: @"fswatcher"];    
+                
+      [startAppWin showWindowWithTitle: @"Recycler"
+                               appName: @"fswatcher"
+                          maxProgValue: 40.0];
+    
+      [NSTask launchedTaskWithLaunchPath: cmd arguments: nil];
+   
+      for (i = 1; i <= 40; i++) {
+        [startAppWin updateProgressBy: 1.0];
+	      [[NSRunLoop currentRunLoop] runUntilDate:
+		                     [NSDate dateWithTimeIntervalSinceNow: 0.1]];
 
-	    [nc addObserver: self
-	           selector: @selector(fswatcherConnectionDidDie:)
-		             name: NSConnectionDidDieNotification
-		           object: c];
-      
-      fswatcher = fsw;
-	    [fswatcher setProtocolForProxy: @protocol(FSWatcherProtocol)];
-      RETAIN (fswatcher);
-                                   
-	    [fswatcher registerClient: (id <FSWClientProtocol>)self];
-      
-	  } else {
-	    static BOOL recursion = NO;
-	    static NSString	*cmd = nil;
-
-	    if (recursion == NO) {
-        if (cmd == nil) {
-            cmd = RETAIN ([[NSSearchPathForDirectoriesInDomains(
-                      GSToolsDirectory, NSSystemDomainMask, YES) objectAtIndex: 0]
-                            stringByAppendingPathComponent: @"fswatcher"]);
-		    }
-      }
-	  
-      if (recursion == NO && cmd != nil) {
-        int i;
-        
-        [startAppWin showWindowWithTitle: @"Recycler"
-                                 appName: @"fswatcher"
-                            maxProgValue: 40.0];
-
-	      [NSTask launchedTaskWithLaunchPath: cmd arguments: nil];
-        RELEASE (cmd);
-        
-        for (i = 1; i <= 40; i++) {
-          [startAppWin updateProgressBy: 1.0];
-	        [[NSRunLoop currentRunLoop] runUntilDate:
-		                       [NSDate dateWithTimeIntervalSinceNow: 0.1]];
-                           
-          fsw = [NSConnection rootProxyForConnectionWithRegisteredName: @"fswatcher" 
-                                                                  host: @""];                  
-          if (fsw) {
-            [startAppWin updateProgressBy: 40.0 - i];
-            break;
-          }
+        fswatcher = [NSConnection rootProxyForConnectionWithRegisteredName: @"fswatcher" 
+                                                                      host: @""];                  
+        if (fswatcher) {
+          [startAppWin updateProgressBy: 40.0 - i];
+          break;
         }
-        
-        [[startAppWin win] close];
-        
-	      recursion = YES;
-	      [self connectFSWatcher];
-	      recursion = NO;
-        
-	    } else { 
-	      recursion = NO;
-        fswnotifications = NO;
-        NSRunAlertPanel(nil,
-                NSLocalizedString(@"unable to contact fswatcher\nfswatcher notifications disabled!", @""),
-                NSLocalizedString(@"Ok", @""),
-                nil, 
-                nil);  
       }
-	  }
+
+      [[startAppWin win] close];
+    }
+    
+    if (fswatcher) {
+      RETAIN (fswatcher);
+      [fswatcher setProtocolForProxy: @protocol(FSWatcherProtocol)];
+    
+	    [[NSNotificationCenter defaultCenter] addObserver: self
+	                   selector: @selector(fswatcherConnectionDidDie:)
+		                     name: NSConnectionDidDieNotification
+		                   object: [fswatcher connectionForProxy]];
+                       
+	    [fswatcher registerClient: (id <FSWClientProtocol>)self 
+                isGlobalWatcher: NO];
+    } else {
+      fswnotifications = NO;
+      NSRunAlertPanel(nil,
+              NSLocalizedString(@"unable to contact fswatcher\nfswatcher notifications disabled!", @""),
+              NSLocalizedString(@"Ok", @""),
+              nil, 
+              nil);  
+    }
   }
 }
 
@@ -388,85 +354,6 @@ static Recycler *recycler = nil;
   }
 }
 
-- (void)connectOperation
-{
-  if (operationsApp == nil) {
-    id opr = [NSConnection rootProxyForConnectionWithRegisteredName: @"Operation" 
-                                                               host: @""];
-
-    if (opr) {
-      NSConnection *c = [opr connectionForProxy];
-
-	    [nc addObserver: self
-	           selector: @selector(operationConnectionDidDie:)
-		             name: NSConnectionDidDieNotification
-		           object: c];
-      
-      operationsApp = opr;
-	    [operationsApp setProtocolForProxy: @protocol(OperationProtocol)];
-      RETAIN (operationsApp);
-      
-	  } else {
-	    static BOOL recursion = NO;
-	  
-      if (recursion == NO) {
-        int i;
-        
-        [startAppWin showWindowWithTitle: @"Recycler"
-                                 appName: @"Operation"
-                            maxProgValue: 80.0];
-
-        [ws launchApplication: @"Operation"];
-
-        for (i = 1; i <= 80; i++) {
-          [startAppWin updateProgressBy: 1.0];
-	        [[NSRunLoop currentRunLoop] runUntilDate:
-		                       [NSDate dateWithTimeIntervalSinceNow: 0.1]];
-          opr = [NSConnection rootProxyForConnectionWithRegisteredName: @"Operation" 
-                                                                  host: @""];                  
-          if (opr) {
-            [startAppWin updateProgressBy: 80.0 - i];
-            break;
-          }
-        }
-        
-        [[startAppWin win] close];
-        
-	      recursion = YES;
-	      [self connectOperation];
-	      recursion = NO;
-        
-	    } else { 
-	      recursion = NO;
-        NSRunAlertPanel(nil,
-                NSLocalizedString(@"unable to contact Operation!", @""),
-                NSLocalizedString(@"Ok", @""),
-                nil, 
-                nil);  
-      }
-	  }
-  }
-}
-
-- (void)operationConnectionDidDie:(NSNotification *)notif
-{
-  id connection = [notif object];
-
-  [nc removeObserver: self
-	              name: NSConnectionDidDieNotification
-	            object: connection];
-
-  NSAssert(connection == [operationsApp connectionForProxy],
-		                                  NSInternalInconsistencyException);
-  RELEASE (operationsApp);
-  operationsApp = nil;
-
-  NSRunAlertPanel(nil, 
-       NSLocalizedString(@"The Operation connection died. File operations disabled!", @""), 
-                              NSLocalizedString(@"OK", @""), nil, nil);                                     
-}
-
-
 //
 // NSServicesRequests protocol
 //
@@ -510,16 +397,20 @@ static Recycler *recycler = nil;
 {
 }
 
-- (void)performFileOperation:(NSDictionary *)opinfo
+- (void)performFileOperation:(NSString *)operation
+		                  source:(NSString *)source
+		             destination:(NSString *)destination
+		                   files:(NSArray *)files
 {
-  [self connectOperation];
+  int tag;
 
-  if (operationsApp) {  
-    NSData *data = [NSArchiver archivedDataWithRootObject: opinfo];
-    [(id <OperationProtocol>)operationsApp performOperation: data];
-  } else {
+  if ([ws performFileOperation: operation 
+                        source: source
+		               destination: destination
+		                     files: files
+			                     tag: &tag] == NO) {
     NSRunAlertPanel(nil, 
-        NSLocalizedString(@"File operations disabled!", @""), 
+        NSLocalizedString(@"Unable to contact GWorkspace", @""), 
                             NSLocalizedString(@"OK", @""), nil, nil);                                     
   }
 }
@@ -558,31 +449,53 @@ static Recycler *recycler = nil;
   return workspaceApplication;
 }
 
+- (oneway void)terminateApplication
+{
+  [NSApp terminate: self];
+}
+
+- (BOOL)terminating
+{
+  return terminating;
+}
+
 
 //
 // Menu Operations
 //
-- (void)emptyTrash:(id)sender
+- (void)emptyTrashFromMenu:(id)sender
 {
+  CREATE_AUTORELEASE_POOL(arp);
   FSNode *node = [FSNode nodeWithPath: trashPath];
-  NSArray *subNodes = [node subNodes];
+  NSMutableArray *subNodes = [[node subNodes] mutableCopy];
+  int count = [subNodes count];
+  int i;  
+
+  for (i = 0; i < count; i++) {
+    FSNode *nd = [subNodes objectAtIndex: i];
+  
+    if ([nd isReserved]) {
+      [subNodes removeObjectAtIndex: i];
+      i--;
+      count --;
+    }
+  }  
   
   if ([subNodes count]) {
     NSMutableArray *files = [NSMutableArray array];
-    NSMutableDictionary *opinfo = [NSMutableDictionary dictionary];
-    int i;  
 
     for (i = 0; i < [subNodes count]; i++) {
       [files addObject: [(FSNode *)[subNodes objectAtIndex: i] name]];
     }
-
-    [opinfo setObject: @"GWorkspaceEmptyRecyclerOperation" forKey: @"operation"];
-    [opinfo setObject: trashPath forKey: @"source"];
-    [opinfo setObject: trashPath forKey: @"destination"];
-    [opinfo setObject: files forKey: @"files"];
-
-    [self performFileOperation: opinfo];
+    
+    [self performFileOperation: @"GWorkspaceEmptyRecyclerOperation"
+		                    source: trashPath
+		               destination: trashPath
+		                     files: files];
   }
+
+  RELEASE (subNodes);
+  RELEASE (arp);
 }
 
 - (void)paste:(id)sender
@@ -592,14 +505,12 @@ static Recycler *recycler = nil;
   if ([[pb types] containsObject: NSFilenamesPboardType]) {
     NSArray *sourcePaths = [pb propertyListForType: NSFilenamesPboardType];   
 
-    if (sourcePaths) {
-      [self connectOperation];
+      [self contactWorkspaceApp];
 
-      if (operationsApp) {
-        BOOL cutted = [(id <OperationProtocol>)operationsApp filenamesWasCutted];
+      if (workspaceApplication) {
+        BOOL cutted = [(id <OperationProtocol>)workspaceApplication filenamesWasCutted];
         
         if ([recview validatePasteOfFilenames: sourcePaths wasCutted: cutted]) {
-          NSMutableDictionary *opDict = [NSMutableDictionary dictionary];
           NSString *source = [[sourcePaths objectAtIndex: 0] stringByDeletingLastPathComponent];
           NSString *destination = trashPath;
           NSMutableArray *files = [NSMutableArray array];
@@ -620,13 +531,11 @@ static Recycler *recycler = nil;
           } else {
 		        operation = NSWorkspaceCopyOperation;
           }
-        
-	        [opDict setObject: operation forKey: @"operation"];
-	        [opDict setObject: source forKey: @"source"];
-	        [opDict setObject: destination forKey: @"destination"];
-	        [opDict setObject: files forKey: @"files"];
 
-	        [self performFileOperation: opDict];	
+          [self performFileOperation: operation
+		                          source: source
+		                     destination: destination
+		                           files: files];
         }
         
       } else {
@@ -635,7 +544,7 @@ static Recycler *recycler = nil;
                                 NSLocalizedString(@"OK", @""), nil, nil); 
         return;                                    
       }
-    }
+
   }
 }
 
@@ -646,21 +555,7 @@ static Recycler *recycler = nil;
 
 - (void)showInfo:(id)sender
 {
-  NSMutableDictionary *d = AUTORELEASE ([NSMutableDictionary new]);
-  [d setObject: @"Recycler" forKey: @"ApplicationName"];
-  [d setObject: NSLocalizedString(@"-----------------------", @"")
-      	forKey: @"ApplicationDescription"];
-  [d setObject: @"Desktop 0.7" forKey: @"ApplicationRelease"];
-  [d setObject: @"06 2004" forKey: @"FullVersionID"];
-  [d setObject: [NSArray arrayWithObjects: @"Enrico Sersale <enrico@imago.ro>.", nil]
-        forKey: @"Authors"];
-  [d setObject: NSLocalizedString(@"See http://www.gnustep.it/enrico/gworkspace", @"") forKey: @"URL"];
-  [d setObject: @"Copyright (C) 2004 Free Software Foundation, Inc."
-        forKey: @"Copyright"];
-  [d setObject: NSLocalizedString(@"Released under the GNU General Public License 2.0", @"")
-        forKey: @"CopyrightDescription"];
-  
-  [NSApp orderFrontStandardInfoPanelWithOptions: d];
+  [NSApp orderFrontStandardInfoPanel: self];
 }
 
 @end
