@@ -1,6 +1,6 @@
 /* lsfupdater.m
  *  
- * Copyright (C) 2004 Free Software Foundation, Inc.
+ * Copyright (C) 2004-2010 Free Software Foundation, Inc.
  *
  * Author: Enrico Sersale <enrico@imago.ro>
  * Date: January 2005
@@ -19,7 +19,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
  */
 
 #include <Foundation/Foundation.h>
@@ -27,6 +27,10 @@
 #include "FinderModulesProtocol.h"
 #include "config.h"
 
+#ifndef GW_DEBUG_LOG
+#define GW_DEBUG_LOG 0
+#endif
+
 #define GWDebugLog(format, args...) \
   do { if (GW_DEBUG_LOG) \
     NSLog(format , ## args); } while (0)
@@ -50,7 +54,6 @@ BOOL subPathOfPath(NSString *p1, NSString *p2);
 
 @protocol	DDBd
 
-- (BOOL)dbactive;
 - (oneway void)insertPath:(NSString *)path;
 - (oneway void)insertDirectoryTreesFromPaths:(NSData *)info;
 - (oneway void)removeTreesFromPaths:(NSData *)info;
@@ -155,16 +158,16 @@ BOOL subPathOfPath(NSString *p1, NSString *p2);
     DESTROY (autoupdateTmr);
   }
   
-	DESTROY (lsfolder);
+  DESTROY (lsfolder);
   DESTROY (ddbd);
 
   RELEASE (modules);
-  TEST_RELEASE (searchPaths);
-  TEST_RELEASE (searchCriteria);
-  TEST_RELEASE (lastUpdate);
-  TEST_RELEASE (startSearch);
+  RELEASE (searchPaths);
+  RELEASE (searchCriteria);
+  RELEASE (lastUpdate);
+  RELEASE (startSearch);
   RELEASE (foundPaths);
-  TEST_RELEASE (directories);
+  RELEASE (directories);
   
   [super dealloc];
 }
@@ -264,17 +267,22 @@ BOOL subPathOfPath(NSString *p1, NSString *p2);
 - (void)loadModules
 {
   CREATE_AUTORELEASE_POOL(arp);
+  NSEnumerator *enumerator;
   NSString *bundlesDir;
   BOOL isdir;
   NSMutableArray *bundlesPaths;
   NSArray *classNames;
   int i;
 
-  bundlesDir = [NSSearchPathForDirectoriesInDomains(NSLibraryDirectory, NSSystemDomainMask, YES) lastObject];
-  bundlesDir = [bundlesDir stringByAppendingPathComponent: @"Bundles"];
   bundlesPaths = [NSMutableArray array];
-  [bundlesPaths addObjectsFromArray: [self bundlesWithExtension: @"finder" 
-                                                         inPath: bundlesDir]];
+  enumerator = [NSSearchPathForDirectoriesInDomains
+    (NSLibraryDirectory, NSAllDomainsMask, YES) objectEnumerator];
+  while ((bundlesDir = [enumerator nextObject]) != nil)
+    {
+      bundlesDir = [bundlesDir stringByAppendingPathComponent: @"Bundles"];
+      [bundlesPaths addObjectsFromArray:
+	[self bundlesWithExtension: @"finder" inPath: bundlesDir]];
+    }
 
   bundlesDir = [NSSearchPathForDirectoriesInDomains(NSLibraryDirectory, NSUserDomainMask, YES) lastObject];
   bundlesDir = [bundlesDir stringByAppendingPathComponent: @"GWorkspace"];
@@ -720,22 +728,29 @@ BOOL subPathOfPath(NSString *p1, NSString *p2);
   NSMutableArray *founds = [NSMutableArray array];
   NSDirectoryEnumerator *enumerator = [fm enumeratorAtPath: dirpath];
   IMP nxtImp = [enumerator methodForSelector: @selector(nextObject)];    
-  NSString *path;
 
-  while ((path = (*nxtImp)(enumerator, @selector(nextObject))) != nil) { 
+  while (1) {
     CREATE_AUTORELEASE_POOL(arp1); 
-    NSString *fullPath = [dirpath stringByAppendingPathComponent: path];
-    NSDictionary *attrs = [enumerator fileAttributes];
-    
-    if ([self checkPath: fullPath attributes: attrs]) {    
-      [founds addObject: fullPath];
-    }
+    NSString *path = (*nxtImp)(enumerator, @selector(nextObject));
 
-    if (([attrs fileType] == NSFileTypeDirectory) && norecursion) {
-      [enumerator skipDescendents];
+    if (path) {
+      NSString *fullPath = [dirpath stringByAppendingPathComponent: path];
+      NSDictionary *attrs = [enumerator fileAttributes];
+
+      if ([self checkPath: fullPath attributes: attrs]) {    
+        [founds addObject: fullPath];
+      }
+
+      if (([attrs fileType] == NSFileTypeDirectory) && norecursion) {
+        [enumerator skipDescendents];
+      }
+
+    } else {
+      RELEASE (arp1);
+      break;
     }
     
-    RELEASE (arp1);
+    RELEASE (arp1); 
   }
   
   RETAIN (founds);
@@ -816,64 +831,43 @@ BOOL subPathOfPath(NSString *p1, NSString *p2);
 - (void)connectDDBd
 {
   if (ddbd == nil) {
-    id db = [NSConnection rootProxyForConnectionWithRegisteredName: @"ddbd" 
-                                                              host: @""];
+    ddbd = [NSConnection rootProxyForConnectionWithRegisteredName: @"ddbd" 
+                                                             host: @""];
 
-    if (db) {
-      NSConnection *c = [db connectionForProxy];
-
-	    [nc addObserver: self
-	           selector: @selector(ddbdConnectionDidDie:)
-		             name: NSConnectionDidDieNotification
-		           object: c];
-      
-      ddbd = db;
-	    [ddbd setProtocolForProxy: @protocol(DDBd)];
-      RETAIN (ddbd);
-      
-      GWDebugLog(@"ddbd connected!");     
-                                         
-	  } else {
-	    static BOOL recursion = NO;
-	    static NSString	*cmd = nil;
-
-	    if (recursion == NO) {
-        if (cmd == nil) {
-            cmd = RETAIN ([[NSSearchPathForDirectoriesInDomains(
-                      GSToolsDirectory, NSSystemDomainMask, YES) objectAtIndex: 0]
-                            stringByAppendingPathComponent: @"ddbd"]);
-		    }
-      }
-	  
-      if (recursion == NO && cmd != nil) {
-        int i;
-        
-	      [NSTask launchedTaskWithLaunchPath: cmd arguments: nil];
-        DESTROY (cmd);
-        
-        for (i = 1; i <= 40; i++) {
-	        [[NSRunLoop currentRunLoop] runUntilDate:
+    if (ddbd == nil) {
+	    NSString *cmd;
+      int i;
+    
+      cmd = [NSTask launchPathForTool: @"ddbd"];    
+                
+      [NSTask launchedTaskWithLaunchPath: cmd arguments: nil];
+   
+      for (i = 0; i < 40; i++) {
+        [[NSRunLoop currentRunLoop] runUntilDate:
 		                       [NSDate dateWithTimeIntervalSinceNow: 0.1]];
-                           
-          db = [NSConnection rootProxyForConnectionWithRegisteredName: @"ddbd" 
+
+        ddbd = [NSConnection rootProxyForConnectionWithRegisteredName: @"ddbd" 
                                                                  host: @""];                  
-          if (db) {
-            break;
-          }
+        if (ddbd) {
+          break;
         }
-        
-	      recursion = YES;
-	      [self connectDDBd];
-	      recursion = NO;
-        
-	    } else { 
-        DESTROY (cmd);
-	      recursion = NO;
-        ddbd = nil;
-        
-        [lsfolder updaterError: @"unable to contact ddbd."];
       }
-	  }
+    }
+    
+    if (ddbd) {
+      RETAIN (ddbd);
+      [ddbd setProtocolForProxy: @protocol(DDBd)];
+    
+	    [[NSNotificationCenter defaultCenter] addObserver: self
+	                   selector: @selector(ddbdConnectionDidDie:)
+		                     name: NSConnectionDidDieNotification
+		                   object: [ddbd connectionForProxy]];
+                       
+      GWDebugLog(@"ddbd connected!");     
+    } else {
+      NSLog(@"unable to contact ddbd.");
+      [lsfolder updaterError: @"unable to contact ddbd."];
+    }
   }
 }
 
@@ -896,7 +890,7 @@ BOOL subPathOfPath(NSString *p1, NSString *p2);
 - (void)ddbdInsertTrees
 {
   [self connectDDBd];
-  if (ddbd && [ddbd dbactive]) {
+  if (ddbd != nil) {
     NSData *info = [NSArchiver archivedDataWithRootObject: searchPaths];
     
     [NSTimer scheduledTimerWithTimeInterval: 10
@@ -912,7 +906,7 @@ BOOL subPathOfPath(NSString *p1, NSString *p2);
 - (void)ddbdInsertDirectoryTreesFromPaths:(NSArray *)paths
 {
   [self connectDDBd];
-  if (ddbd && [ddbd dbactive]) {
+  if (ddbd != nil) {
     NSData *info = [NSArchiver archivedDataWithRootObject: paths];
     [ddbd insertDirectoryTreesFromPaths: info];
   }
@@ -921,7 +915,7 @@ BOOL subPathOfPath(NSString *p1, NSString *p2);
 - (NSArray *)ddbdGetDirectoryTreeFromPath:(NSString *)path
 {
   [self connectDDBd];
-  if (ddbd && [ddbd dbactive]) {
+  if (ddbd != nil) {
     NSData *data = [ddbd directoryTreeFromPath: path];  
 
     if (data) {
@@ -935,7 +929,7 @@ BOOL subPathOfPath(NSString *p1, NSString *p2);
 - (void)ddbdRemoveTreesFromPaths:(NSArray *)paths
 {
   [self connectDDBd];
-  if (ddbd && [ddbd dbactive]) {
+  if (ddbd != nil) {
     [ddbd removeTreesFromPaths: [NSArchiver archivedDataWithRootObject: paths]];
   }
 }
@@ -943,7 +937,7 @@ BOOL subPathOfPath(NSString *p1, NSString *p2);
 - (NSString *)ddbdGetAnnotationsForPath:(NSString *)path
 {
   [self connectDDBd];
-  if (ddbd && [ddbd dbactive]) {
+  if (ddbd != nil) {
     return [ddbd annotationsForPath: path];
   }
   return nil;
@@ -952,7 +946,7 @@ BOOL subPathOfPath(NSString *p1, NSString *p2);
 - (NSTimeInterval)ddbdGetTimestampOfPath:(NSString *)path
 {
   [self connectDDBd];
-  if (ddbd && [ddbd dbactive]) {
+  if (ddbd != nil) {
     return [ddbd timestampOfPath: path];
   }
   return 0.0;
