@@ -1,6 +1,6 @@
 /* main.m
  *  
- * Copyright (C) 2003 Free Software Foundation, Inc.
+ * Copyright (C) 2003-2010 Free Software Foundation, Inc.
  *
  * Author: Enrico Sersale <enrico@imago.ro>
  * Date: August 2001
@@ -19,15 +19,13 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
  */
 
 #include <Foundation/Foundation.h>
 #include <AppKit/AppKit.h>
 #include <math.h>
-#include <unistd.h>
 #include <limits.h>
-#include "GNUstep.h"
 
 #define TMBMAX (48.0)
 #define RESZLIM 4
@@ -114,10 +112,10 @@ static NSString *GWThumbnailsDidChangeNotification = @"GWThumbnailsDidChangeNoti
   RELEASE (thumbnailers);
   RELEASE (extProviders);
   RELEASE (thumbnailDir);
-  TEST_RELEASE (dictPath);
-  TEST_RELEASE (thumbsDict);
+  RELEASE (dictPath);
+  RELEASE (thumbsDict);
   DESTROY (conn);
-	[super dealloc];
+  [super dealloc];
 }
 
 - (id)init
@@ -189,11 +187,12 @@ static NSString *GWThumbnailsDidChangeNotification = @"GWThumbnailsDidChangeNoti
 - (void)loadThumbnailers
 {
   NSString *bundlesDir;
-	NSMutableArray *bundlesPaths;
-	NSArray *bPaths;
+  NSEnumerator *enumerator;
+  NSMutableArray *bundlesPaths;
+  NSArray *bPaths;
   int i;
   
-  TEST_RELEASE (thumbnailers);
+  RELEASE (thumbnailers);
   thumbnailers = [NSMutableArray new];
   
   bundlesPaths = [NSMutableArray array]; 
@@ -202,14 +201,14 @@ static NSString *GWThumbnailsDidChangeNotification = @"GWThumbnailsDidChangeNoti
                           inDirectory: [[NSBundle mainBundle] resourcePath]];
 	[bundlesPaths addObjectsFromArray: bPaths];
 
-  bundlesDir = [NSSearchPathForDirectoriesInDomains(NSLibraryDirectory, NSUserDomainMask, YES) lastObject];
-  [bundlesPaths addObjectsFromArray: [self bundlesWithExtension: @"thumb" 
-			                                              inDirectory: bundlesDir]];
-
-  bundlesDir = [NSSearchPathForDirectoriesInDomains(NSLibraryDirectory, NSSystemDomainMask, YES) lastObject];
-  bundlesDir = [bundlesDir stringByAppendingPathComponent: @"Bundles"];
-  [bundlesPaths addObjectsFromArray: [self bundlesWithExtension: @"thumb" 
-			                                              inDirectory: bundlesDir]];
+  enumerator = [NSSearchPathForDirectoriesInDomains
+    (NSLibraryDirectory, NSAllDomainsMask, YES) objectEnumerator];
+  while ((bundlesDir = [enumerator nextObject]) != nil)
+    {
+      bundlesDir = [bundlesDir stringByAppendingPathComponent: @"Bundles"];
+      [bundlesPaths addObjectsFromArray:
+	[self bundlesWithExtension: @"thumb" inDirectory: bundlesDir]];
+    }
 
   for (i = 0; i < [bundlesPaths count]; i++) {
 		NSString *bpath = [bundlesPaths objectAtIndex: i];
@@ -656,37 +655,60 @@ static NSString *GWThumbnailsDidChangeNotification = @"GWThumbnailsDidChangeNoti
 
 @end
 
-int main(int argc, char** argv, char **env_c)
+int main(int argc, char** argv)
 {
-  NSAutoreleasePool *pool;
-	Thumbnailer *thumbnailer;
+  CREATE_AUTORELEASE_POOL(pool);
+  NSProcessInfo *info = [NSProcessInfo processInfo];
+  NSMutableArray *args = AUTORELEASE ([[info arguments] mutableCopy]);
+  static BOOL	is_daemon = NO;
+  BOOL subtask = YES;
 
-	switch (fork()) {
-	  case -1:
-	    fprintf(stderr, "Thumbnailer - fork failed - bye.\n");
-	    exit(1);
+  if ([[info arguments] containsObject: @"--daemon"]) {
+    subtask = NO;
+    is_daemon = YES;
+  }
 
-	  case 0:
-	    setsid();
-	    break;
-
-	  default:
-	    exit(0);
-	}
-  
-  pool = [NSAutoreleasePool new];
-	thumbnailer = [[Thumbnailer alloc] init];
-  RELEASE (pool);
-  
-  if (thumbnailer != nil) {
-    pool = [NSAutoreleasePool new];
+  if (subtask) {
+    NSTask *task = [NSTask new];
     
-    [NSApplication sharedApplication];
-    NSRegisterServicesProvider(thumbnailer, @"Thumbnailer");
-    [[NSRunLoop currentRunLoop] run];
-    
-  	RELEASE (pool);
+    NS_DURING
+	    {
+	      [args removeObjectAtIndex: 0];
+	      [args addObject: @"--daemon"];
+	      [task setLaunchPath: [[NSBundle mainBundle] executablePath]];
+	      [task setArguments: args];
+	      [task setEnvironment: [info environment]];
+	      [task launch];
+	      DESTROY (task);
+	    }
+    NS_HANDLER
+	    {
+	      fprintf (stderr, "unable to launch the thumbnailer task. exiting.\n");
+	      DESTROY (task);
+	    }
+    NS_ENDHANDLER
+      
+    exit(EXIT_FAILURE);
   }
   
-  exit(0);
+  RELEASE(pool);
+
+  {
+    CREATE_AUTORELEASE_POOL (pool);
+	  Thumbnailer *thumbnailer = [[Thumbnailer alloc] init];
+    RELEASE (pool);
+
+    if (thumbnailer != nil) {
+	    CREATE_AUTORELEASE_POOL (pool);
+
+      [NSApplication sharedApplication];
+      NSRegisterServicesProvider(thumbnailer, @"Thumbnailer");
+      [[NSRunLoop currentRunLoop] run];
+
+  	  RELEASE (pool);
+    }
+  }
+    
+  exit(EXIT_SUCCESS);
 }
+
